#! /bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/db/Oracle8i/cfgtool/call_procs.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
#Updated 05/02/2009 - valnt00077479
DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`

ret_code=0

# ------------------------------------------------------------------------------
# Check the first parameter (the exit code) and return from the script if 
# non-zero
# ------------------------------------------------------------------------------
check_exit_code() {
   if [ "$1" -ne 0 ]
   then
      exit $1;
   fi
}

# ------------------------------------------------------------------------------
# Unplug the NE tree
# ------------------------------------------------------------------------------
if [ "$ne_unplugin" == "Y" ]
then
   echo " "
   echo "`date +%Y%m%d\ %H:%M:%S` Unplugging NE tree starting from entity $ne_module "
   
   sqlplus -s /NOLOG << !

      variable return_status NUMBER
   
      whenever sqlerror exit :return_status

      set feed off
      set serveroutput on size 1000000
      set timing on
      connect ${DB_CONNECT}   

      begin

         :return_status := 0;
         ne_delete.main('$ne_module');

      exception
  
      when others then
    
         :return_status := 1;
         rollback;
   
      end;
   /
   exit :return_status
!
   # Nothing more to do if unplugging ne tree
   return $?

fi


# ------------------------------------------------------------------------------
# Reload the entity information
# ------------------------------------------------------------------------------
if [ "$update_ctx" == "Y" ]
then
   echo "`date +%Y%m%d\ %H:%M:%S` Reloading transition .... "
else
   echo "`date +%Y%m%d\ %H:%M:%S` Loading entity information into dictionary .... "
fi

## this block is to parse main.xml and load entity info.
sqlplus -s /NOLOG << !
   set feed off
   set serveroutput on size 1000000
   set timing on
   connect ${DB_CONNECT}   

   variable ret_code_sh NUMBER

   whenever sqlerror exit :return_code_sh

   DECLARE
     
      ret_code_sql number;
    
   BEGIN   
    
      :ret_code_sh := 0;

      configtoolparser.main('$FLEXPM_HOME','$DBUSER','$update_config','$clean_procs','$ne_plugin','$ne_module','$update_template','$update_ctx',ret_code_sql);

      :ret_code_sh := ret_code_sql;

   EXCEPTION

      WHEN OTHERS THEN
        
         :ret_code_sh := 1;
         rollback;

   END;
/
EXIT :ret_code_sh
!

check_exit_code $?


# ------------------------------------------------------------------------------
# Load dictionary fields only if it is not report template update
# or entity context update
# ------------------------------------------------------------------------------

if [ "$update_template" == "N" -a "$update_ctx" == "N" ]
then

   echo "`date +%Y%m%d\ %H:%M:%S` Loading field information into dictionary .... "

   # the following block is to load dictionary files
   sqlplus -s /NOLOG << !
      set feed off
      set serverout on size 1000000
      set timing on
      connect ${DB_CONNECT}  

      variable return_status NUMBER

      BEGIN 
         :return_status := 0;
         load_dictionary_fields.load_dictionary;      

      EXCEPTION

         WHEN OTHERS THEN
      
           :return_status := 1;
           rollback;
            
      END;
   /
   EXIT :return_status   
!

   check_exit_code $?

fi

# ------------------------------------------------------------------------------
# Create the schema objects 
# ------------------------------------------------------------------------------
echo "`date +%Y%m%d\ %H:%M:%S` Creating schema objects .... "

## following block is to create tables

sqlplus -s /NOLOG << !
   set feed off
   set serveroutput on size 1000000
   set timing on
   connect ${DB_CONNECT}   

   variable return_status NUMBER
   whenever sqlerror exit :return_status

   BEGIN

      :return_status := 0;

      create_schema_objects.create_ent_tables('$update_config');

   EXCEPTION

      WHEN OTHERS THEN
    
         :return_status := 1;
         rollback;

   END;
/
EXIT :return_status
!

check_exit_code $?


# ------------------------------------------------------------------------------
# Load pm_templatefield and filter
# ------------------------------------------------------------------------------
if [ "$update_ctx" == "N" ]
then

   echo "`date +%Y%m%d\ %H:%M:%S` Loading report template fields .... "

   ## following block is to load pm_templatefield and filter
   sqlplus -s /NOLOG << !
      set feed off
      set serveroutput on size 1000000
      set timing on
      connect ${DB_CONNECT}   

      variable return_status NUMBER
      whenever sqlerror exit :return_status

      BEGIN

         :return_status := 0;

         configtoolparser.load_pm_template_field;

      EXCEPTION

         WHEN OTHERS THEN
 
            :return_status := 1;
            rollback;

      END;
   /
   EXIT :return_status
!

   check_exit_code $?
   
fi

# ------------------------------------------------------------------------------
# Load file table map and group
# ------------------------------------------------------------------------------
echo "`date +%Y%m%d\ %H:%M:%S` Loading file table map and group .... "

## following block is to to load file table map and group
sqlplus -s /NOLOG << !
   set feed off
   set serveroutput on size 1000000
   set timing on
   connect ${DB_CONNECT}  
   variable return_status NUMBER
   whenever sqlerror exit :return_status

   BEGIN

      :return_status := 0;

      configtoolparser.ins_file_table_map;
   EXCEPTION
      WHEN OTHERS THEN

         :return_status := 1;
         rollback;
   
   END;
/
exit :return_status         
!

check_exit_code $?


# ------------------------------------------------------------------------------
# Call addondictupdate to create fields for user_templates
# ------------------------------------------------------------------------------ 
echo "`date +%Y%m%d\ %H:%M:%S` Performing addondictupdate  ... "

sqlplus -s /NOLOG << !
   set feed off
   set serveroutput on size 1000000
   set timing on
   connect ${DB_CONNECT}
   variable return_status NUMBER
   whenever oserror exit 1
   
   @$FLEXPM_HOME/db/Oracle8i/dictionary/scripts/add_on_dict_update.sql
!

check_exit_code $?

# ------------------------------------------------------------------------------
# Call dict_restore
# ------------------------------------------------------------------------------ 
echo "`date +%Y%m%d\ %H:%M:%S` Performing dict restore ... "

sqlplus -s /NOLOG << !
   set feed off
   set serveroutput on size 1000000
   set timing on
   connect ${DB_CONNECT} 
   variable return_status NUMBER
   whenever sqlerror exit :return_status
  
   BEGIN
 
      :return_status := 0;

      dict_update.dict_restore('$update_config');

  EXCEPTION

     WHEN OTHERS THEN
     
        :return_status := 1;
        rollback;
 END;
/
exit :return_status         
!

check_exit_code $? 

# ------------------------------------------------------------------------------
# Check for timestamp
# ------------------------------------------------------------------------------
echo "`date +%Y%m%d\ %H:%M:%S` Check for tstamp ... "
## following block calls check_for_tstamp 
sqlplus -s /NOLOG << !
   set feed off
   set serveroutput on size 1000000
   set timing on
   connect ${DB_CONNECT}  
   variable return_status NUMBER
   whenever sqlerror exit :return_status

   BEGIN

       :return_status := 0;

       configtoolparser.check_for_tstamp;
   
   EXCEPTION

      WHEN OTHERS THEN 

         :return_status := 1;
         rollback;
   
   end;
/
exit :return_status   
!

check_exit_code $?


# ------------------------------------------------------------------------------
# Create the start tables (used by the loaders)
# ------------------------------------------------------------------------------
echo "`date +%Y%m%d\ %H:%M:%S` Creating start tables used by loaders ...."

## following block is to create start tables

sqlplus -s /NOLOG << !
   set feed off
   set serveroutput on size 1000000
   set timing on
   connect ${DB_CONNECT}
   variable return_code_sh NUMBER
   whenever sqlerror exit :return_code_sh

   BEGIN

      :return_code_sh := 0;

      configtoolparser.create_start_tabs;

   EXCEPTION
      WHEN OTHERS THEN
         :return_code_sh := 1;
         rollback;
   
   end;
/
exit :return_code_sh  
!

check_exit_code $?


# ------------------------------------------------------------------------------
# Some misc commands
# ------------------------------------------------------------------------------

# Prepare the templates for an update.  
chmod u+w $FLEXPM_HOME/ds/fx/*
chmod u+w $FLEXPM_HOME/ds/fx/templates/*

