#!/usr/bin/env perl -w
#-----------------------------------------------------------------------
# (#) ./base/db/Oracle8i/cfgtool/cft_xml_exp.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
# Copyright 2003 WatchMark Corp.
#-----------------------------------------------------------------------
#

use strict;
use English;
use Getopt::Long;
use File::Copy;
use File::Basename;

#BEGIN {
#  unshift @INC, $ENV{ FLEXPM_HOME } . "/sw/perl/lib"  if   exists $ENV{ FLEXPM_HOME };
#}

#Updated 11/02/2009 - valnt00077479
use pass_crypt;

use DBI;

sub showUsage;
sub xml_dump($$);
sub xml_header($);
sub create_sbp_file;
sub copy_dtd_files;
sub create_xml_sym_cvt($);
sub get_primary_key($$);
sub create_cft_folder_fullpath($);

############################
# subroutine for main.xml
############################
sub get_context;
sub get_data_source;
sub get_field_group;
sub get_technology;
sub get_filetype;
sub get_system_values;
sub get_tableType;
sub get_tableDefinition;
sub get_product_info($);
sub get_help_file_type($);
sub get_help_file($$);
sub get_entitys($);
sub get_entity($$);
sub get_server_module($);
sub get_loader_source($);
sub get_extra_ent_property($$);
# sub get_loader_ent_property($$); # diasabled as per SEAde34640

############################
# subroutine for dict.xml
############################
sub get_tech($$);
sub get_subType($$);
sub get_alias($$);
sub get_enum($$);
sub getDictTypeE($$);
sub getDictTypeC($$);
sub getDictTypeF($$);
sub getDictTypeS($$);
sub get_weight_field($$);
sub get_col_info($$);
sub get_extra_fld_property($$$);
# sub get_loader_fld_property($$$); # diasabled as per SEAde34640
sub get_MeasurementType($$$);
sub get_FontStyle($$);


#######################################
# START
#######################################
  my $dbh;
  my $dbConnect=pass_crypt::getConnectForUser($ENV{DBUSER});
  my $dbUser;
  my $dbPassword;
  my $dbTnsname;
  my $path;
  my $showHelp;
  my $entity_id;
  my $entity_name;
  my $file_name;
  my ($vendor, $version, $compat);

  &GetOptions( 'connect=s' => \$dbConnect,'help' => \$showHelp, 'dest_dir=s' => \$path,
               'vendor=s' => \$vendor,'version=s' => \$version, 'compat=s' => \$compat);

  showUsage if defined $showHelp;

  if ( not defined $path ) {
    if ( exists $ENV{ FLEXPM_HOME } ) {
      $path = $ENV{ FLEXPM_HOME } . "/";
    }
    else {
      $path = "./" ;
    }
  }
  else {
    $path=$path . "/";
  }

  (defined $vendor) || ($vendor="None");
  (defined $version) || ($version="None");
  (defined $compat) || ($compat="None");

 my $flexpm_home=$ENV{ FLEXPM_HOME };
 my $main_dtd=$flexpm_home."\/peik\/xml\/main.dtd";
 my $dict_dtd=$flexpm_home."\/peik\/xml\/dict.dtd";

 copy_dtd_files;

 create_sbp_file;
 $dbh = DBI->connect( 'dbi:Oracle:', $dbConnect, '',
                         { PrintError => 0, ora_module_name => "SchemaInf.pm" } )
    or die "Error: Can't connect to '$dbConnect' ($DBI::errstr).\n";
 
 my $sth;
 #######################################
 # Start preparing tablespaces snapshot
 #######################################
 print "Starting Creating tableName and tablespaceName snapshot...\n";

 $dbh->{RaiseError}=0;

 $sth=$dbh->prepare( " DROP TABLE wm_xml_tab_tablespace_mv " );
 $sth->execute;
 $sth->finish();
 
 $dbh->{RaiseError}=1;
 $sth = $dbh->prepare( "CREATE TABLE wm_xml_tab_tablespace_mv
                        AS SELECT DISTINCT us.tablespace_name,
                                           table_name
                             FROM user_tables us
                            WHERE us.tablespace_name IS NOT null
                            UNION
                           SELECT DISTINCT us.tablespace_name,
                                            table_name
                             FROM part_tables us
                             where table_name like '%TAB'
                            
                        " );
 $sth->execute;
 $sth->finish();
 $sth = $dbh->prepare( "CREATE INDEX ind_wm_xml_tab_tablespace_mv
                        ON wm_xml_tab_tablespace_mv(table_name)");
 $sth->execute;
 $sth->finish();
 print "Finish Creating tableName and tablespaceName snapshot\n";
 #######################################
 # End preparing tablespaces snapshot
 #######################################

 ###############################################################
 # start creating xml_sym_cvt and cft_folder_fullpath functions
 ###############################################################
 create_xml_sym_cvt('Create');
 create_cft_folder_fullpath('Create');
 ###############################################################
 # end creating xml_sym_cvt and cft_folder_fullpath functions
 ###############################################################

 ##################
 # Starte main.xml
 ##################
 print "Start exporting " . "main.xml" . " ...\n";
 open(MAINFILE, "> $path"."main.xml");
 xml_dump(\*MAINFILE, "<?xml version='1.0'?>\n");
 xml_dump(\*MAINFILE, "<!DOCTYPE vendor SYSTEM \"main.dtd\">\n");
 xml_dump(\*MAINFILE, "<vendor name=\"$vendor\" version=\"$version\" compat=\"$compat\">\n");
 get_context;
 get_data_source;
 get_field_group;
 get_technology;
 get_filetype;
 get_system_values;
 get_server_module(\*MAINFILE);
 get_tableType;
 get_tableDefinition;
 get_loader_source(\*MAINFILE);
 get_product_info(\*MAINFILE);
 get_help_file_type(\*MAINFILE);
 get_entitys(\*MAINFILE);

 xml_dump(\*MAINFILE, "</vendor>\n");
 close(MAINFILE);
 print "End exporting " . "main.xml" . " ...\n";
 ##################
 # End main.xml
 ##################
  
 $sth = $dbh->prepare( "SELECT entity_id,
                               entity_name
                          FROM wmn_entity
                      ORDER BY entity_id" );

 $sth->execute;

 $sth->bind_columns(undef, \$entity_id, \$entity_name);

 while( $sth->fetch() ) {
   $file_name="$path" . "$entity_name" . ".xml";
   print "Start exporting " . $entity_name . " ...\n";
   open(XMLOUTPUT, "> "."$file_name");
   xml_dump(\*XMLOUTPUT, "<?xml version='1.0'?>\n");
   xml_dump(\*XMLOUTPUT, "<!DOCTYPE ROWSET SYSTEM \"dict.dtd\">\n");
   xml_dump(\*XMLOUTPUT, "<ROWSET>\n");
   getDictTypeE(\*XMLOUTPUT, $entity_id);
   getDictTypeC(\*XMLOUTPUT, $entity_id);
   getDictTypeF(\*XMLOUTPUT, $entity_id);
   getDictTypeS(\*XMLOUTPUT, $entity_id);
   xml_dump(\*XMLOUTPUT, "</ROWSET>\n");
   close(XMLOUTPUT);
   print "Finish exporting " . $entity_name . "\n";
 }
 
 
 ################################################################
 # start dropping xml_sym_cvt and cft_folder_fullpath functions
 ################################################################
 create_xml_sym_cvt('Drop');
 create_cft_folder_fullpath('Drop');
 ################################################################
 # end dropping xml_sym_cvt and cft_folder_fullpath functions
 ################################################################

 $sth=$dbh->prepare( " DROP TABLE wm_xml_tab_tablespace_mv " );
 $sth->execute;
 $sth->finish();

 my $rc = $dbh->disconnect;
 print "Completed exporting prospect dictionary in XML files.\n";
##################################
# END 
##################################

sub getDictTypeE($$)
{
 my $FILE=shift;
 my $entity_id=shift;
 my $field_id;
 my $entity_name;
 my $name;
 my $field_label;
 my $Report_Header;
 my $Report_Header2;
 my $datatype;
 my $field_type;
 my $field_desc;
 my $field_aggregator;
 my $field_emits_aggregator;
 my $datasource;
 my $field_group;
 my $weight_field_id;
 my $calc_source;
 my $fontstylemask;
 my $fontcolor;
 my $udr_key_order;

 my $sqlDictTypeE=qq{ SELECT fd.field_id,
                             xml_sym_cvt(en.entity_name),
                             xml_sym_cvt(dn.name),
                             xml_sym_cvt(dn.edit_label),
                             xml_sym_cvt(dn.Report_Header),
                             xml_sym_cvt(dn.Report_Header2),
                             fd.datatype,
                             fd.field_type,
                             xml_sym_cvt(fd.field_desc),
                             field_aggregator,
                             field_emits_aggregator,
                             xml_sym_cvt(ds.datasource_name),
                             xml_sym_cvt(fg.field_group_short),
                             fd.weight_field_id,
                             xml_sym_cvt(cd.calc_source),
                             fs.fontstylemask,
                             fs.fontcolor,
                             uk.field_seq udr_key_order
                        FROM wmd_field_def fd, wmd_dict_name dn,
                             wmn_entity en, wmd_data_source ds,
                             wmd_field_group fg, wmd_calc_def cd,
                             wmd_field_style fs, wmd_udr_keys uk
                       WHERE fd.field_id=dn.field_id
                         AND dn.namespace_id=0
                         AND fd.field_type='E'
                         AND fd.entity_id=en.entity_id
                         AND fd.datasource_id=ds.datasource_id(+)
                         AND fd.field_group_id=fg.field_group_id(+)
                         AND fd.field_id=cd.field_id
                         AND fd.field_id=fs.field_id(+)
                         AND fd.entity_id=uk.entity_id(+)
                         AND fd.field_id=uk.field_id(+)
                         AND fd.entity_id= ?
                       ORDER BY dn.name
                    };

 my $sth_DictTypeE = $dbh->prepare($sqlDictTypeE);
 $sth_DictTypeE->bind_param(1, $entity_id);
 $sth_DictTypeE->execute;
 $sth_DictTypeE->bind_columns(undef,
                              \$field_id, \$entity_name, \$name,
                              \$field_label, \$Report_Header,
                              \$Report_Header2, \$datatype, \$field_type,
                              \$field_desc, \$field_aggregator, \$field_emits_aggregator, \$datasource,
                              \$field_group, \$weight_field_id, \$calc_source,
                              \$fontstylemask, \$fontcolor, \$udr_key_order
                             );

 while ($sth_DictTypeE->fetch() ) {
   xml_dump(\*$FILE, "<ROW>");
   xml_dump(\*$FILE, "<entity_name>$entity_name</entity_name>");
   xml_dump(\*$FILE, "<field_name>$name</field_name>");
   (defined $field_label) && xml_dump(\*$FILE, "<field_label>$field_label</field_label>");
   (defined $Report_Header) && xml_dump(\*$FILE, "<field_header>$Report_Header</field_header>");
   (defined $Report_Header2) && xml_dump(\*$FILE, "<field_header2>$Report_Header2</field_header2>");
   xml_dump(\*$FILE, "<field_type>$field_type</field_type>");
   (defined $field_desc) && xml_dump(\*$FILE, "<field_desc>$field_desc</field_desc>");
   xml_dump(\*$FILE, "<field_aggregator>$field_aggregator</field_aggregator>");
   (defined $field_emits_aggregator) &&
     xml_dump(\*$FILE, "<field_emits_aggregator>$field_emits_aggregator</field_emits_aggregator>");
   (defined $datasource) && xml_dump(\*$FILE, "<datasource>$datasource</datasource>");
   (defined $field_group) && xml_dump(\*$FILE, "<field_group>$field_group</field_group>");
   xml_dump(\*$FILE, "<datatype>$datatype</datatype>");
   (defined $calc_source) && xml_dump(\*$FILE, "<calc_source>$calc_source</calc_source>");
   (defined $weight_field_id) && get_weight_field(\*$FILE, $weight_field_id);

   get_MeasurementType(\*$FILE, $entity_id, $field_id);

   (defined $fontcolor) && xml_dump(\*$FILE, "<color>$fontcolor</color>");
   (defined $fontstylemask) && get_FontStyle(\*$FILE, $fontstylemask);
   (defined $udr_key_order) && xml_dump(\*$FILE, "<udr_key_order>$udr_key_order</udr_key_order>");

   #######################################
   # Field Type "E" has tech, alias
   #######################################

   get_tech(\*$FILE, $field_id);
   get_alias(\*$FILE, $field_id);

   ########################################
   # Start Extra Entity field Property
   ########################################
   get_extra_fld_property(\*$FILE, $entity_id, $field_id);
   #####################################
   # End Extra Entity field Property
   #####################################
 
   ########################################
   # Start Extra Loader field Property
   ########################################
   # get_loader_fld_property(\*$FILE, $entity_id, $field_id); # disabled as per SEAde34640
   #####################################
   # End Extra Loader field Property
   #####################################

   xml_dump(\*$FILE, "</ROW>"."\n");
}
$sth_DictTypeE->finish();

}


sub getDictTypeC($$)
{
 my $FILE=shift;
 my $entity_id=shift;

 my $field_id;
 my $entity_name;
 my $name;
 my $field_label;
 my $Report_Header;
 my $Report_Header2;
 my $datatype;
 my $field_type;
 my $field_desc;
 my $field_aggregator;
 my $field_emits_aggregator;
 my $datasource;
 my $field_group;
 my $weight_field_id;
 my $table_name;
 my $col_name;
 my $precision;
 my $length;
 my $is_read_only;
 my $is_null_ok;
 my $is_frozen;
 my $field_order;
 my $default_value;
 my $min_value;
 my $max_value;
 my $units;
 my $fontstylemask;
 my $fontcolor;
 my $udr_key_order;
 my ($db_data_type, $data_length, $data_precision, $data_scale);

 my $sqlDictTypeC=qq{ SELECT fd.field_id,
                             xml_sym_cvt(en.entity_name),
                             xml_sym_cvt(dn.name),
                             xml_sym_cvt(dn.edit_label),
                             xml_sym_cvt(dn.Report_Header),
                             xml_sym_cvt(dn.Report_Header2),
                             fd.datatype,
                             fd.field_type,
                             xml_sym_cvt(fd.field_desc),
                             field_aggregator,
                             field_emits_aggregator,
                             xml_sym_cvt(ds.datasource_name),
                             xml_sym_cvt(fg.field_group_short),
                             fd.weight_field_id,
                             xml_sym_cvt(dt.table_name),
                             xml_sym_cvt(cd.col_name),
                             cd.precision,
                             cd.length,
                             cd.is_read_only,
                             cd.is_null_ok,
                             cd.is_frozen,
                             cd.field_order,
                             xml_sym_cvt(cd.default_value),
                             cd.min_value,
                             cd.max_value,
                             xml_sym_cvt(cd.units),
                             fs.fontstylemask,
                             fs.fontcolor,
                             uk.field_seq udr_key_order
                        FROM wmd_field_def fd, wmd_dict_name dn,
                             wmn_entity en, wmd_data_source ds,
                             wmd_field_group fg, wmd_col_def cd,
                             wmd_dict_table dt,
                             wmd_field_style fs, wmd_udr_keys uk
                       WHERE fd.field_id=dn.field_id
                         AND dn.namespace_id=0
                         AND fd.field_type='C'
                         AND fd.entity_id=en.entity_id
                         AND fd.datasource_id=ds.datasource_id(+)
                         AND fd.field_group_id=fg.field_group_id(+)
                         AND fd.field_id=cd.field_id
                         AND dt.table_id=cd.table_id
                         AND fd.field_id=fs.field_id(+)
                         AND fd.entity_id=uk.entity_id(+)
                         AND fd.field_id=uk.field_id(+)
                         AND dt.summary_id = 0
                         AND fd.entity_id= ?
                       ORDER BY dn.name
                    };

  my $sth_DictTypeC = $dbh->prepare($sqlDictTypeC);
  $sth_DictTypeC->bind_param(1, $entity_id);
  $sth_DictTypeC->execute;

  $sth_DictTypeC->bind_columns(undef,
                              \$field_id, \$entity_name, \$name,
                              \$field_label, \$Report_Header,
                              \$Report_Header2, \$datatype, \$field_type, \$field_desc,
                              \$field_aggregator, \$field_emits_aggregator, \$datasource,
                              \$field_group, \$weight_field_id, \$table_name,
                              \$col_name, \$precision, \$length, \$is_read_only,
                              \$is_null_ok, \$is_frozen, \$field_order,
                              \$default_value, \$min_value, \$max_value,
                              \$units,
                              \$fontstylemask, \$fontcolor, \$udr_key_order
                             );

  

  while ($sth_DictTypeC->fetch() ) {
   ($db_data_type, $data_length, $data_precision, $data_scale)=get_col_info($table_name, $col_name);
   xml_dump(\*$FILE, "<ROW>");
   xml_dump(\*$FILE, "<entity_name>$entity_name</entity_name>");
   xml_dump(\*$FILE, "<field_name>$name</field_name>");
   (defined $field_label) && xml_dump(\*$FILE, "<field_label>$field_label</field_label>");
   (defined $Report_Header) && xml_dump(\*$FILE, "<field_header>$Report_Header</field_header>");
   (defined $Report_Header2) && xml_dump(\*$FILE, "<field_header2>$Report_Header2</field_header2>");
   xml_dump(\*$FILE, "<field_type>$field_type</field_type>");
   (defined $field_desc) && xml_dump(\*$FILE, "<field_desc>$field_desc</field_desc>");
   xml_dump(\*$FILE, "<field_aggregator>$field_aggregator</field_aggregator>");
   (defined $field_emits_aggregator) &&
     xml_dump(\*$FILE, "<field_emits_aggregator>$field_emits_aggregator</field_emits_aggregator>");
   (defined $datasource) && xml_dump(\*$FILE, "<datasource>$datasource</datasource>");
   (defined $field_group) && xml_dump(\*$FILE, "<field_group>$field_group</field_group>");
   (defined $table_name) && xml_dump(\*$FILE, "<table_name>$table_name</table_name>");
   (defined $col_name) && xml_dump(\*$FILE, "<col_name>$col_name</col_name>");
   xml_dump(\*$FILE, "<datatype>$datatype</datatype>");
   (defined $db_data_type && $datatype eq "E")
     && xml_dump(\*$FILE, "<enum_datatype>$db_data_type</enum_datatype>");
   if ($datatype eq "S")  {
     xml_dump(\*$FILE, "<length>$data_length</length>");
   }
   elsif ($datatype eq "I"  ){
     (defined $data_precision) && xml_dump(\*$FILE, "<precision>$data_precision</precision>");
     (defined $data_scale) && xml_dump(\*$FILE, "<scale>$data_scale</scale>");
   }
   elsif ($datatype eq "E" ){
     if ($db_data_type eq 'VARCHAR2' ){
       xml_dump(\*$FILE, "<length>$data_length</length>");
     } else {
       (defined $data_precision) && xml_dump(\*$FILE, "<precision>$data_precision</precision>");
       (defined $data_scale) && xml_dump(\*$FILE, "<scale>$data_scale</scale>");
     }
   }
   (defined $is_read_only) && xml_dump(\*$FILE, "<is_read_only>$is_read_only</is_read_only>");
   (defined $is_null_ok) && xml_dump(\*$FILE, "<is_null_ok>$is_null_ok</is_null_ok>");
   (defined $is_frozen) && xml_dump(\*$FILE, "<is_frozen>$is_frozen</is_frozen>");
   (defined $field_order) && xml_dump(\*$FILE, "<field_order>$field_order</field_order>");
   (defined $default_value) && xml_dump(\*$FILE, "<default_value>$default_value</default_value>");
   (defined $min_value) && xml_dump(\*$FILE, "<min_value>$min_value</min_value>");
   (defined $max_value) && xml_dump(\*$FILE, "<max_value>$max_value</max_value>");
   (defined $units) && xml_dump(\*$FILE, "<units>$units</units>");
   (defined $weight_field_id) && get_weight_field(\*$FILE, $weight_field_id);

   get_MeasurementType(\*$FILE, $entity_id, $field_id);

   (defined $fontcolor) && xml_dump(\*$FILE, "<color>$fontcolor</color>");
   (defined $fontstylemask) && get_FontStyle(\*$FILE, $fontstylemask);
   (defined $udr_key_order) && xml_dump(\*$FILE, "<udr_key_order>$udr_key_order</udr_key_order>");

   #######################################
   # Field Type "C" has tech, subtype, enum, alias
   #######################################

   get_tech(\*$FILE, $field_id);

   get_subType(\*$FILE, $field_id);
   get_enum(\*$FILE, $field_id);
   get_alias(\*$FILE, $field_id);

   ########################################
   # Start Extra Entity field Property
   ########################################
   get_extra_fld_property(\*$FILE, $entity_id, $field_id);
   #####################################
   # End Extra Entity field Property
   #####################################

   ########################################
   # Start Extra Loader field Property
   ########################################
   # get_loader_fld_property(\*$FILE, $entity_id, $field_id); # disabled as per SEAde34640
   #####################################
   # End Extra Loader field Property
   #####################################

   xml_dump(\*$FILE, "</ROW>"."\n");
  }
  $sth_DictTypeC->finish();
};

sub getDictTypeF($$)
{
 my $FILE=shift;
 my $entity_id=shift;

 my $field_id;
 my $entity_name;
 my $name;
 my $field_label;
 my $Report_Header;
 my $Report_Header2;
 my $datatype;
 my $field_type;
 my $field_desc;
 my $field_aggregator;
 my $field_emits_aggregator;
 my $datasource;
 my $field_group;
 my $weight_field_id;
 my $table_name;
 my $func_string;
 my $fontstylemask;
 my $fontcolor;
 my $udr_key_order;
  
 my $sqlDictTypeF=qq{ SELECT fd.field_id,
                             xml_sym_cvt(en.entity_name),
                             xml_sym_cvt(dn.name),
                             xml_sym_cvt(dn.edit_label),
                             xml_sym_cvt(dn.Report_Header),
                             xml_sym_cvt(dn.Report_Header2),
                             fd.datatype,
                             fd.field_type,
                             xml_sym_cvt(fd.field_desc),
                             field_aggregator,
                             field_emits_aggregator,
                             xml_sym_cvt(ds.datasource_name),
                             xml_sym_cvt(fg.field_group_short),
                             fd.weight_field_id,
                             xml_sym_cvt(dt.table_name),
                             xml_sym_cvt(fud.func_string),
                             fs.fontstylemask,
                             fs.fontcolor,
                             uk.field_seq udr_key_order
                        FROM wmd_field_def fd, wmd_dict_name dn,
                             wmn_entity en, wmd_data_source ds,
                             wmd_field_group fg, wmd_dict_table dt,
                             wmd_func_def fud,
                             wmd_field_style fs, wmd_udr_keys uk
                       WHERE fd.field_id=dn.field_id
                         AND dn.namespace_id=0
                         AND fd.field_type='F'
                         AND fd.entity_id=en.entity_id
                         AND fd.datasource_id=ds.datasource_id(+)
                         AND fd.field_group_id=fg.field_group_id(+)
                         AND fud.field_id=fd.field_id
                         AND dt.table_id=fud.table_id
                         AND fd.field_id=fs.field_id(+)
                         AND fd.entity_id=uk.entity_id(+)
                         AND fd.field_id=uk.field_id(+)
                         AND dt.summary_id = 0
                         AND fd.entity_id= ?
                         AND fd.field_id=fud.field_id
                       ORDER BY dn.name
                    };

 my $sth_DictTypeF = $dbh->prepare($sqlDictTypeF);
 $sth_DictTypeF->bind_param(1, $entity_id);
 $sth_DictTypeF->execute;
 $sth_DictTypeF->bind_columns(undef,
                              \$field_id, \$entity_name, \$name,
                              \$field_label, \$Report_Header,
                              \$Report_Header2, \$datatype, \$field_type,
                              \$field_desc, \$field_aggregator,\$field_emits_aggregator, \$datasource,
                              \$field_group, \$weight_field_id, \$table_name, \$func_string,
                              \$fontstylemask, \$fontcolor, \$udr_key_order
                             );

  while ($sth_DictTypeF->fetch() ) {
   xml_dump(\*$FILE, "<ROW>");
   xml_dump(\*$FILE, "<entity_name>$entity_name</entity_name>");
   xml_dump(\*$FILE, "<field_name>$name</field_name>");
   (defined $field_label) && xml_dump(\*$FILE, "<field_label>$field_label</field_label>");
   (defined $Report_Header) && xml_dump(\*$FILE, "<field_header>$Report_Header</field_header>");
   (defined $Report_Header2) && xml_dump(\*$FILE, "<field_header2>$Report_Header2</field_header2>");
   xml_dump(\*$FILE, "<field_type>$field_type</field_type>");
   (defined $field_desc) && xml_dump(\*$FILE, "<field_desc>$field_desc</field_desc>");
   xml_dump(\*$FILE, "<field_aggregator>$field_aggregator</field_aggregator>");
   (defined $field_emits_aggregator) &&
     xml_dump(\*$FILE, "<field_emits_aggregator>$field_emits_aggregator</field_emits_aggregator>");
   (defined $datasource) && xml_dump(\*$FILE, "<datasource>$datasource</datasource>");
   (defined $field_group) && xml_dump(\*$FILE, "<field_group>$field_group</field_group>");
   (defined $table_name) && xml_dump(\*$FILE, "<table_name>$table_name</table_name>");
   xml_dump(\*$FILE, "<datatype>$datatype</datatype>");
   (defined $weight_field_id) && get_weight_field(\*$FILE, $weight_field_id);

   (defined $func_string) && xml_dump(\*$FILE, "<func_string>$func_string</func_string>");
   get_MeasurementType(\*$FILE, $entity_id, $field_id);

   (defined $fontcolor) && xml_dump(\*$FILE, "<color>$fontcolor</color>");
   (defined $fontstylemask) && get_FontStyle(\*$FILE, $fontstylemask);
   (defined $udr_key_order) && xml_dump(\*$FILE, "<udr_key_order>$udr_key_order</udr_key_order>");
 
   #######################################
   # Field Type "F" has tech, alias
   #######################################
   get_tech(\*$FILE, $field_id);
   get_alias(\*$FILE, $field_id);

   ########################################
   # Start Extra Entity field Property
   ########################################
   get_extra_fld_property(\*$FILE, $entity_id, $field_id);
   #####################################
   # End Extra Entity field Property
   #####################################

   ########################################
   # Start Extra Loader field Property
   ########################################
   # get_loader_fld_property(\*$FILE, $entity_id, $field_id); # disabled as per SEAde34640
   #####################################
   # End Extra Loader field Property
   #####################################

   xml_dump(\*$FILE, "</ROW>"."\n");

  }
  $sth_DictTypeF->finish();
}

sub getDictTypeS($$)
{
 my $FILE=shift;
 my $entity_id=shift;

 my $field_id;
 my $entity_name;
 my $name;
 my $field_label;
 my $Report_Header;
 my $Report_Header2;
 my $datatype;
 my $field_type;
 my $field_desc;
 my $field_aggregator;
 my $field_emits_aggregator;
 my $datasource;
 my $field_group;
 my $weight_field_id;
 my $fontstylemask;
 my $fontcolor;
 my $udr_key_order;
  
 my $sqlDictTypeS=qq{ SELECT fd.field_id,
                             xml_sym_cvt(en.entity_name),
                             xml_sym_cvt(dn.name),
                             xml_sym_cvt(dn.edit_label),
                             xml_sym_cvt(dn.Report_Header),
                             xml_sym_cvt(dn.Report_Header2),
                             fd.datatype,
                             fd.field_type,
                             fd.field_desc,
                             field_aggregator,
                             field_emits_aggregator,
                             xml_sym_cvt(ds.datasource_name),
                             xml_sym_cvt(fg.field_group_short),
                             fd.weight_field_id,
                             fs.fontstylemask,
                             fs.fontcolor,
                             uk.field_seq udr_key_order
                        FROM wmd_field_def fd, wmd_dict_name dn,
                             wmn_entity en, wmd_data_source ds,
                             wmd_field_group fg,
                             wmd_field_style fs, wmd_udr_keys uk
                       WHERE fd.field_id=dn.field_id
                         AND dn.namespace_id=0
                         AND fd.field_type='S'
                         AND fd.entity_id=en.entity_id
                         AND fd.datasource_id=ds.datasource_id(+)
                         AND fd.field_group_id=fg.field_group_id(+)
                         AND fd.field_id=fs.field_id(+)
                         AND fd.entity_id=uk.entity_id(+)
                         AND fd.field_id=uk.field_id(+)
                         AND fd.entity_id= ?
                       ORDER BY dn.name
                    };

 my $sth_DictTypeS = $dbh->prepare($sqlDictTypeS);
 $sth_DictTypeS->bind_param(1, $entity_id);
 $sth_DictTypeS->execute;
 $sth_DictTypeS->bind_columns(undef,
                              \$field_id, \$entity_name, \$name,
                              \$field_label, \$Report_Header,
                              \$Report_Header2, \$datatype, \$field_type,
                              \$field_desc, \$field_aggregator, \$field_emits_aggregator, \$datasource,
                              \$field_group, \$weight_field_id,
                              \$fontstylemask, \$fontcolor, \$udr_key_order
                             );

  while ($sth_DictTypeS->fetch() ) {
   xml_dump(\*$FILE, "<ROW>");
   xml_dump(\*$FILE, "<entity_name>$entity_name</entity_name>");
   xml_dump(\*$FILE, "<field_name>$name</field_name>");
   (defined $field_label) && xml_dump(\*$FILE, "<field_label>$field_label</field_label>");
   (defined $Report_Header) && xml_dump(\*$FILE, "<field_header>$Report_Header</field_header>");
   (defined $Report_Header2) && xml_dump(\*$FILE, "<field_header2>$Report_Header2</field_header2>");
   xml_dump(\*$FILE, "<field_type>$field_type</field_type>");
   (defined $field_desc) && xml_dump(\*$FILE, "<field_desc>$field_desc</field_desc>");
   xml_dump(\*$FILE, "<field_aggregator>$field_aggregator</field_aggregator>");
   (defined $field_emits_aggregator) &&
     xml_dump(\*$FILE, "<field_emits_aggregator>$field_emits_aggregator</field_emits_aggregator>");
   (defined $datasource) && xml_dump(\*$FILE, "<datasource>$datasource</datasource>");
   (defined $field_group) && xml_dump(\*$FILE, "<field_group>$field_group</field_group>");
   xml_dump(\*$FILE, "<datatype>$datatype</datatype>");
   (defined $weight_field_id) && get_weight_field(\*$FILE, $weight_field_id);

   get_MeasurementType(\*$FILE, $entity_id, $field_id);

   (defined $fontcolor) && xml_dump(\*$FILE, "<color>$fontcolor</color>");
   (defined $fontstylemask) && get_FontStyle(\*$FILE, $fontstylemask);
   (defined $udr_key_order) && xml_dump(\*$FILE, "<udr_key_order>$udr_key_order</udr_key_order>");
 
   #######################################
   # Field Type "F" has tech
   #######################################
   get_tech(\*$FILE, $field_id);

   #######################################
   # Getting SQL function strings
   #######################################
   get_sqlfunction(\*$FILE, $field_id);

   #######################################
   # Field Type "F" has alias
   #######################################
   get_alias(\*$FILE, $field_id);

   ########################################
   # Start Extra Entity field Property
   ########################################
   get_extra_fld_property(\*$FILE, $entity_id, $field_id);
   #####################################
   # End Extra Entity field Property
   #####################################

   ########################################
   # Start Extra Loader field Property
   ########################################
   # get_loader_fld_property(\*$FILE, $entity_id, $field_id); # disabled as per SEAde34640
   #####################################
   # End Extra Loader field Property
   #####################################

   xml_dump(\*$FILE, "</ROW>"."\n");
  }
  $sth_DictTypeS->finish();
}


#####################
# getting tech
#####################
sub get_tech($$)
{
  my $FILE=shift;
  my $field_id=shift;
  my $tech;
  my $sth_tech = $dbh->prepare(" select xml_sym_cvt(t.tech_name)
                                   from wmd_technology t, wmd_field_tech tf
                                  where tf.field_id = ? and t.tech_id=tf.tech_id
                                  order by t.tech_name ");
  $sth_tech->bind_param(1, $field_id);
  $sth_tech->execute;
  $sth_tech->bind_columns(undef,\$tech);
  while($sth_tech->fetch() ){
    (defined $tech) && xml_dump(\*$FILE, "<tech>$tech</tech>");
  }
  $sth_tech->finish();
}

#####################
# getting subtype
#####################
sub get_subType($$)
{
   my $FILE=shift;
   my $field_id=shift;
   my $default_value;
   my $max_value;
   my $min_value;
   my $subType_name;
   my $sth_subtype = $dbh->prepare(" select xml_sym_cvt(cds.default_value),
                                            cds.max_value,
                                            cds.min_value,
                                            xml_sym_cvt(subtype_name)
                                       from wmd_col_def_subtype cds, wmn_entity_subtype es
                                      where cds.field_id = ?
                                        and cds.entity_id = es.entity_id
                                        and cds.subtype_id = es.subtype_id
                                   order by subtype_name, cds.default_value,
                                            cds.max_value, cds.min_value  ");
   $sth_subtype->bind_param(1, $field_id);
   $sth_subtype->execute;
   $sth_subtype->bind_columns(undef,\$default_value, \$max_value, \$min_value, \$subType_name);
   while($sth_subtype->fetch() ){
     xml_dump(\*$FILE, "<subType name=\"$subType_name\">");
     (defined $default_value) && xml_dump(\*$FILE, "<default_value>$default_value</default_value>");
     (defined $max_value) && xml_dump(\*$FILE, "<max_value>$max_value</max_value>");
     (defined $min_value) && xml_dump(\*$FILE, "<min_value>$min_value</min_value>");
     xml_dump(\*$FILE, "</subType>");
   }
   $sth_subtype->finish();

}

#####################
# getting enum
#####################
sub get_enum($$)
{
   my $FILE=shift;
   my $field_id=shift;
   my $key;
   my $enum;
   my $sth_enum = $dbh->prepare(" select xml_sym_cvt(enum_key),
                                         xml_sym_cvt(enum_desc)
                                    from wmd_enum_values
                                   where field_id = ? 
                                   order by enum_key,enum_desc");
   $sth_enum->bind_param(1, $field_id);
   $sth_enum->execute;
   $sth_enum->bind_columns(undef,\$key, \$enum);
   while($sth_enum->fetch() ){
     xml_dump(\*$FILE, "<enum key=\"$key\">$enum</enum>");
   }
   $sth_enum->finish();

}

#####################
# getting alias
#####################
sub get_alias($$)
{
   my $FILE=shift;
   my $field_id=shift;
   #####################
   # namespace part
   #####################
   my $namespace_title;
   my $alias_name;
   my $sth_alias = $dbh->prepare(" select xml_sym_cvt(dns.namespace_title),
                                          xml_sym_cvt(dn.name)
                                    from wmd_dict_name dn, wmd_dict_namespace dns
                                   where dn.namespace_id=dns.namespace_id
                                     and dn.field_id= ?
                                     and dns.namespace_id <> 0
                                   order by dns.namespace_title, dn.name ");
   $sth_alias->bind_param(1, $field_id);
   $sth_alias->execute;
   $sth_alias->bind_columns(undef,\$namespace_title, \$alias_name);
   while($sth_alias->fetch() ){
     xml_dump(\*$FILE, "<alias extra_field=\"$namespace_title\">$alias_name</alias>");
   }
   $sth_alias->finish();

   #####################
   # DataSet part
   #####################
   my $dataset_title;
   my $datavalue;
   $sth_alias = $dbh->prepare(" select xml_sym_cvt(fds.dataset_title),
                                       xml_sym_cvt(fd.datavalue)
                                  from wmd_field_data fd, wmd_field_data_set fds
                                 where fd.dataset_id=fds.dataset_id
                                   and fd.field_id=? 
                              order by fds.dataset_title, fd.datavalue");
   $sth_alias->bind_param(1, $field_id);
   $sth_alias->execute;
   $sth_alias->bind_columns(undef,\$dataset_title, \$datavalue);
   while($sth_alias->fetch() ){
#     xml_dump(\*$FILE, "<alias extra_field=\"$dataset_title\">$datavalue</alias>");
     if (defined $dataset_title )
     { xml_dump(\*$FILE, "<alias extra_field=\"$dataset_title\">"); }
     else { xml_dump(\*$FILE, "<alias>"); }
     if (defined $datavalue )
     { xml_dump(\*$FILE, "$datavalue</alias>"); }
     else { xml_dump(\*$FILE, "</alias>"); }
   }
   $sth_alias->finish();
}

#####################
# getting sql function
#####################
sub get_sqlfunction($$)
{
   my $FILE=shift;
   my $field_id=shift;
   my $frequency;
   my $sql_function;
   my $sth_sqlfunction = $dbh->prepare(" select xml_sym_cvt(sqlfunction),
                                                frequency
                                           from wmd_dict_function
                                          where field_id = ? 
                                          order by sqlfunction,frequency");
   $sth_sqlfunction->bind_param(1, $field_id);
   $sth_sqlfunction->execute;
   $sth_sqlfunction->bind_columns(undef,\$sql_function, \$frequency);
   xml_dump(\*$FILE, "<sql_function>");
   while($sth_sqlfunction->fetch() ){
     xml_dump(\*$FILE, "<frequency name=\"$frequency\">$sql_function</frequency>");
   }
   xml_dump(\*$FILE, "</sql_function>");
   $sth_sqlfunction->finish();

}
###########################
# getting sql weight_field
###########################
sub get_weight_field($$)
{
   my $FILE=shift;
   my $field_id=shift;
   my $weight_field;
   my $sth_weight_field = $dbh->prepare(" select xml_sym_cvt(name)
                                           from wmd_dict_name
                                          where field_id = ? 
                                       ");
   $sth_weight_field->bind_param(1, $field_id);
   $sth_weight_field->execute;
   $sth_weight_field->bind_columns(undef,\$weight_field);
   while($sth_weight_field->fetch() ){
     xml_dump(\*$FILE, "<weight_field>$weight_field</weight_field>");
   }
   $sth_weight_field->finish();

}

sub get_col_info($$)
{
  my $table_name=shift;
  my $col_name=shift;
  my ($data_type, $data_length, $data_precision, $data_scale);
  my $sth_col=$dbh->prepare("select data_type, data_length,
                                    data_precision, data_scale
			       from user_tab_columns
                              where table_name= ?
				and column_name= ? ");
  $sth_col->bind_param(1, $table_name);
  $sth_col->bind_param(2, $col_name);
  $sth_col->bind_columns(undef, \$data_type, \$data_length, \$data_precision, \$data_scale);
  $sth_col->execute;
  $sth_col->fetch();
  $sth_col->finish();
  return $data_type, $data_length, $data_precision, $data_scale;
}

sub get_primary_key($$)
{
  my $table_name=shift;
  my $table_period=shift;

  my $col_name;
  my $col_datatype;
  my $col_length;

  my $sth_sql = qq(
       select t.column_name
            , t.data_type
            , decode(t.data_type, 'NUMBER', nvl(t.data_precision,38), t.data_length) length
         from USER_CONS_COLUMNS a
            , USER_CONSTRAINTS b
            , USER_TAB_COLUMNS t
        where a.constraint_name=b.constraint_name
          and a.table_name = t.table_name
          and a.column_name = t.column_name
          and t.table_name=?
          and b.constraint_type='P'
     );

  if ( $table_period eq 'D' )
  {
     $sth_sql = $sth_sql . qq( and t.column_name not in ('TSTAMP', 'INSTANCE_ID'));
  }

  my $sth_col=$dbh->prepare($sth_sql);

  $sth_col->bind_param(1, $table_name);
  $sth_col->bind_columns(undef, \$col_name, \$col_datatype, \$col_length);

  $sth_col->execute;
  while ($sth_col->fetch() ) {

     print MAINFILE "<primary_key "
                  . "name=\"$col_name\" "
                  . "datatype=\"$col_datatype\" "
                  . "length=\"$col_length\" "
                  . "/>\n";
  }
  $sth_col->finish();
}

sub showUsage
{
 my $script_name = basename $0;
 print "usage: $script_name -connect <connectstring> [ -dest_dir <destination directory> ] \n";
 print "             [ -vendor <vendor> ] [ -version <version> ] [ -compat <compat> ] \n";
 die "usage: $script_name -help\n";
}

sub get_context
{
#
# * use is optional
# <Context name="$context_name" use="context_use"/>
#

# Variables
  my ($name, $use);

  my $sqlString=qq{ SELECT xml_sym_cvt(context_name),
                           NVL(context_use,'NULL')
                      FROM wmn_context };

  my $sth = $dbh->prepare($sqlString);

  $sth->execute;

  $sth->bind_columns(undef, \$name, \$use);

  while ($sth->fetch() ) {

    # Check for NULL is needed on $use
    if ( $use eq 'NULL' ) {
       print MAINFILE "<Context name=\"" . $name . "\"/>\n";
    }
    else {
       print MAINFILE "<Context name=\"" . $name . "\" usage=\"" . $use . "\"/>\n";
    }
  }

  $sth->finish();
}


sub get_data_source
{
#
# * context is optional
#
#<Dictionary name="$datasource_name" context="$datasource_app_ctx" useUDC="$datasource_use_udc"/>
#
# * Dictionary can have 0 or more ExtraFields
#   <ExtraField name="" unique=""/>
#   ...
#</Dictionary>
#
  my ($dataSourceID, $dictName, $useUDC,  $context_name);
  my ($extraSQL, $namespaceSQL, $datasetSQL);
  my ($sth_extra, $sth_namespace, $sth_dataset);
  my ($namespace_id, $dataset_id,$editable);
  my ($namespace_title, $dataset_title);

  my $sqlString=qq( select DATASOURCE_ID,
                           xml_sym_cvt(DATASOURCE_NAME) dict_name,
                           decode(DATASOURCE_USE_UDC, 1, 'Y', 'N') useUDC,
                           xml_sym_cvt(ctx.context_name)
                      from wmd_data_source ds, wmn_context ctx
                     where ds.DATASOURCE_APP_CTX=ctx.context_id);

  my $sth = $dbh->prepare($sqlString);

  $sth->execute;

  $sth->bind_columns(undef, \$dataSourceID, \$dictName, \$useUDC, \$context_name);

  while ($sth->fetch() ) {
    print MAINFILE "<Dictionary name=\"$dictName\" context=\"$context_name\" useUDC=\"$useUDC\">\n";
    ############################
    # Start NameSpace or DataSet
    ############################
    $extraSQL=qq( select NAMESPACE_ID, DATASET_ID,editable
                    from wmd_datasource_extra
                   where DATASOURCE_ID= ? );
    $sth_extra=$dbh->prepare($extraSQL);
    $sth_extra->bind_param(1, $dataSourceID);
    $sth_extra->execute;
    $sth_extra->bind_columns(undef, \$namespace_id, \$dataset_id, \$editable);
    while($sth_extra->fetch() ){
      ##################
      # Start NameSpace 
      ##################
      $namespaceSQL=qq(select xml_sym_cvt(NAMESPACE_TITLE)
                         from wmd_dict_namespace
                        where NAMESPACE_ID=?);
      $sth_namespace=$dbh->prepare($namespaceSQL);
      $sth_namespace->bind_param(1, $namespace_id);
      $sth_namespace->execute;
      $sth_namespace->bind_columns(undef, \$namespace_title);
      while($sth_namespace->fetch() ){
        print MAINFILE "<ExtraField name=\"$namespace_title\" unique=\"Y\" Editable=\"$editable\" />\n"; 
      }
      $sth_namespace->finish();
      ##################
      # End NameSpace 
      ##################
      ##################
      # Start DataSet 
      ##################
      $datasetSQL=qq(select xml_sym_cvt(DATASET_TITLE)
                       from wmd_field_data_set 
                      where DATASET_ID=?);
      $sth_dataset=$dbh->prepare($datasetSQL);
      $sth_dataset->bind_param(1, $dataset_id);
      $sth_dataset->execute;
      $sth_dataset->bind_columns(undef, \$dataset_title);
      while($sth_dataset->fetch() ){
        print MAINFILE "<ExtraField name=\"$dataset_title\" Editable=\"$editable\"/>\n"; 
      }
      $sth_dataset->finish();
      ##################
      # End DataSet 
      ##################
    }
    $sth_extra->finish();
    ############################
    # End NameSpace or DataSet
    ############################
    print MAINFILE "</Dictionary>\n";
  }

  # Complete the section

  $sth->finish();

}
sub get_field_group
{
#
#<FieldGroup name="$field_group_name" short="$field_group_short"/>
#

  my ($name, $short);

  my $sqlString=qq{ SELECT xml_sym_cvt(field_group_name),
                           xml_sym_cvt(field_group_short)
                      FROM wmd_field_group
                  ORDER BY field_group_name };

  my $sth = $dbh->prepare($sqlString);

  $sth->execute;

  $sth->bind_columns(undef, \$name, \$short);

  while ($sth->fetch() ) {

    print MAINFILE "<FieldGroup name=\"" . $name . "\" short=\"" . $short . "\"/>\n";
  }

  $sth->finish();
}

sub get_technology
{
#
#<Technology name="$tech_name" char="$tech_char">
#   $tech_desc
#</Technology>
#
  my ($name, $char, $tech_desc);

  my $sqlString=qq{ SELECT xml_sym_cvt(tech_name),
                           tech_char,
                           xml_sym_cvt(tech_desc)
                      FROM wmd_technology
                  ORDER BY tech_name };

  my $sth = $dbh->prepare($sqlString);

  $sth->execute;

  $sth->bind_columns(undef, \$name, \$char, \$tech_desc);

  while ($sth->fetch() ) {

    print MAINFILE "<Technology name=\"" . $name . "\" char=\"" . $char . "\">" . $tech_desc . "</Technology>\n";
  }

  $sth->finish();

}

sub get_filetype
{
#
#<FileType name="$file_type" frequency="$frequency" File_Description="$file_desc ription" entity_name="$entity_name" vendor="$vendor"/>
#
  my ($name, $frequency, $file_description, $entity_name, $vendor);
  my ($module_name, $file_extension, $generate_loader);

  my $sqlString=qq{ SELECT xml_sym_cvt(file_type),
                           xml_sym_cvt(frequency),
                           xml_sym_cvt(file_description),
                           xml_sym_cvt(entity_name),
                           xml_sym_cvt(vendor),
			   xml_sym_cvt(MODULE_NAME),
			   xml_sym_cvt(FILE_EXTENSION),
			   decode(GENERATE_LOADER, null,'N', GENERATE_LOADER)
                  FROM wm_file_control
                  ORDER BY file_type };
#			   decode(activate_scenario, null,'N', activate_scenario)
  my $sth = $dbh->prepare($sqlString);

  $sth->execute;

  $sth->bind_columns(undef, \$name, \$frequency,
		     \$file_description, \$entity_name,
		     \$vendor, \$module_name,
		     \$file_extension, \$generate_loader);
#		     \$activate_scenario);

  while ($sth->fetch() ) {

    print MAINFILE "<FileType name=\"" . $name . "\" frequency=\""
    . $frequency . "\" File_Description=\"" . $file_description 
    . "\" Entity_Name=\"" . $entity_name . "\" vendor=\"" . $vendor . "\" ";
    (defined $module_name) && print MAINFILE "Module_Name=\"$module_name\" ";
    (defined $file_extension) && print MAINFILE "File_Extension=\"$file_extension\" ";
    print MAINFILE "Generate_Loader=\"$generate_loader\" ";
  #  print MAINFILE "activate_scenario=\"$activate_scenario\"/>\n";

 print MAINFILE "/>\n";
  }

  $sth->finish();

}

sub get_system_values
{
#
#<SystemValues>
#   <Variable name="$name">
#      $value
#   </Variable>
#   ...
#</SystemValues>
#

  my ($name, $value);

  my $sqlString=qq{ SELECT xml_sym_cvt(name),
                           xml_sym_cvt(value)
                      FROM wm_system_values_v
                  ORDER BY name };

  my $sth = $dbh->prepare($sqlString);

  $sth->execute;

  $sth->bind_columns(undef, \$name, \$value);

  print MAINFILE "<SystemValues>\n";

  while ($sth->fetch() ) {

    print MAINFILE "<variable name=\"" . $name . "\">";
    (defined $value) && print MAINFILE "$value";
    print MAINFILE "</variable>\n";
  }

  print MAINFILE "</SystemValues>\n";

  $sth->finish();

}

sub get_tableType
{
#
#<TableType name="$name" category="$category" />
#
  my ($name);
  my ($category);

  my $sqlString=qq{ SELECT xml_sym_cvt(name)  name
                         , xml_sym_cvt(decode(upper(name), 'TRAFFIC', name, ' ')) category
                      FROM subclass_table
                  ORDER BY name };

  my $sth = $dbh->prepare($sqlString);

  $sth->execute;

  $sth->bind_columns(undef, \$name, \$category);

  while ($sth->fetch() ) {

    print MAINFILE "<TableType name=\"" . $name . "\" category=\"" . $category . "\" />\n";
  }

  $sth->finish();

}

sub get_tableDefinition
{
#
# * Not clear if we should get all tables or only time series
#
#<TableDefinition name="$table_name" tablespace="$tablespace" is_time_series="is_time_series"/>
#

  my ( $name, $tablespace, $is_time_series, $file_type, $mo_class
     , $entity_name, $activate_scenario, $table_type, $frequency, $period, $category);

  my $sqlString=qq{SELECT xml_sym_cvt(wm.table_name)            table_name,
                          xml_sym_cvt(ut.tablespace_name)       tablespace,
                          wm.is_time_series                     is_time_series,
                          xml_sym_cvt(fc.file_type)             file_type,
                          xml_sym_cvt(wm.mo_class)              mo_class,
                          xml_sym_cvt(e.entity_name)            entity_name,
                          decode(ast.entity_id, NULL, 'N', 'Y') activate_scenario,
                          xml_sym_cvt(sct.name)                 table_type,
                          nvl(pt.period,' ')                    frequency,
                          nvl(pt.period,' ')                    period,
                          xml_sym_cvt(decode(upper(sct.name), NULL, NULL, 'TRAFFIC', sct.name, ' ')) category
                    FROM wmd_dict_table wm,
                         wm_xml_tab_tablespace_mv ut ,
                         wm_file_control fc,
                         wm_file_table_map ftm,
                         wmn_entity e,
                         subclass_table sct,
                         subclass_map scm,
                         ad_scan_tables ast,
                         part_tables pt,
                         user_objects uo
                    WHERE wm.table_name = ut.table_name
                      AND wm.summary_id=0
                      AND ftm.table_id=wm.table_id
                      AND ftm.file_type_id=fc.file_type_id
                      AND wm.entity_id=e.entity_id
                      AND wm.table_name=pt.table_name (+)
                      AND pt.part_table_id=scm.object_id (+)
                      and scm.subclass_id=sct.id (+)
                      and wm.entity_id=ast.entity_id (+)
                      and wm.table_name=ast.table_name (+)
                      and upper(wm.table_name) = uo.object_name
                      and uo.object_type = 'TABLE'
                    ORDER BY wm.table_name };

  my $sth = $dbh->prepare($sqlString);

  $sth->execute;

  $sth->bind_columns(undef, \$name, \$tablespace, \$is_time_series, \$file_type, \$mo_class, \$entity_name, \$activate_scenario, \$table_type, \$frequency, \$period, \$category);

  while ($sth->fetch() ) {

    print MAINFILE "<TableDefinition name=\"" . $name . "\" tablespace=\"" . $tablespace . "\" is_time_series=\"" .  $is_time_series . "\" file_type=\"$file_type\" ";
    (defined $mo_class) && print MAINFILE "mo_class=\"$mo_class\" ";
    print MAINFILE "entityname=\"$entity_name\" ";
    print MAINFILE "activate_scenario=\"$activate_scenario\" ";
    (defined $table_type) && print MAINFILE "table_type=\"$table_type\" ";
    (defined $category) && print MAINFILE "table_category=\"$category\" ";
    print MAINFILE "frequency=\"$frequency\" ";
    print MAINFILE ">\n";
    get_primary_key($name, $period);
    print MAINFILE "</TableDefinition>\n";
  }

  $sth->finish();

}


sub get_entitys($)
{
#
# * use is optional
# <Context name="$context_name" use="context_use"/>
#

# Variables
  my $FILE=shift;
  my $sqlString;
  my $sth;

  my ($context_id, $entity_id);

  $sqlString=qq{select entity_id
                  from wmn_entity
                  order by entity_name
               };

  $sth = $dbh->prepare($sqlString);
  $sth->execute;
  $sth->bind_columns(undef, \$entity_id);
  while ($sth->fetch() ) {
    get_entity(\*$FILE, $entity_id);
  }

  $sth->finish();
}

sub get_entity($$)
{
 my $FILE=shift;
 my $entity_id=shift; 

 my ($context_id, $EntityContext, $Parent, $ParseKey);
 my ($entitySQL, $ctxSQL, $ctx_tranSQL, $subTypeSQL, $RptTemplateSQL,
     $RptTemplateFldSQL, $RptTemplateFldFltSQL);
 my $sth_entity;
 my $sth_ctx;
 my $sth_ctx_tran;
 my $sth_subType;
 my $sth_rpttmp;
 my $sth_rpttmpfld;
 my $sth_rpttmpfldflt;
 my( $name, $Group, $Entity_Desc, $Entity_Table,
     $ImageKey, $DefaultNameExpr, $LabelExpr, $EntityOrderExpr,
     $EntityOrderNum, $ReportNameExpr, $ReportNameHeader, $reparent, $BaseEntity );
 my ($ToContext, $Always, $Translation);
 my $SubType;
 my ($template_id, $template_name, $description, $type, $category, $folder, $folder_category);
 my ($ordinal, $entity, $fieldName, $sortOrder, $isShown);
 my ($filterOrdinal, $criteria);

 $entitySQL = qq
      (select xml_sym_cvt(e1.entity_name),
              upper(e1.isgroup),
              xml_sym_cvt(e1.entity_desc),
              xml_sym_cvt(e1.entity_table),
              xml_sym_cvt(e1.imagekey),
              xml_sym_cvt(e1.DefaultNameexpr),
              xml_sym_cvt(e1.labelexpr),
              xml_sym_cvt(e1.entityorderexpr),
              e1.entityordernum,
              xml_sym_cvt(e1.reportnameexpr),
              xml_sym_cvt(e1.reportnameheader),
              e1.reparent,
              xml_sym_cvt(e2.entity_name)
         from wmn_entity e1,  wmn_entity e2
        where e1.entity_id= ? 
          and e1.baseEntityid=e2.entity_id(+)
     order by e1.entity_name);
 $sth_entity = $dbh->prepare($entitySQL);
 $sth_entity->bind_param(1, $entity_id);
 $sth_entity->execute;
 $sth_entity->bind_columns(undef,\$name, \$Group,\$Entity_Desc, \$Entity_Table,
                    \$ImageKey, \$DefaultNameExpr, \$LabelExpr, \$EntityOrderExpr,
                    \$EntityOrderNum, \$ReportNameExpr,
                    \$ReportNameHeader,\$reparent, \$BaseEntity );
 while ($sth_entity->fetch() ) {
  xml_dump(\*$FILE, "<Entity name=\"$name\" ");
  if (defined $Group) {xml_dump(\*$FILE, "Group=\"$Group\" ");}
  else { xml_dump(\*$FILE, "Group=\"N\" ");}
  (defined $Entity_Desc) && xml_dump(\*$FILE, "Entity_Desc=\"$Entity_Desc\" ");
  (defined $Entity_Table) && xml_dump(\*$FILE, "Entity_Table=\"$Entity_Table\" ");
  (defined $ImageKey) && xml_dump(\*$FILE, "ImageKey=\"$ImageKey\" ");
  (defined $DefaultNameExpr) && xml_dump(\*$FILE, "DefaultNameExpr=\"$DefaultNameExpr\" ");
  (defined $LabelExpr) && xml_dump(\*$FILE, "LabelExpr=\"$LabelExpr\" ");
  (defined $EntityOrderExpr) && xml_dump(\*$FILE, "EntityOrderExpr=\"$EntityOrderExpr\" ");
  (defined $EntityOrderNum) && xml_dump(\*$FILE, "EntityOrderNum=\"$EntityOrderNum\" ");
  (defined $ReportNameExpr) && xml_dump(\*$FILE, "ReportNameExpr=\"$ReportNameExpr\" ");
  (defined $ReportNameHeader) && xml_dump(\*$FILE, "ReportNameHeader=\"$ReportNameHeader\" ");
  (defined $reparent) && xml_dump(\*$FILE, "reparent=\"$reparent\" ");
  (defined $BaseEntity) && xml_dump(\*$FILE, "BaseEntity=\"$BaseEntity\" ");
  print $FILE ">\n"; 

  #################
  # entity context
  #################
  $ctxSQL = qq(select ec.context_id,
                      xml_sym_cvt(context_name) EntityContext,
                      xml_sym_cvt(entity_name) parent,
                      decode(parsekey, null, 'Y', parsekey) parsekey
                 from wmn_entity_context ec, wmn_entity e, wmn_context c
                where ec.context_id=c.context_id
                  and ec.parent_entity_id=e.entity_id(+)
                  and ec.entity_id= ? 
                order by context_name,entity_name
              );
  $sth_ctx = $dbh->prepare($ctxSQL);
  $sth_ctx->bind_param(1, $entity_id);
  $sth_ctx->execute;
  $sth_ctx->bind_columns(undef, \$context_id, \$EntityContext,
                         \$Parent, \$ParseKey);

  while ($sth_ctx->fetch() ) {
    xml_dump(\*$FILE, "<EntityContext context=\"$EntityContext\" "); 

    (defined $Parent) && xml_dump(\*$FILE, "Parent=\"$Parent\" ");
    xml_dump(\*$FILE,"ParseKey=\"$ParseKey\">");
    $ctx_tranSQL=qq(select xml_sym_cvt(ctx.context_name) ToContext,
                           xml_sym_cvt(transition) Translation,
                           decode(always_convert, 0, 'N', 'Y') Always
                      from wmn_context_transition ct, wmn_context ctx
                     where ct.entity_id=? and ct.src_context_id=?
                       and ct.dest_context_id=ctx.context_id
                  order by ctx.context_name);
    $sth_ctx_tran=$dbh->prepare($ctx_tranSQL);
    $sth_ctx_tran->bind_param(1, $entity_id);
    $sth_ctx_tran->bind_param(2, $context_id);
    $sth_ctx_tran->execute;
    $sth_ctx_tran->bind_columns(undef,\$ToContext, \$Translation, \$Always,);
    while ($sth_ctx_tran->fetch() ) {
      xml_dump(\*$FILE, "<Translation ToContext=\"$ToContext\" ");
      xml_dump(\*$FILE, "Always=\"$Always\">");
      (defined $Translation) && xml_dump(\*$FILE, "$Translation");
      xml_dump(\*$FILE, "</Translation>\n");
    }
    $sth_ctx_tran->finish();
    xml_dump(\*$FILE, "</EntityContext>\n");
  }
  
  $sth_ctx->finish();

  ####################
  # End entity context
  ####################

  ####################
  # Entity Dictionary 
  ####################
  xml_dump(\*$FILE, "<EntityDictionary file=\"$name.xml\"/>\n");
  ####################
  # End Entity Dictionary 
  ####################

  #################
  # SubType 
  #################

  $subTypeSQL=qq(select xml_sym_cvt(subtype_name) SubType
                   from wmn_entity_subtype
                  where entity_id= ? 
                  order by subtype_name); 
  $sth_subType=$dbh->prepare($subTypeSQL);
  $sth_subType->bind_param(1, $entity_id);
  $sth_subType->execute;
  $sth_subType->bind_columns(undef, \$SubType);
  while ($sth_subType->fetch() ) {
    xml_dump(\*$FILE, "<SubType name=\"$SubType\"/>\n");
  } 
  $sth_subType->finish();
  #################
  # ENd SubType 
  #################

  #####################
  # Start ReportTemplate
  ##################### 
  $RptTemplateSQL=qq( select pt.id template_id,
                             xml_sym_cvt(pt.name),
                             xml_sym_cvt(pt.description),
                             xml_sym_cvt(pt.type),
                             xml_sym_cvt(pt.category),
                             xml_sym_cvt(cft_folder_fullpath(pf.id)) folder,
                             xml_sym_cvt(pf.category)
                        from pm_template pt, pm_folder pf, pm_user pu
                       where pt.folderid=pf.id
                         and pt.userid=pu.id
                         and lower(pu.name)='system'
                         and pt.entity_id=?
                       order by pt.name);

  $sth_rpttmp=$dbh->prepare($RptTemplateSQL);
  $sth_rpttmp->bind_param(1, $entity_id);
  $sth_rpttmp->execute;
  $sth_rpttmp->bind_columns(undef, \$template_id, \$template_name, \$description,
                            \$type, \$category, \$folder, \$folder_category);
  while ($sth_rpttmp->fetch() ) {

    # Remove the base Templates folder
    $folder =~ s,^/ROOT/Templates,,g;

    xml_dump(\*$FILE, "<ReportTemplate name=\"$template_name\" ");
    (defined $description) && xml_dump(\*$FILE, "description=\"$description\" ");
    xml_dump(\*$FILE, "type=\"$type\" ");
    xml_dump(\*$FILE, "category=\"$category\" ");
    xml_dump(\*$FILE, "folder=\"$folder\" ");
    xml_dump(\*$FILE, "folder_category=\"$folder_category\">\n");
    ##################################
    # Start reportTemplateField
    ##################################

    $RptTemplateFldSQL=qq( select ordinal,
                                  xml_sym_cvt(e.entity_name),
                                  xml_sym_cvt(dn.name),
                                  decode(sortorder, null,'N', sortorder) sortorder,
                                  decode(isshown, null,'Y', isshown) isshown
                             from pm_templatefield tf, wmn_entity e, wmd_dict_name dn
                            where templateid=?
                              and e.entity_id=dn.entity_id
                              and dn.field_id=tf.field_id
                              and dn.namespace_id=0
                         order by ordinal );

    $sth_rpttmpfld=$dbh->prepare($RptTemplateFldSQL);
    $sth_rpttmpfld->bind_param(1, $template_id);
    $sth_rpttmpfld->execute;
    $sth_rpttmpfld->bind_columns(undef, \$ordinal, \$entity,
                              \$fieldName, \$sortOrder, \$isShown);

   while ($sth_rpttmpfld->fetch() ) {
    xml_dump(\*$FILE, "<ReportTemplateField ordinal=\"$ordinal\" ");
    xml_dump(\*$FILE, "entity=\"$entity\" ");
    xml_dump(\*$FILE, "fieldName=\"$fieldName\" ");
    xml_dump(\*$FILE, "sortOrder=\"$sortOrder\" ");
    xml_dump(\*$FILE, "isShown=\"$isShown\">\n");
      ##################################
      # Start reportTemplateField filter
      ##################################
      $RptTemplateFldFltSQL=qq( select ordinal,
                                       xml_sym_cvt(criteria)
                                  from pm_filter
                                 where templateID = ?
                                   and fieldordinal = ?);
      $sth_rpttmpfldflt=$dbh->prepare($RptTemplateFldFltSQL);
      $sth_rpttmpfldflt->bind_param(1, $template_id);
      $sth_rpttmpfldflt->bind_param(2, $ordinal);
      $sth_rpttmpfldflt->execute;
      $sth_rpttmpfldflt->bind_columns(undef, \$filterOrdinal, \$criteria);
      while($sth_rpttmpfldflt->fetch() ){
       xml_dump(\*$FILE, "<ReportFilter ordinal=\"$filterOrdinal\" ");
       if (defined $criteria ){ xml_dump(\*$FILE, "criteria=\"$criteria\"/>\n"); } 
       else { xml_dump(\*$FILE, "/>\n"); }
      }
      $sth_rpttmpfldflt->finish();
      ##################################
      # End reportTemplateField filter
      ##################################
    xml_dump(\*$FILE, "</ReportTemplateField>\n");
   }
   $sth_rpttmpfld->finish();
    ##################################
    # End reportTemplateField
    ##################################
   xml_dump(\*$FILE, "</ReportTemplate>\n");
  }
  $sth_rpttmp->finish();
 
  #####################
  # End ReportTemplate
  ##################### 
 
  ########################
  # Start Extra Entity Property
  ######################## 
  get_extra_ent_property(\*$FILE, $entity_id); 
  #####################
  # End Extra Entity Property
  ##################### 

  ########################
  # Start Extra Loader Property
  ######################## 
  #get_loader_ent_property(\*$FILE, $entity_id);  # disabled as per SEAde34640
  #####################
  # End Extra Loader Property
  ##################### 

 xml_dump(\*$FILE, "</Entity>\n");
 }
 $sth_entity->finish();

}
sub get_server_module($)
{
  
  my $FILE=shift;
  my ($module_name, $module_prefix, $module_icon,
      $version_name, $version_prefix, $version_number,
      $root_entity_name);
  my $sth_serverModule = $dbh->prepare(" select xml_sym_cvt(module_name),
                                                xml_sym_cvt(module_prefix),
                                                xml_sym_cvt(module_icon),
                                                xml_sym_cvt(version_name),
                                                xml_sym_cvt(version_prefix),
                                                version_number,
                                                xml_sym_cvt(root_entity_name)
                                           from wm_server_module
                                          order by module_name
                                       ");
  $sth_serverModule->execute;
  $sth_serverModule->bind_columns(undef,
      \$module_name, \$module_prefix, \$module_icon,
      \$version_name, \$version_prefix, \$version_number,
      \$root_entity_name);
  while($sth_serverModule->fetch() ){
    xml_dump(\*$FILE, "<ServerModule name=\"$module_name\" ");
    xml_dump(\*$FILE, "prefix=\"$module_prefix\" ");
    (defined $module_icon) && xml_dump(\*$FILE, "icon=\"$module_icon\" ");
    xml_dump(\*$FILE, "version_name=\"$version_name\" ");
    xml_dump(\*$FILE, "version_prefix=\"$version_prefix\" ");
    xml_dump(\*$FILE, "version_number=\"$version_number\" ");
    xml_dump(\*$FILE, "root_entity_name=\"$root_entity_name\" />\n");
  }
  $sth_serverModule->finish();

}

sub get_loader_source($)
{
  
  my $FILE=shift;
  my ($loadersource_id, $loadersource_name);
  my $sth_loaderSrc = $dbh->prepare("select loadersource_id,
                                            xml_sym_cvt(loadersource_name)
                                       from wmd_loader_source
                                   order by loadersource_id
                                    ");
  $sth_loaderSrc->execute;
  $sth_loaderSrc->bind_columns(undef,
      \$loadersource_id, \$loadersource_name);
  while($sth_loaderSrc->fetch() ){
    xml_dump(\*$FILE, "<LoaderSource name=\"$loadersource_name\" ");
    xml_dump(\*$FILE, "id=\"$loadersource_id\" />\n");
  }
  $sth_loaderSrc->finish();

}

sub xml_dump($$) 
{
 my $FILE=shift;
 my $xml_string = shift;  
 print $FILE "$xml_string";
}

sub xml_header($)
{
 my $FILE=shift;
 print $FILE "<?xml version='1.0'?>\n";
 print $FILE "<!DOCTYPE vendor SYSTEM \"main.dtd\">\n";
}

sub create_sbp_file
{
  opendir THISDIR, $path or die "Cann't open \"$path\" \n";
  my @allfiles = grep / *\.sbp$/, readdir THISDIR;
  my $fileCount = @allfiles;
  if ( $fileCount > 0 ) {
    print "Southbound project file(s) \"@allfiles\" exists under directory \"$path\"\n";
  } else {
    my $sbp_file=$path.$vendor.".sbp";
    open(SBPFILE, "> $sbp_file");
    close(SBPFILE);
    print "Creating Southbound project file \"$sbp_file\"\n";
  }
}

sub copy_dtd_files
{
  if ( -e $main_dtd)
  {
    copy($main_dtd, $path);
    print "Copying $main_dtd to $path\n";
  } else {
    print "Error: $main_dtd doesn\'t exist!\n";
  }
  if ( -e $dict_dtd)
  {
    copy($dict_dtd, $path);
    print "Copying $dict_dtd to $path\n";
  } else {
    print "Error: $dict_dtd doesn\'t exist!\n";
  }
}

sub create_xml_sym_cvt($)
{
  my $create_flag = shift;
  my $xml_sym_cvtSQL;
  if ($create_flag eq 'Create'){
    $xml_sym_cvtSQL=qq( create or replace function xml_sym_cvt(pStr varchar2)
                         return varchar2
                         is
                         vstr varchar2(4000) := null;
                        begin
                           select replace(replace(replace(replace(replace
                             (pStr, chr(38), chr(38)||'amp;'),
                             '<', chr(38)||'lt;'), '>', chr(38)||'gt;'),
                             '\"',chr(38)||'quot;'), '''',chr(38)||'apos;')
                           into vStr
                           from dual;
                           return vStr;
                        end;);
  }
  else{
       if ($create_flag eq 'Drop')
          {
            $xml_sym_cvtSQL=qq(drop function xml_sym_cvt);
          }
       else
       {
          die "Wrong option for create_xml_sym_cvt";
       }
  }

  my $sth_cvtSQL = $dbh->prepare($xml_sym_cvtSQL);
  $sth_cvtSQL->execute;
  $sth_cvtSQL->finish();
}

sub create_cft_folder_fullpath($)
{
  my $create_flag = shift;
  my $create_func_sql;
  if ($create_flag eq 'Create'){
    $create_func_sql=qq( create or replace function cft_folder_fullpath(id number)
                         return varchar2
                         as
                            cursor c_folders(start_id PM_FOLDER.ID%TYPE) is
                               select name
                                 from PM_FOLDER
                                start with id = start_id
                              connect by id = prior folderid;
                         
                            full_name varchar2(255) := NULL;
                         
                         begin
                         
                            for r_folders in c_folders(id)
                            loop
                               full_name := r_folders.name || '/' || full_name;
                            end loop;

                            if full_name is not null
                            then
                               full_name := '/' || full_name;
                            end if;

                            return full_name;

                        end;);
  }
  else{
       if ($create_flag eq 'Drop')
          {
            $create_func_sql=qq(drop function cft_folder_fullpath);
          }
       else
       {
          die "Wrong option for create_cft_folder_fullpath";
       }
  }

  my $sth_prepped_stmt = $dbh->prepare($create_func_sql);
  $sth_prepped_stmt->execute;
  $sth_prepped_stmt->finish();
}

sub get_product_info($)
{
  my $FILE=shift;
  my ($property, $value);
  my ($PropertySQL, $sth_Property);
  $PropertySQL=qq(select xml_sym_cvt(property), xml_sym_cvt(value)
                       from pm_product_info
                      order by property);
  $sth_Property=$dbh->prepare($PropertySQL);
  $sth_Property->execute;
  $sth_Property->bind_columns(undef, \$property, \$value);
  while ($sth_Property->fetch() ) {
    xml_dump(\*$FILE, "<ProductInfo property=\"$property\" value=\"$value\"/>\n");
  }
  $sth_Property->finish();
}

sub get_help_file_type($)
{
  my $FILE=shift;
  my ($file_type_id, $name, $id_base, $id_range, $has_f1_help);
  my ($helpFileTypeSQL, $sth_helpFileType);
  $helpFileTypeSQL=qq(select id, xml_sym_cvt(name),
                             id_base, id_range,
                             decode(has_f1_help, null, 'N', has_f1_help)
                        from pm_help_file_type
                       order by name);
  $sth_helpFileType=$dbh->prepare($helpFileTypeSQL);
  $sth_helpFileType->execute;
  $sth_helpFileType->bind_columns(undef, \$file_type_id,
                                  \$name, \$id_base,
                                  \$id_range, \$has_f1_help);
  while ($sth_helpFileType->fetch() ) {
    xml_dump(\*$FILE, "<HelpFileType ");
    xml_dump(\*$FILE, "name=\"$name\" id_base=\"$id_base\" ");
    xml_dump(\*$FILE, "id_range=\"$id_range\" has_f1_help=\"$has_f1_help\">\n");
    get_help_file(\*$FILE, $file_type_id);
    xml_dump(\*$FILE, "</HelpFileType>\n");
  }
  $sth_helpFileType->finish();
}

sub get_help_file($$)
{
  my $FILE=shift;
  my $help_file_type_id=shift;
  my ($server_path, $client_folder, $client_name, $menu_text, $menu_position, $version);
  my ($helpFileSQL, $sth_helpFile);
  $helpFileSQL=qq(select xml_sym_cvt(server_path),
                         xml_sym_cvt(client_folder),
                         xml_sym_cvt(client_name),
                         xml_sym_cvt(menu_text),
                         menu_position,
                         xml_sym_cvt(version)
                    from pm_help_file
                   where help_file_type=?
                   order by server_path,client_folder,
                         client_name,menu_text,
                         menu_position,version);
  $sth_helpFile=$dbh->prepare($helpFileSQL);
  $sth_helpFile->bind_param(1, $help_file_type_id);
  $sth_helpFile->execute;
  $sth_helpFile->bind_columns(undef, \$server_path, \$client_folder,
                              \$client_name, \$menu_text,
                              \$menu_position, \$version);
  while ($sth_helpFile->fetch() ) {
    xml_dump(\*$FILE, "<HelpFile ");
    (defined $server_path)   && xml_dump(\*$FILE, "server_path=\"$server_path\" ");
    (defined $client_folder) && xml_dump(\*$FILE, "client_folder=\"$client_folder\" ");
    (defined $client_name)   && xml_dump(\*$FILE, "client_name=\"$client_name\" ");
    (defined $menu_text)     && xml_dump(\*$FILE, "menu_text=\"$menu_text\" ");
    (defined $menu_position) && xml_dump(\*$FILE, "menu_position=\"$menu_position\" ");

 
  xml_dump(\*$FILE, "version=\"");
  (defined $version)       && xml_dump(\*$FILE, "$version");
    xml_dump(\*$FILE,"\" ");
    xml_dump(\*$FILE, "/>\n");
   

   # (defined $version)       && xml_dump(\*$FILE, "version=\"$version\"/>\n");

  }
  $sth_helpFile->finish();
}

sub get_extra_ent_property($$)
{
  my $FILE=shift;
  my $entity_id=shift;
  my ($property_name, $property_value);
  my ($entPropertySQL, $sth_entProperty);
  $entPropertySQL=qq(select xml_sym_cvt(name), xml_sym_cvt(value)
                       from peik_extra_ent_property
                      where entity_id= ?
                        and nvl(upper(loader_property),'N')='N' 
                      order by name);
  $sth_entProperty=$dbh->prepare($entPropertySQL);
  $sth_entProperty->bind_param(1, $entity_id);
  $sth_entProperty->execute;
  $sth_entProperty->bind_columns(undef, \$property_name, \$property_value);
  while ($sth_entProperty->fetch() ) {
    xml_dump(\*$FILE, "<ExtraProperty Name=\"$property_name\" Value=\"$property_value\"/>\n");
  }
  $sth_entProperty->finish();
}

#------------------------------------
# SEAde34640: disable LoaderProperty
#------------------------------------
# sub get_loader_ent_property($$)
# {
#   my $FILE=shift;
#   my $entity_id=shift;
#   my $field_id=shift;
#   my ($property_name, $property_value);
#   my ($entPropertySQL, $sth_entProperty);
#   $entPropertySQL=qq(select xml_sym_cvt(name), xml_sym_cvt(value)
#                        from peik_extra_ent_property
#                       where entity_id= ?
#                         and nvl(upper(loader_property),'N')='Y'
#                       order by name);
#   $sth_entProperty=$dbh->prepare($entPropertySQL);
#   $sth_entProperty->bind_param(1, $entity_id);
#   $sth_entProperty->execute;
#   $sth_entProperty->bind_columns(undef, \$property_name, \$property_value);
#   while ($sth_entProperty->fetch() ) {
#     xml_dump(\*$FILE, "<LoaderProperty Name=\"$property_name\" Value=\"$property_value\"/>\n");
#   }
#   $sth_entProperty->finish();
# }

sub get_extra_fld_property($$$)
{
  my $FILE=shift;
  my $entity_id=shift;
  my $field_id=shift;
  my ($property_name, $property_value);
  my ($fldPropertySQL, $sth_fldProperty); 
  $fldPropertySQL=qq(select xml_sym_cvt(name), xml_sym_cvt(value)
                       from peik_extra_fld_property
                      where entity_id= ?
                        and field_id= ?
                        and nvl(upper(loader_property),'N')='N'
                        and upper(name) != 'FIELDNAMEMAPPING'
                      order by name);
  $sth_fldProperty=$dbh->prepare($fldPropertySQL);
  $sth_fldProperty->bind_param(1, $entity_id);
  $sth_fldProperty->bind_param(2, $field_id);
  $sth_fldProperty->execute;
  $sth_fldProperty->bind_columns(undef, \$property_name, \$property_value);
  while ($sth_fldProperty->fetch() ) {
    xml_dump(\*$FILE, "<ExtraProperty Name=\"$property_name\" Value=\"$property_value\"/>\n");
  }
  $sth_fldProperty->finish();

}
#------------------------------------
# SEAde34640: disable LoaderProperty
#------------------------------------
# sub get_loader_fld_property($$$)
# {
#   my $FILE=shift;
#   my $entity_id=shift;
#   my $field_id=shift;
#   my ($property_name, $property_value);
#   my ($fldPropertySQL, $sth_fldProperty);
#   $fldPropertySQL=qq(select xml_sym_cvt(name), xml_sym_cvt(value)
#                        from peik_extra_fld_property
#                       where entity_id= ?
#                         and field_id= ?
#                         and nvl(upper(loader_property),'N')='Y'
#                       order by name);
#   $sth_fldProperty=$dbh->prepare($fldPropertySQL);
#   $sth_fldProperty->bind_param(1, $entity_id);
#   $sth_fldProperty->bind_param(2, $field_id);
#   $sth_fldProperty->execute;
#   $sth_fldProperty->bind_columns(undef, \$property_name, \$property_value);
#   while ($sth_fldProperty->fetch() ) {
#     xml_dump(\*$FILE, "<LoaderProperty Name=\"$property_name\" Value=\"$property_value\"/>\n");
#   }
#   $sth_fldProperty->finish();
# }

sub get_MeasurementType($$$)
{
  my $FILE=shift;
  my $entity_id=shift;
  my $field_id=shift;
  my ($property_name, $property_value);
  my ($fldPropertySQL, $sth_fldProperty); 
  $fldPropertySQL=qq(select xml_sym_cvt(name), xml_sym_cvt(value)
                       from peik_extra_fld_property
                      where entity_id= ?
                        and field_id= ?
                        and upper(name) = 'FIELDNAMEMAPPING'
                      order by name);
  $sth_fldProperty=$dbh->prepare($fldPropertySQL);
  $sth_fldProperty->bind_param(1, $entity_id);
  $sth_fldProperty->bind_param(2, $field_id);
  $sth_fldProperty->execute;
  $sth_fldProperty->bind_columns(undef, \$property_name, \$property_value);
  while ($sth_fldProperty->fetch() ) {
    xml_dump(\*$FILE, "<measurement_type>$property_value</measurement_type>\n");
  }
  $sth_fldProperty->finish();

}

sub get_FontStyle($$)
{
  my $FILE=shift;
  my $stylemask=shift;

  my $bold = 'NO';
  my $underline = 'NO';
  my $strike_thro = 'NO';

  $bold = 'YES' if ( $stylemask == 1 or $stylemask == 3 or $stylemask == 5 or $stylemask == 7 );
  $underline = 'YES' if ( $stylemask == 2 or $stylemask == 3 or $stylemask == 6 or $stylemask == 7 );
  $strike_thro = 'YES' if ( $stylemask == 4 or $stylemask == 5 or $stylemask == 6 or $stylemask == 7 );

  xml_dump(\*$FILE, "<bold>$bold</bold>");
  xml_dump(\*$FILE, "<underline>$underline</underline>");
  xml_dump(\*$FILE, "<strike_thro>$strike_thro</strike_thro>");
     


}
