---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2006,2009. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/client/language_create_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--

-- -----------------------------------------------------------------------------
-- A language that is supported on the prospect system.
-- -----------------------------------------------------------------------------
CREATE TABLE pm_language
(
    id              INTEGER NOT NULL, 
    display_name    NVARCHAR2(128) NOT NULL,
    code            VARCHAR2(20) NOT NULL,
    is_default      CHAR(1) DEFAULT 'N' NOT NULL,
    CONSTRAINT pk_pm_language PRIMARY KEY(id),
    CONSTRAINT uq_pm_language_code UNIQUE(code),
    CONSTRAINT ck_pm_language_default CHECK (is_default IN ('Y', 'N'))
)
TABLESPACE wm_flexpm;

-- -----------------------------------------------------------------------------
-- Sequence for the pm_language table
-- -----------------------------------------------------------------------------
CREATE SEQUENCE seq_pm_language
  START WITH 1 
  INCREMENT BY 1 
  MINVALUE 1 
  MAXVALUE 99999999999999999999 
  CACHE 20 
  NOCYCLE
  NOORDER;

-- -----------------------------------------------------------------------------
-- A language for a user for a specific scope of functionality within the 
-- system.  
-- The functional scope 'DEFAULT' defines the default language selection 
-- for the user.  
-- -----------------------------------------------------------------------------
CREATE TABLE pm_user_language
(
    user_id         INTEGER NOT NULL, 
    language_id     INTEGER NOT NULL,
    funct_scope     VARCHAR2(128) DEFAULT 'DEFAULT' NOT NULL,
    CONSTRAINT fk_userlanguage_user FOREIGN KEY(user_id) REFERENCES pm_user(id) ON DELETE CASCADE,
    CONSTRAINT fk_userlanguage_language FOREIGN KEY(language_id) REFERENCES pm_language(id),
    CONSTRAINT uq_userlanguage_userid UNIQUE(user_id, language_id, funct_scope)
)
TABLESPACE wm_flexpm;
