#Prospect Time Series LoaderControl AWK script
# (#) ./base/db/Oracle8i/creation_scripts/buildControl.nawk BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
# invoke as follows
#   nawk -f makeControl.nawk <Table Definition File>
#
# where <Table Definition File> is a <TAB> separated file in the following
#   Field 1  =  TABLE_NAME            e.g. CDMA_PAF
#   Field 2  =  Key Field Name        e.g. sectid
#   Field 3  =  Key Field Type        e.g. VARCHAR2(12)
#   Field 4  =  Key Field Expression  e.g. CELL.CELLINFO.ECP.f0 + "-" + substring(CELL.f0,2,3) + "-" + f0
#   Field 5  =  Time Field Name       e.g. tstamp
#   Field 6  =  Time Field Expression e.g. CELL.CELLINFO.f1 + " " + CELL.CELLINFO.f2
#   Field 7  =  Calendar Name         e.g. 'HOURLY'
#   Field 8  =  No of Data Fields     e.g. 48
#   Field 9  =  Miscellaneous Parameters   See global parameters below
#   Field 10 =  Field 3 Use           e.g. data
#   Field 11 =  Field 3 Type          e.g. INTEGER
#   Field 12 =  Field 3 Name          e.g. duration
#   Field 13 =  Field 3 Expression    e.g. F2 + F3
#   Field 14 =  Field 4 Use           e.g. key
#   Field 15 =  Field 4 Type          e.g. VARCHAR2(10)
#   Field 16 =  Field 4 Name          e.g. rate
#   Field 17 =  Field 4 Expression    e.g. substring(F1,0,6)
#   Field 18 =  Field 5 Use    etc....
#
# All fields are <TAB> separated and any line with a hash as the very
# FIRST character = # is considered a comment line and is ignored.
#

BEGIN{
  Release="BL_core_schema_8.0.7_8.0.7.5.02_20100605"
  "who -m | cut -f1 -d \" \"" | getline Author

#GLOBAL PARAMETERS
#Define Parameter Names and Default Global Values
  global_parameter["TIMESERIES"]      = "YES"
  global_parameter["LOADER_NAME"]     = "LucentRelease13p0"
  global_parameter["LOADER_VERSION"]  = "7.0"
  global_parameter["TABLE_NAME"]      = "NULL"
  global_parameter["TABLE_REF"]       = "NULL"
  global_parameter["PACKAGE_NAME"]    = "NULL"
  global_parameter["SKIP_COLS"]       = "NULL"
  global_parameter["SKIP_LOAD_COLS"]  = "NULL"
  global_parameter["COLUMN_DEF_FILE"] = "NULL"
  global_parameter["BOOLEAN_LOAD"]    = 1

  #Read Global Parameters used by "buildTS.nawk"
  FS="\""
  check_files["buildDBTable.nawk"] = 1
  check_files["buildTS.nawk"] = 1
  check_files["buildTruncate.nawk"] = 1
  for (file2check in check_files) while (getline < file2check > 0)
  {
    if ($1 ~ "global_parameter" && $3 ~ "=") other_parameters[ $2 ] = 1
  }
  FS = "\t"

  #Set "exit code" to 0  (indicates NO errors have occurred)
  exit_code=0

  #Define the first LoaderSequence number (incremented for each TableDefLine)
  loader_sequence=1

  #Create sub-directory to store PER-TABLE loadmaps
  system("if [ ! -d LOAD_CONTROL ]; then mkdir LOAD_CONTROL; fi")

  #Clean-up old files at script startup
  #
  #Note: When 2 different load control fields are defined for the same tables
  #      they will be concatenated onto the same file.  Therefore all files
  #      in the LOAD_CONTROL directory MUST be deleted to ensure old configs
  #      are not appended to.
  #
  system("rm -f LOAD_CONTROL/*")
  system("rm -f LoaderControl.txt")
  system("rm -f MakeFullLoadmap.sh")

}


#FUNCTION:reset_local_parameters()
#
#This function sets the array "local_parameter" to the Global or Default values
#
function reset_local_parameters() {
  for ( param in global_parameter) local_parameter[param] = global_parameter[param]
}

#FUNCTION:get_local_parameters()
#
#This function will take a field of comma separated name=value pairs
#and update the parameters stored in array "local_parameter".  If the
#parameter is unknown an error message is printed and the exit code
#is incremented.
#
function get_local_parameters( parameter_csv ) {
  reset_local_parameters()
  #Remove stray spaces
  gsub(" ","",parameter_csv)
  #Remove stray quotes
  gsub("\"","",parameter_csv)
  #Separate each parameter
  no_of_params=split(parameter_csv, pairs, ",")
  #Process each Parameter Pair
  for (position in pairs)
  {
    split(pairs[position],parameter,"=")
    if (parameter["1"] in local_parameter) local_parameter[parameter["1"]] = parameter["2"]
    else if ( ! (parameter["1"] in other_parameters ) )
    {
      print "===================================================="
      print "TABLE parameters are corrupted at line " NR
      printf("Parameter %s is UNKNOWN\n",parameter["1"])
      print "===================================================="
      exit_code += 1
    }
  }
  print "     Non-Default parameter values follow...."
  for ( param in local_parameter)
    if (local_parameter [param] != global_parameter[param] )
      printf("      %-20s= %s\n", param, local_parameter[param])
}


#FUNCTION:set_global_parameters()
#
#This function will take a field of comma separated name=value pairs
#and update the parameters stored in array "global_parameter".  If the
#parameter is unknown an error message is printed and the exit code
#is incremented.
#
function set_global_parameters( parameter_csv ) {
  #Remove stray spaces
  gsub(" ","",parameter_csv)
  #Remove stray quotes
  gsub("\"","",parameter_csv)
  #Separate each parameter
  no_of_params=split(parameter_csv, pairs, ",")
  #Process each Parameter Pair
  for (position in pairs)
  {
    split(pairs[position],parameter,"=")
    if (parameter["1"] in global_parameter) global_parameter[parameter["1"]] = parameter["2"]
    else if ( ! (parameter["1"] in other_parameters ) )
    {
      print "======================================================"
      print "Global TABLE parameters are corrupted at line " NR
      printf("Parameter %s is UNKNOWN\n",parameter["1"])
      print "======================================================"
      exit_code += 1
    }
  }
  print  ""
  printf("GLOBAL Parameters RESET at line %d in %s\n", NR, FILENAME)
  print  "Current values follow...."
  for ( param in global_parameter)
    printf(" %-20s= %s\n", param, global_parameter[param])
}


#FUNCTION:print_parameters()
#
#This function will print to the screen a list of all the parameters
#and their current stored value for a passed parameter array.
#
function print_parameters(param_array) {
  for ( param in param_array) {
    printf("\t%s=%s",param,param_array[param])
    }
  printf("\n")
}


#FUNCTION:get_key_value()
#
#This function when passed a LOADER_VERSION number returns the appropriate
#key string to be inserted into the LoaderControl.txt table.
#
function get_key_value( key_input, loader_release )
{
  if (key_input == "data")
    return( "0" )
  else
    if (key_input == "key") 
      if (loader_ver == "1.2")
        return( "1" )
      else if (loader_ver == "1.3")
         return ("K")
      else if (loader_ver == "7.0")
        return( "KD" )
    else if (key_input == "sql" && loader_ver != "1.2") 
        return( "S" )
    else
      if (loader_ver == "1.2")
      {
        exit_code += 1
        print "*******************************************************************************"
        print "ERROR Loader Release 1.2 should only have a key_values of 'data' or 'key'"
        print "*******************************************************************************"
        return( "0" )
      }

      #If it get's this far it's using an unrecognised "Use" field
      exit_code += 1
      print "*******************************************************************************"
      print "WARNING Unrecognized \"Use\" field = " key_input
      print "*******************************************************************************"
      #Although the above message does not say this, the key_input in from the
      #definition file will be passed straight in to the LoaderControl.txt
      return( key_input )
}

#START OF MAIN SCRIPT BODY
#
#Each line of the TableDefinitionFile will be processed by this part
#
{
  # Skip any comment lines in the TABLES file, check for EOF
  gotline=1
  while (substr($0,1,1) == "#" && gotline > 0) gotline=getline

  #Global Parameters were added to support table configuration
  while (gotline > 0 && NF == 2 && $1 == "GLOBAL_PARAMETERS")
  {
    set_global_parameters( $2 )
    gotline=getline
  }

  # Skip any comment lines that follow GLOBAL_PARAMETERS, check for EOF
    while (gotline > 0 && substr($0,1,1) == "#") gotline=getline

  #If one of the above getlines reads an EOF exit script
  if (gotline <= 0) exit(0)

  # Produce an Error Message when number of fields is less than 8
  # in the TABLES file, also check for comments during errors.
  gotline=1
  while (gotline > 0 && (NF < 8 || (NF > 9 && (NF-9)%4 > 0)))
  {
    print "=========================================================="
    print "Warning TABLE definition file is corrupted at line " NR
    print "Filename = " FILENAME
    print "Contents = {" $0 "}"
    print "                                              ....skipping"
    print "=========================================================="
    exit_code+=1
    gotline=getline
    while (gotline > 0 && substr($0,1,1) == "#") getline
  }
  #If EOF exit script
  if (gotline == 0) exit(99)

  #OK Let's do it !
  print"----------------------------------------------------------"
  printf("PROCESSING Line %d for %s\n", NR, $1)

  #Set Configurable parameter variables for this table
  if ($9 == "") reset_local_parameters()
  else get_local_parameters($9)

  #If BOOLEAN_LOAD is set to false then this line does not specify
  #any LoaderControl data.
  if (local_parameter["BOOLEAN_LOAD"] != 1) next

  if (local_parameter["TABLE_NAME"] == "NULL") ts_name = $1
   else ts_name = local_parameter["TABLE_NAME"]
  if (local_parameter["TABLE_REF"] == "NULL") table_ref = ts_name
   else table_ref = local_parameter["TABLE_REF"]
  if (local_parameter["PACKAGE_NAME"] == "NULL") package = ts_name
   else package = local_parameter["PACKAGE_NAME"]
  table_count[ts_name] += 1

  #Reset skip fields array
  for (i in skip_fields)
    delete skip_fields[i]
  #Store any column names that are required to be skipped for
  #loading and table creation in array "skip_fields"
  if (local_parameter["SKIP_COLS"] != "NULL")
  {
    split(local_parameter["SKIP_COLS"], parts, ";")
    for (i in parts)
      skip_fields[parts[i]] = 1
  }
  #Add any column names that are required to be skipped for
  #loading in array "skip_fields"
  if (local_parameter["SKIP_LOAD_COLS"] != "NULL")
  {
    split(local_parameter["SKIP_LOAD_COLS"], parts, ";")
    for (i in parts)
      skip_fields[parts[i]] = 1
  }


  #Set the Loader Version from the parameters
  #
  loader_ver=local_parameter["LOADER_VERSION"]
  valid_loader_ver["1.2"]=1
  valid_loader_ver["1.3"]=1
  valid_loader_ver["7.0"]=1
  if ( ! (loader_ver in valid_loader_ver) )
  { 
    loader_ver = "1.3"
    exit_code += 1
    print "*******************************************************************************"
    printf("WARNING invalid LOADER_VERSION (%s) defaulting to %s\n", local_parameter["LOADER_VERSION"], loader_ver)
    print "*******************************************************************************"
  }
  # N.B. A single LoaderControl.txt file can only be one type if loader
  #      so it is invalid to change the loader type in the middle of the
  #      file.  The following variable "first_loader_ver" is set the first
  #      time any loader is encountered, if another version is found an
  #      error is produced.
  #
  if (first_loader_ver != "" && first_loader_ver != loader_ver)
  {
    exit_code += 1
    print "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
    print "ERROR only one LOADER_VERSION can be used per LoaderControl.txt file"
    printf("See Table Definition file, line %d\n", NR)
    print "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
  }


  #Create an ARRAYs to store the column names, key info and expression
  #(NOTE: Not stored if field is marked as skipped)
  #
  #N.B. This is slightly different than the build scripts because the
  #     Indentity and timestamp fields are also stored in this array.
  #
  #Set Identity data
  column_name[1] = $2
  column_exp[1]  = $4
  if ( loader_ver == "1.2" ) column_key[1] = "1"
   else if (loader_ver == "1.3") column_key[1] = "K"
   else if (loader_ver == "7.0") column_key[1] = "KD"
  #Set timestamp data
  if (local_parameter["TIMESERIES"] == "NO:STORE_TSTAMP")
  {
    #We want to use the Timestamp as a key but not use TimeSeries Cartridge
    #If you require a Timestamp as part of the data use the colDefFile to
    #define the column and data values
    column_name[2] = $5
    column_exp[2]  = $6
    if (loader_ver != "7.0") column_key[2]  = "K"
      else column_key[2] = "KD"
    column_count   = 2
  }
  else if (substr(local_parameter["TIMESERIES"],1,2) == "NO")
  {
    #This is not a timeseries TABLE
    column_count=1
  }
  else
  {
    #Standard Timeseries Table
    column_name[2] = $5
    column_exp[2]  = $6
    if ( loader_ver == "1.2" ) column_key[2] = "1"
     else column_key[2] = "T"
    column_count=2
  }
  #
  data_fields=$8
  use_field=10
  while ($use_field != "")
  {
    name4column = $(use_field+2)
    if (! ( name4column in skip_fields) )
    {
      column_count += 1
      column_name[column_count]=$(use_field+2)
      column_exp[column_count]=$(use_field+3)
      column_key[column_count] = get_key_value( $use_field, loader_ver )
    }
    use_field=use_field+4
  }



  # Process Column Description Files where configured
  if (local_parameter["COLUMN_DEF_FILE"] != "NULL")
  {
    # This ColumnDefFiles takes has the following fields
    #   Field 1 =  Field 3 Name          e.g. duration
    #   Field 2 =  Field 3 Use           e.g. data OR key
    #   Field 3 =  Field 3 Type          e.g. INTEGER, VARCHAR2(10) etc..
    #   Field 4 =  Field 3 Expression    e.g. f2 + f3
    #
    #Note: The following getline DESTROYS the buffer loaded from the
    #      TableDefFile so direct references to it's parameters
    #      e.g. $1, $2 etc... cannot be used from this point onwards
    #
    gotline=1
    while (gotline > 0)
    {
      gotline = getline < local_parameter["COLUMN_DEF_FILE"]
      if (gotline > 0)
      {
        #Ignore Comment lines in ColDefFile
        if ( ! (substr($0,1,1) == "#") )
        {
          #Check only 4 fields are included in ColDefFile
          if (NF == 4)
          {
            if (! ($1 in skip_fields) )
            {
              column_count += 1
              column_name[column_count] = $1
              column_exp[column_count] = $4
              if ($2 == "key")
              {
                 if (loader_ver != "7.0") column_key[column_count] = "K"
                  else column_key[column_count] = "KD"
	      }
	      else if ($2 == "sql")
		column_key[column_count] = "S"
              else
                column_key[column_count] = 0
            }
          }
          else
          {
            exit_code += 1
            print "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
            printf("ERROR: Column Definition Invalid for file %s CONTENTS={%s}\n", local_parameter["COLUMN_DEF_FILE"], $0)
            print "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
          }
        }
      }
    }
    close(local_parameter["COLUMN_DEF_FILE"])
    if ( gotline < 0 )
    {
      exit_code += 1
      print "==============================================================================="
      printf("ERROR Reading Column Definition file -> %s\n", local_parameter["COLUMN_DEF_FILE"])
      print "==============================================================================="
    }
    close( local_parameter["COLUMN_DEF_FILE"] )
  }


  #Set the Loader Name from the parameters
  loader_name=local_parameter["LOADER_NAME"]
  if (loader_name == "") loader_name = "DEFAULT"


  #Define the Output filename + other stuff
  loadmapfile=sprintf("LOAD_CONTROL/%s_Loadmap", ts_name)
  printf("cat %s >> LoaderControl.txt\n",loadmapfile) >> "MakeFullLoadmap.sh"
  "date +\"%m/%d/%Y %T\"" | getline date_str

  #
  #Start the LoadControl data Generation Loop
  #
  printf ("#############################################\n")	>> loadmapfile
  printf ("## WatchMark Prospect Loader Control File  ##\n")	> loadmapfile
  printf ("##   Loader:     %-26s##\n",loader_name)		> loadmapfile
  printf ("##   Table:      %-26s##\n",ts_name)			> loadmapfile
  printf ("##   Appearance: %-26d##\n",table_count[ts_name])	> loadmapfile
  printf ("##   Script Rel: %-26s##\n",Release)			> loadmapfile
  printf ("##   Run by:     %-26s##\n",Author)			> loadmapfile
  printf ("##   Date:       %-26s##\n",date_str)		> loadmapfile
  printf ("#############################################\n")	> loadmapfile
  print   "#"							> loadmapfile

  if (loader_ver == "1.2")
  {
    #Loader 1.2 headers
    printf ("#loader\tseqNum\tcolName\ttableRef\tsType\tpackage\tkeyFlag\tuser\tdataTime\texpression\n")	> loadmapfile
    print   "#"							> loadmapfile
    for (i = 1; i <= column_count; i++) printf ("%s\t%d\t%s\t%s\t_\t%s\t%s\t1\t1\t%s\n", loader_name, loader_sequence, column_name[i], table_ref, package, column_key[i], column_exp[i]) > loadmapfile
    printf ("\n") > loadmapfile
  }
  else
  {
    #Loader 1.3 headers
    printf ("#loader\tseqNum\tcolName\ttableRef\tsType\tpackage\tkeyFlag\texpression\n")	> loadmapfile
    print   "#"							> loadmapfile
    for (i = 1; i <= column_count; i++) printf ("%s\t%d\t%s\t%s\t_\t%s\t%s\t%s\n", loader_name, loader_sequence, column_name[i], table_ref, package, column_key[i], column_exp[i]) > loadmapfile
    printf ("\n") > loadmapfile
  }
  close(loadmapfile)
  loader_sequence = loader_sequence+1
}
END {
  print"----------------------------------------------------------"
  close("MakeFullLoadmap.sh")
  system("chmod u+x ./MakeFullLoadmap.sh")
  system("./MakeFullLoadmap.sh")
  system("rm -f MakeFullLoadmap.sh")
  if (exit_code != 0)
  {
    print ""
    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    print "ERROR : Problems occured whist running this script"
    print "        Rectify the cause and re-run befors using"
    print "        any SQL scripts created on this run"
    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    print ""
  }
}
