#Prospect Timeseries Table Creation AWK script
# (#) ./base/db/Oracle8i/creation_scripts/buildTS.nawk BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
# invoke as follows
#   nawk -f buildTS.nawk <Table Definition File>
#
# where <Table Definition File> is a <TAB> separated file in the following
#   Field 1  =  TABLE_NAME            e.g. CDMA_PAF
#   Field 2  =  Key Field Name        e.g. sectid
#   Field 3  =  Key Field Type        e.g. VARCHAR2(12)
#   Field 4  =  Key Field Expression  e.g. CELL.CELLINFO.ECP.f0 + "-" + substring(CELL.f0,2,3) + "-" + f0
#   Field 5  =  Time Field Name       e.g. tstamp
#   Field 6  =  Time Field Expression e.g. CELL.CELLINFO.f1 + " " + CELL.CELLINFO.f2
#   Field 7  =  Calendar Name         e.g. 'HOURLY'
#   Field 8  =  No of Data Fields     e.g. 48
#   Field 9  =  Miscellaneous Parameters   See Below for examples
#   Field 10 =  Field 3 Use           e.g. data
#   Field 11 =  Field 3 Type          e.g. INTEGER
#   Field 12 =  Field 3 Name          e.g. duration
#   Field 13 =  Field 3 Expression    e.g. F2 + F3
#   Field 14 =  Field 4 Use           e.g. key
#   Field 15 =  Field 4 Type          e.g. VARCHAR2(10)
#   Field 16 =  Field 4 Name          e.g. rate
#   Field 17 =  Field 4 Expression    e.g. substring(F1,0,6)
#   Field 18 =  Field 5 Use    etc....
#
# All fields are <TAB> separated and any line with a hash as the very
# FIRST character = # is considered a comment line and is ignored.
#
BEGIN{
  Release="BL_core_schema_8.0.7_8.0.7.5.02_20100605"
  "who -m | cut -f1 -d \" \"" | getline Author
  "date '+%Y-%m-%d'" | getline Today
  Today4partition=substr(Today,1,4) substr(Today,6,2) substr(Today,9,2)

#GLOBAL PARAMETERS
#Define Parameter Names and Default Global Values
  global_parameter["ORACLE_RELEASE"]      = "8.1.5"
  global_parameter["TIMESERIES"]          = "YES"
  global_parameter["DATA_TYPE"]           = "NUMBER(8)"
  global_parameter["STORAGE_INITIAL_TAB"] = "1M"
  global_parameter["STORAGE_NEXT_TAB"]    = "1M"
  global_parameter["STORAGE_INITIAL_CAL"] = "1K"
  global_parameter["STORAGE_NEXT_CAL"]    = "1K"
  global_parameter["STORAGE_INITIAL_MAP"] = "32K"
  global_parameter["STORAGE_NEXT_MAP"]    = "32K"
  global_parameter["PCTFREE"]             = "5"
  global_parameter["PCTUSED"]             = "90"
  global_parameter["SCHEMA_NAME"]         = "NULL"
  global_parameter["TABLESPACE"]          = "NULL"
  global_parameter["TABLESPACE_TAB"]      = "NULL"
  global_parameter["TABLESPACE_CAL"]      = "NULL"
  global_parameter["TABLESPACE_MAP"]      = "NULL"
  global_parameter["TABLE_NAME"]          = "NULL"
  global_parameter["SKIP_COLS"]           = "NULL"
  global_parameter["SKIP_BUILD_COLS"]     = "NULL"
  global_parameter["COLUMN_DEF_FILE"]     = "NULL"
  global_parameter["PARTITION_SDATE"]     = "NULL"
  global_parameter["PARTITION_EDATE"]     = "NULL"
  global_parameter["PARTITION_OFFSET"]    = 0
  global_parameter["BOOLEAN_BUILD"]       = 1
  global_parameter["USE_IOT_TABLE"]       = 1
  global_parameter["USE_PUB_SYNONYMS"]    = 1
  global_parameter["LOADER_HISTORY_TAB"]  = "LOADER_HISTORY"
  global_parameter["FREELISTS"]            = 4
  #Read Global Parameters used by "buildControl.nawk"
  FS="\""
  check_files["buildControl.nawk"] = 1
  check_files["buildDBTable.nawk"] = 1
  check_files["buildTruncate.nawk"] = 1
  for (file2check in check_files) while (getline < file2check > 0)
  {
    if ($1 ~ "global_parameter" && $3 ~ "=") other_parameters[ $2 ] = 1
  }
  FS = "\t"

  #Set CONSTANT values 
  #
  MAX_TS_COLS=80
  #
  # NOTE: Max Timeseries fields in release 8.1.5 is 80 columns, this may change
  #       in later releases of Oracle
  #
  MAX_IOT_COLS=512
  #
  # NOTE: Max Columns in an IOT table is 512 so any tables with more columns
  #       than this will automatically become Non-IOT tables

  #Set up Calendar Days
  #
  monthdy["01"]=31
  monthdy["02"]=28
  monthdy["03"]=31
  monthdy["04"]=30
  monthdy["05"]=31
  monthdy["06"]=30
  monthdy["07"]=31
  monthdy["08"]=31
  monthdy["09"]=30
  monthdy["10"]=31
  monthdy["11"]=30
  monthdy["12"]=31

  #Create Per-Table directories if they don't already exist
  system ("if [ ! -d BUILD ]; then mkdir BUILD; fi")
  system ("if [ ! -d DROP ]; then mkdir DROP; fi")

  #Set "exit code" to 0  (indicates NO errors have occurred)
  exit_code=0
  #Store current date into a string variable "script_date"
  "date" | getline script_date

  #Start of Script Creation
  #
  printf ("-- Drop all Timeseries objects Script\n")	> "dropAllTimeSeries.sql"
  print   "--"						> "dropAllTimeSeries.sql"
  print   "-- Script Version:  " Release		> "dropAllTimeSeries.sql"
  print   "-- Script Ran by:   " Author			> "dropAllTimeSeries.sql"
  print   "-- Script Date:     " script_date		> "dropAllTimeSeries.sql"
  print   ""						> "dropAllTimeSeries.sql"
#  print   "PAUSE WARNING This is a DESTRUCTIVE script, Press Ctrl-C if you want ABORT"	> "dropAllTimeSeries.sql"
  printf ("-- Make all Timeseries objects Script\n")	> "makeAllTimeSeries.sql"
  print   "--"						> "makeAllTimeSeries.sql"
  print   "-- Script Version:  " Release		> "makeAllTimeSeries.sql"
  print   "-- Script Ran by:   " Author			> "makeAllTimeSeries.sql"
  print   "-- Script Date:     " script_date		> "makeAllTimeSeries.sql"
  print   ""						> "makeAllTimeSeries.sql"
}


#FUNCTION:reset_local_parameters()
#
#This function sets the array "local_parameter" to the Global or Default values
#
function reset_local_parameters() {
  for ( param in global_parameter) local_parameter[param] = global_parameter[param]
}


#FUNCTION:get_local_parameters()
#
#This function will take a field of comma separated name=value pairs 
#and update the parameters stored in array "local_parameter".  If the
#parameter is unknown an error message is printed and the exit code
#is incremented.
#
function get_local_parameters( parameter_csv ) {
  reset_local_parameters()
  #Remove stray spaces
  gsub(" ","",parameter_csv)
  #Remove stray quotes
  gsub("\"","",parameter_csv)
  #Separate each parameter
  no_of_params=split(parameter_csv, pairs, ",")
  #Process each Parameter Pair
  for (position in pairs)
  {
    split(pairs[position],parameter,"=")
    if (parameter["1"] in local_parameter) local_parameter[parameter["1"]] = parameter["2"]
    else if (! (parameter["1"] in other_parameters) )
    {
      print "===================================================="
      print "TABLE parameters are corrupted at line " NR
      printf("Parameter %s is UNKNOWN\n",parameter["1"])
      print "===================================================="
      exit_code += 1
    }
  }
  print "     Non-Default parameter values follow...."
  for ( param in local_parameter)
    if (local_parameter [param] != global_parameter[param] )
      printf("      %-20s= %s\n", param, local_parameter[param])
}


#FUNCTION:set_global_parameters()
#
#This function will take a field of comma separated name=value pairs 
#and update the parameters stored in array "global_parameter".  If the
#parameter is unknown an error message is printed and the exit code
#is incremented.
#
function set_global_parameters( parameter_csv ) {
  #Remove stray spaces
  gsub(" ","",parameter_csv)
  #Remove stray quotes
  gsub("\"","",parameter_csv)
  #Separate each parameter
  no_of_params=split(parameter_csv, pairs, ",")
  #Process each Parameter Pair
  for (position in pairs)
  {
    split(pairs[position],parameter,"=")
    if (parameter["1"] in global_parameter) global_parameter[parameter["1"]] = parameter["2"]
    else if (! (parameter["1"] in other_parameters) )
    {
      print "======================================================"
      print "Global TABLE parameters are corrupted at line " NR
      printf("Parameter %s is UNKNOWN\n",parameter["1"])
      print "======================================================"
      exit_code += 1
    }
  }
  print  ""
  printf("GLOBAL Parameters RESET at line %d in %s\n", NR, FILENAME)
  print  "Current values follow...."
  for ( param in global_parameter)
    printf(" %-20s= %s\n", param, global_parameter[param])
}


#FUNCTION:chk_leapyear( DATE )
#
#DATE is format YYYYMMDD
#
# This function returns the number of days in the month of
# February dependent on whether the current year is a leap-year.
# The array value monthdy["02"] may be modified by this function.
#
function chk_leapyear( ly_date ) {
  ly_year=substr(ly_date,1,4)
  if (ly_year % 4 == 0)
    if (ly_year % 400 == 0) monthdy["02"]=29
    else if (ly_year % 100 == 0) monthdy["02"]=28
         else monthdy["02"]=29
  else monthdy["02"]=28
  return( monthdy["02"] )
}


#FUNCTION:get_nextday( DATE )
#
#DATE is format YYYYMMDD
#
# This function will derive the next day from the current day.  An array
# monthdy[] must be pre-defined in the BEGIN section of this script and 
# the date formats must be correct.  The array value monthdy["02"] may
# be modified by this function.
#
function get_nextday( current_date ) {
  #Date is format YYYYMMDD
  chk_leapyear(current_date)
  cur_year=substr(current_date,1,4)
  cur_month=substr(current_date,5,2)
  cur_day=substr(current_date,7,2)
  if (cur_day < monthdy[cur_month]) 
  {
    ret_day = cur_day + 1
    ret_mon = cur_month
    ret_year = cur_year
  }
  else 
  {
    ret_day = 1
    if (cur_month == "12") 
    {
      ret_mon = 1
      ret_year = cur_year + 1
    }
    else 
    {
      ret_mon = cur_month + 1
      ret_year = cur_year
    }
  }
  return(sprintf("%04d%02d%02d",ret_year, ret_mon, ret_day))
}

#FUNCTION:chk_partidate( DATE )
#
#DATE is format YYYYMMDD
#
# This function will perform basic checks on Partition date strings
# The array value monthdy["02"] may be modified by this function.
#
function chk_partidate( parti_date ) {
  if ( parti_date == "SYSDATE" ) parti_date = Today4partition
  else if ( parti_date ~ /SYSDATE\+[0-9]+/ )
  {
    days2add = substr(parti_date, 9)
    parti_date = Today4partition
    while ( days2add > 0 ) 
    {
      parti_date = get_nextday( parti_date )
      days2add -= 1
    }
  }
  if ( parti_date == "STARTDATE" ) parti_date="STARTDATE+0"
  if ( parti_date ~ /STARTDATE\+[0-9]+/ )
  {
    days2add = substr(parti_date, 11)
    parti_date = local_parameter["PARTITION_SDATE"]
    if (parti_date ~ /STARTDATE\+[0-9]+/ )
    {
      print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
      print "Cannot use PARTITION_SDATE with STARTDATE+<Num> format" 
      print "Line = " NR
      print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
      exit_code += 1
      parti_date="NULL"
      return(parti_date)
    }
    if (parti_date == "NULL" )
    {
      print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
      print "PARTITION_SDATE is NULL Relative end-date not possible" 
      print "Line = " NR
      print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
      exit_code += 1
      parti_date="NULL"
      return(parti_date)
    }
    else while ( days2add > 0 ) 
    {
        parti_date = get_nextday( parti_date )
        days2add -= 1
    }
  }
  chk_leapyear(parti_date)
  current_month=substr(parti_date,5,2)
  current_day=substr(parti_date,7,2)
  if ( parti_date !~ /[12][0-9][0-9][0-9][01][0-9][0-3][0-9]/ )
  {
    print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
    print "Error PARTITION DATE invalid at line " NR 
    print "Date = " parti_date
    print "Valid Formats:"
    print "    Numerical String in format YYYYMMDD"
    print "    Literal String SYSDATE to use script run date"
    print "    SYSDATE+<Num> to use script run date + Num days"
    print "    STARTDATE to use partition start_date"
    print "    STARTDATE+<Num> to use partition start_date + Num days"
    print "Note: These are case and <space> sensitive strings"
    print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
    exit_code += 1
    parti_date="NULL"
  }
  else if (current_month + 0 > 12 )
  {
    print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
    print "Error PARTITION DATE month TOO LARGE at line " NR 
    print "Date = " parti_date
    print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
    exit_code += 1
    parti_date="NULL"
  }
  else if (current_day + 0 > monthdy[current_month] )
  {
    print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
    print "Error PARTITION DATE day TOO LARGE at line " NR 
    print "Date = " parti_date
    print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
    exit_code += 1
    parti_date="NULL"
  }
  return(parti_date)
}



#START OF MAIN SCRIPT BODY
#
#Each line of the TableDefinitionFile will be processed by this part
#
{
  # Skip any comment lines in the TABLES file, check for EOF
  gotline=1
  while (gotline > 0 && substr($0,1,1) == "#") gotline=getline

  # Check if TableDefFile defines new default Parameters
  while (gotline > 0 && NF == 2 && $1 == "GLOBAL_PARAMETERS")
  {
    set_global_parameters( $2 )
    gotline=getline
  }

  # Skip any comment lines that follow GLOBAL_PARAMETERS, check for EOF
  while (gotline > 0 && substr($0,1,1) == "#") gotline=getline

  #If one of the above getlines reads an EOF exit script
  if (gotline <= 0) exit(0)

  # Produce an Error Message when number of fields is less than 8,
  # or greater than 9 but has an non-valid number of fields.
  gotline=1
  while (gotline > 0 && (NF < 8 || (NF > 9 && (NF-9)%4 > 0)))
  {
    print "=========================================================="
    print "Warning TABLE definition file is corrupted at line " NR
    print "Filename = " FILENAME
    print "Contents = {" $0 "}"
    print "                                              ....skipping"
    print "=========================================================="
    exit_code += 1
    gotline=getline
    while (gotline > 0 && substr($0,1,1) == "#") gotline=getline
  }
  #If EOF exit script
  if (gotline <= 0) exit(0)

  #OK Let's do it !
  print"----------------------------------------------------------"
  printf("PROCESSING Line %d for %s\n", NR, $1)
  #reset the pk_string
  pk_string=""
  #Set Configurable parameter variables for this table
  if ($9 == "") reset_local_parameters()
  else get_local_parameters($9)

  #If BOOLEAN_BUILD is set to false then this line does not
  #define any table building commands so skip to next table
  if (local_parameter["BOOLEAN_BUILD"] != 1) next

  # Start of main loop for each table.
  #Note:ts_name may be reset by parameters later in this script
  ts_name=$1
  key_name = $2
  key_type = $3
  time_name = $5
  ts_calendar = $7
  data_fields = $8
  if (local_parameter["SCHEMA_NAME"] == "NULL") db_schema = ""
  else db_schema=sprintf("%s.",local_parameter["SCHEMA_NAME"])

  #Set table name to parameter if defined otherwise use field 1 of TableDefFile
  if (local_parameter["TABLE_NAME"] == "NULL") ts_name = $1
   else ts_name = local_parameter["TABLE_NAME"]
  tab_name = ts_name "_tab"
  buildfilename=sprintf("BUILD/%s_build.sql",ts_name)
  dropfilename=sprintf("DROP/%s_drop.sql",ts_name)

  #NOTE: The skip field array "skip_fields" contains a list of fields that
  #      are not going to be used to create columns in the database table
  #      It is possible to skip fields in BOTH table creation and loading
  #      using parameter SKIP_COLS, or simply define a set of columns to 
  #      skip when building the table using parameter SKIP_BUILD_COLS.
  #
  #Reset skip fields array
  for (i in skip_fields)
    delete skip_fields[i]
  #Store any column names that are required to be skipped for
  #loading and table creation in array "skip_fields"
  if (local_parameter["SKIP_COLS"] != "NULL")
  {
    split(local_parameter["SKIP_COLS"], parts, ";")
    for (i in parts)
      skip_fields[parts[i]] = 1
  }
  #Add any column names that are required to be skipped for
  #table creation in array "skip_fields"
  if (local_parameter["SKIP_BUILD_COLS"] != "NULL")
  {
    split(local_parameter["SKIP_BUILD_COLS"], parts, ";")
    for (i in parts)
      skip_fields[parts[i]] = 1
  }

  #Create an ARRAYs to store the column name and type, plus generate
  #the primary key string. (NOTE: Not stored if field is marked as skipped)
  #
  column_count=0
  use_field=10
  # If statement moved to 541
  #
  while ($use_field != "")
  {
    name4column = $(use_field+2)
    if (! ( name4column in skip_fields) )
    {
      column_count += 1
      column_name[column_count]=$(use_field+2)
      column_type[column_count]=$(use_field+1)
      if ($use_field == "key") pk_string = sprintf("%s, %s", pk_string, $(use_field + 2))
    }
    use_field=use_field+4
  }
  for (i=1; i <= data_fields; i++) 
  {
    name4column = sprintf("f%d", i)
    if (! (name4column in skip_fields) )
    {
      column_count += 1
      column_name[column_count] = name4column
      column_type[column_count] = local_parameter["DATA_TYPE"]
    }
  }

  # Moving from line 462,463
  # This will cause the TSTAMP column to be placed last after the 
  # columns identified in the tabledeffile, and then add whatever
  # key columns are noted in the column def file.
  if (local_parameter["TIMESERIES"] == "NO") pk_string=sprintf("%s",key_name)
  else pk_string=sprintf("%s%s, %s ",key_name, pk_string, time_name)

  # Process Column Description Files where configured
  if (local_parameter["COLUMN_DEF_FILE"] != "NULL")
  {
    # This ColumnDefFiles takes has the following fields
    #   Field 1 =  Field 3 Name          e.g. duration
    #   Field 2 =  Field 3 Use           e.g. data OR key
    #   Field 3 =  Field 3 Type          e.g. INTEGER, VARCHAR2(10) etc..
    #   Field 4 =  Field 3 Expression    e.g. F2 + F3
    #
    #Note: The following getline DESTROYS the buffer loaded from the
    #      TableDefFile so direct references to it's parameters
    #      e.g. $1, $2 etc... cannot be used from this point onwards
    #
    gotline=1
    while (gotline > 0)
    {
      gotline = getline < local_parameter["COLUMN_DEF_FILE"]
      if (gotline > 0)
      {
        #Ignore Comment lines in ColDefFile
        if ( ! (substr($0,1,1) == "#") )
        {
	  #Check only 4 fields are included in ColDefFile
	  if (NF == 4)
          {
	    if (! ($1 in skip_fields) )
            {
              column_count += 1
              column_name[column_count] = $1
	      if ( $3 == "" ) column_type[column_count] = local_parameter["DATA_TYPE"]
              else column_type[column_count] = $3
              if ($2 == "key") pk_string = sprintf("%s, %s", pk_string, $1)
            }
          }
	  else
	  {
            exit_code += 1
            print "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
	    printf("ERROR: Column Definition Invalid for file %s CONTENTS={%s}\n", local_parameter["COLUMN_DEF_FILE"], $0)
            print "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
	  }
        }
      }
    }
    close(local_parameter["COLUMN_DEF_FILE"])
    if ( gotline < 0 )
    {
      exit_code += 1
      print "==============================================================================="
      printf("ERROR Reading Column Definition file -> %s\n", local_parameter["COLUMN_DEF_FILE"])
      print "==============================================================================="
    }
    close( local_parameter["COLUMN_DEF_FILE"] )
  }
  

  #Set up local TABLESPACE TYPE paramaters
  #
  if (local_parameter["TABLESPACE_TAB"] == "NULL") local_parameter["TABLESPACE_TAB"] = local_parameter["TABLESPACE"]
  if (local_parameter["TABLESPACE_CAL"] == "NULL") local_parameter["TABLESPACE_CAL"] = local_parameter["TABLESPACE"]
  if (local_parameter["TABLESPACE_MAP"] == "NULL") local_parameter["TABLESPACE_MAP"] = local_parameter["TABLESPACE"]


  #
  #Start Creating Scripts
  #
  print   ""
  print   "set time on"						> dropfilename
  print   "set echo on"						> dropfilename
  printf ("spool DROP/%s_drop\n", ts_name)			> dropfilename
  print   "--"							> dropfilename
  printf ("-- Removing Timeseries objects for %s%s\n", db_schema, ts_name)	> dropfilename
  print   "--"							> dropfilename
  print   "-- Script Created by: " Author			> dropfilename
  print   "-- Script Version:    " Release		        > dropfilename
  print   "-- Script Date:	 " script_date		        > dropfilename
  print   ""							> dropfilename
  print   ""							> dropfilename
  print   "set time on"						> buildfilename
  print   "set echo on"						> buildfilename
  printf ("spool BUILD/%s_build\n", ts_name)			> buildfilename
  print   "--"							> buildfilename
  printf ("-- Creating Timeseries objects for %s\n", ts_name)	> buildfilename
  print   "--"							> buildfilename
  print   "-- Script Created by: " Author			> buildfilename
  print   "-- Script Version:    " Release		        > buildfilename
  print   "-- Script Date:       " script_date		        > buildfilename
  print   ""							> buildfilename
  print   ""							> buildfilename
  printf ("@%s\n",buildfilename)				> "makeAllTimeSeries.sql"
  printf ("@%s\n",dropfilename)					> "dropAllTimeSeries.sql"
  #
  # Note: It is possible to disable creation/deletion of PUBLIC SYNONYMS
  #       by setting parameter USE_PUB_SYNONYMS to 0
  #
  if (local_parameter["USE_PUB_SYNONYMS"] == 1)
  {
    print   "-- DROP any PUBLIC synonyms for the base table"	> dropfilename
    printf ("DROP PUBLIC SYNONYM %s;\n",tab_name)		> dropfilename
    print   ""							> dropfilename
    if (substr(local_parameter["TIMESERIES"],1,2) != "NO")
    {
      print   "-- DROP public synonyms for the calendar table"	> dropfilename
      printf ("DROP PUBLIC SYNONYM %s_cal;\n",ts_name)		> dropfilename
      print   ""						> dropfilename
      print   "-- DROP any PUBLIC synonyms for the metadata table"	> dropfilename
      printf ("DROP PUBLIC SYNONYM %s_map;\n",ts_name)		> dropfilename
      print   ""						> dropfilename
      print   "-- DROP public synonyms for the timeseries view"	> dropfilename
      printf ("DROP PUBLIC SYNONYM %s;\n",ts_name)		> dropfilename
      print   ""						> dropfilename
    }
    print   ""							> dropfilename
  }
  else print   "-- SYNONYMS Drop Skipped...."			> dropfilename

  printf ("-- Creating Base Table %s%s\n", db_schema, tab_name)	> buildfilename
  printf ("CREATE TABLE %s%s\n", db_schema, tab_name)		> buildfilename
  printf ("( %-12s %s,\n",key_name,key_type)			> buildfilename
  if ( local_parameter["TIMESERIES"] == "YES" || local_parameter["TIMESERIES"] == "NO:STORE_TSTAMP" )
  {
     printf ("  %-12s DATE,\n",time_name)				> buildfilename
  }
  for (i = 1; i <= column_count; i++) printf ("  %s %s,\n", column_name[i], column_type[i])	> buildfilename
  printf (" CONSTRAINT pk_%s PRIMARY KEY (%s)\n", tab_name, pk_string)	> buildfilename
  print   ")"							> buildfilename
  #
  # Note: There is a Maximum limit to the number of columns an
  #       IOT table can support.  This is defined by the CONSTANT
  #       "MAX_IOT_COLS" further up in this script.  The value 2
  #       is added because there are currently 2 key fields (i.e.
  #       the ID and the TimeStamp) in addition to data fields
  # 
  tabisiot=0
  if (local_parameter["USE_IOT_TABLE"] == 1 && column_count + 2 < MAX_IOT_COLS)
  {
    print   "ORGANIZATION INDEX"				> buildfilename
    tabisiot=1
  }
  if (local_parameter["PARTITION_SDATE"] != "NULL" || local_parameter["PARTITION_EDATE"] != "NULL")
  {
    #If PARTITIONING is required check validity of dates etc...
    local_parameter["PARTITION_SDATE"] = chk_partidate( local_parameter["PARTITION_SDATE"] )
    local_parameter["PARTITION_EDATE"] = chk_partidate( local_parameter["PARTITION_EDATE"] )
    if (local_parameter["PARTITION_EDATE"] < local_parameter["PARTITION_SDATE"])
    {
      print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
      print "PARTITION End date is earlier than Start date at line " NR 
      print "StartDate = " local_parameter["PARTITION_SDATE"]
      print "EndDate   = " local_parameter["PARTITION_EDATE"]
      print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
      exit_code += 1
      local_parameter["PARTITION_SDATE"]="NULL"
      local_parameter["PARTITION_EDATE"]="NULL"
    }
    if (local_parameter["PARTITION_SDATE"] != "NULL" && local_parameter["PARTITION_EDATE"] != "NULL")
    {
      print   "PARTITION BY RANGE(tstamp)"			> buildfilename
      print   "("							> buildfilename
      date_str=local_parameter["PARTITION_SDATE"]
      parti_offset=local_parameter["PARTITION_OFFSET"] % 24
      if (parti_offset < 0) parti_offset = 24 - parti_offset
      while ( date_str != local_parameter["PARTITION_EDATE"] )
      {
        nextdate_str = get_nextday( date_str )
        day_str=substr(nextdate_str,7,2)
        mon_str=substr(nextdate_str,5,2)
        yr_str=substr(nextdate_str,1,4)
        printf ("  PARTITION p%s%02d VALUES LESS THAN(TO_DATE('%s/%s/%s %02d', 'MM/DD/YYYY HH24')),\n",date_str, parti_offset, mon_str, day_str, yr_str, parti_offset) > buildfilename
        date_str = nextdate_str
      }
      nextdate_str = get_nextday( date_str )
      day_str=substr(nextdate_str,7,2)
      mon_str=substr(nextdate_str,5,2)
      yr_str=substr(nextdate_str,1,4)
      printf ("  PARTITION p%s%02d VALUES LESS THAN(TO_DATE('%s/%s/%s %02d', 'MM/DD/YYYY HH24'))\n",local_parameter["PARTITION_EDATE"], parti_offset, mon_str, day_str, yr_str, parti_offset) > buildfilename
      print   ")"							> buildfilename
    }
  #End of PARTITIONING section
  }
  if(tabisiot == 0) printf ("PCTUSED %s\n",local_parameter["PCTUSED"])	> buildfilename
  printf ("PCTFREE %s\n",local_parameter["PCTFREE"])		> buildfilename
  if (local_parameter["TABLESPACE_TAB"] != "NULL") printf ("TABLESPACE %s\n",local_parameter["TABLESPACE_TAB"])	> buildfilename
  if (local_parameter["STORAGE_INITIAL_TAB"] == "NULL" || local_parameter["STORAGE_NEXT_TAB"] == "NULL")
  { 
    print   "STORAGE ("                                         > buildfilename
    printf (" FREELISTS %s\n", local_parameter["FREELISTS"])    > buildfilename
    print   ")"                                                 > buildfilename
    print   ";"							> buildfilename
  }
  else
  {
    print   "STORAGE"						     > buildfilename
    printf ("( INITIAL %s\n",local_parameter["STORAGE_INITIAL_TAB"]) > buildfilename
    printf ("  NEXT %s\n",local_parameter["STORAGE_NEXT_TAB"])	     > buildfilename
    printf (" FREELISTS %s\n", local_parameter["FREELISTS"])         > buildfilename
    print   ");"				                     > buildfilename
  }
  print   ""							> buildfilename
  print   ""							> buildfilename
  if (substr(local_parameter["TIMESERIES"],1,2) != "NO")
  {
    print   "--"							> buildfilename
    print   "--"							> buildfilename
    printf ("-- Create Time Series Cartridge Tables & Views for %s\n", ts_name)   > buildfilename
    print   "--"							> buildfilename
    print   "--"							> buildfilename
    print   "-- Create the Calendar Table"			> buildfilename
    printf ("CREATE TABLE %s%s_cal OF \n", db_schema, ts_name)	> buildfilename
    printf ("  ORDSYS.ORDTCalendar \n")				> buildfilename
    printf ("  ( name CONSTRAINT pk_%s_cal PRIMARY KEY)\n",ts_name)	> buildfilename
    printf ("PCTUSED %s\n",local_parameter["PCTUSED"])		> buildfilename
    printf ("PCTFREE %s\n",local_parameter["PCTFREE"])		> buildfilename
    if (local_parameter["TABLESPACE_CAL"] != "NULL") printf ("TABLESPACE %s\n",local_parameter["TABLESPACE_CAL"])	> buildfilename
    if (local_parameter["STORAGE_INITIAL_CAL"] == "NULL" || local_parameter["STORAGE_NEXT_CAL"] == "NULL")
      print ";"							> buildfilename
    else
    {
      print   "STORAGE"						> buildfilename
      printf ("( INITIAL %s\n",local_parameter["STORAGE_INITIAL_CAL"])	> buildfilename
      printf ("  NEXT %s\n",local_parameter["STORAGE_NEXT_CAL"])	> buildfilename
      print   ");"						> buildfilename
    }
    print   ""							> buildfilename
    print   "-- Now Create the Metadata Table"			> buildfilename
    printf ("CREATE TABLE %s%s_map\n", db_schema, ts_name)	> buildfilename
    printf ("( %-12s  %s,\n", key_name, key_type)			> buildfilename
    printf ("  calname       VARCHAR2(256),\n")			> buildfilename
    printf (" CONSTRAINT pk_%s_map PRIMARY KEY (%s))\n",ts_name,key_name)	> buildfilename
    printf ("PCTUSED %s\n",local_parameter["PCTUSED"])		> buildfilename
    printf ("PCTFREE %s\n",local_parameter["PCTFREE"])		> buildfilename
    if (local_parameter["TABLESPACE_MAP"] != "NULL") printf ("TABLESPACE %s\n",local_parameter["TABLESPACE_MAP"])	> buildfilename
    if (local_parameter["STORAGE_INITIAL_MAP"] == "NULL" || local_parameter["STORAGE_NEXT_MAP"] == "NULL")
      print ";"							> buildfilename
    else
    {
      print   "STORAGE"						> buildfilename
      printf ("( INITIAL %s\n",local_parameter["STORAGE_INITIAL_MAP"])	> buildfilename
      printf ("  NEXT %s\n",local_parameter["STORAGE_NEXT_MAP"])	> buildfilename
      print   ");"							> buildfilename
    }
    print   ""							> buildfilename
    print   ""							> buildfilename
    print   "--"							> buildfilename
    print   "--"							> buildfilename
    printf ("-- Populate Calendar Table for %s\n", ts_name)	> buildfilename
    print   "--"							> buildfilename
    print   "--"							> buildfilename
    print   ""							> buildfilename
    printf ("INSERT INTO %s%s_cal\n", db_schema, ts_name)			> buildfilename
    print   "  VALUES ("						> buildfilename
    print   "    ORDSYS.ORDTCalendar ("				> buildfilename
    print   "      0,"						> buildfilename
    if (ts_calendar == "MINS_15")
    {
      #Valid times stamps NN:00 NN:15 NN:30 NN:45
      print   "      'MINS_15',"					> buildfilename
      print   "      2,"						> buildfilename
      print   "      ORDSYS.ORDTPattern("				> buildfilename
      print   "        ORDSYS.ORDTPatternBits(1,0,0,0,0,0,0,0,0,0,0,0,0,0,0),"			> buildfilename
      printf( "        TO_DATE('%s 00:00','YYYY-MM-DD HH24:MI')", Today)	> buildfilename
    }
    else if (ts_calendar == "MINS_30")
    {
      #Valid times stamps NN:00  NN:30
      print   "      'MINS_30',"					> buildfilename
      print   "      2,"						> buildfilename
      print   "      ORDSYS.ORDTPattern("				> buildfilename
      print   "        ORDSYS.ORDTPatternBits(1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),"	> buildfilename
      printf( "        TO_DATE('%s 00:00','YYYY-MM-DD HH24:MI')", Today)	> buildfilename
    }
    else if (ts_calendar == "HOURLY_15")
    {
      #Valid times stamps NN:15
      print   "      'HOURLY_15',"				> buildfilename
      print   "      3,"						> buildfilename
      print   "      ORDSYS.ORDTPattern("				> buildfilename
      print   "        ORDSYS.ORDTPatternBits(1),"		> buildfilename
      printf( "        TO_DATE('%s 00:15','YYYY-MM-DD HH24:MI')", Today)	> buildfilename
    }
    else if (ts_calendar == "DAILY")
    {
      #Valid times stamps are days
      print   "      'DAILY',"				> buildfilename
      print   "      4,"						> buildfilename
      print   "      ORDSYS.ORDTPattern("				> buildfilename
      print   "        ORDSYS.ORDTPatternBits(1),"		> buildfilename
      printf( "        TO_DATE('%s','YYYY-MM-DD')", Today)	> buildfilename
    }
    else 
    {
      if (ts_calendar != "HOURLY")
      {
        print  "=========================================================="
        print  "Warning CALENDAR name Invalid for TimeSeries " ts_name
        printf("Line %d in TablaDefFile %s\n", NR, FILENAME)
        print  ""
        printf("Defined Calendar = {%s}\n", ts_calendar)
        print  ""
        print  "Valid Calendars are :"
	print  "  DAILY     : Every Day at Midnight (hour not considered)"
	print  "  HOURLY    : Every Hour on the hour e.g. 00:00, 01:00"
	print  "  HOURLY_15 : 15 minutes past every hour e.g. 00:15, 01:15"
	print  "  MINS_15   : Every 15 minutes e.g. 00:15, 00:30, 00:45"
	print  "  MINS_30   : Every 30 minutes e.g. 00:00, 00:30, 01:00"
        print  ""
        print  "WARNING: Defaulting to HOURLY"
        print  "=========================================================="
        exit_code += 1
      }
      print   "      'HOURLY',"					> buildfilename
      print   "      3,"						> buildfilename
      print   "      ORDSYS.ORDTPattern("				> buildfilename
      print   "        ORDSYS.ORDTPatternBits(1),"			> buildfilename
      printf( "        TO_DATE('%s 00:00','YYYY-MM-DD HH24:MI')", Today)	> buildfilename
    }
    print   "      ),"						> buildfilename
    print   "      NULL,"						> buildfilename
    print   "      NULL,"						> buildfilename
    print   "      NULL,"						> buildfilename
    print   "      NULL"						> buildfilename
    print   "    )"						> buildfilename
    print   "  );"						> buildfilename
    print   ""							> buildfilename
    print   ""							> buildfilename
  }
  print   "-- Set Access Privileges"				> buildfilename
  print   "--"							> buildfilename
  print   ""							> buildfilename
  print   "-- Set Privileges for Tables"			> buildfilename
  printf ("GRANT SELECT ON %s%s_tab TO PUBLIC;\n",db_schema, ts_name)	> buildfilename
  if (substr(local_parameter["TIMESERIES"],1,2) != "NO")
  {
    printf ("GRANT SELECT ON %s%s_cal TO PUBLIC;\n",db_schema,ts_name)	> buildfilename
    printf ("GRANT SELECT ON %s%s_map TO PUBLIC;\n",db_schema,ts_name)	> buildfilename
  }
  print   ""							> buildfilename
  print   ""							> buildfilename
  if (local_parameter["USE_PUB_SYNONYMS"] == 1)
  {
    print   "-- Set Synonyms for Tables"			> buildfilename
    printf ("CREATE PUBLIC SYNONYM %s_tab FOR %s%s_tab;\n", ts_name, db_schema, ts_name)	> buildfilename
    if (substr(local_parameter["TIMESERIES"],1,2) != "NO")
    {
      printf ("CREATE PUBLIC SYNONYM %s_cal FOR %s%s_cal;\n", ts_name, db_schema, ts_name)	> buildfilename
      printf ("CREATE PUBLIC SYNONYM %s_map FOR %s%s_map;\n", ts_name, db_schema, ts_name)	> buildfilename
      print   ""							> buildfilename
      print   "-- Set Synonyms for the TimeSeries View"		> buildfilename
      printf ("CREATE PUBLIC SYNONYM %s FOR %s%s;\n", ts_name, db_schema, ts_name)	> buildfilename
    }
  }
  else print   "-- SYNONYM Creation Skipped...."		> buildfilename
  print   ""							> buildfilename
  print   ""							> buildfilename
  if (substr(local_parameter["TIMESERIES"],1,2) != "NO")
  {
    if (substr(local_parameter["ORACLE_RELEASE"],1,3) == "8.0" || column_count > MAX_TS_COLS)
    {
      printf ("NOTE: TsTools section not possible for %s\n", ts_name)
      printf ("-- TsTools section not possible for %s\n", ts_name) > buildfilename
      print   ""							> buildfilename
      print   ""							> buildfilename
      print   "--"						> buildfilename
      print   "--"						> buildfilename
      printf ("-- Time Series Reference View Creation for %s\n", ts_name)   > buildfilename
      print   "--"						> buildfilename
      print   "--"						> buildfilename
      print   ""							> buildfilename
      printf ("CREATE OR REPLACE VIEW %s%s\n", db_schema, ts_name)	> buildfilename
      print   "("							> buildfilename
      printf ("  %s,\n",key_name)					> buildfilename
      for (i = 1; i <= column_count; i++) 
      {
        printf ("  %s", column_name[i])				> buildfilename
          if ( i < column_count ) print   ","			> buildfilename
         else print   ""						> buildfilename
      }
      print   ") AS"						> buildfilename
      printf ("    SELECT meta.%s,\n", key_name)			> buildfilename
      for (i = 1; i <= column_count; i++) 
      {
        print   "      ordsys.ordtNumSeriesIOTRef"		> buildfilename
        print   "      ("						> buildfilename
        printf ("        substr(meta.%s, 1, 230) || ' %s NumSeries',\n", key_name, column_name[i])   > buildfilename
        print   "        Ref(cal),"				> buildfilename
        printf ("        '%s',\n",ts_name)			> buildfilename
        printf ("        '%s',\n",time_name)			> buildfilename
        printf ("        '%s',\n",column_name[i])			> buildfilename
        printf ("        '%s',\n",key_name)			> buildfilename
        printf ("        meta.%s\n",key_name)			> buildfilename
        if ( i < column_count ) print   "      ),"		> buildfilename
        else print   "      )"					> buildfilename
      }
      print   "    FROM"						> buildfilename
      printf ("      %s%s_map meta, %s%s_cal cal\n", db_schema, ts_name, db_schema, ts_name)	> buildfilename
      print   "    WHERE"						> buildfilename
      print   "      meta.calname = cal.name;"			> buildfilename
      print   ""							> buildfilename
      print   ""							> buildfilename
      print   "-- Set Privileges for the TimeSeries View"		> buildfilename
      printf ("GRANT SELECT ON %s%s TO PUBLIC;\n",db_schema, ts_name)	> buildfilename
      print   ""							> buildfilename
      print   ""							> buildfilename
      print   "--"						> buildfilename
      printf ("-- TsTools section not possible for %s\n", ts_name)	> dropfilename
      # NOTE: Views/Triggers are dropped by TSTools in Oracle 8.0+
      print   ""							> dropfilename
      print   ""							> dropfilename
      print   "-- DROP the timeseries view"			> dropfilename
      printf ("DROP VIEW %s%s;\n",db_schema, ts_name)		> dropfilename
      print   ""							> dropfilename
      print   ""							> dropfilename
    }
    else
    {
      #Some NOTES pertaining to TsTools.
      # 1) It is not possible to define the schema that the Timeseries belongs
      #    to with TsTools, therefore this will ONLY work if the tables were
      #    created or already exist in the executing schema.
      # 2) It is not possible to specify the schema name of each individual
      #    table, they are all assumed (by TsTools) to belong to the executor.
      #
      # Now lets Retrofit these tables using the TsTools package (Oracle 8.1 +)
      # (Based on "retrofit.sql" provided in the Timeseries demo dated 05/26/98)
      #
      print  "--"							> buildfilename
      print  "-- Using TSTools to generate the reference view"	> buildfilename
      print  "--"							> buildfilename
      print  "--"							> dropfilename
      print  "-- Using TSTools to drop all defintions"		> dropfilename
      print  "--"							> dropfilename
      print  ""							> dropfilename
      print  "DECLARE"						> dropfilename
      print  ""							> dropfilename
      print  "BEGIN"						> dropfilename
      print  ""							> dropfilename
      printf("  ORDSYS.TSTools.Drop_TS_Group_All('%s');\n", ts_name)	> dropfilename
      print  ""							> dropfilename
      print  "  exception"					> dropfilename
      print  "    when others then"				> dropfilename
      print  "        raise;"					> dropfilename
      print  ""							> dropfilename
      print  "END;"						> dropfilename
      print  "/"							> dropfilename
      print  ""							> buildfilename
      print  "DECLARE"						> buildfilename
      print  ""							> buildfilename
      print  "BEGIN"						> buildfilename
      print  ""							> buildfilename
      print  "--"							> buildfilename
      print  "-- Enable tracing option to show views"		> buildfilename
      print  "--"							> buildfilename
      print  ""							> buildfilename
      print  "ORDSYS.TSTools.Trace_On;"				> buildfilename
      print  ""							> buildfilename
      print  "--"							> buildfilename
      print  "-- Establish time series schema name for"		> buildfilename
      print  "-- administrative tools procedures. "		> buildfilename
      print  "--"							> buildfilename
      print  ""							> buildfilename
      printf("  ORDSYS.TSTools.Begin_Create_TS_Group('%s','flat');\n", ts_name)	> buildfilename
      print  ""							> buildfilename
      print  "  --"						> buildfilename
      print  "  -- Assert that the detail, map, and calendar tables exist,"	> buildfilename
      print  "  -- and define the names for these tables."	> buildfilename
      print  "  --"						> buildfilename
      print  ""							> buildfilename
      print  "  ordsys.tstools.set_flat_attributes("		> buildfilename
      printf("        detail_table_name    => '%s',\n", tab_name)	> buildfilename
      print  "        detail_table_exists  => 1,"			> buildfilename
      printf("        map_table_name       => '%s_map',\n", ts_name)	> buildfilename
      print  "        map_table_exists     => 1,"			> buildfilename
      printf("        cal_table_name       => '%s_cal',\n", ts_name)	> buildfilename
      print  "        cal_table_exists     => 1,"			> buildfilename
      printf("        tstamp_colname       => '%s',\n",time_name)	> buildfilename
      printf("        tsname_colname       => '%s',\n",key_name)	> buildfilename
      printf("        rel_view_name        => '%s_RVW');\n", ts_name)	> buildfilename
      print   ""							> buildfilename
      print   "  --"						> buildfilename
      print   "  -- Tell TSTools the names of existing columns "	> buildfilename
      print   "  --"						> buildfilename
      print   ""							> buildfilename
      for (i = 1; i <= column_count; i++) printf ("  ORDSYS.TSTools.Add_Existing_Column('%s');\n", column_name[i])	> buildfilename
      print   ""							> buildfilename
      print   "-- End the specification of schema objects"	> buildfilename
      print   "-- so now create the objects."			> buildfilename
      print   ""							> buildfilename
      print   "  ORDSYS.TSTools.End_Create_TS_Group;"		> buildfilename
      print   ""							> buildfilename
      print   "  ordsys.tstools.trace_off;"			> buildfilename
      print   ""							> buildfilename
      print   "  exception"					> buildfilename
      print   "    when others then"				> buildfilename
      print   "       begin"					> buildfilename
      print   "          ORDSYS.TSTools.Cancel_Create_TS_Group;"	> buildfilename
      print   "          raise;"					> buildfilename
      print   "       end;       "				> buildfilename
      print   ""							> buildfilename
      print   "END;"						> buildfilename
      print   "/"							> buildfilename
      print   ""							> buildfilename
      print   ""							> buildfilename
    }
  }
  # End of Retrofit
  #
  # Tables must be dropped after TSTools libraries are updated
  #
  print   "-- DROPPING tables"					> dropfilename
  print   ""							> dropfilename
  print   "-- DROP the base data table"				> dropfilename
  printf ("DROP TABLE %s%s;\n", db_schema, tab_name)		> dropfilename
  print   ""							> dropfilename
  if (substr(local_parameter["TIMESERIES"],1,2) != "NO")
  {
    print   "-- DROP the Calendar table"				> dropfilename
    printf ("DROP TABLE %s%s_cal;\n", db_schema, ts_name)		> dropfilename
    print   ""							> dropfilename
    print   "-- DROP the metadata table"				> dropfilename
    printf ("DROP TABLE %s%s_map;\n", db_schema, ts_name)		> dropfilename
    print   ""							> dropfilename
  }
  print   ""							> buildfilename
  print   "spool off"						> buildfilename
  print   ""							> dropfilename
  print   "spool off"						> dropfilename
  close( buildfilename )
  close( dropfilename )
}

#The following part of this script gets run after the complete 
#Table Definition file has been processed.
#
END {
  #Drop/Create the LoaderHistoryFile
#  print   "spool loaderHistoryDrop"		> "DROP/loaderHistoryDrop.sql"
#  print   "-- Drop LoaderHistoryFile Script"	> "DROP/loaderHistoryDrop.sql"
#  print   "--"					> "DROP/loaderHistoryDrop.sql"
#  print   "-- Script Created by: " Author	> "DROP/loaderHistoryDrop.sql"
#  print   "-- Script Version:    " Release	> "DROP/loaderHistoryDrop.sql"
#  print   "-- Script Date:	 " script_date	> "DROP/loaderHistoryDrop.sql"
#  print   ""					> "DROP/loaderHistoryDrop.sql"
#  printf( "DROP TABLE %s;\n", local_parameter["LOADER_HISTORY_TAB"])	> "DROP/loaderHistoryDrop.sql"
#  print   "spool off"				> "DROP/loaderHistoryDrop.sql"
#  print   "@DROP/loaderHistoryDrop.sql"		> "dropAllTimeSeries.sql"
  print   "exit"				> "dropAllTimeSeries.sql"
#  print   "spool BUILD/loaderHistoryCreate"	> "BUILD/loaderHistoryCreate.sql"
#  print   "-- Create LoaderHistoryFile Script"	> "BUILD/loaderHistoryCreate.sql"
#  print   "--"					> "BUILD/loaderHistoryCreate.sql"
#  print   "-- Script Created by: " Author	> "BUILD/loaderHistoryCreate.sql"
#  print   "-- Script Version:    " Release	> "BUILD/loaderHistoryCreate.sql"
#  print   "-- Script Date:	 " script_date	> "BUILD/loaderHistoryCreate.sql"
#  print   ""					> "BUILD/loaderHistoryCreate.sql"
#  printf( "CREATE TABLE %s\n", local_parameter["LOADER_HISTORY_TAB"])	> "BUILD/loaderHistoryCreate.sql"
#  print   "( timeseries_name varchar2(20),"	> "BUILD/loaderHistoryCreate.sql"
#  print   "  pk_name         varchar2(60),"	> "BUILD/loaderHistoryCreate.sql"
#  print   "  last_load       date,"		> "BUILD/loaderHistoryCreate.sql"
#  printf( "  CONSTRAINT %s_idx PRIMARY KEY (timeseries_name, pk_name)\n", local_parameter["LOADER_HISTORY_TAB"])> "BUILD/loaderHistoryCreate.sql"
#  print   ")"					> "BUILD/loaderHistoryCreate.sql"
#  print   "ORGANIZATION INDEX"			> "BUILD/loaderHistoryCreate.sql"
#  print   "PCTFREE 5"				> "BUILD/loaderHistoryCreate.sql"
#  print   "STORAGE"				> "BUILD/loaderHistoryCreate.sql"
#  print   "( INITIAL 1M"			> "BUILD/loaderHistoryCreate.sql"
#  print   "  NEXT    1M"			> "BUILD/loaderHistoryCreate.sql"
#  print   ");"					> "BUILD/loaderHistoryCreate.sql"
#  print   "spool off"				> "BUILD/loaderHistoryCreate.sql"
#  print   "@BUILD/loaderHistoryCreate.sql"	> "makeAllTimeSeries.sql"
  print   "exit"				> "makeAllTimeSeries.sql"
#  print"----------------------------------------------------------"
# 
  if (exit_code != 0)
  {
    print ""
    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    print "ERROR : Problems occured whist running this script"
    print "        Rectify the cause and re-run befors using"
    print "        any SQL scripts created on this run"
    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    print ""
  }
}
