---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2002,2010. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/dictionary/scripts/dict_save.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--

CREATE TABLE save_Dict_Field
(
Entity_ID number(38),
Field_Name varchar2(128),
Field_ID number(38)
)
TABLESPACE wm_flexpm;

insert into save_Dict_Field
   SELECT f.Entity_ID
        , n1.Name
        , f.Field_ID
     FROM wmd_field_def f
        , wmd_dict_name n1
    WHERE f.field_id = n1.field_id
      AND n1.namespace_id = 0
      ;

delete from save_Dict_Field where Field_ID is NULL;

CREATE UNIQUE INDEX pk_save_Dict_Field ON save_Dict_Field ( Entity_ID, Field_Name, Field_ID )
COMPRESS 1
TABLESPACE wm_flexpm
PCTFREE 0;

CREATE TABLE save_TemplateField 
TABLESPACE wm_flexpm AS
  SELECT TemplateID, Ordinal, pm.Field_ID, SortOrder, IsShown, Entity_ID, Field_Name
    FROM PM_TemplateField pm, wmd_Dict_Field fld
    WHERE pm.Field_ID = fld.Field_ID;

CREATE TABLE save_Template 
TABLESPACE wm_flexpm AS
  SELECT ID, CriteriaFieldID, pm.Entity_ID, Field_Name, FolderID, Name, Category
    FROM PM_Template pm, wmd_Dict_Field fld
    WHERE pm.CriteriaFieldID = fld.Field_ID (+);

CREATE TABLE save_SoftAlarm 
TABLESPACE wm_flexpm AS
  SELECT ID,SA.FIELD_ID,FLD.FIELD_NAME,FLD.ENTITY_ID
    FROM SA_THRESHOLD SA, WMD_DICT_FIELD FLD
    WHERE SA.FIELD_ID = FLD.FIELD_ID;


CREATE TABLE save_ReportBH
TABLESPACE wm_flexpm AS
  SELECT Id, r.Field_ID, Entity_Id, Field_Name
    FROM PM_ReportSpec r, wmd_Dict_Field fld
    WHERE r.Field_ID = fld.Field_ID;

CREATE TABLE save_BusyHour
TABLESPACE wm_flexpm AS
  SELECT Id, r.Field_ID, fld.Entity_ID, fld.Field_Name
    FROM PM_BusyHour r, wmd_Dict_Field fld
    WHERE r.Field_ID = fld.Field_ID;

CREATE TABLE save_UDC
TABLESPACE wm_flexpm AS
  SELECT Name Field_Name, Report_Header, Report_Header2, Datatype,  Field_Desc, Field_Aggregator, Field_Emits_Aggregator, Entity_Name, Calc_Source, UDC_Source, Error_Msg, Status
    FROM wmd_Field_Def def, wmd_dict_Name nam, wmd_UDC_Def udc, wmn_Entity ent
    WHERE def.Field_Id = nam.Field_ID 
      AND def.Field_ID = udc.Field_ID
      AND def.Entity_ID = ent.Entity_ID
      AND nam.NameSpace_ID = 0;

CREATE TABLE save_Table
TABLESPACE wm_flexpm AS
  SELECT tab.Table_ID, tab.Table_Name, tab.Entity_ID, tab.Is_Time_Series, tab.Summary_ID, tab.Summary_Col, base.Table_Name Base_Table_Name
    FROM wmd_Dict_Table tab, wmd_Dict_Table base
    WHERE tab.Summary_ID != 0
      AND tab.Table_ID = base.Table_ID
      AND base.Summary_ID = 0;

-- ****************************************************
-- wmd_col_def_subtype for GOS fields.
-- SEAde15778: 5EI, upgrade resetting GOS.
-- ****************************************************

CREATE TABLE save_col_def_subtype
TABLESPACE wm_flexpm AS
SELECT e.entity_name entity_name, 
       es.subtype_name subtype_name,
       dn.name field_name,
       cds.min_value min_value,
       cds.max_value max_value, 
       cds.default_value default_value
  FROM wmd_col_def_subtype cds, wmd_dict_name dn,
       wmn_entity_subtype es, wmn_entity e
 WHERE cds.field_id = dn.field_id
   AND cds.subtype_id = es.subtype_id
   AND cds.entity_id = e.entity_id
   AND dn.namespace_id = 0
   AND dn.name = 'grade_serv';

-- ****************************************************
-- SEAde32583: EDS
-- ****************************************************

create table SAVE_EDS_PRIMARYKEY
TABLESPACE wm_flexpm as
select ep.pk_id
     , ed.entity_id
     , ep.field_id
     , dn.name field_name
     , ep.eds_id
     , ep.col_id
     , ep.ordinal
  from EDS_PRIMARYKEY ep
     , EDS_DOCUMENT   ed
     , WMD_DICT_NAME  dn
 where ed.id = ep.eds_id
   and ep.field_id = dn.field_id
   and dn.namespace_id = 0
     ;

create table SAVE_EDS_DEF
TABLESPACE wm_flexpm as
select ed.entity_id
     , ef.field_id
     , dn.name field_name
     , ef.eds_id
     , ef.col_id
  from WMD_EDS_DEF   ef
     , EDS_DOCUMENT  ed
     , WMD_DICT_NAME dn
 where ed.id = ef.eds_id
   and ef.field_id = dn.field_id
   and dn.namespace_id = 0
     ;

create table SAVE_EDS_FIELD
TABLESPACE wm_flexpm as
select FD.entity_id
     , FD.field_id
     , FD.datatype
     , FD.field_type
     , FD.field_desc
     , FD.field_aggregator
     , FD.field_emits_aggregator
     , FD.datasource_id
     , FD.field_owner
     , FD.field_group_id
     , DN.namespace_id
     , DN.name
     , DN.report_header
     , DN.report_header2
  from WMD_FIELD_DEF FD
     , WMD_DICT_NAME DN
 where FD.field_id = DN.field_id
   and DN.namespace_id = 0
   and exists ( select 1 from WMD_EDS_DEF EF
                 where EF.field_id = FD.field_id
              );

create table SAVE_EDS_TECH
TABLESPACE wm_flexpm as
select FT.field_id
     , FT.tech_id
  from WMD_FIELD_TECH FT
 where exists ( select 1 from WMD_EDS_DEF EF
                 where EF.field_id = FT.field_id
              );

-- ****************************************************
-- SEAde40172: FIELD STYLE
-- ****************************************************
create table SAVE_FIELD_STYLE
TABLESPACE wm_flexpm as
select fs.field_id
     , dn.entity_id
     , dn.name field_name
     , fs.fontstylemask
     , fs.fontcolor
  from WMD_FIELD_STYLE fs
     , WMD_DICT_NAME   dn
 where fs.field_id = dn.field_id
   and dn.namespace_id = 0
     ;
     

-- ****************************************************
-- SEAde40534: UDR KEYS
-- ****************************************************
create table SAVE_UDR_KEYS
TABLESPACE wm_flexpm as
select uk.entity_id
     , uk.field_id
     , dn.name field_name
     , uk.field_seq
  from WMD_UDR_KEYS uk
     , WMD_DICT_NAME dn
 where uk.field_id = dn.field_id
   and dn.namespace_id = 0
     ;
-- ****************************************************
-- valnt00100745 adding in support for wmn_field_attribute
-- ****************************************************
CREATE TABLE SAVE_FIELD_ATTRIBUTE TABLESPACE wm_flexpm AS 
SELECT fa.field_id,
       dn.entity_id,
       dn.NAME field_name,
       fa.attribute_name,
       fa.default_num_value,
       fa.user_num_value
  FROM wmn_field_attribute fa,
       wmd_dict_name       dn
 WHERE fa.field_id = dn.field_id
   AND dn.namespace_id = 0;


-- ****************************************************
-- PEIK
-- ****************************************************
-- This table may not exist in some systems, so we use
-- dynamic SQL.
--
declare
  vSQL varchar2(2000);
     x char(1);
begin

   select 'x' into x
     from user_tables
    where table_name = 'PEIK_EXTRA_FLD_PROPERTY';

   vSQL := 'create table SAVE_PEIK_EXTRA_FLD_PROPERTY '
        ||  'TABLESPACE wm_flexpm ' 
        || 'as select p.entity_id '
        ||         ', p.field_id '
        ||         ', p.name '
        ||         ', p.value '
        ||         ', p.loader_property '
        ||         ', n.name field_name '
        ||      'from PEIK_EXTRA_FLD_PROPERTY p '
        ||         ', WMD_DICT_NAME n '
        ||     'where p.field_id = n.field_id '
        ||       'and n.namespace_id = 0 '
         ;
   execute immediate vSQL;

exception
when NO_DATA_FOUND then
     NULL;
end;
/
EXIT;
