CREATE TABLE wmd_Aggregator_Base (
  Aggr_ID   CHAR(1)   NOT NULL
      CONSTRAINT wmd_aggr_pky PRIMARY KEY,
  Aggr_Name   VARCHAR(20) NOT NULL
      CONSTRAINT wmd_aggr_name_unq UNIQUE,
  Aggr_Desc   VARCHAR(500),
  Aggr_BH_OK    CHAR(1)   DEFAULT 'Y' NOT NULL
      CONSTRAINT wmd_aggr_bh_chk CHECK ( Aggr_BH_OK IN ( 'Y', 'N' ) ),
  Aggr_Visible          NUMBER(1)      DEFAULT 1  NOT NULL
      CONSTRAINT wmd_aggr_vis_chk CHECK ( Aggr_Visible IN ( 0, 1 ) ),
  Func_Time             VARCHAR(20)
      CONSTRAINT wmd_aggr_tim_chk CHECK (Func_Time IN ('avg','max','min','sum','nil')),
  Func_Element           VARCHAR(20)
      CONSTRAINT wmd_aggr_elmt_chk CHECK (Func_Element IN ('avg','max','min','sum','nil')),
  Label_Time    varchar2(20)
      CONSTRAINT wmd_aggr_lbl_tim_chk CHECK 
        (Label_Time IN('Sum', 'Average', 'Max', 'Min', 'Null')),
  Label_Element varchar2(20)
      CONSTRAINT wmd_aggr_lbl_elmt_chk CHECK 
        (Label_Element IN('Sum', 'Average', 'Max', 'Min', 'Null'))
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Aggregator_Base IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE VIEW wmd_Aggregator AS
    SELECT Aggr_ID, Aggr_Name, Aggr_Desc, Aggr_BH_OK,
          Func_Time, Func_Element, Label_Time, Label_Element
      FROM wmd_Aggregator_Base WHERE Aggr_Visible = 1;
COMMENT ON TABLE wmd_Aggregator IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wmd_Frequency (
  Freq_ID   CHAR(1)   NOT NULL
      CONSTRAINT wmd_freq_pky PRIMARY KEY,
  Freq_Name   VARCHAR(20) NOT NULL
      CONSTRAINT wmd_freq_name_unq UNIQUE,
  Freq_Desc   VARCHAR(500)
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Frequency IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wmd_Summary (
  Summary_ID    INTEGER   NOT NULL
      CONSTRAINT wmd_sum_pky PRIMARY KEY,
  Summary_Name    VARCHAR2(40)  NOT NULL
      CONSTRAINT wmd_sum_name_unq UNIQUE,
  Frequency   CHAR(1)
      CONSTRAINT wmd_sum_frq_fky REFERENCES wmd_Frequency ( Freq_ID ),
  DTScopeID   INTEGER,
  BusyHourDeterminer  INTEGER
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Summary IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE SEQUENCE wmd_Summary_Seq START WITH 1;
CREATE TRIGGER wmd_Summary_Tr_BI 
   BEFORE INSERT ON wmd_Summary 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   IF ( :new.Summary_ID IS NULL ) THEN
      SELECT wmd_Summary_Seq.NextVal INTO newID FROM DUAL;
      :new.Summary_ID := newID;
   END IF;
END;
/
SHOW ERRORS;
CREATE TABLE wmd_Dict_Table (
  Table_ID    INTEGER   NOT NULL,
  Table_Name    VARCHAR2(40)  NOT NULL,
  Entity_ID   INTEGER         -- NOT NULL
      CONSTRAINT dict_tbl_ent_fky REFERENCES wmn_Entity (Entity_ID),
  Is_Time_Series  CHAR(1)   DEFAULT 'N' NOT NULL
      CONSTRAINT dict_tbl_ts_chk CHECK ( Is_Time_Series IN ( 'Y', 'N' ) ),
  Summary_ID    INTEGER   DEFAULT 0 NOT NULL,
  Summary_Col   VARCHAR2(40),
  Summarize             NUMBER(2) DEFAULT 0 NOT NULL,
  mo_class              VARCHAR2(60),
  CONSTRAINT dict_tbl_pky PRIMARY KEY (Table_ID, Summary_ID ),
  CONSTRAINT dict_tbl_name_unq UNIQUE (Table_Name, Summary_ID)
)
TABLESPACE wm_flexpm;
COMMENT ON TABLE wmd_Dict_Table IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE SEQUENCE wmd_Dict_Table_Seq;
CREATE TRIGGER wmd_Dict_Table_Tr_BI 
   BEFORE INSERT ON wmd_Dict_Table 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   IF ( :new.Table_ID IS NULL ) THEN
      SELECT wmd_Dict_Table_Seq.NextVal INTO newID FROM DUAL;
      :new.Table_ID := newID;
   END IF;
END;
/
SHOW ERRORS;
Create Trigger WMD_Dict_Table_Tr_BIU
 BEFORE INSERT OR UPDATE ON WMD_DICT_TABLE
 FOR EACH ROW
BEGIN
  :New.Table_Name := Upper (:New.Table_Name);
End;
/
SHOW ERRORS;
CREATE OR REPLACE TRIGGER trg_wmn_entity_reparent
AFTER INSERT OR UPDATE
ON wmn_entity
DECLARE
 vCount         PLS_INTEGER:=0;
BEGIN
  SELECT COUNT(entity_id) INTO vCount
    FROM wmn_entity
   WHERE reparent='Y';
  IF vCount > 1 THEN
    RAISE_APPLICATION_ERROR(-20101,
       'You cannot have more than one rehoming entity.');
  END IF;
END;
/
SHOW ERRORS TRIGGER trg_wmn_entity_reparent
CREATE TRIGGER wmd_Dict_Table_Tr_AIUD
  AFTER INSERT OR UPDATE OR DELETE ON WMD_DICT_TABLE
Begin
  wm_alert.signal('WMD_DICT_UPDATE');
End;
/
Show Errors
CREATE TABLE wmd_Data_Source (
  DataSource_ID   INTEGER   NOT NULL
      CONSTRAINT dsrc_pky PRIMARY KEY,
  DataSource_Name VARCHAR2(50)  NOT NULL
      CONSTRAINT dsrc_name_unq UNIQUE,
  DataSource_App_Ctx  INTEGER   DEFAULT NULL
      CONSTRAINT dsrc_app_fky REFERENCES wmn_Context ( Context_ID ),
  DataSource_Use_UDC    INTEGER         DEFAULT 0     NOT NULL
)
TABLESPACE wm_flexpm;
COMMENT ON TABLE wmd_Data_Source IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE SEQUENCE wmd_Data_Source_Seq;
CREATE TRIGGER wmd_Data_Source_Tr_BI 
   BEFORE INSERT ON wmd_Data_Source 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   SELECT wmd_Data_Source_Seq.NextVal INTO newID FROM DUAL;
   :new.DataSource_ID := newID;
END;
/
SHOW ERRORS;
CREATE TABLE wmd_Loader_Source (
  LoaderSource_ID   INTEGER   NOT NULL
      CONSTRAINT lsrc_pky PRIMARY KEY,
  LoaderSource_Name VARCHAR2(50)  NOT NULL
      CONSTRAINT lsrc_name_unq UNIQUE
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Loader_Source IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wmd_Field_Group (
  Field_Group_ID  INTEGER   NOT NULL
      CONSTRAINT fld_grp_pky PRIMARY KEY,
  Field_Group_Short VARCHAR2(5) NOT NULL
      CONSTRAINT fld_grp_short_unq UNIQUE,
  Field_Group_Name  VARCHAR2(20)  NOT NULL
      CONSTRAINT fld_grp_name_unq UNIQUE
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Field_Group IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE SEQUENCE wmd_Field_Group_Seq;
CREATE TRIGGER wmd_Field_Group_Tr_BI 
   BEFORE INSERT ON wmd_Field_Group 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   SELECT wmd_Field_Group_Seq.NextVal INTO newID FROM DUAL;
   :new.Field_Group_ID := newID;
   :new.Field_Group_Short := UPPER(:new.Field_Group_Short);
END;
/
SHOW ERRORS;
CREATE TABLE wmd_Field_Def (
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT field_pky PRIMARY KEY,
  Entity_ID   INTEGER   NOT NULL
      CONSTRAINT field_ent_fky REFERENCES wmn_Entity ( Entity_ID ),
  DataType    CHAR(1)   NOT NULL
      CONSTRAINT field_dtyp_chk CHECK ( Datatype IN ( 'I', 'F', 'S', 'E', 'D' ) ),
  Field_Type    CHAR(1)   NOT NULL
      CONSTRAINT field_typ_chk CHECK ( Field_Type IN ( 'C', 'E', 'S', 'U','F','X' ) ),
  Field_Group_ID  INTEGER   DEFAULT NULL
      CONSTRAINT field_grp_fky REFERENCES wmd_Field_Group ( Field_Group_ID ),
  Field_Access    CHAR(1)   DEFAULT 'S' NOT NULL
      CONSTRAINT field_acc_chk CHECK ( Field_Access IN ( 'S', 'U' ) ),
  Field_Owner   INTEGER   DEFAULT 1,
  Field_Desc    VARCHAR2(1000),
  Field_Aggregator  CHAR(1)   NOT NULL
      CONSTRAINT field_agg_fky REFERENCES wmd_Aggregator_Base ( Aggr_ID )
      CONSTRAINT field_aff_chk CHECK ( Field_Aggregator != 'U' ),
  Field_Emits_Aggregator  CHAR(1)   NOT NULL
      CONSTRAINT field_eagg_fky REFERENCES wmd_Aggregator_Base ( Aggr_ID ),
  DataSource_ID   INTEGER
      CONSTRAINT field_dsrc_fky REFERENCES wmd_Data_Source ( DataSource_ID ),
      CONSTRAINT field_entity_unq UNIQUE ( Field_ID, Entity_ID ),
  Weight_Field_ID       INTEGER
      CONSTRAINT field_weight_field_fk REFERENCES wmd_Field_Def ( Field_ID )
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Field_Def IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE SEQUENCE wmd_Field_Def_Seq;
CREATE TRIGGER wmd_Field_Def_Tr_BI 
   BEFORE INSERT ON wmd_Field_Def 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   IF ( :new.Field_ID IS NULL ) THEN
     SELECT wmd_Field_Def_Seq.NextVal INTO newID FROM DUAL;
     :new.Field_ID := newID;
   END IF;
END;
/
SHOW ERRORS;
CREATE TABLE WMD_FIELD_STYLE(
    Field_Id         INTEGER       NOT NULL,
    FontStyleMask    NUMBER        DEFAULT 0 NOT NULL,
    FontColor        NUMBER        DEFAULT 0 NOT NULL,
    CONSTRAINT WMD_FIELD_STYLE_C1 FOREIGN KEY(Field_Id) 
    REFERENCES WMD_FIELD_DEF(Field_Id) ON DELETE CASCADE,
    CONSTRAINT WMD_FIELD_STYLE_PK PRIMARY KEY(Field_Id)
)
TABLESPACE WM_FLEXPM;
CREATE TABLE wmd_udr_keys (
    entity_id       INTEGER
     CONSTRAINT udrkey_ent_fky REFERENCES wmn_Entity,
    field_id        INTEGER
            CONSTRAINT udrkey_field_fky REFERENCES wmd_Field_Def on delete cascade,
    field_seq       INTEGER,
    CONSTRAINT udrkey_pky PRIMARY KEY ( entity_id, field_id, field_seq )
)
TABLESPACE WM_FLEXPM;
CREATE TABLE wmd_udr_joins (
   child_entity   INTEGER,
   child_table    VARCHAR2(100),
   parent_entity  INTEGER, 
   parent_table   VARCHAR2(100), 
   field_seq      INTEGER, 
   child_field    VARCHAR2(100), 
   parent_field   VARCHAR2(100)
)
TABLESPACE WM_FLEXPM;
CREATE VIEW wmd_Application_Field_Group AS
   SELECT def.DataSource_ID, grp.Field_Group_ID, grp.Field_Group_Name, grp.Field_Group_Short
     FROM wmd_Field_Group grp, wmd_Data_Source src, 
          ( SELECT DISTINCT DataSource_ID, Field_Group_ID
              FROM wmd_Field_Def
            UNION
            SELECT DataSource_ID, Field_Group_ID
              FROM wmd_Data_Source, wmd_Field_Group
              WHERE Field_Group_Short = 'UDC'
                AND DataSource_Use_UDC != 0 ) def
     WHERE grp.Field_Group_ID = def.Field_Group_ID
       AND def.DataSource_ID = src.DataSource_ID;
COMMENT ON TABLE wmd_Application_Field_Group IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wmd_Calc_Def (
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT calc_fld_fky REFERENCES wmd_Field_Def ( Field_ID )
      CONSTRAINT calc_pky PRIMARY KEY,
  Calc_Source   VARCHAR2(4000)  NOT NULL
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Calc_Def IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wmd_UDC_Def (
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT udc_fld_fky REFERENCES wmd_Field_Def ( Field_ID )
      CONSTRAINT udc_pky PRIMARY KEY,
  Entity_ID   INTEGER   NOT NULL,
  UDC_Source    VARCHAR2(4000)  NOT NULL,
  Calc_Source   CLOB,
  Changed   DATE    DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
  Error_Msg   VARCHAR2(2000),
  Status    CHAR(1)   NOT NULL,
  COMPLEX   CHAR(1)   DEFAULT 'N' NOT NULL 
      CONSTRAINT udc_complex_chk CHECK ( Complex IN ('N','Y','U') ),
      CONSTRAINT udc_status_chk CHECK ( Status IN ( 'N', 'U', 'V', 'S', 'I' ) )
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_UDC_Def IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TRIGGER wmd_UDC_Def_Tr_AI 
   AFTER INSERT OR DELETE OR UPDATE OF UDC_Source ON wmd_UDC_Def 
BEGIN
   SYS.DBMS_ALERT.signal(user ||'_wmd_udc_def', '');
END;
/
SHOW ERRORS;
CREATE OR REPLACE TRIGGER wmd_UDC_Def_Tr_BIU
   BEFORE INSERT OR UPDATE OF UDC_Source ON wmd_UDC_Def 
   FOR EACH ROW
BEGIN
    SELECT Entity_ID INTO :new.Entity_ID
      FROM wmd_Field_Def
     WHERE Field_ID = :new.Field_ID;
    IF INSERTING THEN
        :new.Changed := TRUNC(SYSDATE,'MI');
    ELSIF UPDATING THEN
        IF :new.UDC_Source != :old.UDC_Source THEN
           :new.Changed := TRUNC(SYSDATE,'MI');
        END IF;
   END IF;
END;
/
SHOW ERRORS;
CREATE TABLE wmd_Col_Def (
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT col_fld_fky REFERENCES wmd_Field_Def ( Field_ID )
      CONSTRAINT col_pky PRIMARY KEY,
  Table_ID    INTEGER   NOT NULL,
  Col_Name    VARCHAR2(100) NOT NULL,
  Precision   INTEGER,  /* Float */
  Length    INTEGER,  /* String, Float, Int */
  Min_Value   FLOAT,    /* Float, Int */
  Max_Value   FLOAT,    /* Float, Int */
  Is_Read_Only    CHAR(1)   DEFAULT 'Y' NOT NULL
      CONSTRAINT col_ro_chk CHECK ( Is_Read_Only IN ( 'Y', 'N' ) ),
  Is_Null_OK    CHAR(1)   DEFAULT 'N' NOT NULL
      CONSTRAINT col_null_chk CHECK ( Is_Null_OK IN ('Y', 'N') ),
  Is_Frozen   CHAR(1)   DEFAULT 'N' NOT NULL
      CONSTRAINT col_frzn_chk CHECK ( Is_Frozen IN ('Y', 'N') ),
  Field_Order   INTEGER,
  Default_Value   VARCHAR2(1000),
  Units     VARCHAR2(10)
)tablespace wm_flexpm;
COMMENT ON TABLE wmd_Col_Def IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TRIGGER wmd_Col_Def_Tr_BI 
   BEFORE INSERT ON wmd_Col_Def 
   FOR EACH ROW
DECLARE
   vEntityID INTEGER;
BEGIN
   SELECT Entity_ID INTO vEntityId
     FROM wmd_Dict_Table
     WHERE Table_ID = :new.Table_ID AND Summary_ID = 0;
   IF ( vEntityID IS NULL ) THEN
      SELECT Entity_ID INTO vEntityID
        FROM wmd_Field_Def
        WHERE Field_ID = :new.Field_ID;
      UPDATE wmd_Dict_Table
        SET Entity_ID = vEntityID
        WHERE Table_ID = :new.Table_ID;
   END IF;
END;
/
SHOW ERRORS;
Create Trigger WMD_COL_DEF_TR_BIU
  Before Insert OR Update on WMD_COL_DEF
  FOR EACH ROW
Begin
  :New.Col_Name := Upper(:New.Col_Name);
        IF LENGTH(:new.col_name)>30 THEN
           RAISE_APPLICATION_ERROR(-20200, 'The column name("'||
                :new.Col_Name||'") is too long. The limit is 30 characters');
        END IF;
End;
/
Show Errors;
CREATE TABLE wmd_Col_Def_SubType (
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT colSub_fld_fky REFERENCES wmd_Col_Def ( Field_ID ),
  SubType_ID    INTEGER   NOT NULL,
  Entity_ID   INTEGER   NOT NULL,  /* Set by trigger */
  Min_Value   FLOAT,    /* Float, Int */
  Max_Value   FLOAT,    /* Float, Int */
  Default_Value   VARCHAR2(1000),
  CONSTRAINT colSub_pky PRIMARY KEY ( Field_ID, SubType_ID ),
  CONSTRAINT colSub_sub_fky FOREIGN KEY ( Entity_Id, SubType_ID ) 
    REFERENCES wmn_Entity_SubType ( Entity_ID, SubType_ID )
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Col_Def_SubType IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TRIGGER wmd_Col_Def_SubType_Tr_BI 
   BEFORE INSERT OR UPDATE ON wmd_Col_Def_SubType 
   FOR EACH ROW
BEGIN
   SELECT Entity_ID INTO :new.Entity_ID
     FROM wmd_Field_Def
     WHERE Field_ID = :new.Field_ID;
END;
/
SHOW ERRORS;
Create Table WMD_FUNC_DEF (
    Field_Id        Integer     Not Null,
    Table_Id        Integer     Not Null,
    Func_String     Varchar2(4000)   Not Null,
    Constraints PK_FUNC_DEF Primary Key (Field_Id),
    CONSTRAINT FK_Func_Def_Field_Id
    Foreign Key (Field_Id) References WMD_FIELD_DEF (Field_Id)
 )
 tablespace wm_flexpm
/
CREATE OR REPLACE TRIGGER WMD_FUNC_DEF_TR_BIU 
   BEFORE INSERT OR UPDATE ON wmd_Func_Def 
   FOR EACH ROW
DECLARE
   vEntityID INTEGER;
BEGIN
   SELECT Entity_ID INTO vEntityId
     FROM wmd_Dict_Table
     WHERE Table_ID = :new.Table_ID AND Summary_ID = 0;
   IF ( vEntityID IS NULL ) THEN
      SELECT Entity_ID INTO vEntityID
        FROM wmd_Field_Def
        WHERE Field_ID = :new.Field_ID;
      UPDATE wmd_Dict_Table
        SET Entity_ID = vEntityID
        WHERE Table_ID = :new.Table_ID;
   END IF;
END;
/
SHOW ERRORS;
CREATE TABLE wmd_Enum_Values (
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT enum_col_fky REFERENCES wmd_Col_Def ( Field_ID ),
  Enum_Key    VARCHAR2(40)  NOT NULL,
  Enum_Desc   VARCHAR2(40),
  CONSTRAINT enum_pky PRIMARY KEY ( Field_ID, Enum_Key )
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Enum_Values IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wmd_Dict_Namespace (
   Namespace_ID   INTEGER   NOT NULL
      CONSTRAINT namespace_pky PRIMARY KEY,
   Namespace_Title  VARCHAR2(40)  NOT NULL
      CONSTRAINT namespace_title_unq UNIQUE
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Dict_Namespace IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE SEQUENCE wmd_Dict_Namespace_Seq START WITH 1;
/* ****** Need Triggers to enforce Default namespace *********** */
CREATE TRIGGER wmd_Dict_Namespace_Tr_BI 
   BEFORE INSERT ON wmd_Dict_Namespace 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   IF ( :new.Namespace_ID IS NULL ) THEN
      SELECT wmd_Dict_Namespace_Seq.NextVal INTO newID FROM DUAL;
      :new.Namespace_ID := newID;
   END IF;
END;
/
SHOW ERRORS;
CREATE TABLE wmd_Dict_Name (
  Namespace_ID    INTEGER   DEFAULT NULL
      CONSTRAINT dict_nam_namspc_fky REFERENCES wmd_Dict_Namespace ( Namespace_ID ),
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT dict_nam_fld_fky REFERENCES wmd_Field_Def ( Field_ID ),
  Entity_ID   INTEGER   NOT NULL
      CONSTRAINT dict_nam_ent_fky REFERENCES wmn_Entity ( Entity_ID ),
  Name      VARCHAR2(128) NOT NULL,
  Report_Header   VARCHAR2(40)  DEFAULT NULL,
  Report_Header2  VARCHAR2(40)  DEFAULT NULL,
  Edit_Label    VARCHAR2(40)  DEFAULT NULL,
  Name_Desc   VARCHAR2(1000)  DEFAULT NULL,
  CONSTRAINT dict_name_pky PRIMARY KEY ( Namespace_ID, Field_ID ),
  CONSTRAINT dict_name_fldent_fky FOREIGN KEY ( Field_ID, Entity_ID )
        REFERENCES wmd_Field_Def ( Field_ID, Entity_ID ),
  CONSTRAINT dict_name_legal_chk CHECK ( Namespace_ID != 0 
     OR ( (    SUBSTR( Name, 1, 1) =  '_'  
            OR SUBSTR( Name, 1, 1) BETWEEN 'a' AND 'z'
            OR SUBSTR( Name, 1, 1) BETWEEN 'A' AND 'Z'
          )
          AND TRANSLATE ( Name, '?_%' || 'abcdefghijklmnopqrstuvwxyz' ||
                   'ABCDEFGHIJKLMNOPQRSTUVWXYZ' || '0123456789', '?'  ) IS NULL 
        )
     )
)
tablespace wm_flexpm;
Create UNIQUE Index dict_name_unq ON Wmd_Dict_Name (Entity_Id, NameSpace_Id, UPPER(Name)) tablespace wm_flexpm;
COMMENT ON TABLE wmd_Dict_Name IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TRIGGER wmd_Dict_Name_Tr_BI 
   BEFORE INSERT OR UPDATE ON wmd_Dict_Name 
   FOR EACH ROW
BEGIN
   SELECT Entity_ID INTO :new.Entity_ID
      FROM wmd_Field_Def WHERE Field_ID = :new.Field_ID;
END;
/
SHOW ERRORS;
CREATE TABLE wmd_Field_Data_Set (
  DataSet_ID    INTEGER   NOT NULL
      CONSTRAINT dataset_pky PRIMARY KEY,
  DataSet_Title VARCHAR2(40)  NOT NULL
      CONSTRAINT dataset_name_unq UNIQUE,
  Report_Header   VARCHAR2(16)  DEFAULT NULL,
  Report_Header2  VARCHAR2(16)  DEFAULT NULL
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Field_Data_Set IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE SEQUENCE wmd_Field_Data_Set_Seq START WITH 1;
CREATE TRIGGER wmd_Field_Data_Set_Tr_BI 
   BEFORE INSERT ON wmd_Field_Data_Set 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   if :new.DataSet_ID is NULL
   then
      SELECT wmd_Field_Data_Set_Seq.NextVal INTO newID FROM DUAL;
      :new.DataSet_ID := newID;
   end if;
END;
/
SHOW ERRORS;
CREATE TABLE wmd_Field_Data (
  DataSet_ID    INTEGER   DEFAULT NULL
      CONSTRAINT flddata_set_fky REFERENCES wmd_Field_Data_Set ( DataSet_ID ),
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT flddata_fld_fky REFERENCES wmd_Field_Def ( Field_ID ),
  Entity_ID   INTEGER   NOT NULL
      CONSTRAINT flddata_ent_fky REFERENCES wmn_Entity ( Entity_ID ),
  DataValue   VARCHAR2(4000),
  CONSTRAINT flddata_pky PRIMARY KEY ( DataSet_ID, Field_ID )
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Field_Data IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TRIGGER wmd_Field_Data_Tr_BI 
   BEFORE INSERT OR UPDATE ON wmd_Field_Data 
   FOR EACH ROW
BEGIN
   SELECT Entity_ID INTO :new.Entity_ID
      FROM wmd_Field_Def WHERE Field_ID = :new.Field_ID;
END;
/
SHOW ERRORS;
CREATE TABLE wmd_DataSource_Extra (
  DataSource_ID   INTEGER   NOT NULL
      CONSTRAINT dsextra_dsrc_fky REFERENCES wmd_Data_Source ( DataSource_ID ),
  Namespace_ID    INTEGER   DEFAULT NULL
      CONSTRAINT dsextra_namspc_fky REFERENCES wmd_Dict_Namespace ( Namespace_ID ),
  DataSet_ID    INTEGER   DEFAULT NULL
      CONSTRAINT dsextra_set_fky REFERENCES wmd_Field_Data_Set ( DataSet_ID ),
  Editable              CHAR(1)         DEFAULT 'N' NOT NULL,
  CONSTRAINT dsextra_unq UNIQUE ( DataSource_ID, Namespace_ID, DataSet_ID ),
  CONSTRAINT dsextra_chk CHECK ( Namespace_ID IS NULL OR DataSet_ID IS NULL ),
  CONSTRAINT dsextra_edit_chk CHECK ( Editable IN ( 'Y', 'N' ) )
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_DataSource_Extra IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE VIEW wmd_DataSource_Extra_Cols AS
  SELECT DataSource_Id, 'A' e_type, e.Namespace_Id e_id, Namespace_Title col_title, e.editable
    FROM wmd_Dict_Namespace ns, wmd_DataSource_Extra e
    WHERE ns.Namespace_Id = e.Namespace_Id
  UNION
  SELECT DataSource_Id, 'S', e.DataSet_Id, DataSet_Title, e.editable
    FROM wmd_Field_Data_Set ds, wmd_DataSource_Extra e
    WHERE ds.DataSet_Id = e.DataSet_Id;
COMMENT ON TABLE wmd_DataSource_Extra_Cols IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE OR REPLACE VIEW wmd_Dict_Field AS
  SELECT fld.Field_ID, fld.Entity_ID, nam.Name Field_Name,
        nam.Report_Header Field_Header, nam.Report_Header2 Field_Header2,
        fld.Field_Desc, DECODE(calc.Field_ID, NULL, 0, 1) Field_Calc,
        DECODE (fld.Field_Type,'C',col.Table_ID,'F',func.Table_Id) Table_Id,
    DECODE (fld.Field_Type,'C',col.Col_Name,'F',func.func_string) Table_Column,
        calc.Calc_Source Field_Calc_Source, 
    udc.Calc_Source UDC_Calc_Source, 
        fld.Datatype Field_Datatype, col.Is_Null_OK Field_NullOK,
        fld.Field_Aggregator, fld.Field_Emits_Aggregator,
  fld.Field_Type, fld.DataSource_ID
    FROM  wmd_Field_Def fld,
    wmd_Dict_Name nam,
    wmd_Calc_Def calc,
                wmd_UDC_def udc,
    wmd_Col_Def col,
    WMD_Func_Def func
    WHERE fld.Field_ID = nam.Field_ID
  AND nam.Namespace_ID = 0
  AND fld.Field_ID = calc.Field_ID (+)
  AND fld.Field_ID = udc.Field_ID (+)
  AND fld.Field_ID = col.Field_ID (+)
  AND fld.Field_Id = func.Field_Id (+);
COMMENT ON TABLE wmd_Dict_Field IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wmd_Technology (
   Tech_ID  INTEGER   NOT NULL
      CONSTRAINT tech_pky PRIMARY KEY,
   Tech_Name  VARCHAR2(8) NOT NULL
      CONSTRAINT tech_name_unq UNIQUE,
   Tech_Char  CHAR(1)
      CONSTRAINT tech_char_unq UNIQUE,
   Tech_Bit INTEGER
      CONSTRAINT tech_bit_unq UNIQUE,
   Tech_Desc  VARCHAR2(100)
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Technology IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE SEQUENCE wmd_Technology_Seq;
CREATE TRIGGER wmd_Technology_Tr_BI 
   BEFORE INSERT ON wmd_Technology 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   SELECT wmd_Technology_Seq.NextVal INTO newID FROM DUAL;
   :new.Tech_ID := newID;
END;
/
SHOW ERRORS;
CREATE VIEW wmd_Technology_Mask AS
   SELECT Tech_ID, Tech_Name, Tech_Char, POWER(2, Tech_Bit - 1) Tech_Mask, Tech_Desc
     FROM wmd_Technology
     WHERE Tech_Bit IS NOT NULL;
COMMENT ON TABLE wmd_Technology_Mask IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wmd_Field_Tech (
  Field_ID    INTEGER   NOT NULL
      CONSTRAINT fld_tech_fld_fky REFERENCES wmd_Field_Def ( Field_ID ),
  Tech_ID   INTEGER   NOT NULL
      CONSTRAINT fld_tech_tech_fky REFERENCES wmd_Technology ( Tech_ID ),
  CONSTRAINT fld_tech_pky PRIMARY KEY ( Field_ID, Tech_ID )
)
tablespace wm_flexpm;
COMMENT ON TABLE wmd_Field_Tech IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE VIEW wmd_Field_Tech_Mask AS
   SELECT f.Field_Id, sum(m.Tech_Mask) Tech_Mask
     FROM WMD_Field_Tech f, WMD_Technology_Mask m
     WHERE f.Tech_Id = m.Tech_Id
     GROUP BY f.Field_Id;
COMMENT ON TABLE wmd_Field_Tech_Mask IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE OR REPLACE VIEW wmd_DataSource_Field AS
   SELECT /*+ USE_HASH(x) */
          d.Datasource_ID, d.Field_Id, d.Entity_Id, n.Name, n.Report_Header,
          n.Report_Header2, d.Field_Desc, d.DataType, d.Field_Type, d.Field_Owner,
          d.Field_Aggregator, d.Field_Emits_Aggregator, NVL(e.Calc_Source, dbms_lob.substr(u.Calc_Source,1,4000)) Calc_Source,
          u.UDC_Source, u.Status, c.Length, c.Precision, c.Is_Null_OK, g.Field_Group_Short, x.Tech_Mask, u.Changed, c.Units,
          u.COMPLEX, d.weight_field_id
     FROM WMD_Field_Def d, WMD_Dict_Name n, WMD_Col_Def c, WMD_Calc_Def e, wmd_UDC_def u, WMD_Field_Group g,
       ( SELECT f.Field_Id, sum(t.Tech_Mask) Tech_Mask
           FROM WMD_Field_Tech f, WMD_Technology_Mask t
          WHERE f.Tech_Id = t.Tech_Id
          GROUP BY f.Field_Id ) x
    WHERE d.Field_ID = n.Field_ID
      AND n.Namespace_ID = 0
      AND d.Field_Id = c.Field_Id (+)
      AND d.Field_Id = e.Field_Id (+)
      AND d.Field_Id = u.Field_Id (+)
      AND d.Field_Group_Id = g.Field_Group_Id (+)
      AND d.Field_ID = x.Field_ID (+);
COMMENT ON TABLE wmd_Datasource_Field IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE OR REPLACE FUNCTION WMD_AllowReparenting wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
1f
2 :e:
1FUNCTION:
1WMD_ALLOWREPARENTING:
1P_ENTITY_ID:
1WMN_ENTITY:
1ENTITY_ID:
1TYPE:
1RETURN:
1CHAR:
1LV_CONTEXT_ID:
1WMN_CONTEXT:
1CONTEXT_ID:
1LV_DUMMY:
1NUMBER:
1LV_RVAL:
11:
1CURSOR:
1LV_C1:
1CV_ENTITY_ID:
1WMN_ENTITY_CONTEXT:
1CV_CONTEXT_ID:
1DUMMY:
1REPARENT:
1PARENT_ENTITY_ID:
1SELECT 1 dummy:n      FROM wmn_entity_context:n     WHERE entity_id in ( SELE+
1CT entity_id:n                            FROM wmn_entity:n                  +
1         WHERE reparent = 'Y' ):n     START WITH entity_id = cv_entity_id:n  +
1     AND context_id = cv_context_id:n   CONNECT BY entity_id = PRIOR parent_e+
1ntity_id:n       AND context_id = cv_context_id:
1CONTEXT_USE:
1SELECT context_id:n    INTO lv_context_id:n    FROM wmn_context:n   WHERE con+
1text_use = 'R':
1OPEN:
1FOUND:
1Y:
1N:
1CLOSE:
0

0
0
81
2
0 a0 8d 8f :2 a0 6b :2 a0 f
b0 3d b4 :2 a0 2c 6a a3 :2 a0
6b :2 a0 f 1c 81 b0 a3 a0
1c 81 b0 a3 a0 51 a5 1c
81 b0 a0 f4 8f :2 a0 6b :2 a0
f b0 3d 8f :2 a0 6b :2 a0 f
b0 3d b4 bf c8 :e a0 12a bd
b7 11 a4 b1 :4 a0 12a :4 a0 a5
dd e9 :2 a0 e9 d3 :2 a0 f 5a
a0 6e d b7 a0 6e d b7
:2 19 3c :2 a0 e9 c1 :2 a0 65 b7
a4 b1 11 68 4f 1d 17 b5 
81
2
0 3 7 37 1f 23 27 2a
2e 32 1e 3f 1b 44 48 4c
50 7d 58 5c 60 63 67 6b
70 78 57 99 88 8c 94 54
b5 a0 a4 a7 a8 b0 87 bc
c0 ea d5 d9 84 dd e1 e5
d4 f2 110 fb ff d1 103 107
10b fa 118 f7 11d 120 124 128
12c 130 134 138 13c 140 144 148
14c 150 154 158 15c 168 16d 16f
17b 17f 181 185 189 18d 191 19d
1a1 1a5 1a9 1ad 1b5 1b0 1b9 1bd
1c1 1c6 1cb 1cf 1d3 1d8 1db 1df
1e4 1e8 1ea 1ee 1f3 1f7 1f9 1fd
201 204 208 20c 211 213 217 21b
21f 221 225 227 233 237 239 23a
243 
81
2
0 1 a 20 2f 3a 2f :2 44
2f :2 20 1e 3 a :2 1 3 11
1d 11 :2 28 :3 11 :2 3 :3 11 :2 3 11
16 15 :2 11 :2 3 a 12 20 33
20 :2 3d 20 :3 12 20 2c 20 :2 37
20 :2 12 10 :2 3 e :2 c :3 22 11
1d c 19 f 21 c 19 5
:5 3 :4 a :2 3 8 10 1d :2 8 3
9 14 :2 3 8 e 8 6 5
10 5 16 5 10 5 :5 3 9
:3 3 a 3 :9 1 
81
4
0 :c 1 :2 2 :2 1
:a 3 :5 4 :7 5 :b 6
:9 7 :3 6 8 9
:2 a b c :2 d
:2 e :2 f :2 10 8
:5 6 12 13 14
15 12 :7 16 :4 17
:4 18 :3 19 18 :3 1b
1a :3 18 :4 1d :3 1e
:2 11 :7 1 
245
4
:3 0 1 :3 0 2
:a 0 7c 1 :7 0
5 :2 0 3 4
:3 0 5 :2 0 :2 4
5 0 6 :3 0
6 :2 0 1 6
8 :3 0 3 :7 0
a 9 :3 0 7
:3 0 8 :3 0 c
e 0 7c 2
f :2 0 9 84
0 7 a :3 0
b :2 0 4 12
13 0 6 :3 0
6 :2 0 1 14
16 :3 0 17 :7 0
1a 18 0 7a
0 9 :6 0 2a
2b 0 :2 d :3 0
1c :7 0 1f 1d
0 7a 0 c
:6 0 8 :3 0 f
:2 0 b 21 23
:6 0 26 24 0
7a 0 e :6 0
10 :3 0 11 :a 0
2 4d :4 0 33
34 0 f 13
:3 0 5 :2 0 4
6 :3 0 6 :2 0
1 2c 2e :3 0
12 :7 0 30 2f
:3 0 13 :2 0 11
a :3 0 b :2 0
4 6 :3 0 6
:2 0 1 35 37
:3 0 14 :7 0 39
38 :3 0 28 3d
0 3b :3 0 15
:3 0 13 :3 0 5
:3 0 5 :3 0 4
:3 0 16 :3 0 5
:3 0 12 :3 0 b
:3 0 14 :3 0 5
:3 0 17 :3 0 b
:3 0 14 :4 0 18
1 :8 0 4e 28
3d 4f 0 7a
16 4f 51 4e
50 :6 0 4d :7 0
4f b :3 0 9
:3 0 a :3 0 19
:4 0 1a 1 :8 0
78 1b :3 0 11
:3 0 3 :3 0 9
:3 0 18 58 5b
0 5c :2 0 78
58 5b :2 0 11
:3 0 c :4 0 61
:2 0 78 5e 5f
:3 0 11 :3 0 1c
:3 0 62 63 :3 0
64 :2 0 e :3 0
1d :4 0 66 67
0 69 1b 6e
e :3 0 1e :4 0
6a 6b 0 6d
1d 6f 65 69
0 70 0 6d
0 70 1f 0
78 1f :3 0 11
:4 0 74 :2 0 78
72 0 7 :3 0
e :3 0 76 :2 0
78 22 7b :3 0
7b 29 7b 7a
78 79 :6 0 7c
:2 0 2 f 7b
7f :3 0 7e 7c
80 :8 0 
2e
4
:3 0 1 3 1
b 1 11 1
1b 1 22 1
20 1 29 1
32 2 31 3a
1 4c 2 59
5a 1 68 1
6c 2 6e 6f
6 56 5d 60
70 73 77 4
19 1e 25 4d

1
4
0 
7f
0
1
14
2
8
0 1 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
2 0 1
32 2 0
29 2 0
20 1 0
28 1 2
3 1 0
11 1 0
1b 1 0
0

/
CREATE OR REPLACE VIEW wmn_Entity_Hier AS
  SELECT ent.Entity_Id, ec.Parent_Entity_Id, ec.Context_Id, ent.Entity_Name,
   ent.Entity_Desc, ec.Depth, NVL2(Time_Count,'y','n') Can_Sum,
   DECODE(nam.name, NULL, 'n', 'y') Can_Rpt,
         ec.context_entity_label,
         WMD_AllowReparenting( ent.Entity_Id ) AllowReparenting
    FROM wmn_Entity ent,
   (SELECT Entity_Id, Parent_Entity_Id, Context_Id, Level Depth, context_entity_label
      FROM wmn_Entity_Context
      START WITH Parent_Entity_Id IS NULL
      CONNECT BY Context_ID = PRIOR Context_ID
       AND Parent_Entity_Id = PRIOR Entity_Id ) ec,
   (SELECT Entity_Id, count(*) Time_Count FROM wmd_Dict_Table WHERE is_Time_Series = 'Y' GROUP BY Entity_ID ) dt,
   wmd_dict_name nam
    WHERE ent.Entity_Id = ec.Entity_Id
      AND ent.entity_id = dt.entity_id(+)
      AND ent.entity_id = nam.entity_id(+)
      AND nam.name(+) = 'rg_reap'
      AND nam.namespace_id(+) = 0
    ORDER BY ec.Depth , ent.EntityOrderNum;
COMMENT ON TABLE wmn_Entity_Hier IS 'Network Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
CREATE TABLE wm_frequency
(
  frequency_id      INTEGER   NOT NULL,
  tag               VARCHAR2(10) NOT NULL,
  description       VARCHAR2(64),
  seconds           INTEGER,
  truncCode         VARCHAR2(10),
  CONSTRAINT pk_wm_frequency PRIMARY KEY (frequency_id)
)
TABLESPACE WM_FLEXPM;
COMMENT ON TABLE wm_Frequency IS 'Data Dictionary Schema (./base/db/Oracle8i/dictionary/wmd_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
INSERT INTO wm_frequency VALUES (1, 'MI',   'Minute',      60, 'MI');
INSERT INTO wm_frequency VALUES (2, '2MI',  '2 Minute',   120, NULL);
INSERT INTO wm_frequency VALUES (3, '5MI',  '5 Minute',   300, NULL);
INSERT INTO wm_frequency VALUES (4, '10MI', '10 Minute',  600, NULL);
INSERT INTO wm_frequency VALUES (5, '15MI', '15 Minute',  900, NULL);
INSERT INTO wm_frequency VALUES (6, '30MI', '30 Minute', 1800, NULL);
INSERT INTO wm_frequency VALUES (7, 'H',    'Hourly',    3600, 'HH24');
INSERT INTO wm_frequency VALUES (8, 'D',    'Daily',     NULL, 'DD');
INSERT INTO wm_frequency VALUES (9, 'W',    'Weekly',    NULL, 'IW');
INSERT INTO wm_frequency VALUES (10, 'M',   'Monthly',   NULL, 'MM');
INSERT INTO wm_frequency VALUES (11, 'Y',   'Yearly',    NULL, 'IY');
INSERT INTO wm_frequency VALUES (12, 'A',   'All Times',    NULL, 'ALL');
INSERT INTO wm_frequency VALUES (13, 'WH',   'WeekHour',   NULL, 'WH');
INSERT INTO wm_frequency VALUES (14, 'MH',   'MonthHour',   NULL, 'MH');
CREATE TABLE wm_frequency_match
(
  report_frequency_id INTEGER NOT NULL,
  source_frequency_id INTEGER NOT NULL,
  CONSTRAINT pk_wm_frequency_match PRIMARY KEY
  (report_frequency_id, source_frequency_id)
  ,CONSTRAINT fk_wm_frequency_match_rfi FOREIGN KEY (report_frequency_id)
              REFERENCES wm_frequency (frequency_id)
  ,CONSTRAINT fk_wm_frequency_match_sfi FOREIGN KEY (source_frequency_id)
              REFERENCES wm_frequency (frequency_id)
)
  TABLESPACE WM_FLEXPM;
INSERT INTO wm_frequency_match VALUES (1, 1);
INSERT INTO wm_frequency_match VALUES (2, 1);
INSERT INTO wm_frequency_match VALUES (2, 2);
INSERT INTO wm_frequency_match VALUES (3, 1);
INSERT INTO wm_frequency_match VALUES (3, 3);
INSERT INTO wm_frequency_match VALUES (4, 1);
INSERT INTO wm_frequency_match VALUES (4, 2);
INSERT INTO wm_frequency_match VALUES (4, 3);
INSERT INTO wm_frequency_match VALUES (4, 4);
INSERT INTO wm_frequency_match VALUES (5, 1);
INSERT INTO wm_frequency_match VALUES (5, 3);
INSERT INTO wm_frequency_match VALUES (5, 5);
INSERT INTO wm_frequency_match 
  SELECT a.frequency_id, b.frequency_id FROM wm_frequency a, wm_frequency b
    WHERE a.frequency_id >= 6 AND b.frequency_id < a.frequency_id
    ORDER BY a.frequency_id, b.frequency_id;
CREATE TABLE wmd_summary_type
(
  summary_type_id   INTEGER      NOT NULL,
  table_extension   VARCHAR2(10) NOT NULL,
  summary_type      CHAR(1)      NOT NULL,
  CONSTRAINT pk_wmd_summary_type PRIMARY KEY (summary_type_id)
) TABLESPACE WM_FLEXPM;
INSERT INTO wmd_summary_type VALUES (1, '_TAB', 'R');
INSERT INTO wmd_summary_type VALUES (2, '_DBH', 'B');
INSERT INTO wmd_summary_type VALUES (3, '_DSM', 'S');
CREATE TABLE wmd_table_group
(
  table_id          INTEGER      NOT NULL,
  table_base_name   VARCHAR2(40) NOT NULL,
  entity_id         INTEGER      NOT NULL,
  frequency_id      INTEGER,
  CONSTRAINT pk_wmd_table_group PRIMARY KEY (table_id),
  CONSTRAINT fk_wmd_table_group_eid FOREIGN KEY (entity_id)
              REFERENCES wmn_entity (entity_id),
  CONSTRAINT fk_wmd_table_group_fid FOREIGN KEY (frequency_id)
              REFERENCES wm_frequency (frequency_id)
) TABLESPACE WM_FLEXPM;
CREATE SEQUENCE WM_File_Control_Seq;
CREATE TABLE WM_File_Control
(
 file_type_id     NUMBER       NOT NULL,
 file_type        VARCHAR2(30) NOT NULL,
 frequency        VARCHAR2(30) NOT NULL,
 frequency_id     Number NOT Null,
 file_description VARCHAR2(50) NOT NULL,
 entity_name      VARCHAR2(60),
 entity_id    Number,
 vendor           VARCHAR2(50) NOT NULL,
 offset_minutes   NUMBER,
 default_filetype_timeout NUMBER NOT NULL,
 module_name varchar2(30),
 file_extension varchar2(20),
 generate_loader char(1) default 'N',
 activate_scenario char(1) default 'N',
 is_show char(1) default 'Y',
 CONSTRAINT pk_WM_File_Control PRIMARY KEY (file_type_id),
 CONSTRAINT FK_File_Control_Entity 
   FOREIGN KEY(entity_id) REFERENCES WMN_ENTITY(entity_id),
 CONSTRAINT FK_File_Control_Frequency
   FOREIGN KEY(frequency_id) REFERENCES WM_FREQUENCY(frequency_id)
)
TABLESPACE WM_CLIENT
;
CREATE UNIQUE INDEX WM_FILE_CONTROL_UDX1
on WM_FILE_CONTROL (file_type) TABLESPACE WM_CLIENT;
Create Trigger WM_file_control_Tr_BI
 BEFORE INSERT ON WM_file_control
 FOR EACH ROW
DECLARE
   i_sec number := 0;
BEGIN
   select nvl(seconds,3600)
    into i_sec
   from wm_frequency
   where frequency_id=:new.frequency_id;
   if lower(:new.frequency)=lower('daily') then
       :New.default_filetype_timeout := 4*3600 ;
   else
     :New.default_filetype_timeout := i_sec*3;
   end if;
End;
/
SHOW ERRORS;
CREATE TABLE wm_min_partition (
  file_type_id       NUMBER,
  min_partition_date DATE,
  max_partition_date DATE,
    CONSTRAINT wm_min_partition_pk PRIMARY KEY ( file_type_id ),
    CONSTRAINT wm_min_partition_fk FOREIGN KEY ( file_type_id )
      REFERENCES wm_file_control( file_type_id )
)
TABLESPACE wm_flexpm;
CREATE TABLE WM_Available_Data
(
 timestamp     DATE NOT NULL,
 instance_id   NUMBER,
 entity_id     NUMBER NOT NULL,
 file_type_id NUMBER NOT NULL,
 hour NUMBER,
 event_id      NUMBER,
 CONSTRAINT pk_WM_Available_Data PRIMARY KEY (timestamp,file_type_id, instance_id, entity_id),
 CONSTRAINT FK_Avai_Data_File_type_id
   FOREIGN KEY(file_type_id) REFERENCES WM_FILE_CONTROL(file_type_id),
 CONSTRAINT FK_Avai_Data_Entity_Id
  FOREIGN KEY(entity_id)    REFERENCES WMN_ENTITY(entity_id)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm;
CREATE OR REPLACE TRIGGER wm_available_data_tr
AFTER INSERT OR UPDATE ON wm_available_data FOR EACH ROW
DECLARE
  lv_old_min_tstamp wm_available_data.timestamp%TYPE;
  lv_old_max_tstamp wm_available_data.timestamp%TYPE;
BEGIN
  SELECT min_partition_date, max_partition_date
    INTO lv_old_min_tstamp, lv_old_max_tstamp
    FROM wm_min_partition
   WHERE file_type_id = :NEW.file_type_id;
  IF ( lv_old_min_tstamp > :NEW.timestamp )  OR ( lv_old_min_tstamp IS NULL) THEN
    UPDATE wm_min_partition SET min_partition_date = :NEW.timestamp
      WHERE file_type_id = :NEW.file_type_id;
  END IF;
  IF ( lv_old_max_tstamp < :NEW.timestamp )  OR ( lv_old_max_tstamp IS NULL) THEN
    UPDATE wm_min_partition SET max_partition_date = :NEW.timestamp
      WHERE file_type_id = :NEW.file_type_id;
  END IF;
EXCEPTION
  WHEN NO_DATA_FOUND THEN
    INSERT INTO wm_min_partition ( file_type_id, min_partition_date, max_partition_date )
      VALUES ( :NEW.file_type_id, :NEW.timestamp, :NEW.timestamp );
END;
/
SHOW ERRORS TRIGGER wm_available_data_tr
Create or Replace VIEW WM_Available_DATA_VW
as
select wmc.file_type,
       wml.instance_id,
       wml.timestamp,
       wml.hour,
       wml.event_id
from   WM_File_Control wmc,
       WM_Available_Data wml,
       WMN_Instance    wme
where  wmc.file_type_id = wml.file_type_id
and    wme.instance_id = wml.instance_id;
CREATE OR REPLACE VIEW wm_min_partition_vw AS
  SELECT a.min_partition_date min_partition_date,
         a.max_partition_date max_partition_date, 
         b.file_type          file_type
    FROM wm_min_partition a, wm_file_control b
   WHERE a.file_type_id(+) = b.file_type_id;
CREATE TABLE wm_file_table_map
(
  file_type_id INTEGER NOT NULL,
  table_id     INTEGER NOT NULL,
  CONSTRAINT pk_wm_file_table_map PRIMARY KEY (file_type_id, table_id),
  CONSTRAINT fk_wm_file_table_map_fti FOREIGN KEY (file_type_id)
              REFERENCES wm_file_control (file_type_id),
  CONSTRAINT fk_wm_file_table_map_tid FOREIGN KEY (table_id)
              REFERENCES wmd_table_group (table_id)
) TABLESPACE WM_FLEXPM;
CREATE TABLE wmd_dict_field_depend
(Parent_Field_ID   INTEGER 
    constraint FK_field_def_Parent REFERENCES wmd_field_def (field_ID),
 Child_Field_ID    INTEGER NOT NULL
    constraint FK_field_def_Child REFERENCES wmd_field_def (field_ID),
 IS_UDC   NUMBER(1) )
Tablespace WM_FLEXPM;
CREATE INDEX wmd_dict_field_depend_idx1 on wmd_dict_field_depend
( Parent_Field_ID, Child_Field_ID)
tablespace wm_Flexpm storage (freelists 4) compute statistics; 
CREATE INDEX wmd_dict_field_depend_idx2 on wmd_dict_field_depend
( Child_Field_ID, Parent_Field_ID)
tablespace wm_Flexpm storage (freelists 4) compute statistics; 
CREATE INDEX wmd_dict_field_depend_idx3 on wmd_dict_field_depend
(  IS_UDC)
tablespace wm_Flexpm storage (freelists 4) compute statistics; 
CREATE GLOBAL TEMPORARY table wmd_dict_field_tmp
(field_id Integer)
ON COMMIT PRESERVE ROWS;
CREATE OR REPLACE VIEW wmd_field_file_map_vw AS
select /*+ use_hash(m f col func fd) */
dn.field_id,dn.name field_name, m.table_id,m.file_type_id
from wm_file_table_map m,
     wmd_dict_name dn, wmd_field_def f, wmd_col_def col, wmd_func_def func,
     (select distinct parent_field_id,child_field_id
        from wmd_dict_field_depend
       start with (parent_field_id in (select field_id from
wmd_dict_field_tmp))
       connect by parent_field_id= PRIOR child_field_id
     union
     select distinct parent_field_id,child_field_id
       from wmd_dict_field_depend
      start with (parent_field_id is NULL and child_field_id in (select
field_id from wmd_dict_field_tmp))
      connect by parent_field_id= PRIOR child_field_id
     ) fd
where decode(f.field_type, 'C', col.table_id, 'F', func.table_id, -9999) =
m.table_id
and   dn.field_id = fd.child_field_id
and   dn.field_id = f.field_id
and   f.field_id = col.field_id(+)
and   f.field_id = func.field_id(+)
and   dn.namespace_id=0
;
CREATE OR REPLACE VIEW wmd_field_anc_vw
AS select level depth,parent_field_id,child_field_id
from wmd_dict_field_depend
start with child_field_id in (select field_id from wmd_dict_field_tmp)
connect by child_field_id= PRIOR parent_field_id;
CREATE OR REPLACE VIEW wmd_field_des_vw
AS select level depth,parent_field_id,child_field_id
from wmd_dict_field_depend
start with parent_field_id in (select field_id from wmd_dict_field_tmp)
connect by parent_field_id= PRIOR child_field_id;
CREATE TABLE wmd_udc_renameHist
(
  change_id    INTEGER          NOT NULL
    CONSTRAINT wmd_udc_rename_change_id_unq UNIQUE,
  field_ID     INTEGER          NOT NULL
      CONSTRAINT udc_rename_fld_fky REFERENCES wmd_Field_Def ( Field_ID ) on delete cascade,
  entity_ID   INTEGER   NOT NULL
      CONSTRAINT udc_rename_ent_fky REFERENCES wmn_Entity ( Entity_ID ),
  oldName      VARCHAR2(128)  NOT NULL,    -- wmd_dict_name.name%TYPE,
  newName      VARCHAR2(128)  NOT NULL,    -- wmd_dict_name.name%TYPE,
  tStamp       DATE             NOT NULL
)
TABLESPACE WM_FLEXPM;
CREATE SEQUENCE wmd_udc_renameHist_seq;
CREATE OR REPLACE TRIGGER wmd_udc_renameHist_tr_BI
  BEFORE INSERT ON wmd_udc_renameHist
  FOR EACH ROW
DECLARE
  newID INTEGER;
BEGIN
  IF ( :new.change_id IS NULL ) THEN
    SELECT wmd_udc_renameHist_seq.NextVal INTO newID FROM DUAL;
      :new.change_id := newID;
  END IF;
END;
/
SHOW ERRORS;
