---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 1999,2010. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/dictionary/wmn_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--
-- **********************
-- *** Network Schema ***
-- **********************

-- *****************************
-- ** Hierarchy Contexts
-- *****************************

CREATE TABLE wmn_Context (
  Context_ID    INTEGER     NOT NULL
      CONSTRAINT context_pky PRIMARY KEY,
  Context_Name  VARCHAR2(40)    NOT NULL
      CONSTRAINT context_name_unq UNIQUE,
  Context_Use   CHAR(1)
      CONSTRAINT context_use_chk CHECK ( Context_Use IN ( 'R', 'S', 'U' ) )
      -- (R)eport, (S)cenario
      CONSTRAINT context_use_unq UNIQUE
)
tablespace wm_flexpm;

CREATE SEQUENCE wmn_Context_Seq;

CREATE TRIGGER wmn_Context_Tr_BI 
   BEFORE INSERT ON wmn_Context 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   SELECT wmn_Context_Seq.NextVal INTO newID FROM DUAL;
   :new.Context_ID := newID;
END;
/
SHOW ERRORS;

COMMENT ON TABLE wmn_Context IS 'Network Schema (./base/db/Oracle8i/dictionary/wmn_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';


-- ******************
-- ** Entity (Class)
-- ******************

CREATE TABLE wmn_Entity (
  Entity_ID INTEGER     NOT NULL
      CONSTRAINT entity_pky PRIMARY KEY,
  Entity_Name   VARCHAR2(30)    NOT NULL
      CONSTRAINT entity_name_unq UNIQUE
      /* CONSTRAINT entity_name_nospc_chk CHECK ( Entity_Name NOT LIKE '% %' ) */
      CONSTRAINT  entity_name_legal_chk CHECK ( 
         (    SUBSTR( Entity_Name, 1, 1) BETWEEN 'a' AND 'z'
           OR SUBSTR( Entity_Name, 1, 1) BETWEEN 'A' AND 'Z'
         )
         AND TRANSLATE ( Entity_Name, '?_%' || 'abcdefghijklmnopqrstuvwxyz' ||
                     'ABCDEFGHIJKLMNOPQRSTUVWXYZ' || '0123456789', '?'  ) IS NULL 
      ),
  Entity_Label  VARCHAR2(50)    NOT NULL
      CONSTRAINT entity_label_unq UNIQUE,
  Entity_Desc   VARCHAR2(100),
  IsGroup   CHAR(1)     DEFAULT 'n' NOT NULL
      CONSTRAINT entity_group_chk CHECK ( IsGroup IN ('y', 'n') ),
  Entity_Table  VARCHAR2(40),
  /* Scenario-related Fields - formerly in wms_Entity_Scenario */
  ImageKey      VARCHAR2(20),
  DefaultNameExpr   VARCHAR2(4000),
  LabelExpr     VARCHAR2(255),
  EntityOrderExpr   VARCHAR2(255),
  EntityOrderNum    INTEGER,
  ReportNameExpr        VARCHAR2(2000),
  ReportNameHeader      VARCHAR2(500),
  BaseEntityID          INTEGER
      CONSTRAINT entity_base_fky REFERENCES wmn_Entity ( Entity_ID ),
  LocalKey_Type         CHAR(1) DEFAULT 'I' NOT NULL
      CONSTRAINT entity_lktype_chk CHECK ( LocalKey_Type IN ( 'S', 'I', 'F' ) ),
  reparent      CHAR(1) DEFAULT 'N',
      CONSTRAINT entity_reparent_ck CHECK ( reparent IN ( 'Y', 'N' ) ),
  fdn_attr VARCHAR2(30)   DEFAULT   'LOCALKEY'   NOT NULL,
             CONSTRAINT fdn_attr_check check ( fdn_attr in ( 'NAME', 'LABEL', 'LOCALKEY' ) ),
  maptablename          VARCHAR2(30)
)
tablespace wm_flexpm;

CREATE UNIQUE INDEX UNQ_entity_name ON wmn_entity(UPPER(entity_name));

CREATE SEQUENCE wmn_Entity_Seq;

CREATE TRIGGER wmn_Entity_Tr_BI 
   BEFORE INSERT ON wmn_Entity 
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   SELECT wmn_Entity_Seq.NextVal INTO newID FROM DUAL;
   :new.Entity_ID := newID;
END;
/
SHOW ERRORS;

COMMENT ON TABLE wmn_Entity IS 'Network Schema (./base/db/Oracle8i/dictionary/wmn_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';


-- ******************************
-- ** Entity Sub-type (SubClass)
-- ******************************

CREATE TABLE wmn_Entity_SubType (
  Entity_ID INTEGER     NOT NULL
      CONSTRAINT entSub_entity_fky REFERENCES wmn_Entity ( Entity_ID ),
  SubType_ID    INTEGER     NOT NULL,
  SubType_Name  VARCHAR2(40)    NOT NULL,
  CONSTRAINT entSub_pky PRIMARY KEY ( Entity_ID, SubType_ID ),
  CONSTRAINT entSub_name_unq UNIQUE ( Entity_ID, SubType_Name )
)
tablespace wm_flexpm;

CREATE SEQUENCE wmn_Entity_SubType_Seq;

CREATE TRIGGER wmn_Entity__SubType_Tr_BI 
   BEFORE INSERT ON wmn_Entity_SubType
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   IF ( :new.SubType_ID IS NULL ) THEN
      SELECT wmn_Entity_SubType_Seq.NextVal INTO newID FROM DUAL;
   END IF;
   :new.SubType_ID := newID;
END;
/
SHOW ERRORS;

COMMENT ON TABLE wmn_Entity_SubType IS 'Network Schema (./base/db/Oracle8i/dictionary/wmn_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';
     

-- *************************************************
-- ** Entity Context
-- **    An Entity's Context specific data (parent)
-- *************************************************

CREATE TABLE wmn_Entity_Context (
  Entity_ID INTEGER     NOT NULL
      CONSTRAINT entity_ctx_entity_fky REFERENCES wmn_Entity ( Entity_ID ),
  Context_ID    INTEGER     NOT NULL
      CONSTRAINT entity_ctx_context_fky REFERENCES wmn_Context (Context_ID),
  ParseKey  CHAR(1)     DEFAULT 'y' NOT NULL
      CONSTRAINT entity_ctx_parse_chk CHECK ( ParseKey IN ('y', 'n') ),
  Parent_Entity_ID  INTEGER,
  Context_Entity_Label  VARCHAR2(30),
  CONSTRAINT entity_cxt_pky PRIMARY KEY ( Entity_ID, Context_ID ),
  CONSTRAINT entity_cxt_parent_fky FOREIGN KEY ( Parent_Entity_ID, Context_ID )
      REFERENCES wmn_Entity_Context ( Entity_ID, Context_ID )
)
tablespace wm_flexpm;

COMMENT ON TABLE wmn_Entity_Context IS 'Network Schema (./base/db/Oracle8i/dictionary/wmn_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';


-- ***************************
-- ** Entity Instances
-- ***************************

CREATE TABLE wmn_Instance (
  Instance_ID   INTEGER     NOT NULL
      CONSTRAINT inst_pky PRIMARY KEY
      USING INDEX STORAGE (FREELISTS 4),
  Entity_ID INTEGER     NOT NULL
      CONSTRAINT inst_entity_fky REFERENCES wmn_Entity (Entity_ID),
  SubType_ID    INTEGER,
  Instance_Name VARCHAR2(30),
  /* Scenario-related Fields - formerly in wms_Instance_Scenario */
  Label     VARCHAR2(20)    DEFAULT NULL,
  Active    CHAR(1)     DEFAULT 'y' NOT NULL 
      CONSTRAINT inst_ent_active_chk CHECK (Active IN ('y', 'n')),
  PlaceHldr CHAR(1)     DEFAULT 'n' NOT NULL
      CONSTRAINT inst_ent_place_chk CHECK (PlaceHldr IN ('y', 'n')),
  CONSTRAINT inst_ent_subtype_fky FOREIGN KEY ( Entity_ID, SubType_ID )
    REFERENCES wmn_Entity_SubType (Entity_ID, SubType_ID)
)
tablespace wm_flexpm
STORAGE (
  FREELISTS 4
);

CREATE INDEX ind_Instance_Entity_ID ON wmn_Instance ( Entity_ID )
tablespace wm_flexpm;

CREATE INDEX ind_Instance_Entity_subtype ON wmn_Instance ( Entity_ID, SubType_ID )
tablespace wm_flexpm;

CREATE SEQUENCE wmn_Instance_Seq;

CREATE TRIGGER wmn_Instance_Tr_BI
   BEFORE INSERT ON wmn_Instance
   FOR EACH ROW
DECLARE
   newID INTEGER;
BEGIN
   SELECT wmn_Instance_Seq.NextVal INTO newID FROM DUAL;
   :new.Instance_ID := newID;
END;
/
SHOW ERRORS;

CREATE OR REPLACE TRIGGER wmn_Instance_Tr_AIUD
   AFTER INSERT OR UPDATE OR DELETE ON WMN_INSTANCE
   FOR EACH ROW
BEGIN
   IF INSERTING OR UPDATING THEN
      wm_scenario.Entity_Alert( :new.Entity_ID );
   ELSE
      wm_scenario.Entity_Alert( :old.Entity_ID );
   END IF;  
END;
/
SHOW ERRORS;

COMMENT ON TABLE wmn_Instance IS 'Network Schema (./base/db/Oracle8i/dictionary/wmn_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';


-- *************************************************
-- ** Instance Context
-- **    An Instance's Context specific data
-- *************************************************

CREATE TABLE wmn_Instance_Context (
  Instance_ID   INTEGER     NOT NULL
      CONSTRAINT inst_cxt_inst_fky REFERENCES wmn_Instance (Instance_ID) ON DELETE CASCADE,
  Context_ID    INTEGER     NOT NULL
      CONSTRAINT inst_cxt_cxt_fky REFERENCES wmn_Context (Context_ID),
  Entity_ID INTEGER     NOT NULL
      CONSTRAINT inst_cxt_ent_fky REFERENCES wmn_Entity (Entity_ID),
  ParentInstance_ID INTEGER         DEFAULT NULL
      CONSTRAINT inst_parent_fky REFERENCES wmn_Instance (Instance_ID) ON DELETE CASCADE,
  LocalKey  VARCHAR2(90)    NOT NULL,
  FullKey   VARCHAR2(900)   NOT NULL,
  CONSTRAINT inst_cxt_pky PRIMARY KEY ( Context_ID, Instance_ID )
             USING INDEX STORAGE (FREELISTS 4),
  CONSTRAINT inst_cxt_pnt_unq UNIQUE ( Context_ID, Instance_ID, ParentInstance_ID )
             USING INDEX STORAGE (FREELISTS 4),
  CONSTRAINT inst_cxt_parent_fky FOREIGN KEY ( Context_ID, ParentInstance_ID )
      REFERENCES wmn_Instance_Context ( Context_ID, Instance_ID ) ON DELETE CASCADE
)
PARTITION BY HASH( Context_ID, Entity_ID )
(
  PARTITION p1,
  PARTITION p2,
  PARTITION p3,
  PARTITION p4,
  PARTITION p5,
  PARTITION p6,
  PARTITION p7,
  PARTITION p8,
  PARTITION p9,
  PARTITION p10,
  PARTITION p11,
  PARTITION p12,
  PARTITION p13,
  PARTITION p14,
  PARTITION p15,
  PARTITION p16,
  PARTITION p17,
  PARTITION p18,
  PARTITION p19,
  PARTITION p20,
  PARTITION p21,
  PARTITION p22,
  PARTITION p23,
  PARTITION p24,
  PARTITION p25,
  PARTITION p26,
  PARTITION p27,
  PARTITION p28,
  PARTITION p29,
  PARTITION p30,
  PARTITION p31,
  PARTITION p32
)
TABLESPACE wm_FlexPM
STORAGE (
  FREELISTS 4
);

CREATE UNIQUE INDEX inst_ctx_key_unq ON wmn_Instance_Context
    ( Context_ID, Entity_ID, FullKey )
    STORAGE (FREELISTS 4) LOCAL;

CREATE UNIQUE INDEX inst_ctx_lkey_unq ON wmn_Instance_Context
    ( Context_ID,  Entity_ID, ParentInstance_ID, LocalKey )
    STORAGE (FREELISTS 4) LOCAL;

CREATE INDEX inst_cxt_parent_instance ON wmn_Instance_Context
    ( ParentInstance_Id, Instance_Id)
    TABLESPACE wm_flexpm STORAGE (FREELISTS 4);

CREATE INDEX inst_cxt_instance_id ON wmn_Instance_Context
    (Instance_Id)
    TABLESPACE wm_flexpm STORAGE (FREELISTS 4);

CREATE INDEX inst_cxt_entity_id ON wmn_Instance_Context
    (entity_Id)
    TABLESPACE wm_flexpm STORAGE (FREELISTS 4);

CREATE INDEX inst_cxt_context_parent 
ON wmn_instance_context (context_id, parentinstance_id)
COMPRESS
PCTFREE 5
TABLESPACE wm_flexpm STORAGE (FREELISTS 4);

-- For insert or update, set Entity_ID
CREATE TRIGGER wmn_Inst_Ctx_Tr_BIU
   BEFORE INSERT OR UPDATE ON wmn_Instance_Context
   FOR EACH ROW
BEGIN
   SELECT Entity_ID INTO :new.Entity_ID
      FROM wmn_Instance WHERE Instance_ID = :new.Instance_ID;
END;
/
SHOW ERRORS;

Create OR Replace Trigger WMN_Instance_Context_Tr_AIUD
    AFTER INSERT OR UPDATE OR DELETE ON WMN_INSTANCE_CONTEXT
        FOR EACH ROW
Begin
   IF INSERTING OR UPDATING THEN
      IF (:new.context_id=wm_scenario.get_context) THEN
          wm_scenario.Entity_Alert( :new.Entity_ID );
      END IF;
   ELSE
      IF (:old.context_id=wm_scenario.get_context) THEN
          wm_scenario.Entity_Alert( :old.Entity_ID );
      END IF;
   END IF;  
End;
/
Show Errors

COMMENT ON TABLE wmn_Instance_Context IS 'Network Schema (./base/db/Oracle8i/dictionary/wmn_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';

CREATE TABLE wmn_Context_Transition (
  Entity_ID     INTEGER     NOT NULL
      CONSTRAINT cxt_xt_ent_fky REFERENCES wmn_Entity ( Entity_ID ),
  Src_Context_ID    INTEGER     NOT NULL
      CONSTRAINT cxt_xt_src_fky REFERENCES wmn_Context ( Context_ID ),
  Dest_Context_ID   INTEGER     NOT NULL
      CONSTRAINT cxt_xt_dst_fky REFERENCES wmn_Context ( Context_ID ),
  Transition        VARCHAR2(4000),
  Always_Convert        INTEGER         DEFAULT 0     NOT NULL,
  CONSTRAINT cxt_xt_pky PRIMARY KEY ( Entity_ID, Src_Context_ID, Dest_Context_ID ),
  CONSTRAINT cxt_xt_dup_cxt_chk CHECK ( Src_Context_ID != Dest_Context_ID )
)
tablespace wm_flexpm;

COMMENT ON TABLE wmn_Context_Transition IS 'Network Schema (./base/db/Oracle8i/dictionary/wmn_schema.sql), Version BL_core_schema_8.0.7_8.0.7.5.02_20100605';

--This table will create a hierarchy list of instance that need to be deleted from the parent instance.
CREATE GLOBAL TEMPORARY TABLE wmn_delete_instance_tmp
(
  LEV NUMBER,
  INSTANCE_ID INTEGER,
  ENTITY_ID   INTEGER,
  FULLKEY     VARCHAR2(900)
); 
CREATE INDEX wmn_delete_instance_idx ON wmn_delete_instance_tmp (entity_id, instance_id);

--This table will hold a list of table that need to be checked by the delete instance procedure.. 
CREATE TABLE wmn_delete_instance_tables
(
  Table_name        VARCHAR2(30),
  Column_name       VARCHAR2(30),
  Findkey     	    CHAR(1),
  Entity_Column     VARCHAR2(30),
  Entity_type       CHAR(1),
  CONSTRAINT FindKey_ck CHECK (FINDKEY IN ('I','F')),
  CONSTRAINT Entitytype_ck CHECK (Entity_type IN ('N','I')),
  CONSTRAINT Instable_pky PRIMARY KEY (Table_name, Column_name,Findkey)
)
TABLESPACE wm_flexpm;

--This table will hold detail and summary messages which will be written out to a Summary and Details file.
CREATE TABLE wmn_delete_instance_msg_log
(
  Line_num      INTEGER,
  Message       VARCHAR2(1000),
  Report_type 	CHAR(1)       NOT NULL,
  CONSTRAINT Rpt_Sum_Det_ck CHECK (REPORT_TYPE IN ('S','D'))
)
TABLESPACE wm_flexpm;
