CREATE SEQUENCE en_event_typegrp_seq;
CREATE SEQUENCE en_event_type_seq;
CREATE SEQUENCE en_event_seq;
 CREATE TABLE en_event_type_group(
	group_id	INTEGER,
	group_name      VARCHAR2(64),
	group_label     VARCHAR2(64),
	 CONSTRAINT pk_en_event_type_group PRIMARY KEY(group_id)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE TABLE en_event_type (
    event_type_id    INTEGER,
    name	     VARCHAR2(32) NOT NULL,
    label            VARCHAR2(64) NOT NULL,
    view_name        VARCHAR2(30),
    msg_proc         VARCHAR2(64),
    can_schedule     CHAR(1) DEFAULT 'N',
      CONSTRAINT en_event_type_can_schedule_ck check (can_schedule in ('Y','N')),
    can_notify       CHAR(1) DEFAULT 'Y',
      CONSTRAINT en_event_type_can_notify_ck check (can_notify in ('Y','N')),
    group_id         INTEGER NOT NULL,
    forward          CHAR(1) DEFAULT 'Y' NOT NULL,
      CONSTRAINT en_event_type_forward_ck check(forward in ('Y', 'N')),
    CONSTRAINT pk_en_event_type PRIMARY KEY (event_type_id),
    CONSTRAINT en_event_type_name_unq UNIQUE (name) USING INDEX PCTFREE 0 TABLESPACE wm_flexpm,
    CONSTRAINT en_event_type_label_unq UNIQUE (label) USING INDEX PCTFREE 0 TABLESPACE wm_flexpm,
    CONSTRAINT fk_en_event_type_groupid 
		   FOREIGN KEY (group_id) REFERENCES en_event_type_group(group_id)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE TABLE en_event (
    event_id        INTEGER,
    event_time      DATE DEFAULT SYSDATE,
    event_type_id   INTEGER, 
    data_tstamp     DATE,
    handled_tstamp  DATE,
    is_read         CHAR(1) DEFAULT 'N',
    CONSTRAINT pk_en_event PRIMARY KEY (event_id),
    CONSTRAINT fk_en_event_event_type
               FOREIGN KEY (event_type_id) 
               REFERENCES en_event_type (event_type_id),
    CONSTRAINT chk_en_event_is_read CHECK (is_read IN ('Y', 'N')),
    CONSTRAINT en_event_event_time_nn CHECK (event_time IS NOT NULL),
    CONSTRAINT en_event_event_type_id_nn CHECK (event_type_id IS NOT NULL)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 25;
CREATE INDEX en_event_is_read ON en_event (is_read)
TABLESPACE wm_flexpm
;
CREATE TABLE en_event_data_load (
    event_id        INTEGER,
    instance_id     INTEGER,
    entity_id       INTEGER,
    file_type_id     INTEGER, 
    CONSTRAINT pk_event_data_load PRIMARY KEY (event_id),
    CONSTRAINT fk_en_event_data_load 
               FOREIGN KEY (event_id)
               REFERENCES en_event (event_id) ON DELETE CASCADE,
    CONSTRAINT fk_en_event_data_load_file
               FOREIGN KEY (file_type_id)
               REFERENCES wm_file_control (file_type_id)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE OR REPLACE VIEW en_event_data_load_v AS
SELECT e.event_id,e.event_time,e.event_type_id,e.data_tstamp,e.handled_tstamp,
edl.instance_id, edl.entity_id, edl.file_type_id
FROM en_event e, en_event_data_load edl, en_event_type et
WHERE e.event_id = edl.event_id
AND e.event_type_id = et.event_type_id
AND et.name = 'Data load';
CREATE TABLE schedule_event ( 
    schedule_id     INTEGER,
    event_type_id   INTEGER,
    CONSTRAINT pk_schedule_event PRIMARY KEY (schedule_id),
    CONSTRAINT fk_schedule_event_sched_id
               FOREIGN KEY (schedule_id)
	       REFERENCES schedule (schedule_id) ON DELETE CASCADE,
    CONSTRAINT fk_schedule_event_type_id
               FOREIGN KEY (event_type_id)
               REFERENCES en_event_type (event_type_id),
    CONSTRAINT schedule_event_type_nn CHECK (event_type_id IS NOT NULL)	   
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE GLOBAL TEMPORARY TABLE en_event_filter_tmp 
ON COMMIT DELETE ROWS
  AS SELECT event_id,event_time,event_type_id,data_tstamp,handled_tstamp
       FROM en_event WHERE 0 = 1;
ALTER TABLE en_event_filter_tmp ADD CONSTRAINT pk_en_event_filter_tmp PRIMARY KEY (event_id);
CREATE TABLE en_event_file_load(
    event_id           INTEGER,
    error_message      VARCHAR2(128),
    file_name          VARCHAR2(1024),
    log_file           VARCHAR2(1024), 
    instance_id        INTEGER,
    entity_id          INTEGER,
    file_type_id       INTEGER,
    NUMOFINTERVAL      INTEGER,
    CONSTRAINT pk_event_file_load PRIMARY KEY (event_id),
    CONSTRAINT fk_en_event_file_load 
               FOREIGN KEY (event_id)
               REFERENCES en_event (event_id) ON DELETE CASCADE,
    CONSTRAINT fk_en_event_file_load_file
               FOREIGN KEY (file_type_id)
               REFERENCES wm_file_control (file_type_id)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE OR REPLACE VIEW en_event_file_load_v AS
SELECT e.event_id,e.event_time,e.event_type_id,e.data_tstamp,e.handled_tstamp,
 efl.error_message,efl.file_name,efl.log_file,
  efl.INSTANCE_ID,efl.ENTITY_ID,efl.FILE_TYPE_ID,
  efl.NUMOFINTERVAL
FROM en_event e, en_event_file_load efl, en_event_type et
WHERE e.event_id    = efl.event_id
AND e.event_type_id = et.event_type_id
AND et.name in ('FILE_LOAD_S','FILE_LOAD_F');
CREATE TABLE en_event_file_missing(
    event_id           INTEGER,
    error_message      VARCHAR2(128),
    file_name          VARCHAR2(1024),
    log_file           VARCHAR2(1024), 
    instance_id        INTEGER,
    entity_id          INTEGER,
    file_type_id       INTEGER,
    NUMOFINTERVAL      INTEGER,
    CONSTRAINT pk_event_file_missing PRIMARY KEY (event_id),
    CONSTRAINT fk_en_event_file_missing 
               FOREIGN KEY (event_id)
               REFERENCES en_event (event_id) ON DELETE CASCADE,
    CONSTRAINT fk_en_event_file_missing_file
               FOREIGN KEY (file_type_id)
               REFERENCES wm_file_control (file_type_id)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE OR REPLACE VIEW en_event_file_missing_v AS
SELECT e.event_id,e.event_time,e.event_type_id,e.data_tstamp,e.handled_tstamp,
 efl.error_message,efl.file_name,efl.log_file,
  efl.INSTANCE_ID,efl.ENTITY_ID,efl.FILE_TYPE_ID,
  efl.NUMOFINTERVAL
FROM en_event e, en_event_file_missing efl, en_event_type et
WHERE e.event_id    = efl.event_id
AND e.event_type_id = et.event_type_id
AND et.name in ('FILE_MISSING');
CREATE TABLE en_event_sa_file_missing(
    event_id           INTEGER,
    sa_sched_id        INTEGER,
    filemissing_msg    VARCHAR2(4000),
    file_name          VARCHAR2(1024),
    log_file           VARCHAR2(1024),
    CONSTRAINT pk_en_sa_file_missing PRIMARY KEY (event_id),
    CONSTRAINT fk_en_sa_file_missing
               FOREIGN KEY (event_id)
               REFERENCES en_event (event_id) ON DELETE CASCADE
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE OR REPLACE VIEW en_event_sa_file_missing_v AS
SELECT e.event_id, e.event_time, e.event_type_id, e.data_tstamp, e.handled_tstamp,
       esfm.sa_sched_id, 
       esfm.filemissing_msg, esfm.file_name, esfm.log_file
  FROM en_event e, en_event_sa_file_missing esfm, en_event_type et
 WHERE e.event_id      = esfm.event_id
   AND e.event_type_id = et.event_type_id
   AND et.name in ('SA_FILE_MISSING');
CREATE TABLE pm_notification_event_map( 
    notify_id       INTEGER,
    event_type_id   INTEGER,
    CONSTRAINT pk_notif_eventmap PRIMARY KEY (notify_id,event_type_id),
    CONSTRAINT fk_notif_eventmap_notify_id
               FOREIGN KEY (notify_id) REFERENCES pm_notification (id) ON DELETE CASCADE,
    CONSTRAINT fk_notif_eventmap_eventtype_id
               FOREIGN KEY (event_type_id) REFERENCES en_event_type (event_type_id)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE OR REPLACE FUNCTION get_eventype_id wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
c
2 :e:
1FUNCTION:
1GET_EVENTYPE_ID:
1PNAME:
1VARCHAR2:
1RETURN:
1INTEGER:
1V_EVNTTYPE_ID:
1EVENT_TYPE_ID:
1EN_EVENT_TYPE:
1NAME:
1SELECT event_type_id into v_evnttype_id:n  from en_event_type:n  where  name +
1= pname:
1NO_DATA_FOUND:
0

0
0
2a
2
0 a0 8d 8f a0 b0 3d b4
:2 a0 2c 6a a3 a0 1c 81 b0
:5 a0 12a :2 a0 65 b7 :2 a0 4d 65
b7 a6 9 a4 b1 11 68 4f
1d 17 b5 
2a
2
0 3 7 23 1f 1e 2b 1b
30 34 38 3c 55 44 48 50
43 5c 60 64 68 6c 70 7c
80 84 40 88 8c 90 91 95
97 98 9d a1 a3 af b3 b5
b6 bf 
2a
2
0 1 a 1a 23 :2 1a 19 1
8 :3 1 :3 f 1 a 1d 8 a
11 :2 3 a 3 1 10 3 a
3 1e :2 b :8 1 
2a
4
0 :7 1 :2 2 :2 1
:5 4 :2 8 9 :2 a
8 :3 c 6 e
:3 10 :4 e :7 1 
c1
4
:3 0 1 :3 0 2
:a 0 25 1 :7 0
5 :2 0 3 4
:3 0 3 :7 0 5
4 :3 0 5 :3 0
6 :3 0 7 9
0 25 2 a
:2 0 9 24 0
7 6 :3 0 d
:7 0 10 e 0
23 0 7 :6 0
8 :3 0 7 :3 0
9 :3 0 a :3 0
3 :4 0 b 1
:8 0 1a 5 :3 0
7 :3 0 18 :2 0
1a c :3 0 5
:4 0 1d :2 0 1f
c 21 e 20
1f :2 0 22 10
:2 0 24 12 24
23 1a 22 :6 0
25 :2 0 2 a
24 28 :3 0 27
25 29 :8 0 
14
4
:3 0 1 3 1
6 1 c 2
16 19 1 1e
1 1b 1 21
1 f 
1
4
0 
28
0
1
14
1
3
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
2 0 1
c 1 0
3 1 0
0

/
SHOW ERRORS
CREATE TABLE en_event_status_change(event_id           INTEGER,
			            status_message     VARCHAR2(4000),
				    summary            VARCHAR2(256),
			            long_message       VARCHAR2(4000),
				    log_file           VARCHAR2(255),
				    start_time         DATE,
				    end_time           DATE,
    CONSTRAINT pk_event_status_change PRIMARY KEY (event_id),
    CONSTRAINT fk_en_event_status_change FOREIGN KEY (event_id) 
     REFERENCES en_event (event_id) ON DELETE CASCADE
)
TABLESPACE wm_flexpm
PCTFREE 0;
CREATE OR REPLACE VIEW en_event_status_change_v AS
SELECT e.event_id,e.event_time,e.event_type_id,e.data_tstamp,e.handled_tstamp,
  esc.status_message,esc.summary,esc.long_message,
         esc.start_time,esc.end_time,esc.log_file
FROM en_event e, en_event_status_change esc, en_event_type et
 WHERE e.event_id    = esc.event_id
   AND e.event_type_id = et.event_type_id
   AND et.name in ('SCHED_REPORT_GROUP_S','SCHED_REPORT_GROUP_F','BUSYHOUR_S','BUSYHOUR_F',
   'SUMMATION_S','SUMMATION_F','PARTMAINT_D_S','PARTMAINT_D_F',
   'PARTMAINT_W_S','PARTMAINT_W_F', 'PARTMAINT_M_S','PARTMAINT_M_F',
   'ANALYZE_S','ANALYZE_F','RGFP_S','RGFP_F',
   'SCENARIO_AD_S','SCENARIO_AD_F','SOFTALARM_S','SOFTALARM_F',
   'SUMMARY_CLEAN_S','SUMMARY_CLEAN_F','AUTOCLEAN_S','AUTOCLEAN_F',
   'BUSYHOUR_CLEAN_S','BUSYHOUR_CLEAN_F','EVENT_CLEAN_S','EVENT_CLEAN_F',
   'EXPECTATION_S','EXPECTATION_F',
   'SOFTALARM_S', 'SOFTALARM_F',
   'AGGREGATOR_S', 'AGGREGATOR_F',
   'SUMMARY_CLEAN_S', 'SUMMARY_CLEAN_F',
   'AUTOCLEAN_S', 'AUTOCLEAN_F',
   'BUSYHOUR_CLEAN_S', 'BUSYHOUR_CLEAN_F',
   'EVENT_CLEAN_S', 'EVENT_CLEAN_F',
   'EXPECTATION_S', 'EXPECTATION_F',
   'SAEXT_PUSH_S', 'SAEXT_PUSH_F');
CREATE GLOBAL TEMPORARY TABLE en_event_unprocessed(event_id      NUMBER,
                                                   event_type_id NUMBER,
                                                   data_tstamp   DATE,
                                                   is_scheduled char(1) DEFAULT 'N' ,
                                                   is_notify    char(1) DEFAULT 'N')
ON COMMIT DELETE ROWS;
exit;
