----------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2009. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
----------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/instance_exclusion/instance_exclusion_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
-- 

-- ***************************
-- Create the table WM_FEATURE
-- ***************************
PROMPT CREATE TABLE WM_FEATURE...
CREATE TABLE WM_FEATURE (
FEATURE_ID NUMBER NOT NULL,
FEATURE_NAME VARCHAR2(100) NOT NULL,
CONSTRAINT "PK_WM_FEATURE" PRIMARY KEY ("FEATURE_ID")
);

-- ***********************************
-- Create the table WM_DEVICE_EXCLUDED
-- ***********************************
PROMPT CREATE TABLE WM_DEVICE_EXCLUDED...
CREATE TABLE WM_DEVICE_EXCLUDED (
DEVICE_EXCLUDED_ID NUMBER(*,0) NOT NULL, 
FEATURE_ID NUMBER(*,0) NOT NULL, 
ENTITY_ID NUMBER(*,0) NOT NULL, 
INSTANCE_ID NUMBER(*,0),
FULL_KEY VARCHAR2(900) NOT NULL,
CREATED_DATE DATE,
CONSTRAINT "PK_DEVICE_EXCLUDED" PRIMARY KEY ("DEVICE_EXCLUDED_ID"),
CONSTRAINT "FK_FEATURE" FOREIGN KEY ("FEATURE_ID") REFERENCES WM_FEATURE ("FEATURE_ID"), 
CONSTRAINT "FK_ENTITY" FOREIGN KEY ("ENTITY_ID") REFERENCES WMN_ENTITY ("ENTITY_ID"), 
CONSTRAINT "FK_INSTANCE" FOREIGN KEY ("INSTANCE_ID") REFERENCES WMN_INSTANCE ("INSTANCE_ID") ON DELETE CASCADE);

PROMPT CREATE INDEX WM_DEVICE_EXCLUDED_INSTANCE...
CREATE INDEX WM_DEVICE_EXCLUDED_INSTANCE ON WM_DEVICE_EXCLUDED(INSTANCE_ID);

PROMPT CREATE INDEX WM_DEVICE_EXCLUDED_ENTITY...
CREATE INDEX WM_DEVICE_EXCLUDED_ENTITY ON WM_DEVICE_EXCLUDED(ENTITY_ID);

-- Added trigger to activate feature
PROMPT CREATE TRIGGER WM_DEVICE_EXCLUDED_INSERT...
CREATE OR REPLACE TRIGGER WM_DEVICE_EXCLUDED_INSERT
AFTER INSERT 
ON WM_DEVICE_EXCLUDED
BEGIN
    wm_sysval.mergesysval(pname  => 'PWebInstanceExclusion',
                          pvalue => 'TRUE'); 
END;
/

-- Added trigger to check for deactivation of feature
PROMPT CREATE TRIGGER WM_DEVICE_EXCLUDED_DELETE...
CREATE OR REPLACE TRIGGER WM_DEVICE_EXCLUDED_DELETE
AFTER DELETE 
ON WM_DEVICE_EXCLUDED
DECLARE
 vCount         PLS_INTEGER:=0;
BEGIN
  SELECT COUNT(*) INTO vCount
    FROM WM_DEVICE_EXCLUDED;

  IF vCount = 0 THEN
    wm_sysval.mergesysval(pname  => 'PWebInstanceExclusion',
                          pvalue => 'FALSE');
  END IF;
 
END;
/


-- ******************************************
-- Create the sequence WM_DEVICE_EXCLUDED_SEQ
-- ******************************************
PROMPT CREATE SEQUENCE WM_DEVICE_EXCLUDED_SEQ...
CREATE SEQUENCE WM_DEVICE_EXCLUDED_SEQ START WITH 1;


-- *************************************
-- Create the view WM_DEVICE_EXCLUDED_VW
-- *************************************
PROMPT CREATE VIEW WM_DEVICE_EXCLUDED_VW...
CREATE OR REPLACE VIEW WM_DEVICE_EXCLUDED_VW
AS
SELECT connect_by_root onion.feature_id feature_id,
       wic.entity_id   entity_id,
       wic.instance_id instance_id
  FROM wmn_instance_context wic,
       (SELECT DISTINCT (wde.instance_id),         --get all instances already defined in list
                         wde.feature_id
          FROM wm_device_excluded wde
         WHERE wde.instance_id IS NOT NULL
        UNION         -- now deal with any 'ALL' entries in list
        SELECT wic.instance_id,
               wde.feature_id
          FROM wmn_instance_context wic,
               wmn_entity           we,
               wmn_context          wc,
               wm_device_excluded   wde
         WHERE upper(wde.full_key) = 'ALL'
           AND we.entity_id = wic.entity_id
           AND we.entity_id = wde.entity_id
           AND wic.context_id = wc.context_id
           AND wc.context_use = 'R'
        UNION -- finally get instances that are in list but do not have an instance id in the list (i.e they may have been added to the system later)
        SELECT wic.instance_id,
               wde.feature_id
          FROM wmn_instance_context wic,
               wmn_entity           we,
               wmn_context          wc,
               wm_device_excluded   wde
         WHERE wic.fullkey = wde.full_key
           AND we.entity_id = wic.entity_id
           AND we.entity_id = wde.entity_id
           AND wic.context_id = wc.context_id
           AND wc.context_use = 'R'
           AND wde.instance_id IS NULL
           AND upper(wde.full_key) != 'ALL') onion
 WHERE wic.context_id =
       (SELECT wc.context_id FROM wmn_context wc WHERE wc.context_use = 'R')
   AND onion.instance_id(+) = wic.instance_id
 START WITH wic.instance_id = onion.instance_id
CONNECT BY PRIOR wic.instance_id = wic.parentinstance_id
       AND wic.context_id = PRIOR wic.context_id;

-- **************************************************
-- Create the materialized view WM_DEVICE_EXCLUDED_REPORTS_MV
-- **************************************************
PROMPT CREATE MATERIALIZED VIEW WM_DEVICE_EXCLUDED_REPORTS_MV...
CREATE MATERIALIZED VIEW WM_DEVICE_EXCLUDED_REPORTS_MV
      REFRESH COMPLETE 
    START WITH SYSDATE 
    NEXT  SYSDATE + 1/48
    WITH PRIMARY KEY 
    AS 
  SELECT DISTINCT(instance_id) FROM wm_device_excluded_vw wdev
  where wdev.feature_id IN (SELECT feature_id from wm_feature where feature_name in ('Reporting', 'All'));

-- **************************************************
-- Create the materialized view WM_DEVICE_EXCLUDED_AVAIL_MV
-- **************************************************
PROMPT CREATE MATERIALIZED VIEW WM_DEVICE_EXCLUDED_AVAIL_MV...
CREATE MATERIALIZED VIEW WM_DEVICE_EXCLUDED_AVAIL_MV
      REFRESH COMPLETE 
    START WITH SYSDATE 
    NEXT  SYSDATE + 1/48
    WITH PRIMARY KEY 
    AS 
  SELECT DISTINCT(instance_id) FROM wm_device_excluded_vw wdev
  where wdev.feature_id IN (SELECT feature_id from wm_feature where feature_name in ('DataAvailability', 'All'));

  
