-------------------------------------------------
---
-- View to simplify retrieving the details of 
-- the PDO configuration
--
-------------------------------------------------
CREATE OR REPLACE VIEW jloader_pdo_config_v AS 
     SELECT config.jloader_table_id,
            version_name,
            record_type, 
            CASE WHEN table_name IS NULL THEN NULL ELSE wml_load.trans_to_temp(table_name) END table_name,
            table_name real_table,
            post_load_proc,
            max_batch_count,
            date_format,
            key_columns,
            post_load_trace,
            dn_rewrite_func,
            dn_print_class
       FROM jloader_pdo_config config, 
            jloader_pdo_record_type rec_type
      WHERE config.record_type_id (+)= rec_type.record_type_id
;

--
-- This view is a complete list of the counters in each counter group.
-- Note that as of Ericsson JLoader Phase II, counter groups can contain
-- additional counters that are not actually mapped to a table/column
-- This means we can now support the ability to manage counters in
-- a counter group that we want to roll up/across to another counter
-- group before it is loaded into the database. Shockingly powerful
-- stuff.
-- It also more acurately models the separation of input counter
-- group and output column map. 

CREATE OR REPLACE VIEW jloader_pdo_counters_v AS
SELECT rt.record_type_id,
       c.counter_id,
       rt.version_name,
       rt.record_type,
       c.counter_name,
       c.accumulator,
       c.reset_test,
       c.bit_size
  FROM jloader_pdo_counters c,
       jloader_pdo_record_type rt
 WHERE c.record_type_id = rt.record_type_id
;


--
-- This view contains everything JLoader needs to map counters onto table columns
-- We can use the view above to simplify the writing of the outer join
--
CREATE OR REPLACE VIEW jloader_colmap_mv AS
SELECT cv.version_name,
       cv.record_type,
       cv.counter_name,
       CASE WHEN tc.table_name IS NULL THEN NULL ELSE wml_load.trans_to_temp(tc.table_name) END table_name,
       tc.table_name real_table,
       tc.column_name,
       tc.data_type,
       tc.data_scale,
       tc.array_index,
       cv.accumulator,
       cv.reset_test,
       cv.bit_size
  FROM jloader_pdo_counters_v cv,
      (SELECT cfg.record_type_id, cfg.counter_id, 
              cfg.table_name, cfg.column_name, NVL(tc.data_type, 'VARCHAR2') data_type, tc.data_scale, cfg.array_index
         FROM (SELECT pc.record_type_id, cm.counter_id, pc.table_name, cm.column_name, cm.array_index
                 FROM jloader_pdo_config pc,
                      jloader_pdo_colmap cm
                WHERE pc.jloader_table_id = cm.jloader_table_id) cfg,
              user_tab_columns tc
        WHERE tc.table_name  (+)= cfg.table_name
          AND tc.column_name (+)= cfg.column_name) tc
 WHERE cv.record_type_id = tc.record_type_id(+)
   AND cv.counter_id = tc.counter_id(+)
;


