---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2006,2009. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/jloader/create_jloader_pdo_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--
-------------------------------------------------
-- File: create_jloader_pdo_schema.sql
--
-- Desc: Creates the portion of the jloader 
-- schema associated with the PDOStreamProcessor
-- implementation. Logical split is made between
-- the core jloader functionality and the PDO
-- specific tables
--
-- Author: ofagan
--         Robin Arnold 
--
-- Since: Sep06, AEMS-C2.1 Scalability
--
-------------------------------------------------


-------------------------------------------------
--
-- Global temporary table which the PDOStreamProcessor
-- loads the ne_list from ProspectBase/vendor/Common/NE/1.3/ne_list
-- into
-- the AE Post load procedures make use of these
-- entries to create the fullkey data during the 
-- loading process
-------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE jloader_ne_list_tmp (
    ne_name VARCHAR2(100) NOT NULL,
    system_id VARCHAR2(20) NOT NULL,
    ne_id   VARCHAR(20) NOT NULL,
    rev     VARCHAR2(20) NOT NULL,
    prefix  VARCHAR2(50) NOT NULL,
  ne_system VARCHAR2(100) NOT NULL,
  ne_type VARCHAR2(100) NOT NULL,
  CONSTRAINT pk_jloader_ne_list_tmp PRIMARY KEY (ne_name)
) 
ON COMMIT PRESERVE ROWS;

-------------------------------------------------
--
-- Table used to store the record types recognised
-- by the Applications loader.
--
-------------------------------------------------
CREATE TABLE jloader_pdo_record_type (
    record_type_id INTEGER NOT NULL,
        version_name   VARCHAR2(64) DEFAULT 'default' NOT NULL,
    record_type    VARCHAR2(128) NOT NULL, -- counter_group
        dn_rewrite_func VARCHAR2(256),
    CONSTRAINT pk_jloader_pdo_record_type PRIMARY KEY (record_type_id),
    CONSTRAINT uq_jloader_pdo_record_type UNIQUE (version_name, record_type)
) 
TABLESPACE wm_flexpm;

-------------------------------------------------
--
-- Stores the configuration for each record type
-- Used by the PDOStreamProcessor to map the record
-- type to a particular temp table, and then
-- call the correct post load procedure to 
-- map this data to the traffic table
--
-- UPGRADE: alter table jloader_pdo_config rename column MOID_REWRITE_CLASS to dn_print_class;
-------------------------------------------------
CREATE TABLE jloader_pdo_config 
(
        jloader_table_id   INTEGER NOT NULL,
    record_type_id     INTEGER NOT NULL,
    table_name         VARCHAR2(30) NOT NULL,
    post_load_proc     VARCHAR2(200),
    max_batch_count    INTEGER NOT NULL,
    date_format        VARCHAR2(50),
    key_columns        VARCHAR2(1000),
    post_load_trace    CHAR(1) CHECK (post_load_trace IN ( 'Y', 'N' )),
        dn_print_class     VARCHAR2(200) NOT NULL,
        CONSTRAINT pk_jloader_pdo_config PRIMARY KEY (jloader_table_id),
    CONSTRAINT fk_jloader_pdo_record_type FOREIGN KEY (record_type_id) REFERENCES jloader_pdo_record_type(record_type_id)
        ON DELETE CASCADE
)
TABLESPACE wm_flexpm;


CREATE SEQUENCE seq_jloader_pdo_record_type
       START WITH 1 
       INCREMENT BY 1 
       MINVALUE 1 
       MAXVALUE 99999999999999999999 
       CACHE 20 
       NOCYCLE
       NOORDER;

CREATE SEQUENCE seq_jloader_table_id
       START WITH 1 
       INCREMENT BY 1 
       MINVALUE 1 
       MAXVALUE 99999999999999999999 
       CACHE 20 
       NOCYCLE
       NOORDER;

CREATE SEQUENCE seq_jloader_pdo_counter_id
       START WITH 1 
       INCREMENT BY 1 
       MINVALUE 1 
       MAXVALUE 99999999999999999999 
       CACHE 20 
       NOCYCLE
       NOORDER;
-- ----------------------------------------------
-- The following table is used to define all the
-- counters that can belong to a counter-group
-- We need this because for Ericsson JLoader
-- phase II, we have counters that are rolled up
-- to other counter groups. And some counters
-- in the input might not even be mapped to a
-- physical table column, but they are still used
-- in rollups or other runtime calculations.
-- ----------------------------------------------
CREATE TABLE jloader_pdo_counters
(
  counter_id     INTEGER NOT NULL,
  record_type_id INTEGER NOT NULL,
  counter_name   VARCHAR2(256) NOT NULL,
  accumulator    CHAR(1) DEFAULT 'N' NOT NULL,
  reset_test     CHAR(1) DEFAULT 'N' NOT NULL,
  bit_size       NUMBER(2),
  CONSTRAINT pk_jloader_pdo_counters PRIMARY KEY (counter_id),
  CONSTRAINT fk_jloader_pdo_counters_rtid FOREIGN KEY (record_type_id) 
  REFERENCES jloader_pdo_record_type (record_type_id) ON DELETE CASCADE
)
TABLESPACE wm_flexpm;

CREATE UNIQUE INDEX idx_jloader_pdo_counters_unq ON jloader_pdo_counters(record_type_id, counter_name)
TABLESPACE wm_flexpm;

-------------------------------------------------
-- Table to support mapping of Oracle column
-- names to counters in a CSV file that includes
-- a header row. Incredibly useful since it
-- protects us against many file format changes
-- since we only pick out data that we know about
-- Note the array_index column. This is used to
-- reference multi-valued counters where we have
-- a single counter that contains a list of values.
-- This array_index allows us to map to individual
-- counters.
-------------------------------------------------
CREATE TABLE jloader_pdo_colmap
(
  jloader_table_id INTEGER      NOT NULL,
  column_name      VARCHAR2(30) NOT NULL,
  counter_id       INTEGER      NOT NULL,
  array_index      INTEGER,
  CONSTRAINT pk_jloader_pdo_colmap PRIMARY KEY (jloader_table_id, column_name),
  CONSTRAINT fk_jloader_pdo_colmap_rtid FOREIGN KEY (jloader_table_id) 
  REFERENCES jloader_pdo_config (jloader_table_id) ON DELETE CASCADE,
  CONSTRAINT fk_jloader_pdo_colmap_ctrid FOREIGN KEY (counter_id) 
  REFERENCES jloader_pdo_counters (counter_id) ON DELETE CASCADE
)
TABLESPACE wm_flexpm;

-- ===================================================================================
--              META-DATA LOADING TABLE DEFINITIONS
-- ===================================================================================

CREATE GLOBAL TEMPORARY TABLE jloader_pdo_record_tmp 
(
  version_name          VARCHAR2(64)    NOT NULL,
  record_type           VARCHAR2(128)   NOT NULL,
  dn_rewrite_func       VARCHAR2(200),
  CONSTRAINT pk_jloader_pdo_record_tmp PRIMARY KEY (version_name, record_type)
)
ON COMMIT PRESERVE ROWS;

-- -----------------------------------------------
-- GTT used to support loading JLOADER compatible
-- configuration information into the WML_LOAD
-- wml_load_tables table. Populate this GTT then
-- call jloader_pdo_config_api.configure_wml_load_from_tmp
--
-- As of JLoader for Ericsson Phase II, we support
-- NULL table names to support counter groups on
-- the input that haven't been mapped to a table.
-- This is needed to support rollups and various
-- other transformations that happen before we hit
-- the database.
-- -----------------------------------------------
CREATE GLOBAL TEMPORARY TABLE jloader_pdo_table_tmp 
(
   version_name         VARCHAR2(64)   NOT NULL,
   table_name           VARCHAR2(30)   NOT NULL,
   entity_name          VARCHAR2(30)   NOT NULL,
   record_type          VARCHAR2(128)  NOT NULL,
   post_load_proc       VARCHAR2(200),
   max_batch_count      INTEGER        NOT NULL,
   date_format          VARCHAR2(50),
   key_columns          VARCHAR2(1000),
   dn_rewrite_func      VARCHAR2(200),
   dn_print_class       VARCHAR2(200),
   --
   -- for wml_load --
   id_column_name       VARCHAR2(30) NOT NULL,
   needs_entity_map     CHAR(1) DEFAULT 'Y' CHECK (needs_entity_map    IN ('Y', 'N')) NOT NULL,
   needs_rg_reap        CHAR(1) DEFAULT 'Y' CHECK (needs_rg_reap       IN ('Y', 'N')) NOT NULL,
   needs_traf           CHAR(1) DEFAULT 'Y' CHECK (needs_traf          IN ('Y', 'N')) NOT NULL,
   needs_truncate       CHAR(1) DEFAULT 'Y' CHECK (needs_truncate      IN ('Y', 'N')) NOT NULL,
   needs_instance       CHAR(1) DEFAULT 'N' CHECK (needs_instance      IN ('Y', 'N')) NOT NULL,
   needs_trace          CHAR(1) DEFAULT 'N' CHECK (needs_trace         IN ('Y', 'N')) NOT NULL,
   needs_change_event   CHAR(1) DEFAULT 'N' CHECK (needs_change_event  IN ('Y', 'N')) NOT NULL,
   needs_last_seen      CHAR(1) DEFAULT 'N' CHECK (needs_last_seen     IN ('Y', 'N')) NOT NULL,
   find_instance_func   VARCHAR2(200),
   instance_lookup_func VARCHAR2(200),
   instance_name_colname VARCHAR2(30),
   CONSTRAINT pk_jloader_pdo_table_tmp PRIMARY KEY (version_name, table_name, record_type)
)
ON COMMIT PRESERVE ROWS;



-- -----------------------------------------------
-- Real table used to support loading JLOADER compatible
-- configuration information into the WML_LOAD
-- wml_load_tables table. Populate this table (e.g
-- via sqlldr) then call 
-- jloader_pdo_config_api.configure_wml_load_from_stage
-- -----------------------------------------------
CREATE TABLE jloader_pdo_table_stage
(
   version_name         VARCHAR2(64)   NOT NULL,
   table_name           VARCHAR2(30)   NOT NULL,
   entity_name          VARCHAR2(30)   NOT NULL,
   record_type          VARCHAR2(128)  NOT NULL,
   post_load_proc       VARCHAR2(200),
   max_batch_count      INTEGER        NOT NULL,
   date_format          VARCHAR2(50),
   key_columns          VARCHAR2(1000),
   dn_rewrite_func      VARCHAR2(200),
   dn_print_class       VARCHAR2(200),
   --
   -- for wml_load --
   id_column_name       VARCHAR2(30) NOT NULL,
   needs_entity_map     CHAR(1) DEFAULT 'Y' CHECK (needs_entity_map    IN ('Y', 'N')) NOT NULL,
   needs_rg_reap        CHAR(1) DEFAULT 'Y' CHECK (needs_rg_reap       IN ('Y', 'N')) NOT NULL,
   needs_traf           CHAR(1) DEFAULT 'Y' CHECK (needs_traf          IN ('Y', 'N')) NOT NULL,
   needs_truncate       CHAR(1) DEFAULT 'Y' CHECK (needs_truncate      IN ('Y', 'N')) NOT NULL,
   needs_instance       CHAR(1) DEFAULT 'N' CHECK (needs_instance      IN ('Y', 'N')) NOT NULL,
   needs_trace          CHAR(1) DEFAULT 'N' CHECK (needs_trace         IN ('Y', 'N')) NOT NULL,
   needs_change_event   CHAR(1) DEFAULT 'N' CHECK (needs_change_event  IN ('Y', 'N')) NOT NULL,
   needs_last_seen      CHAR(1) DEFAULT 'N' CHECK (needs_last_seen     IN ('Y', 'N')) NOT NULL,
   find_instance_func   VARCHAR2(200),
   instance_lookup_func VARCHAR2(200),
   instance_name_colname VARCHAR2(30),
   CONSTRAINT pk_jloader_pdo_table_stage PRIMARY KEY (version_name, table_name, record_type)
)
TABLESPACE wm_flexpm;




-------------------------------------------------
-- Temp table useful for populating the real
-- colmap table with a merge
-------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE jloader_pdo_colmap_tmp
(
  version_name VARCHAR2(64) DEFAULT 'default' NOT NULL,
  group_name   VARCHAR2(128) NOT NULL,
  counter_name VARCHAR2(256) NOT NULL,
  table_name   VARCHAR2(30),
  column_name  VARCHAR2(30),
  array_index  INTEGER,
  accumulator  CHAR(1) DEFAULT 'N' NOT NULL,
  reset_test   CHAR(1) DEFAULT 'N' NOT NULL,
  bit_size     NUMBER(2)
)
ON COMMIT PRESERVE ROWS;

-------------------------------------------------
-- Same thing but allows us to load the meta-data
-- using sqlldr (table has to be permanent).
-------------------------------------------------
CREATE TABLE jloader_pdo_colmap_stage
(
  version_name VARCHAR2(64)  DEFAULT 'default' NOT NULL,
  group_name   VARCHAR2(128) NOT NULL,
  counter_name VARCHAR2(256) NOT NULL,
  table_name   VARCHAR2(30),
  column_name  VARCHAR2(30),
  array_index  INTEGER,
  accumulator  CHAR(1)       DEFAULT 'N' NOT NULL,
  reset_test   CHAR(1)       DEFAULT 'N' NOT NULL,
  bit_size     INTEGER
)
TABLESPACE wm_flexpm;


-- ----------------------------------------------------------------------------------------------
-- This table is a home-made materialized view (doing a real materialized view is a little tricky
-- because the view uses Oracle data-dictionary tables...and that causes access issues)
-- So this table is just rebuilt with a SELECT * FROM jloader_colmap_mv every time the meta-data
-- is updated. It provides significantly faster access for jloader, being able to read an entire
-- column mapping for one version in around a second (well, on decent hardware :-).
-- We can't use an IOT in this case because there's not a good PK (table_name and column_name
-- can be null). So for a cool little performance tweak, we use hash partitioning to reduce
-- the amount of data we need to scan when asking "WHEN version_name = ?". In fact, this is ideal
-- because we'll end up with a fast full scan of the partition, so as long as we don't get too
-- many hash collisions, it doesn't really get much better
-- ----------------------------------------------------------------------------------------------
CREATE TABLE jloader_colmap_tv
(
  version_name      VARCHAR2(64) NOT NULL,
  record_type       VARCHAR2(64) NOT NULL,
  counter_name      VARCHAR2(256) NOT NULL,
  table_name        VARCHAR2(30),
  real_table        VARCHAR2(30),
  column_name       VARCHAR2(30),
  data_type         VARCHAR2(106),
  data_scale        NUMBER,
  array_index       INTEGER,
  accumulator       CHAR(1) NOT NULL,
  reset_test        CHAR(1) NOT NULL,
  bit_size          NUMBER(2)
)
PARTITION BY HASH(version_name)
PARTITIONS 11
TABLESPACE wm_flexpm;

