--
-- (#) ./base/db/Oracle8i/rg/create_rg_schedule_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--


-- SEAde13550:
--   Table, view and triggers to allow generalized scheduling of report groups.
-- History
-------------------------------
-- 2001.04.20 - Kevin Wagner - Authored.
--

PROMPT Creating table rg_schedule...
CREATE TABLE rg_schedule (
    rg_schedule_Id     INTEGER       NOT NULL,
    FolderId           INTEGER       NOT NULL,
    UserId             INTEGER       NOT NULL,
    Name               VARCHAR2(64)  NOT NULL,
    Description        VARCHAR2(255),
    Changed            DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    ClientMachineId    INTEGER       NOT NULL, -- sub-object id
    Group_id           INTEGER       NOT NULL, -- folder document id
    OvrScenarioId      INTEGER,                -- folder document id
    OvrChanSetId       INTEGER,                -- folder document id
    OvrDestinationId   INTEGER,                -- sub-object id
    OvrDTScopeId       INTEGER,                -- sub-object id
    RunMode            CHAR(1)       DEFAULT 'I' NOT NULL,
      CONSTRAINT rg_schedule_RunMode_ck CHECK (RunMode IN ('I','S')),
    RetrieveMode       CHAR(1)       DEFAULT 'D' NOT NULL,
      CONSTRAINT rg_schedule_RetrieveMode_ck CHECK (RetrieveMode IN ('I','D')),
    AutoPurge          CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT rg_schedule_AutoPurge_ck CHECK (AutoPurge IN ('Y','N')),
    PurgeInterval      INTEGER       DEFAULT 0 NOT NULL, -- 0 means immediate purge
    SubFolderName      VARCHAR2(255),
    LastRunDate        DATE,         -- NULL until set by server
    auto_organize      VARCHAR2(256),-- report result auto-organize string
    pe_resultfolder    INTEGER,
    CONSTRAINT FK_rg_Schd_ClientMachineIdx
               FOREIGN KEY(ClientMachineId)     REFERENCES PM_ClientMachine(Id),
    CONSTRAINT FK_rg_Schd_OvrDestIdx
               FOREIGN KEY(OvrDestinationId)    REFERENCES PM_Destination(Id),
    CONSTRAINT FK_rg_Schd_OvrDtScopeIdx
               FOREIGN KEY(OvrDTScopeId)        REFERENCES PM_DTScope(Id),
    CONSTRAINT FK_rg_Schd_FolderIdx
               FOREIGN KEY(FolderId)            REFERENCES PM_Folder(Id),
    CONSTRAINT FK_rg_Schd_UserIdx
               FOREIGN KEY(UserId)              REFERENCES PM_User(Id),
    CONSTRAINT rg_Schedule_pkx PRIMARY KEY(rg_Schedule_Id),
    CONSTRAINT FK_rg_Schd_resultfolder
               FOREIGN KEY(pe_resultfolder) REFERENCES PM_Folder(Id)
)
TABLESPACE WM_Client;

PROMPT Creating unique index rg_Schedule_unq...
CREATE UNIQUE INDEX rg_Schedule_unq ON rg_Schedule(FolderId, UPPER(Name))
TABLESPACE WM_Client;

PROMPT Creating trigger rg_schedule_del_tr...
-- SEAde15088: Client not deleting rows from PM_DTSCOPE.
-- SEAde43750: User can not delete a report from Prospect Client
CREATE OR REPLACE TRIGGER rg_schedule_del_tr BEFORE DELETE ON rg_schedule FOR EACH ROW
DECLARE
   constraint_error EXCEPTION;
   PRAGMA EXCEPTION_INIT(constraint_error, -02292);
BEGIN
  DELETE FROM PM_DTScope WHERE ID = :OLD.OvrDTScopeId;

  EXCEPTION
     WHEN constraint_error THEN
        -- Silently allow the delete to fail. It must be in use by PM_REPORT
        NULL;
END;
/
SHOW ERRORS TRIGGER rg_schedule_del_tr

PROMPT Creating rg_schedule_view and triggers...
exec gen_scheduler_view.write_code( 'RG_SCHEDULE', 1 );

EXIT

-- Done.
