---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2003,2009. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/rg/rg_util_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--

-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE TABLE rg_dtscope
(
  task_id          INTEGER NOT NULL,
  tstamp           DATE    NOT NULL,
  tstamp_id        INTEGER NOT NULL,
  CONSTRAINT pk_rg_dtscope PRIMARY KEY (task_id, tstamp, tstamp_id)
)
  ORGANIZATION INDEX 
  NOLOGGING
  PCTFREE 0
  TABLESPACE TRAFFIC_JUMBO;

-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_dtscope_tmp
(
  tstamp           DATE    NOT NULL,
  tstamp_id        INTEGER NOT NULL,
  CONSTRAINT pk_rg_dtscope_tmp PRIMARY KEY (tstamp, tstamp_id)
)
  ON COMMIT PRESERVE ROWS;

-- ----------------------------------------------------------------------
-- table containing grouped timestamps allowing construction of group by
-- statements in repgen packages to effect the aggregation of data in
-- the database
-- -----------------------------------------------------------------------
CREATE TABLE rg_trunc
(
  task_id           INTEGER   NOT NULL,
  tstamp_id         INTEGER   NOT NULL,
  tstamp            DATE      NOT NULL,
  CONSTRAINT pk_rg_trunc PRIMARY KEY (task_id, tstamp_id)
)
  ORGANIZATION INDEX
  NOLOGGING
PCTFREE 0
TABLESPACE TRAFFIC_JUMBO;

-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_trunc_tmp
(
  tstamp_id         INTEGER   NOT NULL,
  tstamp            DATE      NOT NULL,
  CONSTRAINT pk_rg_trunc_tmp PRIMARY KEY (tstamp_id)
)
  ON COMMIT PRESERVE ROWS;

-- -----------------------------------------------------------------------
-- TSTAMP RANGE TABLE
--  for efficient BETWEEN queries.
-- -----------------------------------------------------------------------
CREATE TABLE rg_tstamp_range
(
  task_id           INTEGER   NOT NULL,
  min_tstamp        DATE NOT NULL,
  max_tstamp        DATE NOT NULL,
  CONSTRAINT pk_rg_tstamp_range PRIMARY KEY(task_id, min_tstamp)
)
  ORGANIZATION INDEX
  NOLOGGING
PCTFREE 0
TABLESPACE TRAFFIC_JUMBO;  


CREATE GLOBAL TEMPORARY TABLE rg_tstamp_range_tmp
(
  min_tstamp        DATE      NOT NULL,
  max_tstamp        DATE      NOT NULL
)
 ON COMMIT PRESERVE ROWS;

-- -----------------------------------------------------------------------
--
-- -----------------------------------------------------------------------
CREATE TABLE rg_scenario
(
  task_id           INTEGER       not null,
  entity_id         INTEGER       not null,
  instance_id       INTEGER       not null,
  parentinstance_id INTEGER       not null,
  localkey          VARCHAR2(90),
  CONSTRAINT pk_rg_scenario PRIMARY KEY (task_id, entity_id, instance_id)
)
  ORGANIZATION INDEX
  NOLOGGING
  PCTFREE 0
  TABLESPACE TRAFFIC_JUMBO;

-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_scenario_tmp
(
  entity_id          INTEGER       not null,
  instance_id        INTEGER       not null,
  parentinstance_id  INTEGER,
  localkey           VARCHAR2(90),
  CONSTRAINT pk_rg_scenario_tmp PRIMARY KEY (entity_id, instance_id)
)
  ON COMMIT PRESERVE ROWS;

-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE TABLE rg_scenario_remap
(
  task_id              INTEGER   NOT NULL,
  entity_id            INTEGER   NOT NULL,
  instance_id          INTEGER   NOT NULL,
  starttime            DATE      NOT NULL,
  endtime              DATE      NOT NULL,
  synthetic_id         INTEGER   NOT NULL,
  parent_id            INTEGER,
  CONSTRAINT pk_rg_scenario_remap PRIMARY KEY( 
      task_id, entity_id, instance_id, starttime )  
)
  ORGANIZATION INDEX
  NOLOGGING
  PCTFREE 0
  TABLESPACE TRAFFIC_JUMBO;

-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_scenario_remap_tmp
(
  entity_id            INTEGER   NOT NULL,
  instance_id          INTEGER   NOT NULL,
  starttime            DATE      NOT NULL,
  endtime              DATE      NOT NULL,
  synthetic_id         INTEGER   NOT NULL,
  parent_id            INTEGER,
  CONSTRAINT pk_rg_scenario_remap_tmp PRIMARY KEY( 
  entity_id, instance_id, starttime )
)
  ON COMMIT PRESERVE ROWS;


-- -----------------------------------------------------------------------
-- rg_busyhour_tree_mv replaced by rg_instance_tree_mv
-- and rg_busyhour_mv as a performance boost for
-- busyhour reporting and stored busyhour generation
-- -----------------------------------------------------------------------



-- -----------------------------------------------------------------------
-- call wmda_dictionary_schema.parse_scenario
-- fill rg_stamp_tmp
-- call rg_util.expandBusyHour(taskId, busyHourId)
-- -----------------------------------------------------------------------
-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE TABLE rg_instance_tree_mv
(
  task_id           INTEGER   NOT NULL,
  entity_id         INTEGER   NOT NULL,
  instance_id       INTEGER   NOT NULL,
  parentinstance_id INTEGER   NOT NULL,
  bhinstance_id     INTEGER   NOT NULL,
  localkey          VARCHAR2(90),
  CONSTRAINT pk_rg_instance_tree_mv PRIMARY KEY
  (task_id, entity_id, instance_id, bhinstance_id)
)
  ORGANIZATION INDEX
  NOLOGGING
PCTFREE 0
TABLESPACE TRAFFIC_JUMBO;


-- -----------------------------------------------------------------------
-- call wmda_dictionary_schema.parse_scenario
-- fill rg_tstamp_tmp
-- call rg_util.expandBusyHour(taskId, busyHourId)
-- -----------------------------------------------------------------------
CREATE TABLE rg_busyhour_mv
(
  task_id           INTEGER   NOT NULL,
  instance_id       INTEGER   NOT NULL,
  tstamp_id         INTEGER   NOT NULL,
  tstamp            DATE      NOT NULL,
  value             FLOAT,
  CONSTRAINT pk_rg_busyhour_mv PRIMARY KEY
  (task_id, instance_id, tstamp_id, tstamp)
)
  ORGANIZATION INDEX
  NOLOGGING
PCTFREE 0
TABLESPACE TRAFFIC_JUMBO;

-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------

CREATE GLOBAL TEMPORARY TABLE rg_busyhour_mv_stg
(
  instance_id       INTEGER   NOT NULL,
  tstamp_id         INTEGER   NOT NULL,
  tstamp            DATE      NOT NULL,
  value             FLOAT
)
ON COMMIT DELETE ROWS;  

-- -----------------------------------------------------------------------
--  Tables below are helper tables for particular routines,
--  Do not depend on the values.
-- -----------------------------------------------------------------------

-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_busyhour_tmp
(
  instance_id INTEGER       NOT NULL,
  tstamp      DATE          NOT NULL,
  rank        INTEGER,
  VALUE       FLOAT,
  CONSTRAINT pk_rg_busyhour_tmp PRIMARY KEY (instance_id, tstamp, rank)
)
  ON COMMIT PRESERVE ROWS;

-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_bhfilter_tmp
(
  instance_id INTEGER       NOT NULL,
  tstamp      DATE          NOT NULL,
  rank        INTEGER,
  VALUE       FLOAT,
  CONSTRAINT pk_rg_bhfilter_tmp PRIMARY KEY (instance_id, tstamp, rank)
)
  ON COMMIT DELETE ROWS;


-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_tstamp_tmp(
  tstamp     DATE    NOT NULL,
  CONSTRAINT pk_rg_tstamp_tmp PRIMARY KEY (tstamp)
)
  ON COMMIT DELETE ROWS;

-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_calendardef_tmp(
  calendarDate     DATE,
  firstSeconds     INTEGER NOT NULL,
  lastSeconds      INTEGER NOT NULL
)
  ON COMMIT DELETE ROWS;

-- -----------------------------------------------------------------------
-- preloaded table of frequency_ids to tell resolve_dtscope what kind of
-- real timestamps to include in rg_dtscope
-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_frequency_tmp(
  frequency_id     INTEGER NOT NULL
)
  ON COMMIT PRESERVE ROWS;

-- -----------------------------------------------------------------------
-- useful for some really clever cartesian product calculations to get
-- mixes of timestamps
-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_timeoffset_tmp(
  seconds     INTEGER NOT NULL
)
  ON COMMIT PRESERVE ROWS;


-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_busyhour_tree_tmp
(
  instance_id           integer not null,
  bhinstance_id         integer not null,
  entity_id             INTEGER NOT NULL,
  parentinstance_id     INTEGER NOT NULL,
  CONSTRAINT pk_rg_busyhour_tree_tmp PRIMARY KEY (instance_id, bhinstance_id)
)
  ON COMMIT DELETE ROWS;


-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE rg_dtscopedef_tmp(
  scopeType        CHAR(1) NOT NULL,
  timeUnit         CHAR(1) NOT NULL,
  howManyUnits     INTEGER NOT NULL,
  includeCurrent   CHAR(1) NOT NULL,
  sameHoursEachDay CHAR(1) NOT NULL,
  includeHolidays  CHAR(1) NOT NULL,
  holidaysOnly     CHAR(1) NOT NULL,
  includeWeekends  CHAR(1) NOT NULL,
  weekendsOnly     CHAR(1) NOT NULL,
  dtPeriod         INTEGER NOT NULL,
  extendRange      CHAR(1) NOT NULL,
  quickRange       CHAR(1) NOT NULL,
  sundays          CHAR(1),
  mondays          CHAR(1),
  tuesdays         CHAR(1),
  wednesdays       CHAR(1),
  thursdays        CHAR(1),
  fridays          CHAR(1),
  saturdays        CHAR(1),
  tzregion         VARCHAR2(64) default NULL
)
  ON COMMIT DELETE ROWS;


-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------
CREATE OR REPLACE VIEW rg_busyhour_tree_v AS
  SELECT /*+ USE_HASH(tree, bh, scenario) */
bh.summary_id,
scenario.entity_id,
tree.instance_id,
tree.parentinstance_id,
bh.tstamp,
bh.VALUE,
scenario.localkey
FROM rg_busyhour_tree_tmp  tree,
wmda_scenario_instance_tmp scenario,
rg_stored_dbh              bh
WHERE tree.instance_id = scenario.instance_id
AND tree.bhinstance_id = bh.instance_id;



-- -----------------------------------------------------------------------
-- These busyhour generation tables need to be built in rg because they
-- are used in RG_UTIL procedures
-- -----------------------------------------------------------------------
CREATE TABLE BH_ReportSpec(
    Id            INTEGER, -- taskId
    BusyHourId    INTEGER       NOT NULL,    -- FK to busy hour determiner defn
    ScenarioId    INTEGER,
    BusyHourType  VARCHAR2(16),
    CONSTRAINT pk_BH_ReportSpec PRIMARY KEY (Id),
    CONSTRAINT fk_BusyHourId    FOREIGN KEY (BusyHourId)
               REFERENCES PM_BusyHour (Id) ON DELETE CASCADE,
    CONSTRAINT fk_BH_ReportSpec FOREIGN KEY (Id)
               REFERENCES active_task (task_id) ON DELETE CASCADE
)
tablespace wm_flexpm;

CREATE TABLE BH_Calendar(
    ReportSpecId     INTEGER       NOT NULL,
    FirstHour        INTEGER       NOT NULL,
    LastHour         INTEGER       NOT NULL,
    CalendarDate     DATE          NOT NULL,
    CONSTRAINT pk_BH_Calendar PRIMARY KEY
               (ReportSpecId, CalendarDate, FirstHour, LastHour),
    CONSTRAINT fk_BH_Calendar FOREIGN KEY (ReportSpecId)
        REFERENCES BH_ReportSpec(Id) ON DELETE CASCADE
) ORGANIZATION INDEX
TABLESPACE wm_flexpm;

-- ----------------------------------------------------------------------
-- static list of all the sub-hourly intervals needed to fill an hour
-- with each particular frequency of data. Used to help populate 
-- rg_timeoffset_tmp
-- -----------------------------------------------------------------------
CREATE TABLE rg_timeoffset
(
  frequency_id      INTEGER   NOT NULL,
  seconds           INTEGER   NOT NULL
)
TABLESPACE WM_FLEXPM;


-- -----------------------------------------------------------------------
-- populate rg_timeoffset with its static data
-- -----------------------------------------------------------------------
DECLARE
   CURSOR FREQUENCIES IS SELECT * FROM WM_FREQUENCY;
   f WM_FREQUENCY%RowType;
   t INTEGER;
BEGIN
   DELETE FROM rg_timeoffset;
   
   FOR f IN FREQUENCIES
   LOOP
      t := 0;
      WHILE (t < 3600)
      LOOP
     INSERT INTO rg_timeoffset(frequency_id, seconds) VALUES(f.frequency_id, t);
     t := t + f.seconds;
      END LOOP;
   END LOOP;
   
   COMMIT;
END;
/

CREATE OR REPLACE TYPE rg_tstamp_rowtype AS OBJECT( tstamp DATE );
/

CREATE OR REPLACE TYPE rg_tstamp_tabletype AS TABLE OF rg_tstamp_rowtype;
/



-- ----------------------------------------------------------------------------
--
-- Local Variables:
-- mode: pls
-- End:
