---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2006,2010. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/rg_trending/rg_trending_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--
--
-- Author: ofagan
--
-- Date: 09Mar06
--
-- Desc: Schema components for support of the Classic (C++)
-- Repgen 
-- 
--- 

-- -----------------------------------------------------------------------
-- 
-- -----------------------------------------------------------------------

CREATE SEQUENCE seq_rg_trending_c_fingerprint
	   START WITH 1 
	   INCREMENT BY 1 
	   MINVALUE 1 
	   MAXVALUE 99999999999999999999 
	   CACHE 20 
	   NOCYCLE
	   NOORDER;
	   
-- table used to store the value of the UDC at the busy
-- hour timestamp. This is populated with from the inner report
-- and then the oracle linear regression functions are run
-- over it grouped by task_id, regression_id and instance_id
-- to derive the slope and intercept.
-- Cannot be a temp table as it crosses a transaction boundary
-- from the outer to the inner report

CREATE TABLE rg_trending_c_bhvalue
(
	task_id			  INTEGER,
	fingerprint_id INTEGER,
	instance_id   NUMBER(38),
	utc_timestamp NUMBER(38),
	bh_value      FLOAT,
	CONSTRAINT pk_rg_trending_c_bhvalue PRIMARY KEY
  (task_id, fingerprint_id, instance_id, utc_timestamp)
) 
	ORGANIZATION INDEX NOLOGGING PCTFREE 0 
	TABLESPACE TRAFFIC_JUMBO;


CREATE TABLE rg_trending_c_fingerprint
(
	fingerprint_id INTEGER,
	task_id			  INTEGER,
	regression_id VARCHAR(200),
	CONSTRAINT pk_rg_trending_c_fingerprint PRIMARY KEY 
	(fingerprint_id)
)
	ORGANIZATION INDEX NOLOGGING PCTFREE 0 
	TABLESPACE TRAFFIC_JUMBO;
	

-- store the slope and intercept for classic reports
-- this is used to store the output of each report run
-- which then can be checked and the slope and intercept 
-- data reused. Since the reports are SBH reports the busy hour
-- is only updated once a day.
create table rg_trending_c_data_store 
	(
		task_id			   INTEGER,
		scenario_id		 INTEGER,
		reportdate		 DATE,
		fingerprint_id INTEGER,
		instance_id		 INTEGER,
		slope					 FLOAT,
		intercept		   FLOAT,
    nrows          INTEGER,
		CONSTRAINT pk_rg_trending_c_data_store PRIMARY KEY
	  (scenario_id, reportdate, fingerprint_id, instance_id)
 ) 
	ORGANIZATION INDEX PCTFREE 10 
	TABLESPACE TRAFFIC_JUMBO;
	
	

-- Table used to store regression values calculated in the java pipeline.
-- Only exists for the lifetime of the report
CREATE GLOBAL TEMPORARY TABLE rg_regression_tmp(
     regression_id    VARCHAR2(200) NOT NULL,
     tstamp           DATE NOT NULL,
     instance_id      INTEGER,
     slope            FLOAT,
     intercept        FLOAT,
     FULLKEY          VARCHAR2(900)
)
ON COMMIT PRESERVE ROWS;


-- Table used to stored the result of a trending UDC's SBH report in the java 
-- pipeline. Only exists for the lifetime of the report.
CREATE GLOBAL TEMPORARY TABLE RG_REGRESSION_DATA_TMP
(
  FULLKEY VARCHAR2(900),
  TSTAMP  DATE NOT NULL,
  VALUE   FLOAT
)
ON COMMIT PRESERVE ROWS;

	
-- views useful for debugging classic trending reports 

CREATE OR REPLACE VIEW rg_trending_c_bhvalue_v AS
   SELECT bhv.task_id, bhv.fingerprint_id, f.regression_id,
   		 bhv.instance_id, bhv.bh_value, bhv.utc_timestamp			       
   FROM rg_trending_c_bhvalue bhv, rg_trending_c_fingerprint f
   WHERE bhv.fingerprint_id = f.fingerprint_id;

 
CREATE OR REPLACE VIEW rg_trending_c_store_v AS
   SELECT s.task_id, s.scenario_id, s.reportdate, s.fingerprint_id, f.regression_id,
   			  s.instance_id, s.slope, s.intercept, s.nrows
   FROM rg_trending_c_data_store s, rg_trending_c_fingerprint f
   WHERE s.fingerprint_id = f.fingerprint_id
   ORDER by s.task_id DESC;
  
  
 
