CREATE OR REPLACE TRIGGER schedule_trigger
  BEFORE INSERT OR DELETE OR UPDATE OF start_time,next_time,period,sched_time,tzregion ON schedule
  FOR EACH ROW
WHEN ((new.needs_msg_update IS NULL OR new.needs_msg_update = 'Y') AND (new.period IS NULL OR new.period != 'N'))
DECLARE
  ns               schedule%rowtype;
  invalid_schedule EXCEPTION;
  undef_trigger    EXCEPTION;
   PRAGMA EXCEPTION_INIT(undef_trigger, -20001);
BEGIN
   IF( (INSERTING OR UPDATING) AND ( :new.sched_time IS NULL ) ) THEN
     :new.sched_time := :new.start_time;
   END IF;

   IF ( INSERTING OR 
     (UPDATING AND (:new.is_suspended = :old.is_suspended)) ) THEN
      ns.schedule_id        := :new.schedule_id;
      ns.job_type           := :new.job_type;
      ns.start_time         := :new.start_time;
      ns.end_time           := :new.end_time;
      ns.next_time          := :new.next_time;
      ns.which_month_date   := :new.which_month_date;
      ns.which_week         := :new.which_week; 
      ns.which_weekday      := :new.which_weekday;
      ns.period             := :new.period;
      ns.frequency          := :new.frequency;
      ns.admin_task         := :new.admin_task;
      ns.sched_fk           := :new.sched_fk;
      ns.inc_mondays        := :new.inc_mondays;
      ns.inc_tuesdays       := :new.inc_tuesdays;
      ns.inc_wednesdays     := :new.inc_wednesdays;
      ns.inc_thursdays      := :new.inc_thursdays;
      ns.inc_fridays        := :new.inc_fridays;
      ns.inc_saturdays      := :new.inc_saturdays;
      ns.inc_sundays        := :new.inc_sundays;
      ns.inc_holidays       := :new.inc_holidays;
      ns.inc_day_after_hol  := :new.inc_day_after_hol;
      ns.needs_msg_update   := :new.needs_msg_update;
      ns.sched_time         := :new.sched_time;
      ns.tzregion           := :new.tzregion;

      ns.next_time := ns.start_time;
      
      -- do the reschedule logic      
      ns.next_time := sched.reschedule_in_trigger(ns);
      
      :new.next_time := ns.next_time;
      :new.needs_msg_update := 'N';
            
      IF ns.next_time IS NULL THEN      
	 NULL;
      ELSIF (INSERTING) THEN
	 SCHED.SCHEDULE_INSERT(ns);
      ELSIF (UPDATING) THEN
	 SCHED.SCHEDULE_UPDATE(ns);
      END IF;
   ELSIF (DELETING) THEN
      SCHED.SCHEDULE_DELETE(:old.schedule_id);
   END IF;
   
END;
/
show errors;

CREATE OR REPLACE TRIGGER schedule_post_trigger
  AFTER INSERT OR UPDATE OF next_time ON schedule
  FOR EACH ROW
WHEN  (new.next_time IS NULL AND new.period != 'N')
DECLARE
BEGIN
   IF (inserting OR 
     (updating AND (:new.is_suspended = :old.is_suspended)) ) THEN
      raise_application_error(-20113, 'Invalid schedule.  Schedule will never run.');
   END IF;
END;
/
show errors;  
-- ------------------------------------------------------------------------
-- Local Variables:
-- mode: pls
-- End:

