---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2001, 2008. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/softalarm/create_soft_alarm_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--
--

-- This sequence moved from the R23 file "createQueues.sql"
-- Create the sequence we use to enqueue messages
CREATE SEQUENCE SA_ENQUEUE_SEQ
  MINVALUE 1
  MAXVALUE 2147483647
  START WITH 1;


-- SA_ALARM_TYPE
--

CREATE TABLE SA_ALARM_TYPE (
  	Id   			Integer  		NOT NULL,
  	Description    	Varchar2(255)  	NOT NULL,
	--
	-- Primary Key
	--
	CONSTRAINT PK_SA_ALARM_TYPE PRIMARY KEY (Id)
);

--
-- SA_ALARM_CAUSE
--
CREATE TABLE SA_ALARM_CAUSE (
	ID				Integer			NOT NULL,
	Alarm_Type_Id		Integer			NOT NULL,
	Description		Varchar2(255)	NOT NULL,
	--
	-- Foreign Key
	--
	CONSTRAINT FK_Alarm_Type_Id FOREIGN KEY (Alarm_Type_Id) References SA_ALARM_TYPE (Id),
	--
	-- Primary Key
	--
	CONSTRAINT PK_SA_ALARM_CAUSE  PRIMARY KEY (ID)
);

--
-- SA_MESSAGE_TOKEN
--
CREATE TABLE SA_MESSAGE_TOKEN (
	Id				Integer			NOT NULL,
	Token			Varchar2(30)	NOT NULL,
	Description		Varchar2(255)	NOT NULL,
	--
	-- Primary Key
	--
	CONSTRAINT PK_SA_MESSAGE_TOKEN PRIMARY KEY (Id)
);


--
-- SA_AGGREGATOR
--
-- Class - (D)efault, (E)xternal
--
CREATE TABLE SA_AGGREGATOR (
	Id		Integer		NOT NULL,
	Class 		CHAR(1) DEFAULT 'D' NOT NULL,
	Name		Varchar2(30)	NOT NULL,
	Description	Varchar2(255)	NOT NULL,
	--
	-- Primary Key
	--
	CONSTRAINT PK_SA_AGGREGATOR PRIMARY KEY (Id),
	CONSTRAINT saext_ag_class_chk CHECK (class IN ('D','E'))
);


--
-- SA_SEVERITY  
--
CREATE TABLE SA_SEVERITY (
	Id				Integer			NOT NULL,
	Description		Varchar2(255)	NOT NULL,
	--
	-- Primary Key
	--
	CONSTRAINT PK_SA_SEVERITY PRIMARY KEY (Id)
);


--
-- SA_TEMPLATE
--
CREATE TABLE SA_TEMPLATE (
        Id              Integer         NOT NULL,
        UserId          Integer         NOT NULL,
        FolderId        Integer         NOT NULL,
        Name            Varchar2(64)    NOT NULL,
        Description     Varchar2(255),
        CauseId         Integer         NOT NULL,
        Alarm_Type_Id   Integer         NOT NULL,
        Category        Varchar2(64)    NOT NULL,
        Type            Varchar2(64)    NOT NULL,
        -- Category ,Type column kept for Client GUI requirement.
        Summary         VARCHAR2(128)   NOT NULL,
        Message         Varchar2(2000)  NOT NULL,
        Changed         Date            Default Trunc(Sysdate,'MI'),
--
-- Foreign Key
--
        CONSTRAINT FK_TMPL_User_Id  Foreign Key (UserId) References PM_USER (Id),
        CONSTRAINT FK_TMPL_Folder_Id Foreign Key (FolderID) References PM_FOLDER (Id),
        CONSTRAINT FK_TMPL_Cause_Id Foreign Key (CauseId) References SA_ALARM_CAUSE (Id),
--
-- Unique key
--
        CONSTRAINT uni_tmpl_sa_def_key unique (Alarm_Type_Id, CauseId, Summary),
--
-- Primary Key
--
        CONSTRAINT PK_SA_TEMPLATE PRIMARY KEY (Id)
);

--
-- index required to prevent duplicate documents in folder
--
create unique index uni_SA_template_name on sa_template(FolderId, UPPER(Name), Category);

--
-- sequence for SA_TEMPLATE.Template_Id
--
CREATE SEQUENCE SQ_SA_TEMPLATE
		START WITH 1000
		INCREMENT BY 1
		NOMAXVALUE
		NOCYCLE
;

--
-- SA_TIME_RANGE
--
CREATE TABLE SA_TIME_RANGE (
	Id	Integer		NOT NULL,
	Class   CHAR(1) 	DEFAULT 'D' NOT NULL,
	Description		Varchar2(255)	NOT NULL,
 	Time_Range_Key		Varchar2(255)	NOT NULL,	
	--
	-- Primary Key
	--
	CONSTRAINT PK_SA_TIME_RANGE PRIMARY KEY (Id),
	CONSTRAINT saext_tr_class_chk CHECK (class IN ('D','E'))
);


--
-- SOFTALARM
--
CREATE TABLE SA_SOFTALARM (
	Id	Integer			NOT NULL,
	ScenarioId		Integer			NOT NULL,
	UserId			Integer			NOT NULL,
	FolderId		Integer			NOT NULL,
	Entity_Id		Integer			NOT NULL,
	Name			Varchar2(64)	NOT NULL,
	Description		Varchar2(255)	,
	Category		Varchar2(64) 	NOT NULL,
	Type			Varchar2(64)	NOT NULL,
	-- Category , Type column kept for Client GUI requirement.
	SATemplateId	Integer			NOT NULL,
	Time_Range_Id	Integer,
	Time_Range_Value	Integer		NOT NULL,
	Exclude_WeekEnds CHAR(1)	Default 'Y' NOT NULL,
          CONSTRAINT SA_SOFTALARM_Exclude_WeekEn_ck CHECK (Exclude_WeekEnds IN ('Y','N')),
	Exclude_Holidays CHAR(1)	Default 'Y' NOT NULL,
          CONSTRAINT SA_SOFTALARM_Exclude_Holida_ck CHECK (Exclude_Holidays IN ('Y','N')),
        Active			CHAR(1)		Default 'Y' NOT NULL,
          CONSTRAINT SA_SOFTALARM_Active_ck CHECK (Active IN ('Y','N')),
        Changed			Date	Default Trunc(Sysdate,'MI'),
        Auto_Clear            CHAR(1)    DEFAULT 'Y',
        ALARM_ID               NUMBER(*,0) NULL,
          CONSTRAINT SA_SOFTALARM_Auto_Clear_ck CHECK (Auto_Clear IN ('Y','N')),
	--
	-- Foreign Key
	--
	CONSTRAINT FK_SA_Scn_Id Foreign Key (ScenarioId) References PM_SCENARIO (ID),
	CONSTRAINT FK_SA_User_ID Foreign Key (UserId) References PM_USER (ID),
	CONSTRAINT FK_SA_Folder_Id Foreign Key (FolderId) References PM_FOLDER (Id),
	CONSTRAINT FK_SA_Entity_ID Foreign Key (Entity_Id) References WMN_ENTITY (Entity_ID),
	CONSTRAINT FK_SA_Tmpl_Id Foreign Key (SATemplateId) References SA_TEMPLATE (Id),
	CONSTRAINT FK_SA_Time_Range_Id Foreign Key  (Time_Range_Id) References SA_Time_Range (Id),
	--
	-- Primary Key
	--
	CONSTRAINT PK_SA_Soft_Alarm PRIMARY KEY (Id)
);

CREATE UNIQUE INDEX SA_ALARM_ID_UNQ ON SA_SOFTALARM (ALARM_ID); 

--
-- index required to prevent duplicate documents in folder
--
create unique index uni_SA_name on sa_softalarm(FolderId, UPPER(Name), Category);

--
-- sequence for SA_SOFTALARM.SoftAlarm_Id
--
-- Sequence is created in create_client_schema.sql
CREATE SEQUENCE SQ_SOFTALARM
        START WITH 1000
        INCREMENT BY 1
        NOMAXVALUE
        NOCYCLE
;

--
-- SA_LOGICAL_OPERATOR
--
CREATE TABLE SA_LOGICAL_OPERATOR
(
        id         integer
          constraint Pk_SA_OPERATOR primary key,
        operator   varchar2(30) not null,
        parent_id  integer
          constraint FK_SA_OPR_PARENT references sa_logical_operator(id),
        sa_id      integer not null
          constraint FK_SA_OPR_SA_ID references sa_softalarm(id) on delete cascade
)
;

--
-- sequence for SA_LOGICAL_OPERATOR.Id
--
CREATE SEQUENCE SQ_SA_LOGICAL_OPERATOR
        START WITH 1000
        INCREMENT BY 1
        NOMAXVALUE
        NOCYCLE
;

create global temporary table SA_LOGOPR_TMP
(
  source_id         integer,
  destination_id    integer
)
on commit delete rows
;

--
-- SA_THRESHOLD
--
CREATE TABLE SA_THRESHOLD (
	Id	Integer			NOT NULL,
        Ordinal          INTEGER       NOT NULL,
        Ordinal_Group    INTEGER       NOT NULL,
	SoftAlarm_Id	Integer			NOT NULL,
	Field_Id	Integer			NOT NULL,
	Operator		VARCHAR2(2)	NOT NULL,
          CONSTRAINT SA_THRESHOLD_Operator_ck CHECK (Operator IN ('=','!=','>','>=','<','<=')),
	Aggregator_Id	Integer			NOT NULL,
	Threshold_Value			NUMBER	NOT NULL,		
	Severity_Id		Integer,
	Hyst_Count		Integer			Default 1 NOT NULL ,
	Hyst_Range		Integer			Default 1 NOT NULL,
        sa_logopr_id     Integer
          constraint fk_sa_thres_sa_logopr references sa_logical_operator(id),
        logopr_disp      VARCHAR2(10)
          constraint CHK_logopr_disp CHECK (logopr_disp in ('AND', 'OR')),
        Open_Parens             VARCHAR2(10),
        Close_Parens            VARCHAR2(10),
        Changed			Date	Default Trunc(Sysdate,'MI'),
        --
	-- Foreign Key
	--
	CONSTRAINT FK_SA_Thrs_SA_ID	Foreign Key (SoftAlarm_Id) References SA_SOFTALARM (Id),
	CONSTRAINT FK_SA_Thrs_Field_Id Foreign Key (Field_Id) References WMD_FIELD_DEF (Field_Id),
	CONSTRAINT FK_SA_Thrs_Aggr_Id Foreign Key (Aggregator_Id) References SA_AGGREGATOR (Id),
	CONSTRAINT FK_SA_Thrs_Svrty_Id Foreign Key (Severity_Id) References SA_SEVERITY (Id),
	--
	-- Primary Key
	--
	CONSTRAINT PK_SA_THRESHOLD PRIMARY KEY (Id)
);


--
-- sequence for SA_THRESHOLD.Threshold_Id
--
CREATE SEQUENCE SQ_THRESHOLD
        START WITH 1000
        INCREMENT BY 1
        NOMAXVALUE
        NOCYCLE
;

CREATE SEQUENCE pm_saviewer_seq
  START WITH 100;
CREATE SEQUENCE SQ_SAVIEWER
  START WITH 1;

CREATE TABLE PM_SAVIEWER (
  ID 		NUMBER NOT NULL,
  FOLDERID 	NUMBER NOT NULL,
  USERID 	NUMBER NOT NULL,
  NAME 		VARCHAR2(64) NOT NULL,
  DESCRIPTION 	VARCHAR2(255),
  CHANGED 	DATE DEFAULT TRUNC(SYSDATE,'MI') NOT NULL, 
  SCENARIO_ID   NUMBER,
  TOOLBAR       CHAR(1) DEFAULT 'Y',
  STATUSBAR     CHAR(1) DEFAULT 'Y',
  WIDTH         NUMBER DEFAULT 8715,
  HEIGHT        NUMBER DEFAULT 7410,
  SEVERITY      NUMBER DEFAULT 0,
  ENTITY        NUMBER DEFAULT 0,
  Category      Varchar2(64) default 'Soft Alarm View' NOT NULL,
  TYPE 		Varchar2(64) default 'Soft Alarm View' NOT NULL,
  SYSTEMID 	NUMBER, 
  Auto_Updates  CHAR(1) DEFAULT 'Y' NOT NULL,
    CONSTRAINT PM_SAVIEWER_Auto_Updates_chk CHECK ( Auto_Updates IN ( 'Y', 'N' ) ),
  StringTable_ID integer, 
  StringTable_Ordinal integer default 1
   constraint CHK_PM_SAVIEWER_PM_ST_ORDIANL check(StringTable_Ordinal=1),
    CONSTRAINT pm_saviewer_pk PRIMARY KEY(ID) using index tablespace wm_client, 
    CONSTRAINT pm_saviewer_folderid_fk FOREIGN KEY(FOLDERID) REFERENCES PM_FOLDER(ID),
    CONSTRAINT pm_saviewer_userid_fk FOREIGN KEY(USERID) REFERENCES PM_USER(ID),
    CONSTRAINT pm_saviewer_pm_st_fk FOREIGN KEY(StringTable_ID,StringTable_Ordinal)
       REFERENCES PM_STRINGTABLE(ID,Ordinal)
) tablespace wm_client;

--
-- index required to prevent duplicate documents in folder
--
create unique index uni_saviewer_name on pm_saviewer(FolderId, UPPER(Name), Category);

Exit
--
------------------------------------------------------------------------
--
-- Local Variables:
-- mode: pls
-- End:
