-- @(#) create_component_schema.sql 1.2.0 02/11/01 13:01:39
-- @(#) 
--
-- The object schemas below comprises the product component registry.  The
-- purpose is to provide a repository for storing information about the
-- components of the products installed.
--
-- it is encouraged that the package component_registry be used to create,
-- update or remove entries, and the view be used access the data.
--
-- 6FEB2002/nick.talon@watchmark.com
--

CREATE TABLE component_types
(
  component_type   VARCHAR2(15)
    CONSTRAINT component_type_name_nn NOT NULL
    CONSTRAINT component_type_pk PRIMARY KEY,
  component_order  INTEGER  
);


CREATE SEQUENCE component_seq START WITH 1;


CREATE TABLE component
(
  component_id    INTEGER   
    CONSTRAINT component_comp_id_nn  NOT NULL
    CONSTRAINT component_comp_id_pk  PRIMARY KEY,
  component_type  VARCHAR2(15)
    CONSTRAINT component_comp_type_nn NOT NULL
    CONSTRAINT component_comp_type_fk REFERENCES component_types,
  component_name  VARCHAR2(30)
    CONSTRAINT component_comp_name_nn NOT NULL,
  CONSTRAINT component_comp_type_name_unq UNIQUE ( component_type, component_name )
);


CREATE SEQUENCE component_version_seq START WITH 1;


CREATE TABLE component_version
(
  component_version_id    INTEGER
    CONSTRAINT component_ver_id_nn  NOT NULL
    CONSTRAINT component_ver_id_pk  PRIMARY KEY,
  component_id            INTEGER
    CONSTRAINT component_ver_comp_id_nn  NOT NULL
    CONSTRAINT component_ver_comp_id_fk  REFERENCES component,
  revision                VARCHAR2(15)
    CONSTRAINT component_ver_revision_nn NOT NULL,
  build_number            INTEGER,
  replaced_by             INTEGER
    CONSTRAINT component_ver_repl_fk REFERENCES component_version,
  CONSTRAINT component_version_unq UNIQUE ( component_id, revision, build_number )
);


CREATE TABLE component_install_types
(
  install_type        VARCHAR2(10)
    CONSTRAINT component_inst_types_nn NOT NULL 
    CONSTRAINT component_inst_types_pk PRIMARY KEY,
  install_priority    INTEGER 
);


CREATE TABLE component_installed
(
  component_version_id   INTEGER
    CONSTRAINT component_inst_vers_id_nn NOT NULL
    CONSTRAINT component_inst_vers_id_fk REFERENCES component_version
    CONSTRAINT component_inst_vers_id_pk PRIMARY KEY,
  install_type    VARCHAR2(10)
    CONSTRAINT component_inst_type_nn NOT NULL
    CONSTRAINT component_inst_type_fk REFERENCES component_install_types,
  install_date    DATE           
    DEFAULT sysdate
    CONSTRAINT component_inst_date_nn NOT NULL
);


CREATE TABLE component_installing
(
  component_version_id   INTEGER
    CONSTRAINT component_instg_vers_id_nn NOT NULL
    CONSTRAINT component_instg_vers_id_fk REFERENCES component_version
    CONSTRAINT component_instg_pk PRIMARY KEY,
  install_type           VARCHAR2(10)
    CONSTRAINT component_instg_type_fk REFERENCES component_install_types,
  updated    DATE
    DEFAULT sysdate
    CONSTRAINT component_instg_upd_nn NOT NULL
);



CREATE TABLE component_replaces
(
  component_version_id    INTEGER
    CONSTRAINT component_repl_vid_nn NOT NULL
    CONSTRAINT component_repl_vid_fk REFERENCES component_version,
  replaced_component_version_id   INTEGER
    CONSTRAINT component_repl_repld_vid_nn NOT NULL
    CONSTRAINT component_repl_repld_vid_fk REFERENCES component_version,
  CONSTRAINT component_repl_pk PRIMARY KEY ( component_version_id, replaced_component_version_id )
);


CREATE SEQUENCE component_history_seq START WITH 1;


CREATE TABLE component_history
(
  component_hist_id   INTEGER
    CONSTRAINT comp_hist_id_pk   PRIMARY KEY,
  component_version_id   INTEGER
    CONSTRAINT component_hist_vers_id_nn NOT NULL
    CONSTRAINT component_hist_vers_id_fk REFERENCES component_version,
  install_type    VARCHAR2(10)
    CONSTRAINT comp_hist_inst_type_nn NOT NULL
    CONSTRAINT comp_hist_inst_type_fk REFERENCES component_install_types,
  install_action  VARCHAR2(20)
    CONSTRAINT comp_hist_action_nn NOT NULL,
  action_date     DATE           
    DEFAULT sysdate
    CONSTRAINT comp_hist_action_date_nn NOT NULL
);  
  

CREATE OR REPLACE VIEW current_components AS
  SELECT c.component_type || ' ' || c.component_name || ' rev ' || cv.revision || ' b' || nvl(cv.build_number,0)  component,
         ci.install_type, ci.install_date
    FROM component_installed ci, component c, component_version cv, 
         component_types ct
    WHERE ci.component_version_id = cv.component_version_id
      AND cv.component_id = c.component_id
      AND ct.component_type = c.component_type
    ORDER BY ct.component_order;
    
CREATE OR REPLACE VIEW installing_components AS
  SELECT c.component_type || ' ' || c.component_name || ' rev ' || cv.revision || ' b' || nvl(cv.build_number,0)  component,
         ci.install_type, ci.updated
    FROM component_installing ci, component c, component_version cv, 
         component_types ct
    WHERE ci.component_version_id = cv.component_version_id
      AND cv.component_id = c.component_id
      AND ct.component_type = c.component_type;
    
CREATE OR REPLACE VIEW install_history AS
  SELECT h.action_date timestamp, h.install_action action, h.install_type, 
         c.component_type || ' ' || c.component_name || ' rev ' || cv.revision || ' b' || nvl(cv.build_number,0)  component
    FROM component_history h, component c, component_version cv
    WHERE h.component_version_id = cv.component_version_id
      AND cv.component_id = c.component_id
    ORDER BY component_hist_id;
    
-- create view pm_component_registry
-- as
--   select c.product_name 
--          || decode(c.product_name, NULL, '', ' ')
--          || c.component_name
--          || decode(c.component_name, NULL, '', ' ')
--          || i.info_type as property
--        , i.value
--     from pm_product_component c
--        , pm_component_info    i
--    where c.id = i.component_id (+)
-- ;

-- ----------------------------------------------------------------------------
--
-- Local Variables:
-- mode: pls
-- End:
