create table wm_system_variable
(
   Id INTEGER NOT NULL,
   Name VARCHAR2(64) NOT NULL,
   CONSTRAINT PK_PM_SYSTEM_VARIABLE PRIMARY KEY(Id)
)
tablespace wm_client;

CREATE UNIQUE INDEX UNQ_SYSTEM_VARIABLE ON WM_SYSTEM_VARIABLE(Name)
TABLESPACE WM_Client;


create table wm_system_value
(
   Id INTEGER NOT NULL,
   Variable_Id INTEGER NOT NULL,
   Value VARCHAR2(64),
   CONSTRAINT PK_PM_SYSTEM_VALUE PRIMARY KEY(Id),
   CONSTRAINT FK_SYSTEM_VALUE_VARIABLE_ID
	      FOREIGN KEY(Variable_Id) REFERENCES WM_SYSTEM_VARIABLE(Id)
              ON DELETE CASCADE
)
tablespace wm_client;


--
-- sequence for PM_SYSTEM_VARIABLE.Id
--
CREATE SEQUENCE SQ_SYSTEM_VARIABLE
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

--
-- sequence for PM_SYSTEM_VALUE.Id
--
CREATE SEQUENCE SQ_SYSTEM_VALUE
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;


CREATE OR REPLACE VIEW WM_SYSTEM_VALUES_V AS
select
    n.name, v.value
from
    WM_SYSTEM_VARIABLE n, WM_SYSTEM_VALUE v
where
    n.Id = v.variable_id
order by
    v.Id;


-- populated in vendor/*/*/Network/load_client_metadata.sql


CREATE OR REPLACE TRIGGER
  -- ------------------------------------------------------------------------
  --     COPYRIGHT WatchMark Corp 2003
  -- *** (#) ./base/db/Oracle8i/wmlib/WM_SystemTable.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
  -- *** (#) 
  -- ------------------------------------------------------------------------
trg_sys_var_v
 INSTEAD OF INSERT OR UPDATE OR DELETE
 ON wm_system_values_v
FOR EACH ROW
DECLARE
   vVarID  wm_system_variable.id%TYPE := 0; 

BEGIN

    -- The wm_system_variable, and wm_system_value seem to be designed
    -- to handle a system variable with muti-values, but the view is
    -- no consistant with the design. Good thing is so far all system
    -- variable has only one value. And hope this will be our rule to
    -- use these tables. 
    -- This trigger is consistant with the concept that one system has
    -- only one value, basically consistant with the view 

    IF INSERTING THEN

        INSERT INTO wm_system_variable(id, Name)
        VALUES(sq_system_variable.NEXTVAL, :new.name);

        INSERT INTO wm_system_value(id, Variable_Id, Value)
        VALUES(SQ_SYSTEM_VALUE.NEXTVAL, sq_system_variable.CURRVAL, :new.value);

    ELSIF DELETING then

            SELECT id INTO vVarID
              FROM wm_system_variable
             WHERE name=:old.name;

            DELETE FROM wm_system_value WHERE Variable_Id=vVarID;
            DELETE FROM wm_system_variable WHERE id=vVarID;

    ELSIF UPDATING then

            SELECT id INTO vVarID
              FROM wm_system_variable
             WHERE name=:old.name;
            UPDATE wm_system_variable SET name=:new.name WHERE Id=vVarID;
            UPDATE wm_system_value SET value=:new.value WHERE Variable_Id=vVarID;

    END IF;
END trg_sys_var_v;
/ 

SHOW ERRORS TRIGGER trg_sys_var_v
exit;
