package wm_audit;

###################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###################################################################
# Script   : wm_audit                                             #
# Function : wminstall audit functions                            #
###################################################################
# (#) ./install/lib/wm_audit.pm ; BL_wminstall_8.0.7_8.0.7.5.02_20100605  065855

END {
    if ($summary_open) {
        print SUMMARY
          "+-------------------------------------------------------------\n";
        close SUMMARY or die "Unable to close $summary_rpt : $!";
    }

    if ($detail_open) {
        print DETAIL
          "+-------------------------------------------------------------\n";
        close DETAIL or die "Unable to close $detail_rpt : $!";
    }
    print STDOUT
      "+-------------------------------------------------------------\n";
}

###################################################################
#              Standard/Supplied Perl Modules                     #
###################################################################
use strict;
use Cwd;
use File::Basename;
use File::Copy;
use File::Path;
use IO::Handle;

###################################################################
#                      Watchmark Perl Modules                     #
###################################################################
use wm_utilities;


###################################################################
#                     Global Vars                                 #
###################################################################

my $log_dir           = "";
my $report_start_date = "";
my $report_start_time = "";
my $summary_rpt  = "summary.log";
my $detail_rpt   = "detail.log";
my $summary_open = 0;
my $detail_open  = 0;
my $split_line   =
  "+-------------------------------------------------------------";
my $left_column = "|  ";

sub check_for_open_logs {
###################################################################
    #                                                                 #
###################################################################
    my $log_dir = shift;

    ( $report_start_date, $report_start_time ) = &wm_utilities::get_date_time;

    if ( !$summary_open ) {
        my $summary_handle = join "/", "$log_dir", "$summary_rpt";
        open SUMMARY, ">$summary_handle"
          or die "Unable to open $summary_rpt  : $!";
        $summary_open = 1;
        SUMMARY->autoflush(1);
    }

    if ( !$detail_open ) {
        my $detail_handle = join "/", "$log_dir", "$detail_rpt";
        open DETAIL, ">$detail_handle"
          or die "Unable to open $detail_rpt  : $!";
        $detail_open = 1;
        DETAIL->autoflush(1);
    }
}

sub print_headers ( ) {
###################################################################
    #                                                                 #
###################################################################    
    print STDOUT "$split_line\n";
    print STDOUT "$left_column Processing : $0 \n";
    print STDOUT "$split_line\n";
    print STDOUT "$left_column Start Time : $report_start_time \n";
    print STDOUT "$left_column       Date : $report_start_date\n";

    print SUMMARY "$split_line\n";
    print SUMMARY "$left_column Processing : $0 \n";
    print SUMMARY "$left_column Log File   : summary.log\n";
    print SUMMARY "$split_line\n";
    print SUMMARY "$left_column Start Time : $report_start_time \n";
    print SUMMARY "$left_column       Date : $report_start_date\n";

    print DETAIL "$split_line\n";
    print DETAIL "$left_column Processing : $0 \n";
    print DETAIL "$left_column Log File   : detail.log\n";
    print DETAIL "$split_line\n";
    print DETAIL "$left_column Start Time : $report_start_time \n";
    print DETAIL "$left_column       Date : $report_start_date\n";

}

sub print_section ( $$$$ ) {
###################################################################
#                                                                 #
###################################################################

    my $section_name = shift;
    my $detail       = shift;
    my $summary      = shift;
    my $stdout       = shift;    

    $section_name .= " : ";
    if ( $stdout eq 'Y' ) {
        print STDOUT "$split_line\n";
        print STDOUT "$left_column $section_name\n";
    }

    if ( $detail eq 'Y' ) {
        print DETAIL "$split_line\n";
        print DETAIL "$left_column $section_name\n";
    }

    if ( $summary eq 'Y' ) {
        print SUMMARY "$split_line\n";
        print SUMMARY "$left_column $section_name\n";
    }
}

sub print_detail ( $$$$$ ){
###################################################################
#                                                                 #
###################################################################
    my $literal = shift;
    my $value = shift;
    my $detail = shift;
    my $summary = shift;
    my $stdout = shift;

    if ( $stdout eq 'Y' ) {
        printf STDOUT "%-10s %-40s %-1s %-20s\n", $left_column, $literal, ":",
          $value;
    }
    if ( $detail eq 'Y' ) {
        printf DETAIL "%-10s %-40s %-1s %-20s\n", $left_column, $literal, ":",
          $value;
    }

    if ( $summary eq 'Y' ) {
        printf SUMMARY "%-10s %-40s %-1s %-20s\n", $left_column, $literal, ":",
          $value;
    }
}

sub print_error ( $$$ ){
###################################################################
#                                                                 #
###################################################################    
    my $err_warn = shift;
    my $step = shift;
    my $message = shift;

    printf DETAIL "%-2s\t%-10s\t%-80s\t%-20s\n", "|", $err_warn, $step,
      $message;
    printf STDOUT "%-2s\t%-10s\t%-80s\t%-20s\n", "|", $err_warn, $step,
      $message;
    printf SUMMARY "%-2s\t%-10s\t%-80s\t%-20s\n", "|", $err_warn, $step,
      $message;
}

sub close_reports ( ){
###################################################################
    #                                                                 #
###################################################################

    ( $report_start_date, $report_start_time ) = &wm_utilities::get_date_time;

    if ($summary_open) {
        print SUMMARY "$split_line\n";
        print SUMMARY "$left_column End Time   : $report_start_time \n";
        print SUMMARY "$left_column     Date   : $report_start_date\n";
        print SUMMARY "$split_line\n";
        close SUMMARY or die "Unable to close $summary_rpt : $!";
        $summary_open = 0;
    }

    if ($detail_open) {
        print DETAIL "$split_line\n";
        print DETAIL "$left_column End Time   : $report_start_time \n";
        print DETAIL "$left_column     Date   : $report_start_date\n";
        print DETAIL "$split_line\n";

        close DETAIL or die "Unable to close $detail_rpt : $!";
        $detail_open = 0;
    }
        print STDOUT "$split_line\n";
        print STDOUT "$left_column End Time   : $report_start_time \n";
        print STDOUT "$left_column     Date   : $report_start_date\n";

}

sub start_audit_report( $ ){
###################################################################
#                                                                 #
###################################################################
    my $log_dir = shift;

    &check_for_open_logs( $log_dir );

    &print_headers;
}

1;
