package wm_err;

###################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###################################################################
# Script   : wm_err                                               #
# Function : Collect Error Messages for summary/eoj processing    #
###################################################################
# (#) ./install/lib/wm_err.pm ; BL_wminstall_8.0.7_8.0.7.5.02_20100605  065855

###################################################################
#              Standard/Supplied Perl Modules                     #
###################################################################
use strict;
use Cwd;
use File::Basename;
use File::Copy;
use File::Path;

###################################################################
#                      Watchmark Perl Modules                     #
###################################################################
use wm_audit;

###################################################################
#                     Global Vars                                 #
###################################################################

my @err_array;
my $error_count;
my $warning_count;

sub dump_errors ( $ ){
###################################################################
    #                                                                 #
###################################################################

    my $verbosity = shift;

    &wm_audit::print_section( "Error/Warning Summary ", "Y", "Y", "Y" );

    if ( $#err_array lt 0 ) {
        &wm_audit::print_error( "Success", "No errors/warnings found", "" );
        return;
    }

    for ( my $x = 0 ; $x <= $#err_array ; ) {
        &wm_audit::print_error( "$err_array[$x+1]", "$err_array[$x]",
            "$err_array[$x+2]" );
        $x += 3;
    }
}

sub accum_error ( $$$ ){
###################################################################
    #                                                                 #
###################################################################

    my ( $error_step_name, $error_type, $error_message ) = @_;

    push @err_array, @_;

    if ( $error_type =~ /^[Ww]/ ) {
        ++$warning_count;
    }
    elsif ( $error_type =~ /^[Ee]/ ) {
        ++$error_count;
    }
}

sub return_error_counts { return ( $error_count, $warning_count ); }

1;
