#!/bin/ksh
#
# (#) ./base/ldap/scripts/drop_ldap_schema.ksh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
# COPYRIGHT Watchmark Corp. 2003
#
#
#   Script deletes attributes and object classes
#   in the namespaces used by Watchmark.
#
# History
# 06-01-2004 kwagner initial version.
#

schemadn="cn=schema"
attributespace="1.3.6.1.4.1.8066.2.2.1"
objectspace="1.3.6.1.4.1.8066.2.2.2"

function usage {
   echo "Usage: $0 [-n] -h host [-p port] -D bind_dn -w password"
   echo "options:"
   echo "  -n         pretend to do what would be done but don't actually do"
   echo "  -h host    ldap server"
   echo "  -p port    port on ldap server (default=389)"
   echo "  -D binddn  bind dn"
   echo "  -w passwd  bind passwd (for authentication)"
   exit 1
}

function delete {
  typeset    filter=$1
  typeset    namespace=$2
  typeset    result
  set -A     result $( ldapsearch -h "$host" -p "$port" -s base -F ":" \
                                  -D "$binddn" -w "$bindpw" -b "$schemadn" \
                                  "objectClass=*" "$filter" \
                                  | grep -i "$namespace" | tr " " "^" )
  typeset -i idx=0;
  while [[ -n ${result[$idx]} ]]
  do

    ldapmodify -v $pretend -h "$host" -p "$port" -D "$binddn" -w "$bindpw" \
<<EOF
dn: cn=schema
changetype: modify
delete: $filter
$( echo ${result[$idx]} | tr "^" " " )

EOF

    idx=$(($idx+1))
  done
}

# Main entry point of the script

  port="389" # default ldap port

  while getopts "nh:p:D:w:" opt
  do
     case $opt in
        n) pretend="-n";;
        h) host=${OPTARG};;
        p) port=${OPTARG};;
        D) binddn=${OPTARG};;
        w) bindpw=${OPTARG};;
        *) usage;;
     esac
  done
  
  if [[ -z $host || -z $binddn || -z $bindpw ]]
  then
     usage
  fi

  echo
  echo "LDAP Schema Pruner"
  echo
  echo "Options:"
  echo "  host:    $host"
  echo "  port:    $port"
  echo "  bind dn: $binddn"
  echo
  
  if [[ ! -z ${pretend} ]]
  then
    delete "objectClasses" $objectspace
    delete "attributeTypes" $attributespace 
  else
    echo "Warning: This action is non-recoverable. Are you "
    echo "sure you want to prune this schema? (yes|[no]) "
    read answer
    if [[ "yes" == $answer ]]
    then
      echo
    delete "objectClasses" $objectspace
    delete "attributeTypes" $attributespace 
    else
      echo "Goodbye."
    fi
  fi

#Done.

