#!/bin/ksh
#
# (#) ./base/ldap/scripts/drop_ldap_subtree.ksh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
# COPYRIGHT Watchmark Corp. 2003
#
#   Script recursively deletes a tree or subtree
#   of entries from an ldap server.
#
# Note: Script does not delete the specified root, only 
#       it's children. I've this nightmare vision of 
#       users joyfully deleting thier base contexts, then
#       calling in the wee hours to find out where thier
#       servers have gone.
# 
# History
# 05-27-2004 kwagner initial version.
#

function usage {
   echo "Usage: $0 [-n] -h host [-p port] -D bind_dn -w password -b rootdn"
   echo "options:"
   echo "  -n         pretend to do what would be done but don't actually do"
   echo "  -h host    ldap server"
   echo "  -p port    port on ldap server (default=389)"
   echo "  -D binddn  bind dn"
   echo "  -w passwd  bind passwd (for authentication)"
   echo "  -b rootdn  root dn or domain suffix for delete"
   exit 1
}

# Recursive function traverses the tree, deleting the nodes, depthfirst.

function rdelete {
  typeset    basedn=$( echo $1 | tr "^" " " )
  typeset    result
  set -A     result $( ldapsearch -h "$host" -p "$port" -s one  \
                                  -D "$binddn" -w "$bindpw" -b "$basedn" \
                                  "objectClass=*" dn | tr " " "^" )
  typeset -i idx=0;
  while [[ -n ${result[$idx]} ]]
  do
    rdelete ${result[$idx]}
    ldapdelete -v $pretend -h "$host" -p "$port" -D "$binddn" \
               -w "$bindpw" "$( echo ${result[$idx]} | tr "^" " " )"
    idx=$(($idx+1))
  done
}


# Main entry point of the script

  port="389" # default ldap port

  while getopts "nh:p:D:w:b:" opt
  do
     case $opt in
        n) pretend="-n";;
        h) host=${OPTARG};;
        p) port=${OPTARG};;
        D) binddn=${OPTARG};;
        w) bindpw=${OPTARG};;
        b) basedn=${OPTARG};;
        *) usage;;
     esac
  done
  
  if [[ -z $host || -z $binddn || -z $bindpw || -z $basedn ]]
  then
     usage
  fi

  echo
  echo "LDAP Entry Pruner"
  echo
  echo "Options:"
  echo "  host:    $host"
  echo "  port:    $port"
  echo "  bind dn: $binddn"
  echo "  root dn: $basedn"
  echo
  
  if [[ ! -z ${pretend} ]]
  then
    rdelete $basedn
  else
    echo "Warning: This action is non-recoverable. Are you "
    echo "sure you want to prune this subtree? (yes|[no]) "
    read answer
    if [[ "yes" == $answer ]]
    then
      echo
      rdelete $basedn
    else
      echo "Goodbye."
    fi
  fi

#Done.
