--  @(#) ../sql/cleanupExport.sql = generate ASCII file for 'export'
--  @(#) ./base/migrate/prospect/sql/exportUserData.sql ; BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527

--  NOTES:
--      Sep-12-2001  Balaji V  Added Hawaii tables at the end of file
--	wmq_query done by James
--	wmq_query_text done by James
--	pm_fileformat: system static data, not migrated
--	pm_cpfailcouttype: system static data, not migrated
--	pm_groupby: system static data, not migrated
--	pm_cfgtpl: system owned, not migrated
--  NOTES:
--	Only export user created data where id >= 1000
--	If a table has some pre-loaded data,
--	id look up is used in child tables to find the new ids
--	since pre-loaded data might change.
--	If a table only contains user-created data,
--	id is directly copied to child tables.
--  NOTES:
--	Do not change the order that tables are exported
--	because once table size is getting bigger,
--	we can use release action to separate the table load.
--	There is a "RELEASE" token after each table
--	(except for PM_User and PM_Folder)
--	which should be loaded together.
--	If they are getting really bigger,
--	we might need to split the table.
--	For now, we can just load them together.)
--	Always select parent table first then child table.

    set linesize 32767
    set heading off
    set feedback off
    set null ""
    set tab off
      SET trimspool ON
      
--  table PM_User

select 'PM_User:'			|| '|' ||
	(PM_User.Id)			|| '|' ||
	getFolderPathForMigration(folderid)			|| '|' ||
	decode(PM_User.Name,'flexpm','prospect',pm_user.name)	|| '|' ||
	PM_User.Description 		|| '|' ||
--	PM_User.Password		|| '|' ||
	PM_User.Type 			|| '|' ||
	PM_User.AutoPurgeRequired 	|| '|' ||
	PM_User.MaxPurgeInterval 	|| '|' ||
	PM_User.MaxUsageMB  		|| '|'
from	PM_User
where	id >= 1000
order by id
;

--  table PM_User Password field

--  password is different from others --
--  <TAB> is used as delimiter
--  begin and end marks are used for easy parsing

select  substr('BEGIN_PASSWORD_SECTION', 1) from dual;

select 'Password:'	|| '	' ||
	name 		|| '	' ||
	password
from 	pm_user
where	id >= 1000
;

select  substr('END_PASSWORD_SECTION', 1) from dual;

--  table PM_Folder

select 'PM_Folder:'			|| '|' ||
	(PM_Folder.Id)			|| '|' ||
	getFolderPathForMigration(pm_folder.folderid)		|| '|' ||
	Decode(Pm_user.Name,'flexpm','prospect',Pm_user.Name)	|| '|' ||
	decode(PM_Folder.Name,'flexpm','prospect',pm_folder.name)	|| '|' ||
	PM_Folder.Description 		|| '|' ||
	to_char(pm_folder.Changed, 'MM/DD/RR HH24:MI') 		|| '|' ||
	PM_Folder.Type			|| '|' ||
	PM_Folder.Category 		|| '|'
from	PM_Folder, PM_User
where	PM_Folder.id >= 1000 and
	PM_User.id = PM_Folder.userid
order by pm_folder.id
;

select  substr('RELEASE', 1) from dual;

--  load user defined field back to dictionary.

--  table wmd_udc_def

select 'WMD_UDC_DEF:'			|| '|' ||
	wmd_dict_name.name		|| '|' ||
	wmn_entity.entity_name		|| '|' ||
	wmd_field_def.datasource_id	|| '|' ||
	wmd_field_def.field_owner	|| '|' ||
	wmd_field_def.datatype		|| '|' ||
	wmd_udc_def.udc_source		|| '|' ||
	wmd_dict_name.report_header	|| '|' ||
	wmd_dict_name.report_header2	|| '|' ||
	wmd_field_def.field_desc	|| '|' ||
	getTechMaskForMigration(wmd_udc_def.field_id)		|| '|' ||
	wmd_field_def.field_aggregator 	|| '|'
from 	wmd_field_def, wmd_udc_def, wmd_dict_name, wmn_entity
where	wmd_udc_def.field_id=wmd_field_def.field_id and
	wmd_udc_def.entity_id=wmn_entity.entity_id and
	wmd_dict_name.field_id=wmd_udc_def.field_id and
	wmd_dict_name.namespace_id=0
;

select  substr('RELEASE', 1) from dual;

--  table PM_Template

select 'PM_Template:'			|| '|' ||
	(PM_Template.Id)		|| '|' ||
	getFolderPathForMigration (pm_template.folderid)	|| '|' ||
	decode(pm_user.name,'flexpm','prospect',pm_user.name)	|| '|' ||
	PM_Template.Name		|| '|' ||
	PM_Template.Description		|| '|' ||
	to_char(pm_template.Changed, 'MM/DD/RR HH24:MI') 	|| '|' ||
	PM_Template.Category		|| '|' ||
	PM_Template.Type		|| '|' ||
	wmn_Entity_1.Entity_Name	|| '|' ||
	wmn_Entity_2.Entity_Name	|| '|' ||
	wmd_Dict_Name.Name 		|| '|'
from	PM_Template, wmn_Entity wmn_entity_1, wmn_entity wmn_entity_2, wmd_Dict_Name, pm_user
where	pm_template.id >= 1000 and
	PM_Template.Entity_Id = wmn_Entity_1.Entity_Id(+) and
	PM_Template.CriteriaFieldId = wmd_Dict_Name.Field_Id(+) and
	wmd_Dict_Name.Namespace_ID (+) = 0 and
	wmd_dict_name.Entity_Id = wmn_Entity_2.Entity_Id(+) and
	pm_user.id = userid
order by pm_template.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_TemplateField

select 'PM_TemplateField:'		|| '|' ||
	getFolderPathForMigration(PM_Template.folderid)		|| '|' ||
	(pm_template.name)		|| '|' ||
	pm_template.category		|| '|' ||
	wmn_entity_1.entity_name	|| '|' ||
	PM_TemplateField.Ordinal	|| '|' ||
	wmn_entity_2.entity_name	|| '|' ||
	wmd_Dict_Name.Name		|| '|' ||
	PM_TemplateField.SortOrder	|| '|' ||
	PM_TemplateField.IsShown 	|| '|'
from	PM_TemplateField, pm_template, wmd_Dict_Name, wmn_Entity wmn_entity_1, wmn_entity wmn_entity_2
where	PM_TemplateField.templateid >= 1000 and
	templateid = pm_template.id and
	pm_template.entity_id = wmn_entity_1.entity_id (+) and
	PM_TemplateField.Field_Id = wmd_Dict_Name.Field_Id and
	wmd_Dict_Name.Entity_Id = wmn_Entity_2.Entity_Id and
	wmd_Dict_Name.Namespace_ID = 0
;

select  substr('RELEASE', 1) from dual;

--  table PM_Filter

select 'PM_Filter:'			|| '|' ||
	getFolderPathForMigration(PM_Template.folderid)		|| '|' ||
	(pm_template.name)		|| '|' ||
	pm_template.category		|| '|' ||
	wmn_entity.entity_name		|| '|' ||
	PM_Filter.FieldOrdinal		|| '|' ||
	PM_Filter.Ordinal		|| '|' ||
	PM_Filter.Criteria 		|| '|'
from	PM_Filter, pm_template, wmn_entity
where 	PM_filter.templateid >= 1000 and
	templateid = pm_template.id and
	pm_template.entity_id = wmn_entity.entity_id(+)
;

select  substr('RELEASE', 1) from dual;

--  table PM_ChanSetPlan

select 'PM_ChanSetPlan:'		|| '|' ||
	(PM_ChanSetPlan.Id)		|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	decode(pm_user.name,'flexpm','prospect',pm_user.name)	|| '|' ||
	PM_ChanSetPlan.Name		|| '|' ||
	PM_ChanSetPlan.Description	|| '|' ||
	to_char(pm_chansetplan.Changed, 'MM/DD/RR HH24:MI')  	|| '|'
from	PM_ChanSetPlan, pm_user
where	pm_chansetplan.userid = pm_user.id and
	pm_chansetplan.id >= 1000
order by pm_chansetplan.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Scenario
--  select user created scenarios.

select 'PM_Scenario:'			|| '|' ||
 	PM_Scenario.Id			|| '|' ||
	getFolderPathForMigration(pm_scenario.folderid)		|| '|' ||
	decode(pm_user.name,'flexpm','prospect',pm_user.name)	|| '|' ||
 	PM_Scenario.Name		|| '|' ||
 	PM_Scenario.Description		|| '|' ||
 	to_char(pm_scenario.Changed, 'MM/DD/RR HH24:MI') 	|| '|' ||
 	PM_Scenario.Type		|| '|' ||
	PM_Scenario.systemid		|| '|' ||
 	PM_Scenario.Query_Id 		|| '|'
from	PM_Scenario, pm_user
where 	pm_scenario.id >= 1000 and	
	pm_scenario.userid = pm_user.id and
	pm_scenario.type = 'U'
order by pm_scenario.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_BusyHour
--  do not export firstavail or lastavail, leave it as null (SEAde08956)

select 'PM_BusyHour:'			|| '|' ||
	(PM_BusyHour.Id)		|| '|' ||
	decode(pm_user.name,'flexpm','prospect',pm_user.name)	|| '|' ||
	PM_BusyHour.Name 		|| '|' ||
	to_char(PM_BusyHour.Changed, 'MM/DD/RR HH24:MI') 	|| '|' ||
	wmn_Entity_1.Entity_Name	|| '|' ||
	wmn_Entity_2.Entity_Name	|| '|' ||
	wmd_Dict_Name.Name		|| '|' ||
 	PM_BusyHour.Description 	|| '|' ||
	to_char(PM_BusyHour.FirstNeeded, 'MM/DD/RR HH24:MI') 	|| '|' ||
	PM_BusyHour.Span 		|| '|' ||
	PM_BusyHour.MinRank		|| '|' ||
	PM_BusyHour.MaxRank		|| '|' ||
	'|' ||
        '|' ||
        PM_BusyHour.DTPeriod            || '|' ||
        getFolderPathForMigration(pm_scenario.folderid)         || '|' ||
        pm_scenario.name                || '|'
from	PM_BusyHour, pm_user, wmn_Entity wmn_Entity_1, wmn_Entity wmn_Entity_2, wmd_Dict_Name, pm_scenario
where	pm_busyhour.userid = pm_user.id (+) and
	PM_BusyHour.Cluster_Entity_Id = wmn_Entity_1.Entity_Id and
	PM_BusyHour.Field_Id = wmd_Dict_Name.Field_Id (+) and
        PM_BusyHour.scenario_id = pm_scenario.id(+) and
	wmd_Dict_Name.Namespace_ID (+) = 0 and
        wmd_dict_name.entity_id = wmn_entity_2.entity_id (+) AND
	pm_busyhour.id >= 1000
order by pm_busyhour.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_HourRange
--  pm_busyhour is not pre-loaded, so just copy busyhourid over.

select 'PM_HourRange:'			|| '|' ||
	(PM_HourRange.Id)		|| '|' ||
	(PM_HourRange.BusyHourId) 	|| '|' ||
	PM_HourRange.FirstHour 		|| '|' ||
        PM_HourRange.LastHour 		|| '|' ||
        PM_HourRange.FirstSeconds       || '|' ||
        PM_HourRange.LastSeconds        || '|'
from	PM_HourRange, pm_busyhour
where 	pm_hourrange.id >= 1000 and
	pm_hourrange.busyhourid = pm_busyhour.id  -- add to ensure FK constraints
order by pm_hourrange.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_DTScope

select 'PM_DTScope:'			|| '|' ||
	(PM_DTScope.Id)			|| '|' ||
	PM_DTScope.ScopeType 		|| '|' ||
	PM_DTScope.TimeUnit 		|| '|' ||
	PM_DTScope.HowManyUnits 	|| '|' ||
	PM_DTScope.IncludeCurrent 	|| '|' ||
	PM_DTScope.SameHoursEachDay 	|| '|' ||
	PM_DTScope.IncludeHolidays 	|| '|' ||
	PM_DTScope.HolidaysOnly 	|| '|' ||
	PM_DTScope.IncludeWeekends 	|| '|' ||
        PM_DTScope.WeekendsOnly 	|| '|' ||
        PM_DTScope.DTPeriod             || '|' ||
        PM_DTScope.ExtendRange          || '|' ||
        PM_DTScope.QuickRange           || '|'
from	PM_DTScope
where 	id >= 1000
order by id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Calendar
--  pm_dtscope is not pre-loaded too.

select 'PM_Calendar:'			|| '|' ||
	(PM_Calendar.Id)		|| '|' ||
	(PM_Calendar.DTScopeId)		|| '|' ||
	to_char(PM_Calendar.CalendarDate, 'MM/DD/RR HH24:MI') 	|| '|' ||
	PM_Calendar.FirstHour 		|| '|' ||
        PM_Calendar.LastHour 		|| '|' ||
        PM_Calendar.FirstSeconds        || '|' ||
        PM_Calendar.LastSeconds         || '|'
from	PM_Calendar, PM_DTScope
where 	PM_Calendar.dtscopeid = PM_DTScope.id	-- added to skip bad record
	and pm_calendar.id >= 1000
order by PM_Calendar.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_PageSetup

select 'PM_PageSetup:'			|| '|' ||
	(PM_PageSetup.Id)		|| '|' ||
	PM_PageSetup.Orientation 	|| '|' ||
	PM_PageSetup.ScalingType 	|| '|' ||
	PM_PageSetup.ReductionRatio 	|| '|' ||
	PM_PageSetup.PagesWide 		|| '|' ||
	PM_PageSetup.PagesTall 		|| '|' ||
	PM_PageSetup.StyleSheet 	|| '|' ||
	PM_PageSetup.UserMacro 		|| '|' ||
	PM_PageSetup.SplitKey 		|| '|' ||
	PM_PageSetup.HideKey 		|| '|'
from	PM_PageSetup
where 	id >= 1000
order by id
;

select  substr('RELEASE', 1) from dual;

--  table PM_ClientMachine

select 'PM_ClientMachine:'		|| '|' ||
	(PM_ClientMachine.Id)		|| '|' ||
	PM_ClientMachine.Domain 	|| '|' ||
	PM_ClientMachine.MachineName	|| '|'
from	PM_ClientMachine
where 	id >= 1000
order by id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Destination

select 'PM_Destination:'		|| '|' ||
	(PM_Destination.Id)		|| '|' ||
	pm_fileformat.name 		|| '|' ||
	PM_Destination.ToDisplay 	|| '|' ||
	PM_Destination.ToPrinter 	|| '|' ||
	PM_Destination.ToFile		|| '|' ||
	PM_Destination.Delimiter	|| '|' ||
	PM_Destination.Directory 	|| '|'
from 	PM_Destination, pm_fileformat
where	pm_destination.formatid = pm_fileformat.id and
	pm_destination.id >= 1000
order by pm_destination.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_ReportSpec
--  copy busyhourid over since pm_busyhour is not pre-loaded.
--  copy groupbyid, cpfailcounttypeid over. pm_groupby and pm_cpfailcounttype are
--  system owned data. Assume those pre-meta data does not delete
--  old one.

select 'PM_ReportSpec:'			|| '|' ||
    	(PM_ReportSpec.Id)		|| '|' ||
    	PM_ReportSpec.ReportKind	|| '|' ||
	PM_ReportSpec.DoIndividual	|| '|' ||
    	PM_ReportSpec.DoSummary		|| '|' ||
	PM_ReportSpec.IndSortType	|| '|' ||
    	PM_ReportSpec.SumSortType	|| '|' ||
    	PM_ReportSpec.IndGraphType	|| '|' ||
    	PM_ReportSpec.SumGraphType	|| '|' ||
    	PM_ReportSpec.DoStored		|| '|' ||
    	PM_ReportSpec.DoMissingData	|| '|' ||
    	PM_ReportSpec.GroupById		|| '|' ||
    	PM_ReportSpec.RangeMin		|| '|' ||
    	PM_ReportSpec.RangeMax		|| '|' ||
    	(PM_ReportSpec.BusyHourId)	|| '|' ||
    	PM_ReportSpec.CPFailCountTypeId	|| '|' ||
    	wmn_Entity_1.Entity_Name	|| '|' ||
    	wmn_Entity_2.Entity_Name	|| '|' ||
    	wmd_Dict_Name.Name		|| '|' ||
    	PM_ReportSpec.DN		|| '|' ||
        PM_ReportSpec.Threshold 	|| '|' ||
        PM_ReportSpec.GraphId           || '|' ||
        PM_ReportSpec.MapId             || '|' ||
        PM_ReportSpec.DimensionTypeid   || '|' ||
        PM_ReportSpec.ForecastThreshold || '|' ||
        PM_ReportSpec.CurrentThreshold  || '|' ||
        PM_ReportSpec.FinalThreshold    || '|' ||
        PM_ReportSpec.CorrelationThreshold || '|' ||
        PM_ReportSpec.N1Days            || '|' ||
        PM_ReportSpec.N2Days            || '|' ||
        PM_ReportSpec.N3Days            || '|' ||
        getFolderPathForMigration(pm_scenario.folderid)         || '|' ||
        pm_scenario.name                || '|'
from 	PM_ReportSpec, pm_busyhour, pm_cpfailcounttype, pm_groupby,
	wmn_Entity wmn_Entity_1, wmn_Entity wmn_Entity_2, wmd_Dict_Name,
        pm_scenario
where	pm_reportspec.busyhourid = pm_busyhour.id (+) and
	pm_reportspec.cpfailcounttypeid = pm_cpfailcounttype.id (+)and
	pm_reportspec.groupbyid = pm_groupby.id (+) and	
	PM_ReportSpec.Cluster_Entity_Id = wmn_Entity_1.Entity_Id (+) and
	PM_ReportSpec.Field_Id = wmd_Dict_Name.Field_Id (+) and
	wmd_Dict_Name.Namespace_ID (+) = 0 and
	wmd_dict_name.entity_Id = wmn_entity_2.entity_id (+) and
        PM_ReportSpec.BHScenario_Id = pm_scenario.ID(+) and
	PM_ReportSpec.id >= 1000
order by pm_reportspec.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Report
--  copy destinationid, dtscopeid, reportspecid and pagesetupid over.
--  those tables are not pre-loaded.

--  udr, traffic report where templateid is from pm_template

select 'PM_Report:'			|| '|' ||
    	(PM_Report.Id)			|| '|' ||
	getFolderPathForMigration(pm_report.folderid)		|| '|' ||
 	decode(pm_user.name,'flexpm','prospect',pm_user.name)	|| '|' ||	
    	PM_Report.Name			|| '|' ||
    	PM_Report.Description		|| '|' ||
	to_char(pm_report.Changed, 'MM/DD/RR HH24:MI') 		|| '|' ||
    	PM_Report.Category		|| '|' ||
    	PM_Report.Type			|| '|' ||
    	wmn_Entity_1.Entity_Name	|| '|' ||
	getFolderPathForMigration(pm_scenario_1.folderid)	|| '|' ||
	(pm_scenario_1.name)		|| '|' ||
	getFolderPathForMigration(pm_template.folderid)		|| '|' ||
	(pm_template.name)		|| '|' ||
	pm_template.category		|| '|' ||
	wmn_entity_2.entity_name	|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	(pm_chansetplan.name)		|| '|' ||
    	(PM_Report.DestinationId)	|| '|' ||
    	(PM_Report.PageSetupId)		|| '|' ||
    	(PM_Report.DTScopeId)		|| '|' ||
    	(PM_Report.ReportSpecId)	|| '|' ||
   	PM_Report.ChannelNumber		|| '|' ||
	getFolderPathForMigration(pm_scenario_2.folderid)	|| '|' ||
	(pm_scenario_2.name)		|| '|' ||
	PM_Report.MapRadiusScale	|| '|' ||
	PM_Report.TestChannels		|| '|' ||
	PM_Report.EquippedOnly		|| '|' ||
    	PM_Report.FileName 		|| '|'
from 	PM_Report, wmn_Entity wmn_entity_1, wmn_entity wmn_entity_2,
	pm_user, pm_scenario pm_scenario_1, pm_scenario pm_scenario_2,
	pm_template, pm_chansetplan
where	pm_report.userid = pm_user.id and
	pm_report.scenarioid = pm_scenario_1.id (+) and
	pm_report.DomainScenarioId = pm_scenario_2.id (+) and
	pm_report.templateid = pm_template.id (+) and
	Upper(pm_report.category) not like 'CONFIG%' and
	pm_template.entity_id = wmn_entity_2.entity_id (+) and
	pm_report.chansetplanid = pm_chansetplan.id (+) and	
	PM_Report.Entity_Id = wmn_Entity_1.Entity_Id (+) and
	pm_report.id >= 1000
order by pm_report.id
;

select  substr('RELEASE', 1) from dual;

--  config report where templateid is from pm_cfgtpl

select 'PM_Report_CfgTpl:'		|| '|' ||
    	(PM_Report.Id)			|| '|' ||
	getFolderPathForMigration(pm_report.folderid)		|| '|' ||
 	decode(pm_user.name,'flexpm','prospect',pm_user.name)	|| '|' ||	
    	PM_Report.Name			|| '|' ||
    	PM_Report.Description		|| '|' ||
	to_char(pm_report.Changed, 'MM/DD/RR HH24:MI') 		|| '|' ||
    	PM_Report.Category		|| '|' ||
    	PM_Report.Type			|| '|' ||
    	wmn_Entity_1.Entity_Name	|| '|' ||
	getFolderPathForMigration(pm_scenario_1.folderid)	|| '|' ||
	(pm_scenario_1.name)		|| '|' ||
	getFolderPathForMigration(pm_cfgtpl.folderid)		|| '|' ||
	(pm_cfgtpl.name)		|| '|' ||
	pm_cfgtpl.category		|| '|' ||
	pm_cfgtpl.type			|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	(pm_chansetplan.name)		|| '|' ||
    	(PM_Report.DestinationId)	|| '|' ||
    	(PM_Report.PageSetupId)		|| '|' ||
    	(PM_Report.DTScopeId)		|| '|' ||
    	(PM_Report.ReportSpecId)	|| '|' ||
    	PM_Report.ChannelNumber		|| '|' ||
	getFolderPathForMigration(pm_scenario_2.folderid)	|| '|' ||
	(pm_scenario_2.name)		|| '|' ||
	PM_Report.MapRadiusScale	|| '|' ||
	PM_Report.TestChannels		|| '|' ||
	PM_Report.EquippedOnly		|| '|' ||
    	PM_Report.FileName 		|| '|'
from 	PM_Report, wmn_Entity wmn_entity_1,
	pm_user, pm_scenario pm_scenario_1, pm_scenario pm_scenario_2, pm_cfgtpl, pm_chansetplan
where	pm_report.userid = pm_user.id and
	pm_report.scenarioid = pm_scenario_1.id (+) and
	pm_report.DomainScenarioId = pm_scenario_2.id (+) and
	pm_report.templateid = pm_cfgtpl.id (+) and
	Upper(pm_report.category) like 'CONFIG%' and
	pm_report.chansetplanid = pm_chansetplan.id (+) and	
	PM_Report.Entity_Id = wmn_Entity_1.Entity_Id (+) and
	pm_report.id >= 1000
order by pm_report.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_RptGrp
-- For Hawaii schema this has been changed to view and this content should go to
-- JOB_GROUP  -- on Sep-13-2001 Balaji V

select 'Job_Group:'			|| '|' ||
   	(PM_RptGrp.Id)			|| '|' ||
    	getFolderPathForMigration(pm_rptgrp.FolderId)		|| '|' ||
    	decode(pm_user.name,'flexpm','prospect',pm_user.name)	|| '|' ||
   	PM_RptGrp.Name			|| '|' ||
	PM_RptGrp.Description		|| '|' ||
	to_char(pm_rptgrp.Changed, 'MM/DD/RR HH24:MI')  	|| '|' ||
	PM_RptGrp.Group_Type		|| '|'
from	PM_RptGrp, pm_user
where 	pm_rptgrp.userid = pm_user.id and
	pm_rptgrp.id >= 1000
order by pm_rptgrp.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_RptGrpItem
--  copy rptgrpid and reportid over
--  pm_rptgrp and pm_report and not pre loaded.

-- For Hawaii schema this has been changed to view and this content should go to
-- JOB_GROUP_ITEM  -- on Sep-13-2001 Balaji V

select 'Job_Group_Item:'			|| '|' ||
   	(PM_RptGrpItem.Id)		|| '|' ||
    	(PM_RptGrpItem.RptGrpId)	|| '|' ||
    	(PM_RptGrpItem.ReportId)	|| '|' ||
   	PM_RptGrpItem.Ordinal 		|| '|'
from	PM_RptGrpItem
where	id >= 1000
order by id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Schedule
--  copy rptgrp, ovrdestinationid, ovrdtscopeid, clientmachineid
--  over since pm_rptgrp, pm_destination, pm_dtsope, pm_clientmachine
--  is not pre-loaded
--  migrate schedules that are not created by the system. (suggested by
--  RAS)
--  system created schedules are putting in folder "\Temp"
--  user created schedules are putting in folder "\Schedule"

select 'PM_Schedule:'			|| '|' ||
   	(PM_Schedule.Id)		|| '|' ||
	getFolderPathForMigration(pm_schedule.folderid)		|| '|' ||
	decode(pm_user.name,'flexpm','prospect',pm_user.name)	|| '|' ||
    	PM_Schedule.Name		|| '|' ||
    	PM_Schedule.Description		|| '|' ||
	to_char(pm_schedule.Changed, 'MM/DD/RR HH24:MI') 	|| '|' ||
    	inactiveSchedule(PM_Schedule.Periodicity,
                         PM_Schedule.ScheduleActive,
                         PM_Schedule.FirstDatetime)	|| '|' ||
    	(PM_Schedule.ClientMachineId)	|| '|' ||
    	(PM_Schedule.RptGrpId)		|| '|' ||
	getFolderPathForMigration(pm_scenario.folderid)		|| '|' ||
	pm_scenario.name		|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	pm_chansetplan.name		|| '|' ||
    	(PM_Schedule.OvrDestinationId)	|| '|' ||
    	(PM_Schedule.OvrDTScopeId)	|| '|' ||
    	PM_Schedule.RunMode		|| '|' ||
    	PM_Schedule.RetrieveMode	|| '|' ||
   	PM_Schedule.AutoPurge		|| '|' ||
   	PM_Schedule.PurgeInterval	|| '|' ||
    	PM_Schedule.SubFolderName	|| '|' ||
	to_char(PM_Schedule.FirstDateTime, 'MM/DD/RR HH24:MI') 	|| '|' ||
	to_char(PM_Schedule.FinalDateTime, 'MM/DD/RR HH24:MI') 	|| '|' ||
     	PM_Schedule.WhichMonthDate	|| '|' ||
     	PM_Schedule.WhichWeek		|| '|' ||
    	PM_Schedule.WhichWeekday	|| '|' ||
     	PM_Schedule.Periodicity		|| '|' ||
    	PM_Schedule.Frequency		|| '|' ||
    	PM_Schedule.IncludeMondays	|| '|' ||
    	PM_Schedule.IncludeTuesdays	|| '|' ||
    	PM_Schedule.IncludeWednesdays	|| '|' ||
    	PM_Schedule.IncludeThursdays	|| '|' ||
    	PM_Schedule.IncludeFridays	|| '|' ||
    	PM_Schedule.IncludeSaturdays	|| '|' ||
    	PM_Schedule.IncludeSundays	|| '|' ||
    	PM_Schedule.IncludeHolidays	|| '|' ||
   	PM_Schedule.IncludeDayAfterHol	|| '|' ||
	to_char(PM_Schedule.NextRunDate, 'MM/DD/RR HH24:MI') 	|| '|' ||
	to_char(PM_Schedule.LastRunDate, 'MM/DD/RR HH24:MI')  	|| '|'
from 	PM_Schedule, pm_user, pm_scenario, pm_chansetplan, pm_folder
where 	pm_schedule.id >= 1000 and
	pm_schedule.folderid = pm_folder.id and
	pm_folder.name = 'Schedule' and
	pm_schedule.userid = pm_user.id and
	pm_schedule.ovrscenarioid = pm_scenario.id (+) and
	pm_schedule.ovrchansetid = pm_chansetplan.id (+)
order by pm_schedule.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Holidays

select 'PM_Holidays:'			|| '|' ||
	to_char(PM_Holidays.Holiday, 'MM/DD/RR HH24:MI') 	|| '|'
from	PM_Holidays
;

select  substr('RELEASE', 1) from dual;

--  table PM_Trigger

select 'PM_Trigger:'			|| '|' ||
	(PM_Trigger.Trigger_Id)		|| '|' ||
	PM_Trigger.Trigger_Name		|| '|' ||
	PM_Trigger.SubTrigger_Name	|| '|' ||
	getFolderPathForMigration(pm_template.folderid)		|| '|' ||
	(pm_template.name)		|| '|' ||
	pm_template.category		|| '|' ||
	wmn_entity.entity_name 		|| '|'
from 	PM_Trigger, pm_template, wmn_entity
where 	trigger_id >= 1000 and
	pm_trigger.template_id = pm_template.id and
	pm_template.entity_id = wmn_entity.entity_id (+)
order by trigger_id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Feature

select 'PM_Feature:'			|| '|' ||
	(PM_Feature.Feature_Id)		|| '|' ||
	PM_Feature.Feature_Name 	|| '|'
from 	PM_Feature
where	feature_id >= 1000
order by feature_id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Trend_Impact
--  copy feature_id over since pm_feature in not pre-loaded

select 'PM_Trend_Impact:'		|| '|' ||
	(pm_trigger.trigger_name)	|| '|' ||
	(pm_trigger.subtrigger_name)	|| '|' ||
	getFolderPathForMigration(pm_template.folderid)		|| '|' ||
	(pm_template.name)		|| '|' ||
	pm_template.category		|| '|' ||
	wmn_entity.entity_name		|| '|' ||
	(PM_Trend_Impact.Feature_Id)	|| '|' ||
	PM_Trend_Impact.Impact 		|| '|'
from 	PM_Trend_Impact, pm_template, pm_trigger, wmn_entity
where	pm_trend_impact.trigger_id = pm_trigger.trigger_id and
	pm_trigger.template_id = pm_template.id and
	pm_template.entity_id = wmn_entity.entity_id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Partition

select 'PM_Partition:'			|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	(pm_chansetplan.name)		|| '|' ||
	PM_Partition.Ordinal		|| '|' ||
	PM_Partition.Name		|| '|' ||
	PM_Partition.NumberOfRows	|| '|' ||
	PM_Partition.IsSetup  		|| '|'
from	PM_Partition, pm_chansetplan
where	pm_partition.chansetplanid >= 1000 and
	pm_partition.chansetplanid = pm_chansetplan.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_ChanGrp

select 'PM_ChanGrp:'			|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	(pm_chansetplan.name)		|| '|' ||
	PM_ChanGrp.Ordinal		|| '|' ||
	PM_ChanGrp.Name			|| '|' ||
	PM_ChanGrp.BackColor 		|| '|'
from	PM_ChanGrp, pm_chansetplan
where	pm_changrp.chansetplanid >= 1000 and
	pm_changrp.chansetplanid = pm_chansetplan.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_ChanSetGrp

select 'PM_ChanSetGrp:'			|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	(pm_chansetplan.name)		|| '|' ||
	PM_ChanSetGrp.Ordinal		|| '|' ||
	PM_ChanSetGrp.Name		|| '|' ||
	PM_ChanSetGrp.BackColor 	|| '|'
from	PM_ChanSetGrp, pm_chansetplan
where	pm_chansetgrp.chansetplanid >= 1000 and	
	pm_chansetgrp.chansetplanid = pm_chansetplan.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_ChanSet

select 'PM_ChanSet:'			|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	(pm_chansetplan.name)		|| '|' ||
	PM_ChanSet.Ordinal		|| '|' ||
	PM_ChanSet.Name			|| '|' ||
	PM_ChanSet.ChanSetGrpName 	|| '|'
from	PM_ChanSet, pm_chansetplan
where	pm_chanset.chansetplanid >= 1000 and
	pm_chanset.chansetplanid = pm_chansetplan.id
;

select  substr('RELEASE', 1) from dual;

--  table PM_Channel

select 'PM_Channel:'			|| '|' ||
	getFolderPathForMigration(pm_chansetplan.folderid)	|| '|' ||
	(pm_chansetplan.name)		|| '|' ||
	PM_Channel.Ordinal		|| '|' ||
	PM_Channel.ChanSetName		|| '|' ||
	PM_Channel.ChanGrpName		|| '|' ||
	PM_Channel.PartitionName	|| '|' ||
	PM_Channel.RowNumber 		|| '|'
from	PM_Channel, pm_chansetplan
where	pm_channel.chansetplanid >= 1000 and
	pm_channel.chansetplanid = pm_chansetplan.id	
;

select  substr('RELEASE', 1) from dual;

-- Following tables Added by Balaji on Sep-12-2001

-- table PM_Pabh

select 'PM_Pabh:'  || '|' ||
  Id               || '|' ||
  FirstAvail       || '|' ||
  LastAvail        || '|'
  FROM PM_PABH
;

select  substr('RELEASE', 1) from dual;

-- table PM_Graph

select 'PM_Graph:'                                || '|' ||
       PG.Id                                      || '|' ||
       getFolderPathForMigration (PG.FolderId)    || '|' ||
       Decode (PM_User.Name,'flexpm','prospect',PM_User.Name) || '|' ||
       PG.Name                                    || '|' ||
       PG.Description                             || '|' ||
       To_Char(PG.Changed, 'MM/DD/RR HH24:MI')    || '|'
  FROM PM_GRAPH PG,
       PM_USER 
 WHERE PM_USER.Id = PG.UserId
;

select  substr('RELEASE', 1) from dual;

-- table PM_GraphItem

select 'PM_GraphItem:'      || '|' ||
       PGI.ID               || '|' ||
       PGI.GraphID          || '|' ||
       PGI.Ordinal          || '|' ||
       PGI.GraphType        || '|' ||
       PGI.Name             || '|' ||
       PGI.PosX             || '|' ||
       PGI.PosY             || '|' ||
       PGI.Width            || '|' ||
       PGI.Height           || '|' ||
       PGI.Field            || '|'
  FROM PM_GRAPHITEM PGI,
       PM_GRAPH PG
 WHERE PG.Id = PGI.GraphId
;

select  substr('RELEASE', 1) from dual;

-- table PM_GraphItemProp

select 'PM_GraphItemProp:'      || '|' ||
       PGIP.ID                  || '|' ||
       PGIP.GraphItemId         || '|' ||
       PGIP.Ordinal             || '|' ||
       PGIP.PropName            || '|' ||
       PGIP.PropValue           || '|'
  FROM PM_GraphItemProp PGIP
 WHERE PGIP.Id >= 1000
;
  
select  substr('RELEASE', 1) from dual;

-- table PM_Map

select 'PM_Map:'                                   || '|' ||
       Map.Id                                      || '|' ||
       getFolderPathForMigration (Map.FolderId)  || '|' ||
       Decode (PM_User.Name,'flexpm','prospect',PM_User.Name) || '|' || 
       Map.Name                                    || '|' ||
       Map.Description                             || '|' ||
       To_Char(Map.Changed, 'MM/DD/RR HH24:MI')    || '|' ||
       Map.TYPE                || '|' ||
       Map.Query_Id            || '|'
  FROM PM_Map Map,
       PM_User
 WHERE Map.Id >= 1000
   AND PM_User.Id = Map.UserId
;

select  substr('RELEASE', 1) from dual;


-- table PM_StringTable

select 'PM_StringTable:'   || '|' ||
       Id                  || '|' ||
       Ordinal             || '|' ||
       StringData          || '|'
  FROM PM_StringTable
 WHERE Id >= 1000
;
  
select  substr('RELEASE', 1) from dual;  

-- table WMD_Col_Def_SubType

select 'WMD_Col_Def_SubType:' || '|' ||
       SUBTYPE_ID             || '|' ||
       DEFAULT_VALUE          || '|'
 from WMD_Col_Def_SubType
where FIELD_ID=WM_Dictionary.Find_Field ('TrunkGroup','grade_serv')
order by SUBTYPE_ID
;

select  substr('RELEASE', 1) from dual;  


-- table SES_STUDY
select 'SES_Study:'                            || '|' ||
       ss.ID                                   || '|' ||
       getFolderPathForMigration(ss.FOLDERID)  || '|' ||
       decode(pmu.NAME,'flexpm','prospect',pmu.NAME)   || '|' ||
       ss.NAME                                 || '|' ||
       ss.DESCRIPTION                          || '|' ||
       to_char(ss.CHANGED, 'MM/DD/RR HH24:MI') || '|' || 
       ss.CATEGORY                             || '|' ||
       ss.TYPE                                 || '|' ||
       getFolderPathForMigration(pms.folderid) || '|' ||
       pms.NAME                                || '|' ||
       ss.DURATION                             || '|' ||
       ss.POLLING_INTERVAL                     || '|' ||
       ss.RETRIEVE_DELAY                       || '|'
  from Ses_study ss, pm_user pmu, pm_scenario pms
 where ss.SCENARIOID=pms.id(+)
   and ss.userid=pmu.id;

select  substr('RELEASE', 1) from dual;

-- table SES_HomStudy
select 'SES_HomStudy:'                         || '|' ||
       ID                                      || '|' ||
       DATAFILE                                || '|' ||
       HOMMODE                                 || '|' ||
       TOAMPS                                  || '|' ||
       FROMAMPS                                || '|' ||
       TOTDMAAMPS                              || '|' ||
       TOTDMA                                  || '|' ||
       SEMISOFT                                || '|' ||
       HARD                                    || '|' ||
       TOCDMAAMPS                              || '|' ||
       SEMISOFTPRIMARY                         || '|' ||
       SOFTPRIMARY                             || '|' ||
       SOFTADD                                 || '|' ||
       SOFTDROP                                || '|' ||
       PATH                                    || '|' ||
       COLLECTDATA                             || '|' ||
       STARTONFAIL                             || '|' ||
       CURSTDDATAFILE                          || '|' ||
       CURSTDPATH                              || '|'
  from SES_HomStudy
 order by ID;

select  substr('RELEASE', 1) from dual;

-- table SES_PlmStudy
select 'SES_PlmStudy:'                         || '|' ||
       ID                                      || '|' ||
       DATAFILE                                || '|' ||
       TAG                                     || '|' ||
       PATH                                    || '|'
  from SES_PlmStudy
 order by ID;

select  substr('RELEASE', 1) from dual;

-- table SES_PLMMODE
select 'SES_PlmMode:'                          || '|' ||
       ID                                      || '|' || 
       PLMMODE                                 || '|' ||
       CARRIER                                 || '|' ||
       SERVICEOPTION                           || '|' ||
       GENERATIONTYPE                          || '|' ||
       SelectionMethod                         || '|' ||
       AmpsDistribution                        || '|' ||
       TdmaDistribution                        || '|' ||
       TotalDistribution                       || '|' ||
       DistributionMethod                      || '|' ||
       SortBy                                  || '|' ||
       SelectFromList                          || '|' ||
       RangeFrom                               || '|' ||
       RangeTo                                 || '|' ||
       ExcludeAce                              || '|'
  from SES_PlmMode
 order by id;

select  substr('RELEASE', 1) from dual;

-- table SES_PLMCHANNELLIST
select 'SES_PlmChannelList:'                   || '|' ||
       ID                                      || '|' ||
       PLMMODE                                 || '|' ||
       CHANNEL                                 || '|'
  from SES_PlmChannelList
 order by id;

select  substr('RELEASE', 1) from dual;

-- table SES_RFCSTUDY
select 'SES_RFCStudy:'                         || '|' ||
       ID                                      || '|' ||
       TAG                                     || '|' ||
       DBM                                     || '|' ||
       CLEARLOG                                || '|' ||
       MULTIPLEDNS                             || '|' ||
       USEECP                                  || '|' ||
       PATH                                    || '|'
  from SES_RFCStudy
 order by id;

select  substr('RELEASE', 1) from dual;

-- table SES_RFCDNLIST
select 'SES_RFCDNList:'                        || '|' ||
       ID                                      || '|' ||
       DN                                      || '|' ||
       DATAFILE                                || '|'
  from SES_RFCDNList
 order by id;

select  substr('RELEASE', 1) from dual;

-- table SES_VCSSTUDY
select 'SES_VCSStudy:'                         || '|' ||
       ID                                      || '|' ||
       EVENTA                                  || '|' ||
       EVENTB                                  || '|' ||
       EVENTC                                  || '|' ||
       EVENTD                                  || '|' ||
       EVENTE                                  || '|' ||
       EVENTF                                  || '|' ||
       EVENTG                                  || '|' ||
       EVENTH                                  || '|' ||
       EVENTI                                  || '|' ||
       EVENTJ                                  || '|' ||
       EVENTK                                  || '|' ||
       EVENTL                                  || '|' ||
       EVENTM                                  || '|' ||
       DATAFILE                                || '|' ||
       DIRECTIONALSETUP                        || '|' ||
       CARRIERDATA                             || '|' ||
       SINGLEDN                                || '|' ||
       PATH                                    || '|'
  from SES_VCSStudy
 order by id;

select  substr('RELEASE', 1) from dual;

-- table SES_ECP_LIST
select 'SES_Ecp_List:'                         || '|' ||
       SES_List_ID                             || '|' ||
       wme.entity_name                         || '|' ||
       wmic.FULLKEY                            || '|' ||
       SystemID                                || '|' ||
       EcpID                                   || '|'
  from SES_Ecp_List ecp,
       wmn_instance_context wmic, wmn_entity wme
 where ecp.InstanceID = wmic.INSTANCE_ID
   and wmic.context_id =
       (select CONTEXT_ID from wmn_context
         where CONTEXT_USE='R')
   and wme.entity_id = wmic.entity_id
 order by SES_List_ID;

select  substr('RELEASE', 1) from dual;

-- table SES_CELL_LIST
select 'SES_Cell_List:'                        || '|' ||
       SES_List_ID                             || '|' ||
       wme.entity_name                         || '|' ||
       wmic.FULLKEY                            || '|' ||
       SystemID                                || '|' ||
       EcpID                                   || '|' ||
       CellID                                  || '|'
  from SES_Cell_List cell,
       wmn_instance_context wmic, wmn_entity wme
 where cell.InstanceID = wmic.INSTANCE_ID
   and wmic.context_id = 
       (select CONTEXT_ID from wmn_context
         where CONTEXT_USE='R')
   and wme.entity_id = wmic.entity_id
 order by SES_List_ID;

select  substr('RELEASE', 1) from dual;

-- table SES_ANTENNA_LIST
select 'SES_Antenna_List:'                     || '|' ||
       SES_List_ID                             || '|' ||
       wme.entity_name                         || '|' ||
       wmic.FULLKEY                            || '|' ||
       SystemID                                || '|' ||
       EcpID                                   || '|' ||
       CellID                                  || '|' ||
       AntennaID                               || '|' ||
       Selected                                || '|'
  from SES_Antenna_List ant,
       wmn_instance_context wmic, wmn_entity wme
 where ant.InstanceID = wmic.INSTANCE_ID
   and wmic.context_id = 
       (select CONTEXT_ID from wmn_context
         where CONTEXT_USE='R')
   and wme.entity_id = wmic.entity_id
 order by SES_List_ID;

select  substr('RELEASE', 1) from dual;

-- table SES_CARRIER_LIST
select 'SES_Carrier_List:'                     || '|' ||
       SES_List_ID                             || '|' ||
       wme.entity_name                         || '|' ||
       wmic.FULLKEY                            || '|' ||
       SystemID                                || '|' ||
       EcpID                                   || '|' ||
       CellID                                  || '|' ||
       CarrierID                               || '|' ||
       Selected                                || '|'
  from SES_Carrier_List carr,
       wmn_instance_context wmic, wmn_entity wme
 where carr.InstanceID = wmic.INSTANCE_ID
   and wmic.context_id = 
       (select CONTEXT_ID from wmn_context
         where CONTEXT_USE='R')
   and wme.entity_id = wmic.entity_id
 order by SES_List_ID;

select  substr('RELEASE', 1) from dual;

-- table SES_FAF
select 'SES_FAF:'                              || '|' ||
       wme.entity_name                         || '|' ||
       wmic.FULLKEY                            || '|' ||
       OMPHostName                             || '|' ||
       f368                                    || '|' ||
       f121                                    || '|' ||
       f545                                    || '|' ||
       f430                                    || '|' ||
       f430                                    || '|' ||
       f559                                    || '|' ||
       f161                                    || '|' ||
       f276                                    || '|' ||
       f297                                    || '|' ||
       f465                                    || '|'
  from SES_FAF faf,
       wmn_instance_context wmic, wmn_entity wme
 where faf.InstanceID = wmic.INSTANCE_ID
   and wmic.context_id = 
       (select CONTEXT_ID from wmn_context
         where CONTEXT_USE='R')
   and wme.entity_id = wmic.entity_id
 order by InstanceID;

select  substr('RELEASE', 1) from dual;

exit;

