#!/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2010. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/migrate/upgrades/bin/core_inst.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#

RUNTYPE=""
MAXCC="2"

if [[ -z "$SQLPATH" ]] ; then
	export SQLPATH
	SQLPATH=$FLEXPM_HOME/config
fi

setup_envar()
{
    if [[ "x$release"  = "x" ]] ; then
         pro_release=`grep release\=\" ${FLEXPM_HOME}/.profile`
         parse_release=`echo ${pro_release}  | sed s/release\=//g | sed s/\ *//g | sed s/\;\#*[a-z]*//g | sed s/\"//g`
         export release
                release=${parse_release}
    fi

    cd /tmp

    export uid
           uid=$DBUSER

    if [[ -d ${uid} ]] ; then
       rm -rf ${uid}
    fi 

    if [[ ! -d ${uid} ]] ; then
       mkdir ${uid} > /dev/null 2>&1
    fi

    MAXCC=$?
    export MAXCC
    return $MAXCC
}

do_upgrade()
{
    # Ensure oracle paths are correct
    cd ${FLEXPM_HOME}
    export ORACLE_VERS=`basename ${ORACLE_HOME}`
    chmod 755 sw/oracle/preset_oracle_paths
    sw/oracle/preset_oracle_paths -f
      
    if [[ $? -ne 0 ]]
    then
       echo "preset_oracle_paths -f failed"
       exit 2
    fi

    chmod -R 755 $FLEXPM_HOME
    #SEAde68695 - change permission to ftpIN subdirectories back to 775.
    `find $FLEXPM_HOME -type d -follow | grep -i ftpIN | xargs chmod 775`

    pm/adm/setup-exec-bits -set

    if [[ $? -ne 0 ]]
    then
       echo "setup-exec-bits -set failed"
       exit 2
    fi

    # If NOPASSWD is in env, then to switch to encrypt .profile.connect
    if [[ "$NOPASSWD" = "1" ]]
    then
        mv ${FLEXPM_HOME}/.profile.connect.encrypt ${FLEXPM_HOME}/.profile.connect
    fi
    
    # Save the DB password since may not survive with if not supposed to be in
    # the environment.  Removed at end of upgrade
    export save_dbpasswd=${DBPASSWD}
    
    export save_vendor=$vendor
    export save_product=$wm_product
    export save_release=$release
    
    . ./.profile

    export vendor=$save_vendor
    export wm_product=$save_product
    export release=$save_release

    ${FLEXPM_HOME}/pm/adm/crypt_util.ksh -p ${DBUSER} ${save_dbpasswd}

    if [[ $? -ne 0 ]]
    then
       echo "crypt_util.ksh -p ${DBUSER} failed"
       exit 2
    fi
    
    # remove password from env.
    unset save_dbpasswd;

    ## SEAde64875 choose correct version .profile.oracle, Oracle.so MW links
    MAKE=$FLEXPM_HOME/pm/adm/make
    cd ${FLEXPM_HOME}
    ${MAKE} bldfix 2>&1 

    if [[ $? -ne 0 ]]
    then 
       echo "${MAKE} bldfix failed"
       exit 2
    fi
    cd -
    
    # Start upgrade
    if [[ -d "$FLEXPM_HOME/migrate/upgrades/bin" ]]
    then
        cd ${FLEXPM_HOME}/migrate/upgrades/bin
    else
        echo "$FLEXPM_HOME/migrate/upgrades/bin doesn't exist"
        echo " ... aborting"
        exit 2
    fi

    if [[ ! -d "$FLEXPM_HOME/.setup" ]]
    then
	mkdir $FLEXPM_HOME/.setup > /dev/null 2>&1
    fi

    if [[ ! -f "$FLEXPM_HOME/.setup/FlexPM-?????.at" ]]
    then
       touch ${FLEXPM_HOME}/.setup/FlexPM-00000.at
    fi

    ./setup-upgrade -r ${release} -schema 2>&1 

    if [[ $? -ne 0 ]]
    then 
       echo "setup-upgrade -r ${release} -schema"
       exit 2
    fi

    ./setup-upgrade -r ${release} -config 2>&1 

    if [[ $? -ne 0 ]]
    then 
       echo "setup-upgrade -r ${release} -config failed"
       exit 2
    fi

    # valnt00089272 options file contains $DB_CONNECT
    [ -f /var/tmp/${DBUSER}/upgrade/options ] && rm -f /var/tmp/${DBUSER}/upgrade/options

    DBCONN=`${FLEXPM_HOME}/pm/adm/crypt_util.ksh -g ${DBUSER} -c`
    
    echo "\nChecking for errors in packages, view, procedures and/or functions."
    ## Ensure no errors exist in compiled objects
    sqlplus -s /nolog <<END

       set serveroutput on size 1000000;
       WHENEVER SQLERROR EXIT FAILURE;
       VARIABLE return_status NUMBER;
       CONNECT ${DBCONN}
           
       DECLARE
          v_errors INTEGER := 0;
       BEGIN
          :return_status := 0;
          SELECT count(*) 
            INTO v_errors
            FROM user_errors;

          IF v_errors != 0 THEN
             dbms_output.put_line('Errors found: ' || v_errors);
             :return_status := 1;
          END IF;
       END;
       /
       EXIT :return_status;
END

    if [[ $? -ne 0 ]]
    then 
       echo "FAILED: Errors exist in packages, views, procedures and/or functions"
       exit 2
    fi   
    echo "No errors found."
}

do_install()
{
   echo "Install not supported using upgrade package"
   echo "ensure FLEXPM_HOME is set properly"
   exit 2
}

do_cleanup()
{
    if [[ -d /tmp/${uid} ]] ; then
       rm -rf /tmp/${uid}
    fi
}

while getopts m: x
do
   case $x in
       m)
           MODE=$OPTARG
           ;;
   esac
done

if [[  -z "$MODE" ]]
then
   echo "Usage : $0 -m <INSTALL|UPGRADE>"
   exit 2
fi

if [[ "$MODE" = "INSTALL" ]] || [[ "$MODE" = "UPGRADE" ]]
then
   RUNTYPE=${MODE}
else
   echo "Invalid Mode - (${MODE}) . Valid modes are INSTALL and UPGRADE"
   exit 2
fi

setup_envar

if [[ "$RUNTYPE" = "UPGRADE" ]]
then
   echo "Doing upgrade"
   do_upgrade
fi

if [[ "$RUNTYPE" = "INSTALL" ]]
then
   echo "Doing installation"
   do_install
fi

do_cleanup

unset uid

exit $MAXCC
