set serveroutput on size 1000000

PROMPT Updating DXS and DXC columns
---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2008. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/migrate/upgrades/sql/Generic/upgrade_agg.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--

--
-- This script is designed to add missing columns to the
-- DXS and DXC tables. We are only adding a subset of columns
-- that use special aggregations.
--
-- This script DOES NOT create missing DXS or DXC tables. To
-- date we have not seen this issue.
--

PROMPT START upgrade_agg.sql

DECLARE

   -- We already handled 'A' field_aggregators but we
   -- missed the other func_time='avg' aggregations
   CURSOR clist IS
      SELECT UPPER(table_name) table_name, 
         UPPER(table_column) table_column, 
         field_aggregator, 
         field_datatype, 
         agg.func_time
      FROM wmd_dict_field f, wmd_dict_table t, wmd_aggregator_base agg
      WHERE f.table_id = t.table_id
      AND f.field_type = 'C'
      AND t.is_time_series = 'Y'
      AND t.summary_id = 0
      AND field_aggregator NOT IN ('a','d')
      AND f.field_aggregator=agg.aggr_id
      AND upper(agg.func_time) = 'AVG'
      AND f.field_aggregator != 'A'
      order by table_name, table_column;

      -- Shadow tables
      vDXSTableName part_tables.table_name%TYPE;
      vDXCTableName part_tables.table_name%TYPE;

      vCurTableName part_tables.table_name%TYPE;

      vDXSCheck PLS_INTEGER :=0;
      vDXCCheck PLS_INTEGER :=0;

      vDXSColCheck PLS_INTEGER :=0;

      vDataLength PLS_INTEGER;

      vAlterStr VARCHAR2(4000);
      vAlterDataType VARCHAR2(4000);

BEGIN

   -- Get the list of missing tables and columns
   FOR rec IN clist LOOP

      -- Simple cache in case we have multiple fields for the same table
      IF ( vCurTableName IS NULL OR vCurTableName != rec.table_name ) THEN

         SELECT MAX(data_length)
         INTO vDataLength
         FROM user_tab_columns
         WHERE data_type = 'VARCHAR2'
         AND table_name = rec.table_name
         AND column_name = rec.table_column; 

         vCurTableName := rec.table_name;

      END IF;
	       
      -- Get the SM data type ... this should always be a float
      vAlterDatatype :=  create_schema_objects.get_sm_data_type(rec.field_aggregator, 
         rec.table_name,
         rec.table_column, 
         rec.field_datatype, 
         vDataLength );

      -- Tell us where we are ...
      DBMS_OUTPUT.PUT_LINE(' Check: '
         || rec.table_name
         || ' :: '
         || rec.table_column );

      -- Check for the DXS table, assume DXC follows
      vDXSTableName := summarize.GenDestName(rec.table_name, 'XS');
      vDXCTableName := summarize.GenDestName(rec.table_name, 'XC');

      vDXSCheck := 0;

      BEGIN
         SELECT 1 INTO vDXSCheck
         FROM user_tables
         WHERE table_name = vDXSTableName;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            -- DBMS_OUTPUT.PUT_LINE( 'DXS MISSING: ' || vDXSTableName );
            NULL;
      END;

      vDXSColCheck := 0;

      -- Check for field, as the table is present
      IF ( vDXSCheck = 1 ) THEN
         BEGIN
            SELECT 1 INTO vDXSColCheck
            FROM user_tab_columns
            WHERE table_name = vDXSTableName
            AND column_name = rec.table_column;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               -- DBMS_OUTPUT.PUT_LINE( 'DXS COL MISSING: ' 
               --    || vDXSTableName
               --    || ' :: ' 
               --    || rec.table_column );
               NULL;
         END;

         IF ( vDXSColCheck = 0 ) THEN
            -- We need to add the column to DXS and DXC
            vAlterStr := 'ALTER TABLE '
               || vDXSTableName
               || ' ADD '
               || rec.table_column
               || ' '
               || vAlterDataType ;
        
            -- No exception handler, throw the error
            BEGIN
               DBMS_OUTPUT.PUT_LINE( vAlterStr );
               EXECUTE IMMEDIATE vAlterStr;
            END;

            -- Always an integer because its a count
            vAlterStr := 'ALTER TABLE '
               || vDXCTableName
               || ' ADD '
               || rec.table_column
               || ' '
               || 'integer' ;

            -- No exception handler, throw the error
            BEGIN
               DBMS_OUTPUT.PUT_LINE( vAlterStr );
               EXECUTE IMMEDIATE vAlterStr;
            END;

         END IF;

      ELSE

         -- We need to create the tables ... we only check DXS
         DBMS_OUTPUT.PUT_LINE( 'ERROR: The DXS and DXC tables for '
            || vCurTableName
            || ' are missing. Please contact Support. ' );

      END IF;

   END LOOP;
END;
/

PROMPT END upgrade_agg.sql DONE
