---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2003,2010. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/migrate/upgrades/sql/Generic/upgrade_schema.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--

PROMPT START upgrade_schema.sql

BEGIN
   -- Compress job tables.  Do this every upgrade or patch
   
   run_sql( ' alter table task_status move tablespace wm_flexpm ' );
   run_sql( ' alter table active_task move tablespace wm_flexpm ' );
   run_sql( ' alter table active_job move tablespace wm_flexpm ' );
   run_sql( ' alter index pk_active_job rebuild ' );
   run_sql( ' alter index pk_active_task rebuild ' );
   run_sql( ' alter index active_task_unq rebuild ' );
   run_sql( ' alter index task_status_unq rebuild ' );
   run_sql( ' alter index task_status_unq_task rebuild ' );
   run_sql( ' alter index task_status_job rebuild ' );

   -- Core 8.0.6 patch start
   run_sql ( 'ALTER TABLE RG_BUSYHOUR_TMP ADD CONSTRAINT pk_rg_busyhour_tmp PRIMARY KEY (INSTANCE_ID, TSTAMP, RANK) ');

   run_sql( ' ALTER TABLE part_tables '
         || ' ADD ( termination_date date )' );

   run_sql( ' ALTER TABLE part_task '
         || ' ADD ( termination_date date )' );

   run_sql( ' CREATE TABLE WM_MESSAGE_LOG '
         || ' (MESSAGE_ID               INTEGER NOT NULL, TSTAMP DATE NOT NULL, COMPONENT VARCHAR2(60) NOT NULL, '
         || '  MESSAGE_TEXT     VARCHAR2(1000) NOT NULL, SEVERITY VARCHAR2(10), '
         || '  CONSTRAINT               PK_WM_LOG_MESSAGE PRIMARY KEY (TSTAMP, MESSAGE_ID), '
         || '  CONSTRAINT               CK_WM_LOG_MESSAGE_SEV CHECK (SEVERITY IN (''HIGH'',''MEDIUM'',''LOW'')) ENABLE '
         || ' ) ORGANIZATION INDEX PARTITION BY RANGE(tstamp) '
         || '( PARTITION ' || 'P' || TO_CHAR(trunc(sysdate+1), 'YYYYMMDDHH24')
         || ' VALUES LESS THAN (TO_DATE(''' || TO_CHAR(trunc(sysdate+2), 'YYYYMMDDHH24') || ''', ''YYYYMMDDHH24'')) ) '
         || ' TABLESPACE TRAFFIC_MEDIUM' );

   run_sql( ' CREATE SEQUENCE  wm_log_message_seq  MINVALUE 1 NOMAXVALUE INCREMENT BY 1 START WITH 1 CACHE 100 NOORDER  NOCYCLE ' );

   run_sql( 'CREATE INDEX ind_Instance_Entity_subtype ON wmn_Instance ( Entity_ID, SubType_ID ) '
         || 'tablespace wm_flexpm' );

   run_sql( 'CREATE INDEX inst_cxt_instance_id ON wmn_Instance_Context '
         || '    (Instance_Id) '
         || '    TABLESPACE wm_flexpm STORAGE (FREELISTS 4) ' );

   run_sql( 'CREATE INDEX inst_cxt_entity_id ON wmn_Instance_Context '
         || '    (entity_Id) '
         || '    TABLESPACE wm_flexpm STORAGE (FREELISTS 4) ' );

   DELETE FROM EDS_PRIMARYKEY e
    WHERE NOT EXISTS (SELECT 1 FROM wmd_field_def f where f.field_id = e.field_id);
   COMMIT;

   run_sql( ' ALTER TABLE EDS_PRIMARYKEY ENABLE CONSTRAINT FK_EDS_PMRYKEYFIELD_ID ' );

   DELETE FROM PEIK_EXTRA_FLD_PROPERTY e
    WHERE NOT EXISTS (SELECT 1 FROM wmd_field_def f where f.field_id = e.field_id);
   COMMIT;

   run_sql( ' ALTER TABLE PEIK_EXTRA_FLD_PROPERTY ENABLE CONSTRAINT FK_PEIK_EXTRA_FLD_PRPT_FLD_ID ' );

   DELETE FROM PM_REPORTGROUPING e
    WHERE NOT EXISTS (SELECT 1 FROM wmd_field_def f where f.field_id = e.field_id);
   COMMIT;

   run_sql( ' ALTER TABLE PM_REPORTGROUPING ENABLE CONSTRAINT FK_PM_RPTGRPING_FIELD_ID ' );

   DELETE FROM WMD_EDS_DEF e
    WHERE NOT EXISTS (SELECT 1 FROM wmd_field_def f where f.field_id = e.field_id);
   COMMIT;

   run_sql( ' ALTER TABLE WMD_EDS_DEF ENABLE CONSTRAINT FK_EDSDEF_FLDID ' );

   DELETE FROM WMD_FIELD_STYLE e
    WHERE NOT EXISTS (SELECT 1 FROM wmd_field_def f where f.field_id = e.field_id);
   COMMIT;

   run_sql( ' ALTER TABLE WMD_FIELD_STYLE ENABLE CONSTRAINT WMD_FIELD_STYLE_C1 ' );

   DELETE FROM WMD_UDC_RENAMEHIST e
    WHERE NOT EXISTS (SELECT 1 FROM wmd_field_def f where f.field_id = e.field_id);
   COMMIT;

   run_sql( ' ALTER TABLE WMD_UDC_RENAMEHIST ENABLE CONSTRAINT UDC_RENAME_FLD_FKY ' );

   DELETE FROM WMD_UDR_KEYS e
    WHERE NOT EXISTS (SELECT 1 FROM wmd_field_def f where f.field_id = e.field_id);
   COMMIT;

   run_sql( ' ALTER TABLE WMD_UDR_KEYS ENABLE CONSTRAINT UDRKEY_FIELD_FKY ' );

   run_sql( ' CREATE INDEX IDX_RG_SNAP_RPT_ATTR_TASKID ON RG_SNAP_REPORT_ATTR(task_id) ' );
   run_sql( ' alter table rg_dtscope drop constraint fk_rg_dtscope ' );
   run_sql( ' alter table rg_trunc drop constraint fk_rg_trunc_tid ' );
   run_sql( ' alter table rg_tstamp_range drop constraint fk_rg_tstamp_range_tid ' );
   run_sql( ' alter table rg_scenario drop constraint fk_rg_scenario ' );
   run_sql( ' alter table rg_scenario_remap drop constraint fk_rg_scenario_remap ' );
   run_sql( ' alter table rg_instance_tree_mv drop constraint fk_rg_instance_tree_mv ' );
   run_sql( ' alter table rg_busyhour_mv drop constraint fk_rg_busyhour_mv ' );
   -- Core 8.0.6 patch end

   --SA/BH Export-Import merge into core - START
   run_sql( 'CREATE OR REPLACE TYPE T_EXPORTIMPORT_IDS IS TABLE OF NUMBER ');
   --SA/BH Export-Import merge into core - END

   -- Action Logging Changes Start
   
   --  pm_action_logging_areas contains meta data on the logging areas . this table is linked to the pm_action_logging.area_id
   run_sql( ' CREATE TABLE pm_action_logging_areas ( '
        || ' ID       INTEGER      NOT NULL, '
        || ' NAME     VARCHAR2(40) NOT NULL, '
        || ' CONSTRAINT PK_ACTION_LOGGING_AREA PRIMARY KEY (ID) '
        || ' ) '
        || 'TABLESPACE WM_CLIENT' );

   -- logs the users actions
   -- one record per user action
   -- it is maitained by partition maintence
   -- the action_log.ksh script querys it to generate logs in the form of files
   run_sql( ' CREATE TABLE pm_action_logging ( '
        || ' id integer not null, '
        || ' priority integer not null, '
        || ' tstamp date not null, '
        || ' user_id integer not null, '
        || ' severity  integer not null, '
        || ' area_id integer not null, '
        || ' message varchar2(2000),  '
        || ' CONSTRAINT CHK_ACTION_LOGGING_SEVERITY CHECK (severity IN (5, 6)) ENABLE, '
        || ' CONSTRAINT CHK_ACTION_LOGGING_PRIORITY CHECK (priority IN (133, 134)) ENABLE, '
        || ' CONSTRAINT FK_ACTION_LOGGING_AREA FOREIGN KEY (AREA_ID) REFERENCES PM_ACTION_LOGGING_AREAS (ID) ENABLE, '
        || ' CONSTRAINT pk_PM_ACTION_LOGGING PRIMARY KEY (id,tstamp)) '
        || ' ORGANIZATION INDEX PARTITION BY RANGE(tstamp) (PARTITION P' ||to_char(wm_date.trunc_to_last_period(SYSDATE, 'D'), 'YYYYMMDDHH24') 
        || ' VALUES LESS THAN(to_date('|| '''' || to_char(wm_date.trunc_to_last_period(SYSDATE, 'D')+1, 'YYYYMMDDHH24')||''''||',''YYYYMMDDHH24''))) ' 
        || ' PCTFREE 5  TABLESPACE TRAFFIC_SMALL ' );

   -- sequence for pm_action_logging.id . 
   run_sql( 'CREATE SEQUENCE PM_ACTION_LOGGING_SEQ START WITH 1 INCREMENT BY 1 NOCYCLE NOCACHE' );

   -- Action Logging Changes End   


   -- Analysis Framework - START

   -- Sequence for Soft Alarms.
   run_sql( 'CREATE SEQUENCE SA_ENQUEUE_SEQ MINVALUE 1 MAXVALUE 2147483647 START WITH 1 ' );


   -- Analysis Framework - END  

   -- Instance Exclusion Changes Start
   
  
   -- wm_feature table
   run_sql( 'CREATE TABLE WM_FEATURE ('
   || ' FEATURE_ID NUMBER NOT NULL,'
   || ' FEATURE_NAME VARCHAR2(100) NOT NULL,'
   || ' CONSTRAINT "PK_WM_FEATURE" PRIMARY KEY ("FEATURE_ID")'
   || ' ) ');
   
   
   -- wm_device_excluded table
   run_sql( 'CREATE TABLE WM_DEVICE_EXCLUDED ( '
   || ' DEVICE_EXCLUDED_ID NUMBER(*,0) NOT NULL, '
   || ' FEATURE_ID NUMBER(*,0) NOT NULL, '
   || ' ENTITY_ID NUMBER(*,0) NOT NULL, '
   || ' INSTANCE_ID NUMBER(*,0), '
   || ' FULL_KEY VARCHAR2(900) NOT NULL, '
   || ' CREATED_DATE DATE, '
   || ' CONSTRAINT "PK_DEVICE_EXCLUDED" PRIMARY KEY ("DEVICE_EXCLUDED_ID"), '
   || ' CONSTRAINT "FK_FEATURE" FOREIGN KEY ("FEATURE_ID") REFERENCES WM_FEATURE ("FEATURE_ID"), '
   || ' CONSTRAINT "FK_ENTITY" FOREIGN KEY ("ENTITY_ID") REFERENCES WMN_ENTITY ("ENTITY_ID"), '
   || ' CONSTRAINT "FK_INSTANCE" FOREIGN KEY ("INSTANCE_ID") REFERENCES WMN_INSTANCE ("INSTANCE_ID") ON DELETE CASCADE)  ');
   
   -- indexes on wm_device_excluded
  run_sql( 'CREATE INDEX WM_DEVICE_EXCLUDED_INSTANCE ON WM_DEVICE_EXCLUDED(INSTANCE_ID) ');
  run_sql( 'CREATE INDEX WM_DEVICE_EXCLUDED_ENTITY ON WM_DEVICE_EXCLUDED(ENTITY_ID) ');


  -- sequence on wm_device_excluded
  run_sql( 'CREATE SEQUENCE WM_DEVICE_EXCLUDED_SEQ START WITH 1 INCREMENT BY 1 NOCYCLE NOCACHE ');

   
   -- Instance exclusion Changes End

   -- ====================================================== -- 
   -- Report Forecasting - START
   -- ====================================================== --
   
   -- C++ repgen sequence
   run_sql( 'CREATE SEQUENCE seq_rg_trending_c_fingerprint '
        || '    START WITH 1 '
        || '    INCREMENT BY 1 '
        || '    MINVALUE 1 '
        || '    MAXVALUE 99999999999999999999 ' 
        || '    CACHE 20 '
        || '    NOCYCLE '
        || '    NOORDER' );

   -- C++ repgen tables
   run_sql( 'CREATE TABLE rg_trending_c_bhvalue ( '
        || '    task_id        INTEGER, '
        || '    fingerprint_id INTEGER, '
        || '    instance_id    NUMBER(38), '
        || '    utc_timestamp  NUMBER(38), '
        || '    bh_value       FLOAT, '
        || '    CONSTRAINT pk_rg_trending_c_bhvalue PRIMARY KEY (task_id, fingerprint_id, instance_id, utc_timestamp)'
        || ')' 
        || 'ORGANIZATION INDEX NOLOGGING PCTFREE 0' 
        || 'TABLESPACE TRAFFIC_JUMBO' );

   run_sql( 'CREATE TABLE rg_trending_c_fingerprint ( '
        || '    fingerprint_id INTEGER, '
        || '    task_id        INTEGER, '
        || '    regression_id  VARCHAR(200),'
        || '    CONSTRAINT pk_rg_trending_c_fingerprint PRIMARY KEY (fingerprint_id) '
        || ') '
        || 'ORGANIZATION INDEX NOLOGGING PCTFREE 0 ' 
        || 'TABLESPACE TRAFFIC_JUMBO' );
   
   run_sql( 'CREATE TABLE rg_trending_c_data_store ( '
        || '    task_id        INTEGER, '
        || '    scenario_id    INTEGER, '
        || '    reportdate     DATE, '
        || '    fingerprint_id INTEGER, '
        || '    instance_id    INTEGER, '
        || '    slope          FLOAT, '
        || '    intercept      FLOAT, '
        || '    nrows          INTEGER, '
        || '    CONSTRAINT pk_rg_trending_c_data_store PRIMARY KEY (scenario_id, reportdate, fingerprint_id, instance_id) '
        || ') '
        || 'ORGANIZATION INDEX PCTFREE 10 '  
        || 'TABLESPACE TRAFFIC_JUMBO' );

   -- Java repgen tables
   run_sql( 'CREATE GLOBAL TEMPORARY TABLE rg_regression_tmp( '
        || '    regression_id VARCHAR2(200) NOT NULL, '
        || '    tstamp        DATE NOT NULL, '
        || '    instance_id   INTEGER, '
        || '    slope         FLOAT, '
        || '    intercept     FLOAT, '
        || '    FULLKEY       VARCHAR2(900) '
        || ') '
        || 'ON COMMIT PRESERVE ROWS' );

   run_sql( 'CREATE GLOBAL TEMPORARY TABLE RG_REGRESSION_DATA_TMP ( '
        || '    FULLKEY VARCHAR2(900), '
        || '    TSTAMP  DATE NOT NULL, '
        || '    VALUE   FLOAT '
        || ') '
        || 'ON COMMIT PRESERVE ROWS' );

   run_sql( 'CREATE GLOBAL TEMPORARY TABLE xr_dtscope_sbh_tmp ( '
        || '    tstamp_id INTEGER, ' 
        || '    tstamp    DATE '
        || ') '
        || 'ON COMMIT PRESERVE ROWS' );   

   -- System Capacity Planning table
   run_sql( 'CREATE TABLE wmn_field_attribute ( '
        || '    field_id          INTEGER NOT NULL, '
        || '    entity_id         INTEGER NOT NULL, '
        || '    attribute_name    VARCHAR(64) NOT NULL, '
        || '    default_num_value FLOAT, '
        || '    user_num_value    FLOAT, '
        || '    CONSTRAINT pk_wmn_field_attribute PRIMARY KEY (field_id, entity_id, attribute_name), '
        || '    CONSTRAINT fk_wmn_field_attribute_eid FOREIGN KEY (entity_id) REFERENCES wmn_entity(entity_id), '
        || '    CONSTRAINT fk_wmn_field_attribute_fid FOREIGN KEY (field_id) REFERENCES wmd_field_def(field_id) '
        || ') '
        || 'TABLESPACE wm_flexpm' );   

   -- ====================================================== -- 
   -- Report Forecasting - FINISH
   -- ====================================================== --
 
   -- JLoader changes START
   
   run_sql( 'ALTER TABLE jloader_pdo_table_tmp ADD (key_columns VARCHAR2(1000))' );
   run_sql( 'ALTER TABLE jloader_pdo_table_stage ADD (key_columns VARCHAR2(1000))' );

   -- JLoader changes FINISH

   -- Rehoming changes START
   
   run_sql( 'DROP TABLE rh_trend_tree' );
   run_sql( 'DROP TABLE rh_map_tmp' );
   run_sql( 'DROP TABLE rh_active_entity_tmp' );
   run_sql( 'DROP TABLE rh_log_table' );
   run_sql( 'DROP INDEX idx_rh_reap_tmp' );
   run_sql( 'DROP INDEX idx1_rh_reap_tmp' );

   run_sql( 'CREATE GLOBAL TEMPORARY TABLE rh_trend_tree'
        || '( '
        || ' trend_key       varchar2(900), '
        || ' entity_ID       INTEGER, '
        || ' instanceID      INTEGER, '
        || ' started         DATE, '
        || ' ended           DATE, '
        || ' fullkey         VARCHAR2(900), '
        || ' localkey        VARCHAR2(90), '
        || ' instance_name   VARCHAR2(30) '
        || ') ON COMMIT PRESERVE ROWS ' );

   run_sql( 'CREATE GLOBAL TEMPORARY TABLE rh_map_tmp'
        || '( '
        || ' entityID       INTEGER, '
        || ' active_insID   INTEGER, '
        || ' instanceID     INTEGER, '
        || ' started        DATE, '
        || ' ended          DATE, '
        || ' localkey       VARCHAR2(90) '
        || ') ON COMMIT PRESERVE ROWS ' );

   run_sql( 'CREATE GLOBAL TEMPORARY TABLE rehoming_reap_tmp '
        || '( '
        || '    entity_id                 INTEGER, '
        || '    active_insid              INTEGER, '
        || '    instance_id               INTEGER, '
        || '    tstamp                    DATE '
        || ') ON COMMIT PRESERVE ROWS ');

   run_sql( 'CREATE GLOBAL TEMPORARY TABLE rh_active_entity_tmp '
        || '( '
        || '   entityID   INTEGER NOT NULL '
        || ' ) ON COMMIT PRESERVE ROWS ');


   run_sql( 'CREATE INDEX idx_rh_reap_tmp_insid on rehoming_reap_tmp(instance_id, tstamp)' );
   run_sql( 'CREATE INDEX idx_rh_reap_tmp_actid on rehoming_reap_tmp(active_insid, tstamp)' );
   run_sql( 'CREATE INDEX ind_rh_trend_tree on rh_trend_tree(trend_key, entity_ID, instanceID)' );
   run_sql( 'CREATE INDEX ind_rh_map_tmp on rh_map_tmp(active_insID)');
   

   -- Rehoming changes FINISH
   
   -- ClassicClient merge start
   run_sql('ALTER TABLE sa_softalarm ADD (ALARM_ID  NUMBER(*,0) NULL) ');    
   run_sql('CREATE UNIQUE INDEX SA_ALARM_ID_UNQ ON SA_SOFTALARM ("ALARM_ID") ');
   
   -- Classic Client merge finish

   -- Japanese Lite START --
   
   run_sql('CREATE TABLE pm_language('
   || ' id              INTEGER NOT NULL,' 
   || ' display_name    NVARCHAR2(128) NOT NULL,'
   || ' code            VARCHAR2(20) NOT NULL,'
   || ' is_default      CHAR(1) DEFAULT ''N'' NOT NULL,'
   || ' CONSTRAINT pk_pm_language PRIMARY KEY(id),'
   || ' CONSTRAINT uq_pm_language_code UNIQUE(code),'
   || ' CONSTRAINT ck_pm_language_default CHECK (is_default IN (''Y'', ''N''))'
   || ' )' 
   || ' TABLESPACE wm_flexpm');


  run_sql('CREATE SEQUENCE seq_pm_language'
    ||   ' START WITH 1' 
    ||   ' INCREMENT BY 1' 
    ||   ' MINVALUE 1' 
    ||   ' MAXVALUE 99999999999999999999' 
    ||   ' CACHE 20' 
    ||   ' NOCYCLE'
    ||   ' NOORDER');
  

  run_sql('CREATE TABLE pm_user_language'
    ||   '('
    ||   '    user_id         INTEGER NOT NULL, '
    ||   '    language_id     INTEGER NOT NULL,'
    ||   '    funct_scope     VARCHAR2(128) DEFAULT ''DEFAULT'' NOT NULL,'
    ||   '    CONSTRAINT fk_userlanguage_user FOREIGN KEY(user_id) REFERENCES pm_user(id) ON DELETE CASCADE,'
    ||   '    CONSTRAINT fk_userlanguage_language FOREIGN KEY(language_id) REFERENCES pm_language(id),'
    ||   '    CONSTRAINT uq_userlanguage_userid UNIQUE(user_id, language_id, funct_scope)'
    ||   ') '
    ||   ' TABLESPACE wm_flexpm');
    
    wm_patch.ALTER_COLUMN_TO_NCHAR('pm_document', 'NAME',256);
    run_sql ( 'alter table pm_document modify (NAME not null)');    
    run_sql ( 'CREATE UNIQUE INDEX unq_pm_doc_fldid_name  ON pm_document(parent_doc_id, UPPER(name))');
    run_sql ( 'CREATE INDEX idx_pm_document_name ON pm_document(name)');

   --Japanese Lite FINISH--
   
  -- ============================ --
  -- Report Import/Export - START
  -- ============================ --
  
  -- Want to extend the 'PM_REPORT.NAME' column from 64 -> 100 characters.
  -- To do this, have to (1) drop the index on the column, (2) alter the column and finally, (3) recreate the index. 
   
  run_sql( 'DROP INDEX UNQ_Report' );

  run_sql( ' ALTER TABLE pm_report '
         || ' MODIFY ( name VARCHAR2(100) )' );
         
  -- valnt00091397: Reports with names longer that 64 characters fail to run - START
  run_sql( ' ALTER TABLE rg_tasklist '
         || ' MODIFY ( reportname VARCHAR2(100) )' );
  -- valnt00091397: Reports with names longer that 64 characters fail to run - END

  run_sql( 'CREATE UNIQUE INDEX UNQ_Report ON PM_Report(FolderId, UPPER(Name), Category, Entity_Id) '
         || ' TABLESPACE WM_Client' );
  
  -- ============================ --
  -- Report Import/Export - END
  -- ============================ --
  
  END;
/

-- Sentry Loader Start

PROMPT Sentry Loader Changes - PM_FILE_CONFIG Table

DECLARE

  TYPE stringtable IS TABLE OF VARCHAR2(64);
  ignoresysvar    wm_system_values_v.NAME%TYPE := 'ignoreZeroLengthDataFileTypes';
  permittedsysvar wm_system_values_v.NAME%TYPE := 'permittedZeroLengthDataFileTypes';
  ignoreflag      wm_system_values_v.VALUE%TYPE;
  filetypes       wm_system_values_v.VALUE%TYPE;
  zerolengths     stringtable;

  FUNCTION str2tbl(p_str   IN VARCHAR2,
                   p_delim IN VARCHAR2 DEFAULT ',') RETURN stringtable AS
    l_str  LONG DEFAULT p_str || p_delim;
    l_n    NUMBER;
    l_data stringtable := stringtable();
  BEGIN
    LOOP
      l_n := instr(l_str, p_delim);
      EXIT WHEN(nvl(l_n, 0) = 0);
      l_data.EXTEND;
      l_data(l_data.COUNT) := ltrim(rtrim(substr(l_str, 1, l_n - 1)));
      l_str := substr(l_str, l_n + length(p_delim));
    END LOOP;
    RETURN l_data;
  END;

BEGIN
  -- make changes to PM_FILE_CONFIG TABLE
  run_sql( ' ALTER TABLE pm_file_config ADD (SWEEP_ACTION VARCHAR2(20)) ' );
  run_sql( ' ALTER TABLE pm_file_config ADD (ALLOW_ZERO_LENGTH CHAR(1)) ' );
  run_sql( ' ALTER TABLE pm_file_config add CONSTRAINT ck_file_config_sweep_action CHECK (sweep_action IN (''MOVE'', ''DELETE'')) ' );   
  -- get the ingore flag
  ignoreflag := wm_sysval.getstring(pname => ignoresysvar);
  -- if it is true then we chack the permitted values and update them to allow zero lengths
  IF ignoreflag = 'true' THEN
    filetypes   := wm_sysval.getstring(pname => permittedsysvar);
    zerolengths := str2tbl(filetypes);
    FOR i IN 1 .. zerolengths.COUNT LOOP
      run_sql( ' UPDATE pm_file_config fc '
            || '    SET fc.allow_zero_length = ''Y'''
            || '  WHERE fc.reg_expression = zerolengths('|| i ||')' );
    END LOOP;
  END IF;
  -- update all remaining values to 'N'
  run_sql( ' UPDATE pm_file_config fc '
        || '    SET fc.allow_zero_length = ''N'''
        || '  WHERE fc.allow_zero_length IS NULL ' );

  COMMIT;

  --now we can change make it not null and add constraint
  run_sql(' ALTER TABLE pm_file_config MODIFY (allow_zero_length NOT NULL) ');
  run_sql(' ALTER TABLE pm_file_config add CONSTRAINT ck_pm_file_config_2 CHECK (allow_zero_length IN (''Y'', ''N'')) '); 
  -- finally remove the system values
  wm_sysval.deletesysval(pname => ignoresysvar);
  wm_sysval.deletesysval(pname => permittedsysvar);

  COMMIT;
END;
/

PROMPT Sentry Loader Changes - Other Tables

BEGIN
  -- pm_sentry_file_completed
  run_sql( ' ALTER TABLE pm_sentry_file_completed ADD (sweep_status VARCHAR2(20)) ' );
  run_sql( ' ALTER TABLE pm_sentry_file_completed ADD CONSTRAINT ck_sentry_file_comp_sweep_sts CHECK (sweep_status IN (''MOVED'', ''DELETED'', ''ERROR'')) ' );   
  
END;
/

PROMPT New column in wm_file_control

BEGIN
  run_sql( ' ALTER TABLE wm_file_control ADD (is_show char(1) default ''Y'') ' );
END;
/

-- Sentry Loader End


PROMPT New column in sa_softalarm_snap

BEGIN
  run_sql( ' ALTER TABLE sa_softalarm_snap ADD (ALARM_ID NUMBER(38)) ' );
END;
/

--Delete NE Feature

BEGIN
--This table will create a hierarchy list of instance that need to be deleted from the parent instance.
run_sql('CREATE GLOBAL TEMPORARY TABLE wmn_delete_instance_tmp ' ||
' ( ' ||
' LEV NUMBER, ' ||
' INSTANCE_ID INTEGER, ' ||
' ENTITY_ID   INTEGER, ' ||
' FULLKEY     VARCHAR2(900) ' ||
' ) ' ); 

run_sql('CREATE INDEX wmn_delete_instance_idx ON wmn_delete_instance_tmp (entity_id, instance_id)');

--This table will hold a list of table that need to be checked by the delete instance procedure.. 
run_sql('CREATE TABLE wmn_delete_instance_tables ' || 
 ' (  ' ||
 ' Table_name         VARCHAR2(30), ' ||
 ' Column_name        VARCHAR2(30), ' ||
 ' Findkey     	    CHAR(1), ' ||
 ' Entity_Column      VARCHAR2(30), ' ||
 ' Entity_type     	CHAR(1), ' ||
 ' CONSTRAINT FindKey_ck CHECK (FINDKEY IN (''I'',''F'')), ' ||
 ' CONSTRAINT Entitytype_ck CHECK (Entity_type IN (''N'',''I'')), ' ||
 ' CONSTRAINT Instable_pky PRIMARY KEY (Table_name, Column_name,Findkey) ' ||
 ' ) ' ||
 ' TABLESPACE wm_flexpm ');

 --This table will hold detail and summary messages which will be written out to a Summary and Details file.
run_sql('CREATE TABLE wmn_delete_instance_msg_log  ' ||
' (  ' ||
' Line_num 	INTEGER,  ' ||
' Message 	VARCHAR2(1000),  ' ||
' Report_type 	CHAR(1)       NOT NULL,  ' ||
'      CONSTRAINT Rpt_Sum_Det_ck CHECK (REPORT_TYPE IN (''S'',''D''))  ' ||
' )  ' ||
' TABLESPACE wm_flexpm ');
 
--
--drop constraint
--
run_sql('ALTER TABLE wm_available_data DROP CONSTRAINT FK_AVAI_DATA_INSTANCE_ID');
 
END;
/
-- 8.0.7.4

PROMPT Fix invalidly named constraints in wmn_field_attribute

BEGIN
  run_sql('ALTER TABLE wmn_field_attribute RENAME CONSTRAINT pk_wmd_field_attribute TO pk_wmn_field_attribute');
  run_sql('ALTER TABLE wmn_field_attribute RENAME CONSTRAINT fk_wmd_field_attribute_eid TO fk_wmn_field_attribute_eid');
  run_sql('ALTER TABLE wmn_field_attribute RENAME CONSTRAINT fk_wmd_field_attribute_fid TO fk_wmn_field_attribute_fid');
END;
/

--Indexes to prevent deadlocking in pm_group_user.deleteuser
PROMPT New indexes in PM_USER

BEGIN
  run_sql( 'CREATE INDEX pm_user_folderid_idx ON PM_USER(FOLDERID)' );
END;
/
PROMPT New indexes in PM_FOLDER

BEGIN
  run_sql( 'CREATE INDEX pm_folder_userid_idx ON PM_FOLDER(USERID)' );
END;
/

PROMPT New indexes in PM_DOCUMENT

BEGIN
  run_sql( 'CREATE INDEX idx_pm_document_userid ON pm_document(user_id)' );
  run_sql( 'CREATE INDEX idx_pm_document_grpid ON pm_document(group_id)' );
  
END;
/

PROMPT New column in Task_Filename

BEGIN
  run_sql( ' ALTER TABLE Task_Filename ADD (SizeInRows Integer) ' );
END;
/
PROMPT END upgrade_schema.sql DONE


