---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2003,2010. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/migrate/upgrades/sql/Generic/upgrade_views.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--

PROMPT START upgrade_views.sql

-- Start 8.0.6 patches
PROMPT
PROMPT Updating en_schedunproc_evnts_v...
@ $FLEXPM_HOME/db/Oracle8i/event/en_schedunproc_evnts_v.sql
-- End 8.0.6 patches


-- BH SA Import Start
PROMPT
PROMPT Updating exp_sa_job_group_view...
CREATE OR REPLACE VIEW exp_sa_job_group_view AS
  SELECT  id,folderid, userid, name, description, changed, group_type
    FROM  job_group jg,
          pm_groups pg
   WHERE  jg.group_type = pg.group_id AND UPPER(PG.group_name) = 'SOFT ALARMS';

show errors view exp_sa_job_group_view;
-- BH SA Import End
   
-- ====================================================== -- 
-- Report Forecasting - START
-- ====================================================== --
PROMPT
PROMPT Updating rg_trending_c_bhvalue_v...
CREATE OR REPLACE VIEW rg_trending_c_bhvalue_v AS
   SELECT bhv.task_id, bhv.fingerprint_id, f.regression_id,
          bhv.instance_id, bhv.bh_value, bhv.utc_timestamp                  
     FROM rg_trending_c_bhvalue bhv, rg_trending_c_fingerprint f
    WHERE bhv.fingerprint_id = f.fingerprint_id;

show errors view rg_trending_c_bhvalue_v;

PROMPT
PROMPT Updating rg_trending_c_store_v...
CREATE OR REPLACE VIEW rg_trending_c_store_v AS
   SELECT s.task_id, s.scenario_id, s.reportdate, s.fingerprint_id, f.regression_id,
          s.instance_id, s.slope, s.intercept, s.nrows
     FROM rg_trending_c_data_store s, rg_trending_c_fingerprint f
    WHERE s.fingerprint_id = f.fingerprint_id
    ORDER by s.task_id DESC;

show errors view rg_trending_c_store_v;
   -- ====================================================== -- 
   -- Report Forecasting - FINISH
   -- ====================================================== --
   

-- Instance Exclusion Start

-- *************************************
-- Create the view WM_DEVICE_EXCLUDED_VW
-- *************************************
PROMPT
PROMPT Updating WM_DEVICE_EXCLUDED_VW...
CREATE OR REPLACE VIEW WM_DEVICE_EXCLUDED_VW
AS
SELECT connect_by_root onion.feature_id feature_id,
       wic.entity_id   entity_id,
       wic.instance_id instance_id
  FROM wmn_instance_context wic,
       (SELECT DISTINCT (wde.instance_id),         --get all instances already defined in list
                         wde.feature_id
          FROM wm_device_excluded wde
         WHERE wde.instance_id IS NOT NULL
        UNION         -- now deal with any 'ALL' entries in list
        SELECT wic.instance_id,
               wde.feature_id
          FROM wmn_instance_context wic,
               wmn_entity           we,
               wmn_context          wc,
               wm_device_excluded   wde
         WHERE upper(wde.full_key) = 'ALL'
           AND we.entity_id = wic.entity_id
           AND we.entity_id = wde.entity_id
           AND wic.context_id = wc.context_id
           AND wc.context_use = 'R'
        UNION -- finally get instances that are in list but do not have an instance id in the list (i.e they may have been added to the system later)
        SELECT wic.instance_id,
               wde.feature_id
          FROM wmn_instance_context wic,
               wmn_entity           we,
               wmn_context          wc,
               wm_device_excluded   wde
         WHERE wic.fullkey = wde.full_key
           AND we.entity_id = wic.entity_id
           AND we.entity_id = wde.entity_id
           AND wic.context_id = wc.context_id
           AND wc.context_use = 'R'
           AND wde.instance_id IS NULL
           AND upper(wde.full_key) != 'ALL') onion
 WHERE wic.context_id =
       (SELECT wc.context_id FROM wmn_context wc WHERE wc.context_use = 'R')
   AND onion.instance_id(+) = wic.instance_id
 START WITH wic.instance_id = onion.instance_id
CONNECT BY PRIOR wic.instance_id = wic.parentinstance_id
       AND wic.context_id = PRIOR wic.context_id;

show errors view WM_DEVICE_EXCLUDED_VW;

PROMPT
BEGIN
-- **************************************************
-- Create the materialized view WM_DEVICE_EXCLUDED_REPORTS_MV  (Using run_sql because can not replace materialized view)
-- **************************************************
run_sql( 'CREATE MATERIALIZED VIEW WM_DEVICE_EXCLUDED_REPORTS_MV '
|| '      REFRESH COMPLETE '
|| '     START WITH SYSDATE '
|| '     NEXT  SYSDATE + 1/48 '
|| '     WITH PRIMARY KEY '
|| '     AS '
|| '   SELECT DISTINCT(instance_id) FROM wm_device_excluded_vw wdev '
|| '   where wdev.feature_id IN (SELECT feature_id from wm_feature where feature_name in (''Reporting'', ''All''))  ');

-- **************************************************
-- Create the materialized view WM_DEVICE_EXCLUDED_AVAIL_MV  (Using run_sql because can not replace materialized view)
-- **************************************************
run_sql( 'CREATE MATERIALIZED VIEW WM_DEVICE_EXCLUDED_AVAIL_MV '
|| '       REFRESH COMPLETE '
|| '     START WITH SYSDATE '
|| '     NEXT  SYSDATE + 1/48 '
|| '     WITH PRIMARY KEY '
|| '     AS '
|| '   SELECT DISTINCT(instance_id) FROM wm_device_excluded_vw wdev '
|| '   where wdev.feature_id IN (SELECT feature_id from wm_feature where feature_name in (''DataAvailability'', ''All''))  ');

-- Instance Exclusion End

END;
/

-- Sentry Loader Changes Start

PROMPT
PROMPT Updating sentry loader views...
@ $FLEXPM_HOME/db/Oracle8i/sentryloader/createViews.sql 
-- Sentry Loader Changes End

PROMPT END upgrade_views.sql DONE
