#!/bin/ksh


#
#  Generate .profile.custom from .profile.custom_
#  as a PRE step for MotoR23 to Core 807 upgrade
#  and preserve custom values found in environment
#


function quit {
	echo ${1:-""}
	exit ${2:-0}
}

function die {
	quit "$@" 1
}


PROFILE_BKUP=".profile.custom.bak-$$"

function rollback {
	[ -f ${PROFILE_BKUP} ] && mv ${PROFILE_BKUP} .profile.custom
	exit
}

trap rollback  1
trap rollback  2
trap rollback  3
trap rollback 15

[ -z "${FLEXPM_HOME}" ] && die "\$FLEXPM_HOME not set."
[ -d "${FLEXPM_HOME}" ] || die "\$FLEXPM_HOME is not a directory [${FLEXPM_HOME}]."
cd ${FLEXPM_HOME}
[ -f .profile.custom_ ] || die ".profile.custom_ not a file."
cp .profile.custom_ .profile.custom
chmod 644 .profile.custom


sed -e 's!#    DB_CONNECT="flexpm@flexpm"!    DB_CONNECT="'${DBUSER}@${DBSID}'"!g' \
	.profile.custom > .profile.tmp1
sed -e 's!#    PORT_GROUP="6440"!    PORT_GROUP="'${PORT_GROUP}'"!g' \
	.profile.tmp1 > .profile.tmp2
sed -e 's!#    . ${FLEXPM_HOME}/.profile.connect!    .  ${FLEXPM_HOME}/.profile.connect!g' \
	.profile.tmp2 > .profile.tmp3

if [ "${TNS_ADMIN}" = "${ORACLE_BASE}/network/admin" ]
then
	sed -e 's!#    TNS_ADMIN="${ORACLE_BASE}/network/admin"!    TNS_ADMIN="${ORACLE_BASE}/network/admin"!g' \
		.profile.tmp3 > .profile.tmp4
	mv .profile.tmp4 .profile.custom
else
	mv .profile.tmp3 .profile.custom
fi
rm -f .profile.tmp*

cp .profile.custom ${PROFILE_BKUP}

# Record each LDAP variable found in the environment
if [ -n "${LDAP_SERVER}" ]
then
	sed -e 's!#    export LDAP_SERVER=localhost!    export LDAP_SERVER="'${LDAP_SERVER}'"!g' \
		.profile.custom > .profile.tmp
	mv .profile.tmp .profile.custom
fi

if [ -n "${LDAP_PORT}" ]
then
	sed -e 's!#    export LDAP_PORT=389!    export LDAP_PORT="'${LDAP_PORT}'"!g' \
		.profile.custom > .profile.tmp
	mv .profile.tmp .profile.custom
fi

if [ -n "${LDAP_SUFFIX}" ]
then
	sed -e 's!#    export LDAP_SUFFIX=wa.watchmark.com!    export LDAP_SUFFIX="'${LDAP_SUFFIX}'"!g' \
		.profile.custom > .profile.tmp
	mv .profile.tmp .profile.custom
fi

# Set Java VM size variables to 2048m
sed -e 's!#    export JAVA_MIN_VM_SIZE=\"512m\"!    export JAVA_MIN_VM_SIZE=\"2048m\"!' \
	.profile.custom > .profile.tmp
mv .profile.tmp .profile.custom

sed -e 's!#    export JAVA_MAX_VM_SIZE=\"512m\"!    export JAVA_MAX_VM_SIZE=\"2048m\"!' \
	.profile.custom > .profile.tmp
mv .profile.tmp .profile.custom
rm ${PROFILE_BKUP}
