-- ----------------------------------------------------------------------
-- @(#) upgrade_misc_fixes.sql 1.11.0 06/08/28 15:15:17
-- @(#) 
-- Copyright 2003 WatchMark Corp.
-- ----------------------------------------------------------------------
PROMPT START upgrade_misc_fixes805.sql

-- SEAde63057 add subclass_map for 'TRAFFIC' tables
insert into subclass_map(object_id,subclass_id)
    select pt.part_table_id, sc.id
      from part_tables pt, subclass_table sc
     where pt.table_name like '%\_ST\_TAB' escape '\'
       and UPPER(sc.name) = 'TRAFFIC' 
       and pt.part_table_id not in 
       (select sm.object_id from subclass_map sm)
/

PROMPT Cleaning up the orphaned schedue entries...
@clean_orphaned_skeds805.sql

PROMPT Updating the pm_groupby...
@pm_groupby805.sql

-- SEAde53786, fixed the pm_user_group
INSERT INTO pm_user_group(USER_ID, GROUP_ID)
SELECT user_id, group_id
FROM
(
SELECT u.id user_id, g.group_id group_id
  FROM pm_user u, pm_group g
 WHERE g.group_name='Everybody'
) tmp
WHERE NOT EXISTS
      (SELECT 1 FROM pm_user_group ug
        WHERE tmp.user_id = ug.user_id
          AND tmp.group_id=ug.group_id)
/

COMMIT;

-- SEAde54088
SET SERVEROUTPUT ON SIZE 1000000

PROMPT inserting system user user_mask ...
BEGIN

INSERT INTO pm_user_acl_mask(ACL_MASK_ID, OWNERUSER_ID, USER_ID, GROUP_ID, READ, WRITE)
SELECT SQ_PM_User_acl_mask.NEXTVAl, id, id, NULL, 'Y', 'Y'
  FROM pm_user
 WHERE UPPER(name)=UPPER('system');

EXCEPTION
  WHEN DUP_VAL_ON_INDEX THEN
    IF SQLERRM like '%UNQ_PM_USER_ACL_MASK%' THEN
        wm_print.print('system user''s user-mask exists already');
    ELSE
        RAISE;
    END IF;
END;
/

PROMPT inserting system user group-mask...
BEGIN

INSERT INTO pm_user_acl_mask(ACL_MASK_ID, OWNERUSER_ID, USER_ID, GROUP_ID, READ, WRITE)
SELECT SQ_PM_User_acl_mask.NEXTVAl, pu.id, NULL, pg.group_id, 'Y', 'N'
  FROM pm_user pu, pm_group pg
 WHERE UPPER(pu.name)=UPPER('system')
   AND UPPER(pg.group_name)=UPPER(pm_group_user.PM_EVERYBODY_GROUP)
   AND pu.group_id=pg.group_id;

EXCEPTION
  WHEN DUP_VAL_ON_INDEX THEN
    IF SQLERRM like '%UNQ_PM_USER_ACL_MASK%' THEN
        wm_print.print('system user''s group-mask exists already');
    ELSE
        RAISE;
    END IF;
END;
/

PROMPT inserting prospect user user-mask ...
BEGIN

INSERT INTO pm_user_acl_mask(ACL_MASK_ID, OWNERUSER_ID, USER_ID, GROUP_ID, READ, WRITE)
SELECT SQ_PM_User_acl_mask.NEXTVAl, id, id, NULL, 'Y', 'Y'
  FROM pm_user
 WHERE UPPER(name)=UPPER('prospect');

EXCEPTION
  WHEN DUP_VAL_ON_INDEX THEN
    IF SQLERRM like '%UNQ_PM_USER_ACL_MASK%' THEN
        wm_print.print('prospect user''s user-mask exists already');
    ELSE
        RAISE;
    END IF;
END;
/

PROMPT inserting prospect user group-mask ...
BEGIN

INSERT INTO pm_user_acl_mask(ACL_MASK_ID, OWNERUSER_ID, USER_ID, GROUP_ID, READ, WRITE)
SELECT SQ_PM_User_acl_mask.NEXTVAl, pu.id, NULL, pg.group_id, 'Y', 'N'
  FROM pm_user pu, pm_group pg
 WHERE UPPER(pu.name)=UPPER('prospect')
   AND UPPER(pg.group_name)=UPPER(pm_group_user.PM_EVERYBODY_GROUP)
   AND pu.group_id=pg.group_id;

EXCEPTION
  WHEN DUP_VAL_ON_INDEX THEN
    IF SQLERRM like '%UNQ_PM_USER_ACL_MASK%' THEN
        wm_print.print('prospect user''s group-mask exists already');
    ELSE
        RAISE;
    END IF;
END;
/

---
--- Update to set the user acl mask to read = 'Y' for Everybody group
---
BEGIN
UPDATE pm_user_acl_mask set read = 'Y'
 WHERE group_id =
    (SELECT group_id
       FROM pm_group
      WHERE UPPER(group_name) = UPPER(pm_group_user.PM_EVERYBODY_GROUP)
    );
END;
/

COMMIT;

-- Ensure folders exist in PM_DOCUMENT
DECLARE
  cursor cGetFolder is
    SELECT f.name, f.folderid, f.userid, f.id, f.description, NVL2(dt.document_id, 'Y', 'N') doc_exists
      FROM pm_folder f, (SELECT d.document_id, d.document_fk 
                           FROM pm_document d, pm_document_type t
                          WHERE t.document_type_id = d.document_type_id
                            AND upper(t.document_type_name) = 'FOLDER') dt
     WHERE dt.document_fk (+)= f.id
	   START WITH FOLDERID is null
	      CONNECT BY FOLDERID= prior ID;
begin
   for r in cGetFolder loop 
   
       IF (r.doc_exists = 'N') THEN
          pm_doc.add_document( r.name
			       , nvl(to_char(r.FOLDERID),-1)
			       , to_char(r.userid)
			       , 'Folder'
			       , to_char(r.id)
			       , 'application/x-watchmark-folder'
			       , r.description);
       END IF;
   end loop;
end;
/ 
COMMIT;

-- SEAde59563 - Back out the workaround given to customers
UPDATE pm_document
   SET group_write = 'N'
 WHERE name = 'Users'
   AND document_type_id = 1;


-- SEAde58327 - Fix doproc
UPDATE sched_job_types set command = 'doproc.sh' where job_type = 20;

-- SEAde62184 - Make sure that all temp schedules have a cleanup_time
UPDATE rg_schedule_view
   SET cleanup_time = SYSDATE+1
 WHERE cleanup_time IS NULL and periodicity = 'I';

-- DALtx00270 merge
INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 8, NULL, 'scenario_activation'
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 8 and sched_fk is null);
  
INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 20,   4, 'summary_clean'
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 20 and sched_fk = 4);

INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 20,   6, 'misc_clean' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 20 and sched_fk = 6);
 
INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 20,   8, 'bh_clean' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 20 and sched_fk = 8);
 
INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 20,  12, 'pm_daily' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 20 and sched_fk = 12);

INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 20,  13, 'pm_weekly' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 20 and sched_fk = 13);

INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 20,   14, 'pm_monthly' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 20 and sched_fk = 14);

INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 20,   19, 'event_clean' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 20 and sched_fk = 19);
 
INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 20,   22, 'clean_iot' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 20 and sched_fk = 22);
 
INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 25, NULL, 'daily_summary'
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 25 and sched_fk is null);

INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 30, NULL, 'bh_summary' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 30 and sched_fk is null);
 
INSERT INTO schedule_maint ( job_type_id, sched_fk, job_type_name ) 
SELECT 15,    6, 'rgfp' 
  FROM dual
 WHERE NOT EXISTS (SELECT 1 FROM schedule_maint where job_type_id = 15 and sched_fk = 6);


-- Updating wmd_Aggregator_Base.AGGR_VISIBLE
PROMPT Updating wmd_Aggregator_Base.AGGR_VISIBLE ...
UPDATE wmd_aggregator_base
   SET aggr_visible = 1
 WHERE aggr_name NOT IN ('Undefined','StdDev','TAverage')
   AND aggr_visible = 0;

-- Updating wmd_Aggregator_Base.AGGR_BH_OK
PROMPT Updating wmd_Aggregator_Base.AGGR_BH_OK ...
UPDATE wmd_aggregator_base 
   SET aggr_bh_ok='Y'
 WHERE func_time IN ('max', 'sum','avg')
   AND func_element NOT IN ('min', 'nil')
   AND aggr_bh_ok='N';

COMMIT;

DECLARE
  nCnt number := 0;
BEGIN

   -- SEAde59141 - Invalid rowid
   SELECT count(*) 
     INTO nCnt FROM rg_software 
    WHERE id = 22;
   IF (nCnt = 0 ) THEN
     INSERT INTO	rg_software(id, program) 
     VALUES(22,  'autoclean.clean_iot');
	
     INSERT INTO schedule
	       (schedule_id,JOB_TYPE,start_time,end_time,which_month_date
	       ,which_week, which_weekday, period,sched_fk,frequency, is_optional)
	   VALUES
         (schedule_seq.NEXTVAL,20,
          trunc(sysdate, 'HH')+ ((ceil(to_number(to_char(sysdate, 'MI')) / 30) * 30 + 15) / (24*60)),
          NULL,0,1,1,'S',22,1800,'Y');
   END IF;
END;
/

COMMIT;
--------

-- SEAde64801 Fix autoclean.clean_iot schedule to be optional
UPDATE schedule
   SET is_optional = 'Y',
       start_time = trunc(sysdate, 'HH')+ ((ceil(to_number(to_char(sysdate, 'MI')) / 30) * 30 + 15) / (24*60)),
       end_time = NULL
 WHERE job_type = 20
   AND sched_fk = 22;

-- SEAde68748 Add clean_iot to schedule_maint
UPDATE rg_software
   SET post_job_proc = 'event_statprocs.POST_AUTOCLEANPROC'
 WHERE id = 22;

INSERT INTO en_event_type (event_type_id, name, label,msg_proc,view_name, can_schedule,can_notify,group_id, forward)
   SELECT 36, 'CLEANIOT_S','Clean IOT Success', 'GET_EVENT_MESSAGE.GENERIC','EN_EVENT_STATUS_CHANGE_V', 'N','Y',2, 'Y'
     FROM dual
    WHERE NOT EXISTS(SELECT 1 FROM en_event_type WHERE event_type_id = 36);

INSERT INTO en_event_type (event_type_id, name, label,msg_proc,view_name, can_schedule,can_notify,group_id, forward)
   SELECT 37, 'CLEANIOT_F','Clean IOT Failure', 'GET_EVENT_MESSAGE.GENERIC','EN_EVENT_STATUS_CHANGE_V', 'N','Y',2, 'Y'
     FROM dual
    WHERE NOT EXISTS(SELECT 1 FROM en_event_type WHERE event_type_id = 37);

COMMIT;
/

-- SEAde63446 BH update fix ensure value is an int if 'Y'
UPDATE wm_system_values_v
   SET value = '1'
 WHERE name = 'UseFirstBHWhenEqual'
   AND value = 'Y';

-- ----------------------------------------------------------------------
exec wm_sysval.newSysValIfNotExists( 'EnableRemoveControlInfo', 'TRUE' );

-- SEAde67900 Truncate folder event if core.  Do not overwrite web setting.
exec wm_sysval.newSysValIfNotExists( 'EventTruncatePmFolderEventTable', 'Y' );

exec wm_sysval.newSysValIfNotExists( 'AutoCleanMaxLoops', '5' );
exec wm_sysval.newSysValIfNotExists( 'AutoCleanIOTMaxLoops', '5' );
exec wm_sysval.newSysValIfNotExists( 'IOTSECONDS2WAIT', '30' );
exec wm_sysval.newSysValIfNotExists( 'DDLProtectTruncate', 'TRUE' );

-- SEAde62916 Add parallel options to part_maint
exec wm_sysval.newSysValIfNotExists( 'PartMaintStatsParallel', '2' );
exec wm_sysval.newSysValIfNotExists( 'PartMaintDropParallel', '2' );
exec wm_sysval.newSysValIfNotExists( 'PartMaintMoveParallel', '2' );
exec wm_sysval.newSysValIfNotExists( 'PartMaintSessions', '1' );
                    
-- SEAde62916 Add HASH option for BH_Update and BH_Clean
-- and additional cleanup
exec wm_sysval.newSysValIfNotExists( 'BHSortAreaSize', 10*1024*1024 );
exec wm_sysval.newSysValIfNotExists( 'BHSortAreaRetainedSize', 10*1024*1024 );
exec wm_sysval.newSysValIfNotExists( 'BHHashAreaSize', 20*1024*1024 );

exec wm_sysval.newSysValIfNotExists( 'BHCleanSortAreaSize', 100*1024*1024 );
exec wm_sysval.newSysValIfNotExists( 'BHCleanSortAreaRetainedSize', 100*1024*1024 );
exec wm_sysval.newSysValIfNotExists( 'BHCleanHashAreaSize', 200*1024*1024 );
exec wm_sysval.newSysValIfNotExists( 'BHCleanMaxRunTime', 60*60*2 );

exec wm_sysval.newSysValIfNotExists( 'wml_load.lock_unsafe_find_instance', '0' );

-- SEAde68939 Add new summation variables
exec wm_sysval.newSysValIfNotExists( 'SummationLastNIDate', TO_CHAR(sysdate-1,'YYYYMMDD' ) );
exec wm_sysval.newSysValIfNotExists( 'SummationForceNI', 'FALSE' );

commit;

-- SEAde64074 Daylight savings change in 2007
UPDATE wm_dst_rules
   SET start_month = 3,
       start_day = 8,
       start_dayofweek = -1,
       end_month = 11,
       end_day = 1,
       end_dayofweek = 1
 WHERE ID = 1;

-- SEAde67403 NZ DST fix
UPDATE wm_dst_rules
   SET start_month = 9,
       start_day = -1,
       start_dayofweek = 1,
       end_month = 4,
       end_day = 1,
       end_dayofweek = 1
 WHERE ID = 7;

-- SEAde68219 - Fix new Venezuela Timezone from -240 to -270 minutes
UPDATE wm_locales
   SET gmt_offset = -270
 WHERE locale_name = 'America/Caracas';

-- SEAde59816 - Fix schedules (also fixes timezone stuff)
UPDATE schedule
   SET next_time = start_time, needs_msg_update = 'Y'
 WHERE period NOT IN ('I', 'O')
   AND NVL(end_time, sysdate) >= sysdate
   AND job_type NOT IN (1, 10);

-- Only change scheduled reports that will run in the future
UPDATE schedule
   SET next_time = start_time, needs_msg_update = 'Y'
 WHERE period NOT IN ('I', 'O')
   AND NVL(end_time, sysdate) >= sysdate
   AND job_type = 10
   AND next_time > sysdate;
    
commit;
PROMPT END upgrade_misc_fixes805.sql DONE
-- ----------------------------------------------------------------------
