-- 
-- 
-- PURPOSE:
--  :  Update Objects not covered by Core scripts
--  
-- 
-- 

PROMPT START upgrade_misc_fixesR23.sql

BEGIN

   run_sql( 'CREATE OR REPLACE TYPE T_EXPORTIMPORT_IDS IS TABLE OF NUMBER ');
   run_sql( 'CREATE OR REPLACE TYPE SentryFileArray AS TABLE OF NUMBER ');
   
END;
/

BEGIN 

 -- create a temp table to hold PM_ACTION_LOGGING data during R23-8.0.7 upgrade
   run_sql( 'CREATE TABLE pm_action_logging_TEMP ( '
||      ' id integer not null, '
||      ' priority integer not null, '
||      ' tstamp date not null, '
||      ' user_id integer not null, '
||      ' severity integer not null, '
||      ' area_id integer not null, '
||      ' message varchar2(2000)) '
||      ' TABLESPACE TRAFFIC_SMALL ' );

 -- copy the PM_ACTION_LOGGING data to the new temp table
      run_sql( 'INSERT INTO pm_action_logging_TEMP (id, priority, tstamp, user_id, severity, area_id, message) '
            || 'SELECT id, priority, tstamp, user_id, severity, area_id, message FROM PM_ACTION_LOGGING ');


 -- drop the existing PM_ACTION_LOGGING table along its constraints
      run_sql( 'DROP TABLE PM_ACTION_LOGGING CASCADE constraints ');

 -- recreate the PM_ACTION_LOGGING table
      declare
           sqlStr         VARCHAR2(4000);
      begin

           sqlStr := ' CREATE TABLE pm_action_logging ( '
                   || ' id integer not null, '
                   || ' priority integer not null, '
                   || ' tstamp date not null, '
                   || ' user_id integer not null, '
                   || ' severity  integer not null, '
                   || ' area_id integer not null, '
                   || ' message varchar2(2000),  '
                   || ' CONSTRAINT CHK_ACTION_LOGGING_SEVERITY CHECK (severity IN (5, 6)) ENABLE, '
                   || ' CONSTRAINT CHK_ACTION_LOGGING_PRIORITY CHECK (priority IN (133, 134)) ENABLE, '
                   || ' CONSTRAINT FK_ACTION_LOGGING_AREA FOREIGN KEY (AREA_ID) REFERENCES PM_ACTION_LOGGING_AREAS (ID) ENABLE,  '
                   || ' CONSTRAINT pk_PM_ACTION_LOGGING PRIMARY KEY (id,tstamp)) '
                   || ' ORGANIZATION INDEX PARTITION BY RANGE(tstamp)(PARTITION P'
                   || to_char(wm_date.trunc_to_last_period(SYSDATE, 'D'), 'YYYYMMDDHH24')
                   || ' VALUES LESS THAN(to_date('
                   || to_char(wm_date.trunc_to_last_period(SYSDATE, 'D')+1, 'YYYYMMDDHH24')
                   || ',''YYYYMMDDHH24''))) '
                   || ' PCTFREE 5  TABLESPACE TRAFFIC_SMALL ' ;

           BEGIN

               run_sql(  sqlStr  );

EXCEPTION
               WHEN OTHERS THEN
                 dbms_output.put_line('ERROR: in create pm_action_logging ' || SQLERRM);
END;
      end;

 --copy data from pm_action_logging_TEMP table into the recreated PM_ACTION_LOGGING table
    run_sql( 'INSERT INTO PM_ACTION_LOGGING (id, priority, tstamp, user_id, severity, area_id, message) '
        || 'SELECT id, priority, tstamp, user_id, severity, area_id, message FROM pm_action_logging_TEMP ');


 -- delete pm_action_logging_TEMP; no longer needed
    run_sql( 'DROP TABLE PM_ACTION_LOGGING_TEMP ');


--valnt00091228
    run_sql( ' delete from part_tables where table_name = ''PM_ACTION_LOGGING'' ');

    begin
      INSERT INTO part_tables(part_table_id, table_name, tablespace_name,
        period,future_part,past_part,Run_Daily,Needs_Analyze,Needs_Move,Needs_Purge)
        VALUES(part_table_seq.NEXTVAL, 'PM_ACTION_LOGGING', 'TRAFFIC_SMALL', 'D', 5, 2, 'Y', 'Y', 'Y', 'Y');

    exception 
      WHEN dup_val_on_index THEN NULL;
    end;
    run_sql( ' delete from part_tables where table_name = ''PE_LOG'' ');
END;
/

--valnt00097751
BEGIN
  INSERT INTO subclass_map( subclass_id , object_id) 
            VALUES((select id as subclass_id from subclass_table where name='PmActionLogging'), (select PART_TABLE_ID as object_id from part_tables where table_name='PM_ACTION_LOGGING'));
EXCEPTION
   WHEN dup_val_on_index THEN
      NULL;
END;
/
COMMIT;

--valnt00091998 
update pe_datasource set max_allowed_agents=10 where max_allowed_agents=100 and local='N' and deleted='N';
commit;

-- Forecasting function signature updates: START
BEGIN
  -- Update UDC function: getKCILimit()
  UPDATE wme_function 
     SET function_signature = 'getKCILimit(STRING,STRING,STRING)',
         function_desc = 'FLOAT getKCILimit(STRING entity, STRING field, STRING limit_name) - Gets the value of the specified limit, defined on the specified field.'
   WHERE upper(function_name) = 'GETKCILIMIT'
     AND function_type_id = (SELECT function_type_id FROM wme_function_type WHERE upper(function_type_name) = 'TRENDING');
EXCEPTION
   WHEN dup_val_on_index THEN
      NULL;
END;
/

BEGIN
  -- Update UDC function: trend()
  UPDATE wme_function 
     SET function_signature = 'trend(RELATION,STRING,STRING,STRING,INT,INT)',
         function_desc      = 'FLOAT trend(RELATION kci_to_trend, STRING bh_cluster_entity, STRING stored_busy_hour_name, STRING regression_type, INT number_of_days_regression, INT min_datapoints) - Gets the best-fit line value of the kci_to_trend calculated for the particular timestamp. Instances must have at least min_datapoints days worth of data to be included in the resultset.'
   WHERE upper(function_name) = 'TREND'
     AND function_type_id = (SELECT function_type_id FROM wme_function_type WHERE upper(function_type_name) = 'TRENDING');
EXCEPTION
   WHEN dup_val_on_index THEN
      NULL;
END;
/

BEGIN
  -- Update UDC function: projectTime()
  UPDATE wme_function 
     SET function_signature = 'projectTime(RELATION,STRING,STRING,STRING,INT,INT,INT)',
         function_desc      = 'FLOAT projectTime(RELATION kci_to_project, STRING bh_cluster_entity, STRING stored_busy_hour_name, STRING regression_type, INT number_of_days_regression, FLOAT limit, INT min_datapoints) - Gets the number of days until the specified limit will be breached. Instances must have at least min_datapoints days worth of data to be included in the resultset.'
   WHERE upper(function_name) = 'PROJECTTIME'
     AND function_type_id = (SELECT function_type_id FROM wme_function_type WHERE upper(function_type_name) = 'TRENDING');
EXCEPTION
   WHEN dup_val_on_index THEN
      NULL;
END;
/

COMMIT;
-- Forecasting function signature updates: END

PROMPT END upgrade_misc_fixesR23.sql DONE
