-- -------------------------------------
-- @(#) upgrade_udc_calc_source.sql 1.4.0 07/12/09 16:32:28
-- @(#) 
-- -------------------------------------

PROMPT START upgrade_udc_calc_source805.sql

DECLARE 
   v_dataType user_tab_columns.data_type%TYPE;
BEGIN
  
  SELECT data_type
    INTO v_dataType
    FROM user_tab_columns
   WHERE table_name = 'WMD_UDC_DEF'
     AND column_name = 'CALC_SOURCE';
     
  IF (v_dataType = 'VARCHAR2') THEN
    BEGIN
        
      dbms_output.put_line('Updating WMD_UDC_DEF');
     
      -- Create temp table
      run_sql('CREATE TABLE WMD_UDC_DEF_TMP ( ' ||
              'Field_ID		INTEGER		NOT NULL, ' ||
              'Entity_ID		INTEGER		NOT NULL, ' ||
              'UDC_Source		VARCHAR2(4000)	NOT NULL, ' ||
              'Calc_Source	VARCHAR2(4000), ' ||
              'Changed		DATE		DEFAULT TRUNC(SYSDATE,''MI'') NOT NULL, ' ||
              'Error_Msg		VARCHAR2(2000), ' ||
              'Status		CHAR(1)		NOT NULL, ' ||
              'COMPLEX		CHAR(1)		DEFAULT ''N'' NOT NULL ' ||
              ') ' ||
              'TABLESPACE WM_FLEXPM', 'Y');
        
      -- Save data
      run_sql('INSERT INTO WMD_UDC_DEF_TMP(field_id, entity_id, udc_source, calc_source, ' ||
              '                              changed, error_msg, status, complex) ' ||
              'SELECT field_id, entity_id, udc_source, calc_source, changed, error_msg, ' ||
              '       status, complex ' ||
              '  FROM WMD_UDC_DEF ', 'Y');
              
      -- Drop old table        		
	    run_sql('DROP TABLE WMD_UDC_DEF CASCADE CONSTRAINTS', 'Y');
	
      -- Create new table
	    run_sql('CREATE TABLE wmd_UDC_Def ( ' ||
              'Field_ID		INTEGER		NOT NULL ' ||
              'CONSTRAINT udc_fld_fky REFERENCES wmd_Field_Def ( Field_ID ) ' ||
              'CONSTRAINT udc_pky PRIMARY KEY, ' ||
              'Entity_ID		INTEGER		NOT NULL, ' ||
              'UDC_Source		VARCHAR2(4000)	NOT NULL, ' ||
              'Calc_Source		CLOB, ' ||
              'Changed		DATE		DEFAULT TRUNC(SYSDATE,''MI'') NOT NULL, ' ||
              'Error_Msg		VARCHAR2(2000), ' ||
              'Status		CHAR(1)		NOT NULL, ' ||
              'COMPLEX		CHAR(1)		DEFAULT ''N'' NOT NULL ' ||
              'CONSTRAINT udc_complex_chk CHECK ( Complex IN (''N'',''Y'',''U'') ), '||
              'CONSTRAINT udc_status_chk CHECK ( Status IN ( ''N'', ''U'', ''V'', ''S'', ''I'' ) ) ' ||
              ')' ||
              'tablespace wm_flexpm', 'Y');

      run_sql('COMMENT ON TABLE wmd_UDC_Def IS ''Data Dictionary Schema (wmd_schema.sql), Version 1.120.0''', 'Y');		
	
      -- Copy saved data from temp table
      run_sql('INSERT INTO WMD_UDC_DEF SELECT * FROM WMD_UDC_DEF_TMP', 'Y');
      
      -- Remove temp table
      run_sql('DROP TABLE WMD_UDC_DEF_TMP');
    END;
  ELSE
    dbms_output.put_line('Table WMD_UDC_DEF is up to date');
  END IF;

  SELECT data_type
    INTO v_dataType
    FROM user_tab_columns
   WHERE table_name = 'UDC_SAVE'
     AND column_name = 'CALC_SOURCE';
     
  IF (v_dataType = 'VARCHAR2') THEN
    BEGIN

      dbms_output.put_line('Updating UDC_SAVE');

      -- Create temp table
      run_sql('CREATE TABLE UDC_SAVE_TMP ( ' ||
	     ' tstamp DATE, ' ||
  	     ' field_name VARCHAR2(128), ' ||
	     ' report_header VARCHAR2(40), ' ||
	     ' report_header2  VARCHAR2(40), ' ||
  	     ' datatype CHAR(1), ' ||
 	     ' field_desc VARCHAR2(1000), ' ||
  	     ' field_aggregator CHAR(1), ' ||
  	     ' field_emits_aggregator CHAR(1), ' ||
  	     ' entity_name VARCHAR2(30), ' ||
  	     ' calc_source VARCHAR2(4000), ' ||
	     ' udc_source VARCHAR2(4000), ' ||
  	     ' error_msg VARCHAR2(2000), ' ||
 	     ' status CHAR(1) ' ||
              ') ' ||
              'TABLESPACE WM_FLEXPM', 'Y');
        
      -- Save data
      run_sql('INSERT INTO UDC_SAVE_TMP(tstamp, field_name, report_header, report_header2, ' ||
         	     '                           datatype, field_desc, field_aggregator, '||
        	     '                           field_emits_aggregator, entity_name, calc_source, ' ||
      	     '                           udc_source, error_msg, status) ' ||
              'SELECT tstamp, field_name, report_header, report_header2, datatype, field_desc, ' ||
              '       field_aggregator, field_emits_aggregator, entity_name, calc_source, ' ||
              '       udc_source, error_msg, status ' ||
              '  FROM UDC_SAVE ', 'Y');
              
      -- Drop old table        		
	    run_sql('DROP TABLE UDC_SAVE CASCADE CONSTRAINTS', 'Y');
	
      -- Create new table
      run_sql('CREATE TABLE UDC_SAVE ( ' ||
	     ' tstamp DATE, ' ||
  	     ' field_name VARCHAR2(128), ' ||
	     ' report_header VARCHAR2(40), ' ||
	     ' report_header2  VARCHAR2(40), ' ||
  	     ' datatype CHAR(1), ' ||
 	     ' field_desc VARCHAR2(1000), ' ||
  	     ' field_aggregator CHAR(1), ' ||
  	     ' field_emits_aggregator CHAR(1), ' ||
  	     ' entity_name VARCHAR2(30), ' ||
  	     ' calc_source CLOB, ' ||
	     ' udc_source VARCHAR2(4000), ' ||
  	     ' error_msg VARCHAR2(2000), ' ||
 	     ' status CHAR(1) ' ||
              ') ' ||
              'TABLESPACE WM_FLEXPM', 'Y');

      -- Copy saved data from temp table
      run_sql('INSERT INTO UDC_SAVE SELECT * FROM UDC_SAVE_TMP', 'Y');
      
      -- Remove temp table
      run_sql('DROP TABLE UDC_SAVE_TMP');
    END;
  ELSE
    dbms_output.put_line('Table UDC_SAVE is up to date');
  END IF;

END;
/

PROMPT END upgrade_udc_calc_source805.sql DONE
