PROMPT START PWEB upgrade_data_repository225.sql
-- ------------------------------------------------------------------------
-- @(#) upgrade_data_repository.sql 1.1.0 06/08/25 17:13:35
-- @(#) 
--
-- SEAde61743 Upgrade existing float columns to the proper precision
-- Only processes columns not at the proper precision
--
DECLARE
  vStartTime      DATE := TRUNC(sysdate, 'HH24');
  vEndTime        DATE := vStartTime - 1/24;
  vEntityName     pe_entity.entity_name%TYPE;
  vEntityId       pe_entity.entity_id%TYPE;
  vTableName      pe_table.table_name%TYPE;
  vTempTableName  pe_table.temp_table_name%TYPE;
  vColumnName     pe_stored_field.column_name%TYPE;
  vDatasourceName pe_datasource.datasource_name%TYPE;
  vActGroupId     pe_activity.activity_group_id%TYPE;
  vDDLSQL         VARCHAR2(500);
  vUpdatedEntity  INTEGER;
  
  -- All entity cursor
  CURSOR C_ENTITY
    IS
    SELECT entity_name, entity_id
      FROM pe_entity;
  
  -- All floating point colunmns that need updating
  CURSOR C_FLOAT_COLUMNS( b_entityid pe_entity.entity_id%TYPE )
    IS
    SELECT t.table_name, t.temp_table_name, f.column_name
      FROM pe_stored_field  f,
           pe_table         t,
           pe_column_type   ct,
           user_tab_columns uc
     WHERE f.table_id = t.table_id
       AND f.column_type_id = ct.column_type_id
       AND uc.table_name = upper(t.table_name)
       AND uc.column_name = upper(f.column_name)
       AND uc.data_precision != 126
       AND ct.type_tag = 'F'
       AND t.status = 'I'
       AND f.status IN ('I', 'D')
       AND t.entity_id = b_entityid
    UNION
    SELECT t.table_name, t.temp_table_name, f.field_name
      FROM pe_field           f,
           pe_dimension_field df,
           pe_table           t,
           pe_column_type     ct,
           user_tab_columns   uc
     WHERE f.field_id = df.field_id
       AND df.table_id = t.table_id
       AND f.column_type_id = ct.column_type_id
       AND uc.table_name = upper(t.table_name)
       AND uc.column_name = upper(f.field_name)
       AND uc.data_precision != 126
       AND ct.type_tag = 'F'
       AND t.status = 'I'
       AND df.status IN ('I', 'D')
       AND t.entity_id = b_entityid
    UNION
    SELECT t.table_name, t.temp_table_name, f.column_name
      FROM pe_dropped_columns f,
           pe_table           t,
           pe_column_type     ct,
           user_tab_columns   uc
     WHERE f.table_id = t.table_id
       AND f.column_type_id = ct.column_type_id
       AND uc.table_name = upper(t.table_name)
       AND uc.column_name = upper(f.column_name)
       AND uc.data_precision != 126
       AND ct.type_tag = 'F'
       AND t.status = 'I'
       AND t.entity_id = b_entityid;
       
  -- Datasources where data should be updated
  CURSOR C_ACTIVITY_DATASOURCE( b_entityid pe_entity.entity_id%TYPE )
    IS
    SELECT DISTINCT d.datasource_name
      FROM pe_activity a, pe_agent_types t, pe_datasource d
     WHERE a.agent_type_id = t.agent_type_id
       AND t.agent_type_name = 'TRAFFIC'
       AND a.state IN ('RUNNING', 'RUNNABLE', 'SCHEDULED')
       AND d.datasource_id = a.datasource_id
       AND a.entity_id = b_entityid;
      
      
BEGIN
  dbms_output.put_line('Updating data respository...');
  OPEN C_ENTITY;
  LOOP
    FETCH C_ENTITY INTO vEntityName, vEntityId;
    EXIT WHEN C_ENTITY%NOTFOUND;
    
    dbms_output.put_line('Updating entity: ' || vEntityName);
    vUpdatedEntity := 0;
    
    OPEN C_FLOAT_COLUMNS(vEntityId);
    LOOP
      FETCH C_FLOAT_COLUMNS INTO vTableName, vTempTableName, vColumnName;
      EXIT WHEN C_FLOAT_COLUMNS%NOTFOUND;
    
      vDDLSQL := 'ALTER TABLE ' || vTableName || ' MODIFY ' || vColumnName || ' FLOAT(126)';
      run_sql(vDDLSQL);
      vDDLSQL := 'ALTER TABLE ' || vTempTableName || ' MODIFY ' || vColumnName || ' FLOAT(126)';
      run_sql(vDDLSQL);
    
      vUpdatedEntity := 1;
    END LOOP;
    CLOSE C_FLOAT_COLUMNS;
  
    IF (vUpdatedEntity = 1) THEN
      OPEN C_ACTIVITY_DATASOURCE(vEntityId);
      LOOP
        FETCH C_ACTIVITY_DATASOURCE INTO vDataSourceName;
        EXIT WHEN C_ACTIVITY_DATASOURCE%NOTFOUND;
        
        vActGroupId := pe_agent.createActivityGroup('Y');
        
        pe_agent.addActivity(vActGroupId, 'TRAFFIC', 'Back-load traffic data', 
                             NULL, vDataSourceName, vEntityName, NULL, vStartTime, vEndTime);
                 
        dbms_output.put_line('Created backload activity on ' || vDataSourceName);            
      END LOOP;
      CLOSE C_ACTIVITY_DATASOURCE;
    END IF;
 
    dbms_output.put_line('Done updating entity: ' || vEntityName);
 
  END LOOP;  
  CLOSE C_ENTITY;
  
  dbms_output.put_line('Done updating data respository.');
  COMMIT;
END;
/
PROMPT END PWEB upgrade_data_repository225.sql DONE
