#!/usr/bin/env perl -w

# (#) ./base/migrate/upgrades/verify/dbver.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 

use strict;
use English;
use Getopt::Long;


BEGIN {
  unshift @INC, $ENV{ FLEXPM_HOME } . "/sw/perl/lib"  if   exists $ENV{ FLEXPM_HOME };
  require DBI;
}


sub readDatabase( $ );
sub readConfig( $ );
sub cleanConfig( $ );
sub compareStats( $$ );
sub checkIndex( $$$$$ );
sub compareIndex( $$$$ );
sub checkConstraint( $$$$$ );
sub compareConstraint( $$$$ );
sub writeConfig( $$ );
sub runSQL ( $$ );
sub printMessage( $$@ );
sub showUsage;


my $verbosity = 3;
my $zipFiles;
my $replSysNames;
my $inclPls;
my $minPerl = 5.005;

{
  my $dbConnect;
  my $baseFile;
  my $createBase;
  my $verbose;
  my $terse;
  my $warn;
  my $showHelp;

  warn "\nWarning: Perl version is unsupported.  Expected version is $minPerl\n\n" if ( $PERL_VERSION < $minPerl or int($PERL_VERSION) > int($minPerl) );

  &GetOptions( 'base=s' => \$baseFile, 'create=s' => \$createBase, 'connect=s' => \$dbConnect, 'verbose:i' => \$verbose, 'terse' => \$terse, 'warn' => \$warn, 'zip!' => \$zipFiles, 'help' => \$showHelp, 'repl!' => \$replSysNames, 'pls!' => \$inclPls );

  showUsage if defined $showHelp;

  if ( not defined $dbConnect ) {
    if ( exists $ENV{ DB_CONNECT } ) {
      $dbConnect = $ENV{ DB_CONNECT };
    }
    else {
      $dbConnect = 'flexpm/flexpm@flexpm';
    }
  }

  if ( defined $createBase ) {
    showUsage if ( defined $baseFile or $#ARGV >= 0 );
    showUsage if ( defined $verbose or defined $terse or defined $warn );
    $zipFiles = 0 unless defined $zipFiles;
    $replSysNames = 0 unless defined $replSysNames;
    $inclPls = 0 unless defined $inclPls;
    my $statRef = readDatabase( $dbConnect );
    writeConfig( $createBase, $statRef );
  }
  elsif ( $#ARGV == 1 ) {
    showUsage if defined $baseFile;
    showUsage if defined $zipFiles or defined $replSysNames;
    showUsage if defined $inclPls;
    showUsage if defined $terse and defined $verbose;
    showUsage if defined $terse and defined $warn;
    showUsage if defined $warn and defined $verbose;
    $verbosity = 3 if defined $terse;
    $verbosity = 2 if defined $warn;
    $verbosity = $verbose if defined $verbose;

    my $srcConfig = readConfig( $ARGV[ 0 ]  );
    my $dstConfig = readConfig( $ARGV[ 1 ] );

    cleanConfig( $srcConfig );
    cleanConfig( $dstConfig );

    compareStats( $srcConfig, $dstConfig );
  }
  else {
    showUsage unless $#ARGV == 0 xor defined $baseFile;
    $baseFile = $ARGV[ 0 ] if $#ARGV == 0;

    showUsage if defined $zipFiles or defined $replSysNames;
    showUsage if defined $terse and defined $verbose;
    showUsage if defined $terse and defined $warn;
    showUsage if defined $warn and defined $verbose;
    $verbosity = 3 if defined $terse;
    $verbosity = 2 if defined $warn;
    $verbosity = $verbose if defined $verbose;
    $inclPls = 0 unless defined $inclPls;

    my $baseStats = readConfig( $baseFile  );
    my $dbStats = readDatabase( $dbConnect );

    cleanConfig( $baseStats );
    cleanConfig( $dbStats );

    compareStats( $baseStats, $dbStats );
  }
}


sub showUsage {
  print "usage:  dbver.pl [-base] <srcfile> [-connect <connectstring>]\n";
  print "                 [-verbose [<level>] | -terse | -warn] [-pls]\n";
  print "        dbver.pl <srcfile> <dstfile> [-verbose [<level>] | -terse | -warn]\n";
  print "        dbver.pl -create <basefile> [-connect <connectstring>] [-zip] [-repl] [-pls]\n";
  die   "        dbver.pl -help\n";
}


sub readDatabase( $ ) {
  my $connectString = shift;

  my $dbName = $connectString;
  $dbName =~ s/\/.*\@/\@/;

  my %statHash = ( input => { type => 'db', 
			      connect => $connectString,
			      name => $dbName
			    }
		 );

  my $statHashRef = \%statHash;
  my %indexList;
  
  my $dbh = DBI->connect( 'dbi:Oracle:', $connectString, '', { PrintError => 0, ora_module_name => "./base/migrate/upgrades/verify/dbver.pl" } )  
    or die "Error: Can't connect to '$connectString' ($DBI::errstr).\n";

  print "Reading database configuration from $dbName .. \n";

  print "  Reading tables from database .. \n";
  {
    my $statement = "SELECT table_name, tablespace_name, monitoring, partitioned, iot_type, temporary, " .
      "pct_free, pct_used, pct_increase, initial_extent, next_extent, min_extents, max_extents, " .
	"ini_trans, max_trans, freelists, freelist_groups, logging, duration " .
	  "FROM USER_TABLES";
      
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } ) {
	print "** Return Row with undefined Table Name\n";
	next;
      }
      my $tabName = $returnRowRef->{ TABLE_NAME };
      
      $statHashRef->{ tables }->{ $tabName } = {}  unless exists $statHashRef->{ tables }->{ $tabName };
      my $tabRef = $statHashRef->{ tables }->{ $tabName };
      
      foreach my $colKey ( keys %{ $returnRowRef } ) {
	$returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
      }
      
      $tabRef->{ type } = 'TABLE';
      $tabRef->{ tablespace } = $returnRowRef->{ TABLESPACE_NAME };
      $tabRef->{ monitoring } = $returnRowRef->{ MONITORING };
      $tabRef->{ partitioned } = $returnRowRef->{ PARTITIONED };
      $tabRef->{ iot } = $returnRowRef->{ IOT_TYPE };
      $tabRef->{ temp } = $returnRowRef->{ TEMPORARY };
      $tabRef->{ pct_free } = $returnRowRef->{ PCT_FREE };
      $tabRef->{ pct_used } = $returnRowRef->{ PCT_USED };
      $tabRef->{ freelists } = $returnRowRef->{ FREELISTS };
      $tabRef->{ freelist_groups } = $returnRowRef->{ FREELIST_GROUPS };
      $tabRef->{ duration } = $returnRowRef->{ DURATION };
      $tabRef->{ columns } = {};
      
    }
  }


  print "  Reading views from database .. \n";
  {
    # get max view text length
    #my $statement = "SELECT MAX(Text_Length) FROM USER_VIEWS";

    my $statement = "SELECT View_Name, Text, Text_Length FROM USER_VIEWS";
    
    my $maxReadLen = 10000;
    $dbh->{LongReadLen} = $maxReadLen;
    $dbh->{LongTruncOk} = 1;
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      unless ( exists $returnRowRef->{ VIEW_NAME } and defined $returnRowRef->{ VIEW_NAME } ) {
	print "** Return Row with undefined View Name\n";
	next;
      }
      my $viewName = $returnRowRef->{ VIEW_NAME };
      $statHashRef->{ tables }->{ $viewName }->{ type } = 'VIEW';
      if ( $inclPls ) {
	my $viewRef =  $statHashRef->{ tables }->{ $viewName };
	$viewRef->{ viewtext } = $returnRowRef->{ TEXT };
	$viewRef->{ viewtext } =~ s/^\s+//g;
	$viewRef->{ viewtext } =~ s/\s+$//g;
	$viewRef->{ viewtext } =~ s/\s*\n\s*/ /g;
	$viewRef->{ viewtext } =~ s/\s+/ /g;

	my $textLen = $returnRowRef->{ TEXT_LENGTH };
	print STDERR "Warning: View Text too long ($textLen) for $viewName\n"      if ( $textLen > $maxReadLen );
      }
    }
  }


  print "  Reading table/view columns from database .. \n";
  {
    my $statement = "SELECT table_name, column_name, data_type, data_length, data_precision, data_scale, nullable, column_id, data_default FROM USER_TAB_COLUMNS ORDER BY table_name, column_id";
    
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      
      unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } ) {
	 print "** Return Row with undefined Table Name\n";
	next;
      }
      my $tabName = $returnRowRef->{ TABLE_NAME };
      
      unless ( exists $returnRowRef->{ COLUMN_NAME } and defined $returnRowRef->{ COLUMN_NAME } ) {
	 print "** Return Row with undefined Column Name for table $tabName\n";
	next;
      }
      my $colName = $returnRowRef->{ COLUMN_NAME };
      
      if ( not exists $statHashRef->{ tables }->{ $tabName } ) {
	 print "** Table does not exist - $tabName for column $colName\n";
	next;
      }
      if ( not defined $returnRowRef->{ COLUMN_ID } ) {
	 print "Column Ordinal not defined for $tabName.$colName\n";
	next;
      }
      
      #printf "%-40s\t%s\n", $tabName, $colName   if $tabName eq 'WMN_INSTANCE';
      
      foreach my $colKey ( keys %{ $returnRowRef } ) {
	$returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
      }
      
      $statHashRef->{ tables }->{ $tabName }->{ columns }->{ $colName } = {}  unless exists $statHashRef->{ tables }->{ $tabName }->{ columns }->{ $colName };
      my $colRef = $statHashRef->{ tables }->{ $tabName }->{ columns }->{ $colName };
      
      $colRef->{ type }        = $returnRowRef->{ DATA_TYPE };
      $colRef->{ len }         = $returnRowRef->{ DATA_LENGTH };
      $colRef->{ prec }        = $returnRowRef->{ DATA_PRECISION };
      $colRef->{ scale }       = $returnRowRef->{ DATA_SCALE };
      $colRef->{ nullable }    = $returnRowRef->{ NULLABLE };
      $colRef->{ ordinal }     = $returnRowRef->{ COLUMN_ID };
      $colRef->{ default }     = $returnRowRef->{ DATA_DEFAULT };
      $colRef->{ constraints } = [];
      $colRef->{ default } =~ s/\s+$//g;
    } 
  }
  

  print "  Reading indexes from database .. \n";
  {
    my $statement = "SELECT index_name, index_type, table_name, uniqueness, tablespace_name, compression, pct_free, freelists, freelist_groups FROM USER_INDEXES";
      
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      print STDERR "Program Error: Return Row with undefined Table Name\n"
	and next
	  unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
      my $tabName = $returnRowRef->{ TABLE_NAME };
      
      print STDERR "Program Error: Return Row with undefined Index Name for table $tabName\n" 
	and next
	  unless ( exists $returnRowRef->{ INDEX_NAME } and defined $returnRowRef->{ INDEX_NAME } );
      my $indName = $returnRowRef->{ INDEX_NAME };
      
      print STDERR "Program Error: Table does not exist - $tabName for index $indName\n"
	and next
	  if ( not exists $statHashRef->{ tables }->{ $tabName } );
      
      $statHashRef->{ tables }->{ $tabName } = {}  unless exists $statHashRef->{ tables }->{ $tabName };
      my $tabRef =$statHashRef->{ tables }->{ $tabName };
      $tabRef->{ indexes }->{ $indName } = {}  unless exists $tabRef->{ indexes }->{ $indName };
      my $indRef =$tabRef->{ indexes }->{ $indName };
      
      $indexList{ $indName } = $tabName;
      
      foreach my $colKey ( keys %{ $returnRowRef } ) {
	$returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
      }
      
      $returnRowRef->{ INDEX_TYPE } =~ s/\s+//g   if exists $returnRowRef->{ INDEX_TYPE } and defined $returnRowRef->{ INDEX_TYPE };
      
      $indRef->{ type } = $returnRowRef->{ INDEX_TYPE };
      $indRef->{ uniqueness } = $returnRowRef->{ UNIQUENESS };
      $indRef->{ tablespace } = $returnRowRef->{ TABLESPACE_NAME };
      $indRef->{ compression } = $returnRowRef->{ COMPRESSION };
      $indRef->{ pct_free } = $returnRowRef->{ PCT_FREE };
      $indRef->{ freelists } = $returnRowRef->{ FREELISTS };
      $indRef->{ freelist_groups } = $returnRowRef->{ FREELIST_GROUPS };
      $indRef->{ columns }= [];
      
      if ( $indRef->{ type } eq 'IOT-TOP' ) {
	$tabRef->{ iot_index } = $indName;
      }
    }
  }


  print "  Reading index columns from database .. \n";
  {
    my $statement = "SELECT ic.index_name, ic.table_name, ic.column_name, ie.column_expression " .
      "FROM USER_IND_COLUMNS ic, USER_IND_EXPRESSIONS ie " .
	"WHERE ic.index_name = ie.index_name (+)  AND ic.table_name = ie.table_name (+) " .
	  "  AND ic.column_position = ie.column_position (+) " . 
	    "ORDER BY ic.index_name, ic.table_name, ic.column_position";
    
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      print STDERR "Program Error:  Return Row with undefined Table Name\n"
	and next
	  unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
      my $tabName = $returnRowRef->{ TABLE_NAME };
      
      print STDERR "Program Error:  Return Row with undefined Index Name for table $tabName\n"
	and next
	  unless ( exists $returnRowRef->{ INDEX_NAME } and defined $returnRowRef->{ INDEX_NAME } );
      my $indName = $returnRowRef->{ INDEX_NAME };
      
      print STDERR "Program Error:  Return Row with undefined Column Name for table $tabName, index $indName\n"
	and next
	  unless ( exists $returnRowRef->{ COLUMN_NAME } and defined $returnRowRef->{ COLUMN_NAME } );
      my $colName = $returnRowRef->{ COLUMN_NAME };
      
      print STDERR "Program Error:  Table does not exist - $tabName for index $indName, column $colName\n"
	and next 
	  if ( not exists $statHashRef->{ tables }->{ $tabName } );
      
      print STDERR "Program Error: Index does not exist - $indName for table $tabName, column $colName\n"
	and next
	  if ( not exists $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName } );
      print STDERR "Program Error: column tag does not exist - $indName for table $tabName, column $colName\n"
	and next
	  if ( not exists $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ columns } );
      
      $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ columns } = []
	unless exists $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ columns };
      
      my $colArrRef = $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ columns };
      if ( exists $returnRowRef->{ COLUMN_EXPRESSION } and defined $returnRowRef->{ COLUMN_EXPRESSION } ) {
	push @{ $colArrRef }, $returnRowRef->{ COLUMN_EXPRESSION };
      }
      else {
	push @{ $colArrRef }, $colName;
      }
    }
  }


  print "  Reading partition table information from database .. \n";
  {
    my $statement = "SELECT table_name, partitioning_type, partitioning_key_count, def_tablespace_name, " .
      "def_pct_free, def_pct_used, def_freelists, def_freelist_groups " .
	"FROM USER_PART_TABLES";
    
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      print STDERR "Program Error:  Return Row with undefined Table Name\n"
	and next
	  unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
      my $tabName = $returnRowRef->{ TABLE_NAME };
      
      print STDERR "Program Error:  Table does not exist - $tabName for partition table info\n"
	and next 
	  if ( not exists $statHashRef->{ tables }->{ $tabName } );
      
      foreach my $colKey ( keys %{ $returnRowRef } ) {
	$returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
      }
      
      $statHashRef->{ tables }->{ $tabName }->{ part_info } = {}  unless exists $statHashRef->{ tables }->{ $tabName }->{ part_info };
      my $partRef = $statHashRef->{ tables }->{ $tabName }->{ part_info };
      
      $partRef->{ part_type } = $returnRowRef->{ PARTITIONING_TYPE };
      $partRef->{ part_key_count } = $returnRowRef->{ PARTITIONING_KEY_COUNT };
      $partRef->{ part_keys } = [];
      $partRef->{ tablespace } = $returnRowRef->{ DEF_TABLESPACE_NAME };
      $partRef->{ pct_free } = $returnRowRef->{ DEF_PCT_FREE };
      $partRef->{ pct_used } = $returnRowRef->{ DEF_PCT_USED };
      $partRef->{ freelists } = $returnRowRef->{ DEF_FREELISTS };
      $partRef->{ freelist_groups } = $returnRowRef->{ DEF_FREELIST_GROUPS };
    }
    
  }
  

  print "  Reading partition index information from database .. \n";
  {
    my $statement = "SELECT table_name, index_name, partitioning_type, partitioning_key_count, def_tablespace_name, " .
      "def_pct_free, def_freelists, def_freelist_groups " .
	"FROM USER_PART_INDEXES";
    
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      print STDERR "Program Error:  Return Row with undefined Table Name\n"
	and next
	  unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
      my $tabName = $returnRowRef->{ TABLE_NAME };
      
      print STDERR "Program Error:  Return Row with undefined Index Name for table $tabName\n"
	and next
	  unless ( exists $returnRowRef->{ INDEX_NAME } and defined $returnRowRef->{ INDEX_NAME } );
      my $indName = $returnRowRef->{ INDEX_NAME };
      
      print STDERR "Program Error:  Table does not exist - $tabName for partition table index\n"
	and next 
	  if ( not exists $statHashRef->{ tables }->{ $tabName } );
      
      print STDERR "Program Error: Index does not exist - $tabName.$indName for partition index\n"
	and next
	  if ( not exists $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName } );
      
      foreach my $colKey ( keys %{ $returnRowRef } ) {
	$returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
      }
      
      $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ part_info } = {}  unless exists $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ part_info };
      my $indPartRef = $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ part_info };
      
      $indPartRef->{ part_type } = $returnRowRef->{ PARTITIONING_TYPE };
      $indPartRef->{ part_key_count } = $returnRowRef->{ PARTITIONING_KEY_COUNT };
      $indPartRef->{ part_keys } = [];
      $indPartRef->{ tablespace } = $returnRowRef->{ DEF_TABLESPACE_NAME };
      $indPartRef->{ pct_free } = $returnRowRef->{ DEF_PCT_FREE };
      $indPartRef->{ freelists } = $returnRowRef->{ DEF_FREELISTS };
      $indPartRef->{ freelist_groups } = $returnRowRef->{ DEF_FREELIST_GROUPS };
    }
  }
  
  
  print "  Reading partition key column information from database .. \n";
  {
    my $statement = "SELECT name, object_type, column_name " .
      "FROM USER_PART_KEY_COLUMNS ORDER BY column_position";
    
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      # for each partition_key, add to table or index part_info
      print STDERR "Program Error:  Return Row with undefined Object Name\n"
	and next
	  unless ( exists $returnRowRef->{ NAME } and defined $returnRowRef->{ NAME } );
      my $objName = $returnRowRef->{ NAME };
      
      print STDERR "Program Error:  Return Row with undefined Object Type for object $objName\n"
	and next
	  unless ( exists $returnRowRef->{ OBJECT_TYPE } and defined $returnRowRef->{ OBJECT_TYPE } );
      my $objType = uc $returnRowRef->{ OBJECT_TYPE };
      $objType =~ s/\s+//g;
      
      print STDERR "Program Error:  Return Row with undefined Column Name for $objType object $objName\n"
	and next
	  unless ( exists $returnRowRef->{ COLUMN_NAME } and defined $returnRowRef->{ COLUMN_NAME } );
      my $colName = $returnRowRef->{ COLUMN_NAME };
      
      print STDERR "Program Error:  Unknown Object Type $objType for object $objName\n"
	and next
	  unless ( $objType eq 'INDEX' or $objType eq 'TABLE' );
      
      if ( $objType eq 'INDEX' ) {
	my $tabName = $indexList{ $objName };
	print STDERR "Program Error: Index does not exist - $objName for partition key $colName\n"
	  and next
	    unless defined $tabName and exists $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $objName };
	print STDERR "Program Error: PartKeys does not exist -  $tabName.$objName - $colName\n"
	  and next
	    unless exists $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $objName }->{ part_info }->{ part_keys };
	
	push @{ $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $objName }->{ part_info }->{ part_keys } }, $colName;
      }
      elsif ( uc $objType eq 'TABLE' ) {
	print STDERR "Program Error: Table does not exist - $objName for partition key $colName\n"
	  and next
	    unless exists $statHashRef->{ tables }->{ $objName };
	print STDERR "Program Error: PartKeys does not exist -  $objName - $colName\n"
	  and next
	    unless exists $statHashRef->{ tables }->{ $objName }->{ part_info }->{ part_keys };
	
	push @{ $statHashRef->{ tables }->{ $objName }->{ part_info }->{ part_keys } }, $colName;
      }
    }
  }


  print "  Reading constraints from database .. \n";
  {
    my $statement = "SELECT Constraint_Name, Constraint_Type, Table_Name, R_Constraint_Name, Delete_Rule, Status, Deferrable, Deferred, Search_Condition " .
      "FROM USER_CONSTRAINTS ORDER BY Constraint_Name";
    
    $dbh->{LongReadLen} = 2000;
    $dbh->{LongTruncOk} = 1;
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      print STDERR "Program Error:  Return Row with undefined Constraint Name\n"
	and next
	  unless ( exists $returnRowRef->{ CONSTRAINT_NAME } and defined $returnRowRef->{ CONSTRAINT_NAME } );
      my $consName = $returnRowRef->{ CONSTRAINT_NAME };
      print STDERR "Program Error:  Return Row with undefined Table Name for constraint $consName\n"
	and next
	  unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
      my $tabName = $returnRowRef->{ TABLE_NAME };
      
      print STDERR "Program Error: Table does not exist - $tabName for constraint $consName\n"
	and next
	  unless exists $statHashRef->{ tables }->{ $tabName };
      
      foreach my $colKey ( keys %{ $returnRowRef } ) {
	$returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
      }
      
      $statHashRef->{ tables }->{ $tabName }->{ constraints }->{ $consName } = {}  unless exists $statHashRef->{ tables }->{ $tabName }->{ constraints }->{ $consName };
      my $consRef = $statHashRef->{ tables }->{ $tabName }->{ constraints }->{ $consName };
      
      $consRef->{ type } = $returnRowRef->{ CONSTRAINT_TYPE };
      $consRef->{ status } = $returnRowRef->{ STATUS };
      $consRef->{ refName } = $returnRowRef->{ R_CONSTRAINT_NAME };
      $consRef->{ delRule } = $returnRowRef->{ DELETE_RULE };
      $consRef->{ deferrable } = $returnRowRef->{ DEFERRABLE };
      $consRef->{ deferred } = $returnRowRef->{ DEFERRED };
      $consRef->{ search } = $returnRowRef->{ SEARCH_CONDITION };
      $consRef->{ search } =~ s/^\s+//g;
      $consRef->{ search } =~ s/\s+$//g;
      $consRef->{ search } =~ s/\s*\n\s*/ /g;
      $consRef->{ search } =~ s/\s+/ /g;
      $consRef->{ columns } = {};
      
      $statHashRef->{ constraints }->{ $consName }->{ table } = $tabName;
    }
  }
  

  print "  Reading constraint columns from database .. \n";
  {
    my $statement = "SELECT Constraint_Name, Table_Name, Column_Name, Position FROM USER_CONS_COLUMNS ORDER BY Constraint_Name, Position, Column_Name";
    
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      print STDERR "Program Error:  Return Row with undefined Constraint Name\n"
	and next
	  unless ( exists $returnRowRef->{ CONSTRAINT_NAME } and defined $returnRowRef->{ CONSTRAINT_NAME } );
      my $consName = $returnRowRef->{ CONSTRAINT_NAME };
      
      print STDERR "Program Error:  Return Row with undefined Table Name for constraint $consName\n"
	and next
	  unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
      my $tabName = $returnRowRef->{ TABLE_NAME };
      
      print STDERR "Program Error:  Return Row with undefined Column Name for constraint $consName, table $tabName\n"
	and next
	  unless ( exists $returnRowRef->{ COLUMN_NAME } and defined $returnRowRef->{ COLUMN_NAME } );
      my $colName = $returnRowRef->{ COLUMN_NAME };
      
      print STDERR "Program Error:  Table does not exist - $tabName for constraint $consName, column $colName\n"
	and next
	  unless exists $statHashRef->{ tables }->{ $tabName };
      
      print STDERR "Program Error: Constraint does not exist - $consName for table $tabName, column $colName\n"
	and next
	  unless exists $statHashRef->{ tables }->{ $tabName }->{ constraints }->{ $consName };
      
      print STDERR "Program Error: Constraint Column does not exist - $colName for constraint $consName, table $tabName\n"
	and next
	  unless exists $statHashRef->{ tables }->{ $tabName }->{ columns }->{ $colName };
      
      unless ( exists $returnRowRef->{ POSITION } and defined $returnRowRef->{ POSITION } ) {
	my $colRef = $statHashRef->{ tables }->{ $tabName }->{ columns }->{ $colName };
	push @{ $colRef->{ constraints } }, $consName;
	my $consRef = $statHashRef->{ tables }->{ $tabName }->{ constraints }->{ $consName };
	if ( $consRef->{ type } eq 'C'  and 
	     $consRef->{ search } =~ m/^\"?$colName\"\s+IS\s+NOT\s+NULL\s*/i and
	     $colRef->{ nullable } = 'N' ) {
	  $colRef->{ notNull } = $consName;
	}
	$statHashRef->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ columns }->{ $colName }->{ position } = 0;
      }
      else {
	$statHashRef->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ columns }->{ $colName }->{ position } =  $returnRowRef->{ POSITION };
      }
    }
  }
  
  
  print "  Reading sequences from database .. \n";
  {
    my $statement = "SELECT Sequence_Name, Cycle_Flag, Order_Flag FROM USER_SEQUENCES";
    
    my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
    foreach my $returnRowRef ( @{ $returnSetRef } ) {
      print STDERR "Program Error:  Return Row with undefined Sequence Name\n"
	and next
	  unless ( exists $returnRowRef->{ SEQUENCE_NAME } and defined $returnRowRef->{ SEQUENCE_NAME } );
      my $seqName = $returnRowRef->{ SEQUENCE_NAME };
      
      foreach my $colKey ( keys %{ $returnRowRef } ) {
	$returnRowRef->{ $colKey } = '<NULL>'    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
      }
      
      $statHashRef->{ sequences }->{ $seqName } = {}  unless exists $statHashRef->{ sequences }->{ $seqName };
      my $seqRef = $statHashRef->{ sequences }->{ $seqName };
      
      $seqRef->{ cycleFlag } = $returnRowRef->{ CYCLE_FLAG };
      $seqRef->{ orderFlag } = $returnRowRef->{ ORDER_FLAG };
    }
  }	
  
  my $rc = $dbh->disconnect;
  return $statHashRef;
}


sub readConfig( $ ) {
  my $configFile = shift;

  my %statHash = ( input => { type => 'file', 
			      name => $configFile 
			    }
		 );

  my $statHashRef = \%statHash;

  my $fType = `file $configFile`;
  if ( $fType =~ m/gzip compressed data/ ) {
    open DATAFILE, "gzip -d -c <$configFile |" or die "Can't open input file $configFile\n";
  }
  else {
    open DATAFILE, "<$configFile" or die "Can't open input file $configFile\n";
  }

  print "Reading stored configuration from $configFile \n";

  my $tabName;
  my $tabRef;
  my $indName;
  my $indRef;
  my $consName;
  my $consRef;

  while ( <DATAFILE> ) {
    chomp;
    my @inArgs = split /\t/, $_, -1;
    #printf "size = %3i\t\$inArgs[0] = '%s'\n", $#inArgs, $inArgs[0];
    my $argKey = shift @inArgs;
    if ( not defined $argKey or substr($argKey,0,7) eq 'COMMENT' or substr($argKey,0,1) eq '#' ) {
      # This is either a blank line or a comment line, ignore it
    }
    elsif ( $argKey eq 'TABLE' ) {
      $tabName = shift @inArgs;
      if ( defined $tabName ) {
	$statHashRef->{ tables }->{ $tabName } = { } if not exists $statHashRef->{ tables }->{ $tabName };
	#printf "Read table %s\n", $tabName;
	$tabRef = $statHashRef->{ tables }->{ $tabName };
	undef $indName;
	undef $indRef;
	undef $consName;
	undef $consRef;
	$tabRef->{ type } = shift @inArgs;
	$tabRef->{ columns } = {};
      }
      else {
	print " ** Discarding TABLE line - table name not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'TABINFO' ) {
      if ( defined $tabRef ) {
	$tabRef->{ tablespace } = shift @inArgs;
	$tabRef->{ monitoring } = shift @inArgs;
	$tabRef->{ partitioned } = shift @inArgs;
	$tabRef->{ iot } = shift @inArgs;
	$tabRef->{ temp } = shift @inArgs;
	$tabRef->{ pct_free } = shift @inArgs;
	$tabRef->{ pct_used } = shift @inArgs; 
	$tabRef->{ freelists } = shift @inArgs;
	$tabRef->{ freelist_groups } = shift @inArgs;
	$tabRef->{ duration } = shift @inArgs;
      }
      else {
	print " ** Discarding TABINFO line - table not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'TABPART' ) {
      if ( defined $tabRef ) {
	$tabRef->{ part_info }->{ tablespace } = shift @inArgs;
	$tabRef->{ part_info }->{ part_type } = shift @inArgs;
	$tabRef->{ part_info }->{ part_key_count } = shift @inArgs;
	$tabRef->{ part_info }->{ pct_free } = shift @inArgs;
	$tabRef->{ part_info }->{ pct_used } = shift @inArgs;
	$tabRef->{ part_info }->{ freelists } = shift @inArgs;
	$tabRef->{ part_info }->{ freelist_groups } = shift @inArgs;
	my $keyString = shift @inArgs;
	if ( defined $keyString ) {
	  my @keyList = split /\;/, $keyString, -1;
	  $tabRef->{ part_info }->{ part_keys } = [ @keyList ];
	}
      }
      else {
	print " ** Discarding TABPART line - table not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'COLUMN' ) {
      my $colName = shift @inArgs;
      if ( defined $tabRef and defined $colName ) {
	$tabRef->{ columns }->{ $colName } = {} if not exists $tabRef->{ columns }->{ $colName };
	my $colRef = $tabRef->{ columns }->{ $colName };
	$colRef->{ type } = shift @inArgs;
	$colRef->{ len } = shift @inArgs;
	$colRef->{ prec } = shift @inArgs;
	$colRef->{ scale } = shift @inArgs;
	$colRef->{ nullable } = shift @inArgs;
	$colRef->{ ordinal } = shift @inArgs;
	$colRef->{ default } = shift @inArgs;
	$colRef->{ constraints } = [];
      }
      elsif ( not defined $tabRef ) {
	print " ** Discarding COLUMN line - table not defined at $INPUT_LINE_NUMBER\n";
      }
      else {
	print " ** Discarding COLUMN line - column name not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'INDEX' ) {
      $indName = shift @inArgs;
      if ( defined $tabRef and defined $indName ) {
	$tabRef->{ indexes }->{ $indName } = {} if not exists $tabRef->{ indexes }->{ $indName };
	$indRef = $tabRef->{ indexes }->{ $indName };
	$indRef->{ type } = shift @inArgs;
	if ( defined $indRef->{ type } and $indRef->{ type } eq 'IOT-TOP' ) {
	  $tabRef->{ iot_index } = $indName;
	}
	$indRef->{ uniqueness } = shift @inArgs;
	$indRef->{ tablespace } = shift @inArgs;
	$indRef->{ compression } = shift @inArgs;
	$indRef->{ pct_free } =shift @inArgs;
	$indRef->{ freelists } = shift @inArgs;
	$indRef->{ freelist_groups } = shift @inArgs;
	$indRef->{ columns }= [];
      }
      elsif ( not defined $tabRef ) {
	print " ** Discarding INDEX line - table not defined at $INPUT_LINE_NUMBER\n";
      }
      else {
	print " ** Discarding INDEX line - index name not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'INDPART' ) {
      if ( defined $indRef ) {
	$indRef->{ part_info }->{ tablespace } = shift @inArgs;
	$indRef->{ part_info }->{ part_type } = shift @inArgs;
	$indRef->{ part_info }->{ part_key_count } = shift @inArgs;
	$indRef->{ part_info }->{ pct_free } = shift @inArgs;
	$indRef->{ part_info }->{ freelists } = shift @inArgs;
	$indRef->{ part_info }->{ freelist_groups } = shift @inArgs;
	my $keyString = shift @inArgs;
	if ( defined $keyString ) {
	  my @keyList = split /\;/, $keyString, -1;
	  $indRef->{ part_info }->{ part_keys } = [ @keyList ];
	}
      }
      else {
	print " ** Discarding INDPART line - index not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'INDCOL') {
      my $colName = shift @inArgs;
      if ( defined $indRef and defined $colName ) {
	push @{ $indRef->{ columns } }, $colName;
      }
      elsif ( not defined $indRef ) {
	print " ** Discarding INDCOL line - index not defined at $INPUT_LINE_NUMBER\n";
      }
      else {
	print " ** Discarding INDCOL line - column name not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'CONSTRAINT' ) {
      $consName = shift @inArgs;
      if ( defined $tabRef and defined $consName ) {
	$tabRef->{ constraints }->{ $consName } = {} if not exists $tabRef->{ constraints }->{ $consName };
	$consRef = $tabRef->{ constraints }->{ $consName };
	$consRef->{ type } = shift @inArgs;
	$consRef->{ status } = shift @inArgs;
	$consRef->{ refName } = shift @inArgs;
	$consRef->{ delRule } = shift @inArgs;
	$consRef->{ deferrable } = shift @inArgs;
	$consRef->{ deferred } = shift @inArgs;
	$consRef->{ search } =shift @inArgs;
	$consRef->{ columns }= {};
	$statHashRef->{ constraints }->{ $consName }->{ table } = $tabName;
      }
      elsif ( not defined $tabRef ) {
	print " ** Discarding CONSTRAINT line - table not defined at $INPUT_LINE_NUMBER\n";
      }
      else {
	print " ** Discarding CONSTRAINT line - constraint name not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'CONSCOL') {
      my $colName = shift @inArgs;
      if ( defined $consRef and defined $colName ) {
	$consRef->{ columns }->{ $colName } = {} if not exists $consRef->{ columns }->{ $colName };
	my $colRef = $consRef->{ columns }->{ $colName };
	my $tabColRef = $tabRef->{ columns }->{ $colName };
	my $position = shift @inArgs;
	$position = $#{ $tabColRef->{ constraints } } + 2 unless defined $position;
	if ( defined $position and $position == 0 ) {
	  push @{ $tabColRef->{ constraints } }, $consName;

	  # Check for a 'NOT NULL' constraint
	  if ( $consRef->{ type } eq 'C'  and 
	       $consRef->{ search } =~ m/^\"?$colName\"\s+IS\s+NOT\s+NULL\s*/i and
	       $tabColRef->{ nullable } = 'N' ) {
	    $tabColRef->{ notNull } = $consName;
	  }
	}
	$colRef->{ position } = $position;
      }
      elsif ( not defined $consRef ) {
	print " ** Discarding CONSCOL line - constraint not defined at $INPUT_LINE_NUMBER\n";
      }
      else {
	print " ** Discarding CONSCOL line - column name not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    elsif ( $argKey eq 'SEQUENCE') {
      my $seqName = shift @inArgs;
      if ( defined $seqName ) {
	$statHashRef->{ sequences }->{ $seqName } = { } if not exists $statHashRef->{ sequences }->{ $seqName };
	my $seqRef = $statHashRef->{ sequences }->{ $seqName };
	$seqRef->{ cycleFlag } = shift @inArgs;
	$seqRef->{ orderFlag } = shift @inArgs;
      }
      else {
	print " ** Discarding SEQUENCE line - sequence name not defined at $INPUT_LINE_NUMBER\n";
      }
    }
    else {

      print " ** Discarding $argKey line - unrecognized key at $INPUT_LINE_NUMBER\n";
    }
  }

  print "Completed reading stored configuration.\n";

  close DATAFILE;

  return $statHashRef
}


{
  my ( $srcConfig, $dstConfig );
  my ( $tabName, $colName );
  my ( $srcName, $dstName );

  sub compareTabColAttr( $;$ ) {
    my $attrName = shift;
    my $chkCons = shift;
    
    if ( exists $srcConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ $attrName } and exists $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ $attrName } ) {
      my $srcAttr = $srcConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ $attrName };
      my $dstAttr = $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ $attrName };
      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  printMessage(2, "Column %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $colName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  my $matchingCons = 0;
	  if ( $chkCons ) {
	    if ( $attrName eq 'nullable' ) {
	      my $tempConst = sprintf("%s IS NOT NULL", $colName);
	      if ( $srcAttr eq 'Y' ) {
		foreach my $dstCons ( sort keys %{ $dstConfig->{ tables }->{ $tabName }->{ constraints } } ) {
		  if ( $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $dstCons }->{ search } eq $tempConst ) {
		    $matchingCons = 1;
		    last;
		  }
		}
	      }
	      else {
		foreach my $srcCons ( sort keys %{ $srcConfig->{ tables }->{ $tabName }->{ constraints } } ) {
		  if ( $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $srcCons }->{ search } eq $tempConst ) {
		    $matchingCons = 1;
		    last;
		  }
		}
	      }
	    }
	  }
	  if ( not $matchingCons ) {
	    printMessage(2, "Column %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $colName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	  }
	  else {
	    printMessage(1, "Column %s.%s - %s mismatch: %s => %s and %s => %s - Handled by Constraint", $tabName, $colName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	  }
	}
      }
      elsif ( defined $dstAttr ) {
	printMessage(2, "Column %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $colName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
      }
    }
  }


  sub compareTableColumn ( $ ) {
    $colName = shift;
    
    if ( not exists $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName } ) {
      printMessage(3, "Column %s.%s in %s is missing in %s.", $tabName, $colName, $srcName, $dstName );
    }
    else {
      compareTabColAttr( 'type' );
      compareTabColAttr( 'len' );
      compareTabColAttr( 'prec' );
      compareTabColAttr( 'scale' );
      compareTabColAttr( 'nullable', 1 );
    }
  }


  sub compareTabAttr( $;$ ) {
    my $attrName = shift;
    my $sectionName = shift;
    
    my $srcTabRef = $srcConfig->{ tables }->{ $tabName };
    my $dstTabRef = $dstConfig->{ tables }->{ $tabName };
    if ( defined $sectionName ) {
      $srcTabRef = $srcTabRef->{ $sectionName };
      $dstTabRef = $dstTabRef->{ $sectionName };
    }

    if ( exists $srcTabRef->{ $attrName } and exists $dstTabRef->{ $attrName } ) {
      my $srcAttr = $srcTabRef->{ $attrName };
      my $dstAttr = $dstTabRef->{ $attrName };

      $srcAttr = join( ":", @{ $srcAttr } )  if ( defined $srcAttr and ref $srcAttr eq 'ARRAY' );
      $dstAttr = join( ":", @{ $dstAttr } )  if ( defined $dstAttr and ref $dstAttr eq 'ARRAY' );

      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  if ( $srcAttr ne '' ) {
	    printMessage(2, "Table %s - %s mismatch: %s => %s and %s => %s", $tabName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
	  }
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  printMessage(2, "Table %s - %s mismatch: %s => %s and %s => %s", $tabName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	}
      }
      elsif ( defined $dstAttr and $dstAttr ne '' ) {
	printMessage(2, "Table %s - %s mismatch: %s => %s and %s => %s", $tabName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
      }
    }
  }


  sub compareTable( $$$ ) {
    $srcConfig = shift;
    $dstConfig = shift;
    $tabName = shift;

    $srcName = $srcConfig->{ input }->{ name };
    $dstName = $dstConfig->{ input }->{ name };

    if ( exists $srcConfig->{ tables }->{ $tabName }->{ type } and exists $dstConfig->{ tables }->{ $tabName }->{ type } ) {
      my $srcType = $srcConfig->{ tables }->{ $tabName }->{ type };
      my $dstType = $dstConfig->{ tables }->{ $tabName }->{ type };
      if (  $srcType ne  $dstType ) {
	printMessage( 3, "Table/View %s mismatch: %s is %s and %s is %s", $tabName, $srcName, $srcType, $dstName, $dstType );
      }
    }

    # Check table attributes
    compareTabAttr( 'tablespace' );
    compareTabAttr( 'monitoring' );
    compareTabAttr( 'partitioned' );
    compareTabAttr( 'iot' );
    compareTabAttr( 'temp' );
    compareTabAttr( 'pct_free' );
    #compareTabAttr( 'pct_used' );
    compareTabAttr( 'freelists' );
    compareTabAttr( 'freelist_groups' );
    compareTabAttr( 'duration' );

    compareTabAttr( 'tablespace', 'part_info' );
    compareTabAttr( 'part_type', 'part_info' );
    compareTabAttr( 'part_key_count', 'part_info' );
    compareTabAttr( 'pct_free', 'part_info' );
    compareTabAttr( 'pct_used', 'part_info' );
    compareTabAttr( 'freelists', 'part_info' );
    compareTabAttr( 'freelist_groups', 'part_info' );
    compareTabAttr( 'part_keys', 'part_info' );
    
    # Check table columns
    foreach my $colName ( sort keys %{ $srcConfig->{ tables }->{ $tabName }->{ columns } } ) {
      compareTableColumn( $colName );
    }
    
    # Check Table indexes
    foreach my $indName ( sort keys %{ $srcConfig->{ tables }->{ $tabName }->{ indexes } } ) {
      my $srcIndRef = $srcConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName };
      if ( not exists $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName } ) {
	# Check to see if we can find this index with another name
	my $indMatch = 0;
	my $dstIndName;
	foreach my $tIndName ( keys %{ $dstConfig->{ tables }->{ $tabName }->{ indexes } } ) {
	  $dstIndName = $tIndName;
	  my $dstIndRef = $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $tIndName };
	  $indMatch = checkIndex( $srcConfig, $dstConfig, $tabName, $indName, $tIndName );
	  last if $indMatch;
	}
	if ( $indMatch ) {
	  printMessage(0, "Index %s on table %s in %s is renamed to %s in %s.", $indName, $tabName, $srcName, $dstIndName, $dstName );
	  $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName } = $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $dstIndName };
	  delete $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $dstIndName };
	  compareIndex( $srcConfig, $dstConfig, $tabName, $indName );
	}
	else {
	  printMessage(3, "Index %s on table %s in %s is missing in %s.", $indName, $tabName, $srcName, $dstName );
	}
      }
      else {
	compareIndex( $srcConfig, $dstConfig, $tabName, $indName );
      }
    }

    # Check Table Constraints
    foreach my $consName ( sort keys %{ $srcConfig->{ tables }->{ $tabName }->{ constraints } } ) {
      my $srcConsRef = $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName };
      if ( not exists $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName } ) {
	# Check to see if we can find this constraint with another name
	my $consMatch = 0;
	my $dstConsName;
	foreach my $tConsName ( keys %{ $dstConfig->{ tables }->{ $tabName }->{ constraints } } ) {
	  $dstConsName = $tConsName;
	  my $dstConsRef = $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $tConsName };
	  $consMatch = checkConstraint( $srcConfig, $dstConfig, $tabName, $consName, $tConsName );
	  last if $consMatch;
	}
	if ( $consMatch ) {
	  printMessage(0, "Constraint %s on table %s in %s is renamed to %s in %s.", $consName, $tabName, $srcName, $dstConsName, $dstName );
	  $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName } = $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $dstConsName };
	  delete $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $dstConsName };
	  compareConstraint( $srcConfig, $dstConfig, $tabName, $consName );
	}
	else {
	  my $redundant = 0;
	  my $srch = $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ search };
	  if ( defined $srch and $srch =~ m/^\s*(\w+)\s+IS\s+NOT\s+NULL\s*$/ ) {
	    my $colName = $1;
	    if ( exists $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName } ) {
	      $redundant = ( $dstConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ nullable } eq 'N' );
	    }
	  }
	  if ( $redundant ) {
	    printMessage(1, "Constraint %s on table %s in %s is missing in %s but handled by 'NOT NULL' column", $consName, $tabName, $srcName, $dstName );
	  }
	  else {
	    printMessage(2, "Constraint %s on table %s in %s is missing in %s.", $consName, $tabName, $srcName, $dstName );
	  }
	}
      }
      else {
	compareConstraint( $srcConfig, $dstConfig, $tabName, $consName );
      }
    }
  }

}


{ 
  my ( $srcConfig, $dstConfig );
  my ( $srcName, $dstName );
  my ( $srcSeqRef, $dstSeqRef );
  my $seqName;


  sub compareSeqAttr( $ ) {
    my $attrName = shift;

    if ( exists $srcSeqRef->{ $attrName } and exists $dstSeqRef->{ $attrName } ) {
      my $srcAttr = $srcSeqRef->{ $attrName };
      my $dstAttr = $dstSeqRef->{ $attrName };
      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  printMessage(2, "Sequence %s - %s mismatch: %s => %s and %s => %s", $seqName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  printMessage(2, "Sequence %s - %s mismatch: %s => %s and %s => %s", $seqName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	}
      }
      elsif ( defined $dstAttr ) {
	printMessage(2, "Sequence %s - %s mismatch: %s => %s and %s => %s", $seqName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
      }
    }
  }


  sub compareSequence( $$$ ) {
    $srcConfig = shift;
    $dstConfig = shift;
    $seqName = shift;

    $srcName = $srcConfig->{ input }->{ name };
    $dstName = $dstConfig->{ input }->{ name };
    
    $srcSeqRef = $srcConfig->{ sequences }->{ $seqName };
    $dstSeqRef = $dstConfig->{ sequences }->{ $seqName };

    compareSeqAttr( 'cycleFlag' );
    compareSeqAttr( 'orderFlag' );
  }
}


sub compareStats ( $$ ) {
  my $sourceConfig = shift;
  my $destConfig = shift;

  my $sourceConfigName = $sourceConfig->{ input }->{ name };
  my $destConfigName = $destConfig->{ input }->{ name };

  printf "Comparing %s configuration to %s configuration...\n", $sourceConfigName, $destConfigName;
  #my @srcKeys = keys %{ $sourceConfig->{ tables } };
  #my @dstKeys = keys %{ $destConfig->{ tables } };
  #printf "\tSource table Count = %i, Destination table count = %i\n", $#srcKeys, $#dstKeys;
  foreach my $tabName ( sort keys %{ $sourceConfig->{ tables } } ) {
    if ( not exists $destConfig->{ tables }->{ $tabName } ) {
      printMessage(3, "Table %s in %s is missing in %s.", $tabName, $sourceConfigName, $destConfigName );
    }
    else {
      compareTable( $sourceConfig, $destConfig, $tabName );
    }
  }

  foreach my $tabName ( sort keys %{ $destConfig->{ tables } } ) {
    if ( not exists $sourceConfig->{ tables }->{ $tabName } ) {
      printMessage(1, "Table %s is extra in %s.", $tabName, $destConfigName );
    }
    else {
      foreach my $colName ( sort keys %{ $destConfig->{ tables }->{ $tabName }->{ columns } } ) {
	if ( not exists $sourceConfig->{ tables }->{ $tabName }->{ columns }->{ $colName } ) {
	  printMessage(1, "Column %s.%s is extra in %s.", $tabName, $colName, $destConfigName );
	}
      }
      foreach my $indName ( sort keys %{ $destConfig->{ tables }->{ $tabName }->{ indexes } } ) {
	if ( not exists $sourceConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName } ) {
	  printMessage(2, "Index %s on table %s is extra in %s.", $indName, $tabName, $destConfigName );
	}
      }
      foreach my $consName ( sort keys %{ $destConfig->{ tables }->{ $tabName }->{ constraints } } ) {
	if ( not exists $sourceConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName } ) {
	  my $redundant = 0;
	  my $srch = $destConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ search };
	  if ( defined $srch and $srch =~ m/^\s*(\w+)\s+IS\s+NOT\s+NULL\s*$/ ) {
	    my $colName = $1;
	    if ( exists $sourceConfig->{ tables }->{ $tabName }->{ columns }->{ $colName } ) {
	      $redundant = ( $sourceConfig->{ tables }->{ $tabName }->{ columns }->{ $colName }->{ nullable } eq 'N' );
	    }
	  }
	  if ( $redundant ) {
	    printMessage(1, "Constraint %s on table %s is extra in %s but equivalent to 'NOT NULL' column", $consName, $tabName, $destConfigName );
	  }
	  else {
	    printMessage(2, "Constraint %s on table %s is extra in %s.", $consName, $tabName, $destConfigName );
	  }
	}
      }
    }
  }

  foreach my $seqName ( sort keys %{ $sourceConfig->{ sequences } } ) {
    if ( not exists $destConfig->{ sequences }->{ $seqName } ) {
      printMessage(3, "Sequence %s in %s is missing in %s.", $seqName, $sourceConfigName, $destConfigName );
    }
    else {
      compareSequence( $sourceConfig, $destConfig, $seqName );
    }
  }

  foreach my $seqName ( sort keys %{ $destConfig->{ sequences } } ) {
    if ( not exists $sourceConfig->{ sequences }->{ $seqName } ) {
      printMessage( 1, "Sequence %s is extra in %s.", $seqName, $destConfigName );
    }
  }
}


sub checkIndex( $$$$$ ) {
  my $srcConfig = shift;
  my $dstConfig = shift;
  my $tabName = shift;
  my $srcIndName = shift;
  my $dstIndName = shift;

  my $result = 1;

  my $saveIndCols = join(":", @{  $srcConfig->{ tables }->{ $tabName }->{ indexes }->{ $srcIndName }->{ columns } });
  my $dbIndCols = join(":", @{ $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $dstIndName }->{ columns } });

  $result = 0 if ( $saveIndCols ne $dbIndCols );

  return $result;
}


sub checkConstraint( $$$$$ ) {
  my $srcConfig = shift;
  my $dstConfig = shift;
  my $tabName = shift;
  my $srcConsName = shift;
  my $dstConsName = shift;

  my $srcConsRef = $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $srcConsName };
  my $dstConsRef = $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $dstConsName };

  my $result = 1;

  #print "Check Constraint: $tabName.$srcConsName with $tabName.$dstConsName\n";
  #print "    Types:  src = '", $srcConsRef->{ type },"',  dst = '",$dstConsRef->{ type },"'\n";
  $result = 0 if ( $srcConsRef->{ type } ne $dstConsRef->{ type } );
  if ( exists $srcConsRef->{ search } and exists $dstConsRef->{ search } ) {
    my $cmprSrc = $srcConsRef->{ search };
    my $cmprDst = $dstConsRef->{ search };
    $cmprSrc =~ s/\s//g;
    $cmprDst =~ s/\s//g;
    $result = 0 if ( $cmprSrc ne $cmprDst );
  }

  my $saveConsCols = join(":", sort { $srcConsRef->{ columns }->{ $a }->{ position } <=> $srcConsRef->{ columns }->{ $b }->{ position } } keys %{  $srcConsRef->{ columns } });
  my $dbConsCols = join(":", sort { $dstConsRef->{ columns }->{ $a }->{ position } <=> $dstConsRef->{ columns }->{ $b }->{ position } } keys %{ $dstConsRef->{ columns } });

  $result = 0 if ( $saveConsCols ne $dbConsCols );

  #print "    Result = $result\n";
  return $result;
}


{ 
  my ( $srcConfig, $dstConfig );
  my ( $srcName, $dstName );
  my ( $srcIndRef, $dstIndRef );
  my $tabName;
  my $indName;


  sub compareIndexAttr( $;$ ) {
    my $attrName = shift;
    my $sectionName = shift;

    my $srcRef = $srcIndRef;
    my $dstRef = $dstIndRef;

    $srcRef = $srcRef->{ $sectionName }  if defined $sectionName;
    $dstRef = $dstRef->{ $sectionName }  if defined $sectionName;

    if ( exists $srcRef->{ $attrName } and exists $dstRef->{ $attrName } ) {
      my $srcAttr = $srcRef->{ $attrName };
      my $dstAttr = $dstRef->{ $attrName };

      $srcAttr = join( ":", @{ $srcAttr } )  if ( defined $srcAttr and ref $srcAttr eq 'ARRAY' );
      $dstAttr = join( ":", @{ $dstAttr } )  if ( defined $dstAttr and ref $dstAttr eq 'ARRAY' );

      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  printMessage(2, "Index %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $indName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  printMessage(2, "Index %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $indName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	}
      }
      elsif ( defined $dstAttr ) {
	printMessage(2, "Index %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $indName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
      }
    }
  }


  sub compareIndex( $$$$ ) {
    $srcConfig = shift;
    $dstConfig = shift;
    $tabName = shift;
    $indName = shift;

    $srcName = $srcConfig->{ input }->{ name };
    $dstName = $dstConfig->{ input }->{ name };
    
    $srcIndRef = $srcConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName };
    $dstIndRef = $dstConfig->{ tables }->{ $tabName }->{ indexes }->{ $indName };

    compareIndexAttr( 'type' );
    compareIndexAttr( 'uniqueness' );
    compareIndexAttr( 'tablespace' );
    compareIndexAttr( 'compression' );
    compareIndexAttr( 'pct_free' );
    compareIndexAttr( 'freelists' );
    compareIndexAttr( 'freelist_groups' );

    compareIndexAttr( 'tablespace', 'part_info' );
    compareIndexAttr( 'part_type', 'part_info' );
    compareIndexAttr( 'part_key_count', 'part_info' );
    compareIndexAttr( 'pct_free', 'part_info' );
    compareIndexAttr( 'freelists', 'part_info' );
    compareIndexAttr( 'freelist_groups', 'part_info' );
    compareIndexAttr( 'part_keys', 'part_info' );

    my $srcIndCols = join(":", @{ $srcIndRef->{ columns } });
    my $dstIndCols = join(":", @{ $dstIndRef->{ columns } });
    if ( $srcIndCols ne $dstIndCols ) {
      printMessage( 3, "Index %s on table %s, col def '%s' in %s differs from '%s' in %s.\n", $indName, $tabName, $dstIndCols, $dstName, $srcIndCols, $srcName );
    }
  }

}


{ 
  my ( $srcConfig, $dstConfig );
  my ( $srcName, $dstName );
  my ( $srcConRef, $dstConRef );
  my $tabName;
  my $consName;


  sub compareConstraintAttr( $;$ ) {
    my $attrName = shift;
    my $compress = shift;

    if ( exists $srcConRef->{ $attrName } and exists $dstConRef->{ $attrName } ) {
      my $srcAttr = $srcConRef->{ $attrName };
      my $dstAttr = $dstConRef->{ $attrName };
      if ( defined $srcAttr ) {
	if ( not defined $dstAttr ) {
	  printMessage(2, "Constraint %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $consName, $attrName, $srcName, $srcAttr, $dstName, '<NULL>' );
	}
	elsif ( $srcAttr ne $dstAttr ) {
	  my $diff = 1;
	  my $wsDiff = 0;
	  if ( defined $compress ) {
	    $wsDiff = 1;
	    my $cSrcAttr = $srcAttr;
	    my $cDstAttr = $dstAttr;
	    $cSrcAttr =~ s/\s//g;
	    $cDstAttr =~ s/\s//g;
	    $diff = ( $cSrcAttr ne $cDstAttr );
	  }
	  if ( not $diff and $wsDiff ) {
	    printMessage(1, "Constraint %s.%s - %s: whitespace differences only", $tabName, $consName, $attrName );
	  }
	  elsif ( $diff and ( length( $srcAttr ) > 30 or length( $dstAttr ) > 30 ) ) {
	    printMessage(2, "Constraint %s.%s - %s mismatch:\n\t%30s => %s\n\t%30s => %s", $tabName, $consName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	  }
	  elsif ( $diff ) {
	    printMessage(2, "Constraint %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $consName, $attrName, $srcName, $srcAttr, $dstName, $dstAttr );
	  }
	}
      }
      elsif ( defined $dstAttr ) {
	printMessage(2, "Constraint %s.%s - %s mismatch: %s => %s and %s => %s", $tabName, $consName, $attrName, $srcName, '<NULL>', $dstName, $dstAttr );
      }
    }
  }


  sub compareConstraint( $$$$ ) {
    $srcConfig = shift;
    $dstConfig = shift;
    $tabName = shift;
    $consName = shift;

    $srcName = $srcConfig->{ input }->{ name };
    $dstName = $dstConfig->{ input }->{ name };
    
    $srcConRef = $srcConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName };
    $dstConRef = $dstConfig->{ tables }->{ $tabName }->{ constraints }->{ $consName };

    compareConstraintAttr( 'type' );
    compareConstraintAttr( 'status' );
    compareConstraintAttr( 'refName' );
    compareConstraintAttr( 'delRule' );
    compareConstraintAttr( 'deferrable' );
    compareConstraintAttr( 'deferred' );
    compareConstraintAttr( 'search', 1 );

    my $srcConCols = join(":", sort { $srcConRef->{ columns }->{ $a }->{ position } <=> $srcConRef->{ columns }->{ $b }->{ position } 
				      or $a cmp $b
				    } keys %{ $srcConRef->{ columns } });
    my $dstConCols = join(":", sort { $dstConRef->{ columns }->{ $a }->{ position } <=> $dstConRef->{ columns }->{ $b }->{ position }
				      or $a cmp $b} keys %{ $dstConRef->{ columns } });
    if ( $srcConCols ne $dstConCols ) {
      printMessage( 3, "Constraint %s on table %s, col def '%s' in %s differs from '%s' in %s.\n", $consName, $tabName, $dstConCols, $dstName, $srcConCols, $srcName );
    }
  }

}


sub cleanConfig ( $ ) {
  my $configRef = shift;

  my $configPath = $configRef->{ input }->{ name };

#  my @ignoreTab = ( 'UPGRADE_CONSTRAINTS', 'WMPUPG_TRIGGER_COLS', 'UPGRADE_SQL', 'PRE_POST_UPGRADE', 'UPGRADE_OBJECTS', 'UPGRADE_INFO', 'UPGRADE_IGNORE', 'PLAN_TABLE', 'EXCEPTION_TABLE', 'ACARR_CCARR_RUP' );
  my @ignoreTab = ( 'ACARR_CCARR_RUP' );
#  my @ignoreCols = ('LUC_TF_TDMA_TAB.TDMA_PAF_F9','LUC_TF_CELL_CP_TAB.CDMA_MCA_F1','LUC_TF_CELL_CP_TAB.CDMA_MCA_F1',
#		    'LUC_TF_CELL_CP_TAB.CDMA_MCA_F2','LUC_TF_CELL_CP_TAB.CDMA_MCA_F3','LUC_TF_CELL_CP_TAB.CDMA_MCA_F4',
#		    'LUC_TF_CELL_CP_TAB.CDMA_MCA_F5','LUC_TF_CELL_CP_TAB.CDMA_MCA_F6',
#		    'LUC_1XEV_TP_TAB.RPCONFAIL','LUC_1XEV_TP_TAB.RPCONATT','LUC_1XEV_HCS_TAB.SFTSFTRHOATTTCA',
#		    'LUC_1XEV_HCS_TAB.CELLREL_1XEV_DOCNTR', 'LUC_1XEV_TP_TAB.TOTACTSESSTP' );
  my @ignoreCols = ( );
  my @ignoreDBHCols = @ignoreCols;
  my @ignoreDSMCols = @ignoreCols;
  my @ignoreInds = ( );
  map s/TAB/DBH/g, @ignoreDBHCols;
  map s/TAB/DSM/g, @ignoreDSMCols;
  foreach my $tabName ( sort keys %{ $configRef->{ tables } } ) {
    if ( grep { /^$tabName$/ } @ignoreTab ) {
      printMessage( 0, "Ignoring table '%s' in %s.", $tabName, $configPath );
      delete $configRef->{ tables }->{ $tabName };
    }
    else {
      foreach my $colName ( keys %{ $configRef->{ tables }->{ $tabName }->{ columns } } ) {
	if ( grep { /^$tabName\.$colName$/ } @ignoreCols ) {
	  printMessage( 0, "Ignoring column '%s.%s' in %s.", $tabName, $colName, $configPath );
	  delete $configRef->{ tables }->{ $tabName }->{ columns }->{ $colName };
	}
	if ( grep { /^$tabName\.$colName$/ } @ignoreDBHCols ) {
	  printMessage( 0, "Ignoring column '%s.%s' in %s.", $tabName, $colName, $configPath );
	  delete $configRef->{ tables }->{ $tabName }->{ columns }->{ $colName };
	}
	if ( grep { /^$tabName\.$colName$/ } @ignoreDSMCols ) {
	  printMessage( 0, "Ignoring column '%s.%s' in %s.", $tabName, $colName, $configPath );
	  delete $configRef->{ tables }->{ $tabName }->{ columns }->{ $colName };
	}
      }
      foreach my $indName ( keys %{ $configRef->{ tables }->{ $tabName }->{ indexes } } ) {
	if ( grep { /^$indName$/ } @ignoreInds ) {
	  printMessage( 0, "Ignoring index '%s' on table '%s' in %s.", $indName, $tabName, $configPath );
	  delete $configRef->{ tables }->{ $tabName }->{ indexes }->{ $indName };
	}
      }

      foreach my $consName ( sort keys %{ $configRef->{ tables }->{ $tabName }->{ constraints } } ) {
	my $consRef = $configRef->{ tables }->{ $tabName }->{ constraints }->{ $consName };
	if ( exists $consRef->{ search } and defined $consRef->{ search } ) {
	  my $oldSrch = $consRef->{ search };
	  my $newSrch = uc $oldSrch;
	  $newSrch =~ s/\"//g;
	  $newSrch =~ s/\s\s+/ /g;
	  if ( $newSrch ne $oldSrch ) {
	    #print "Change: '$oldSrch'\n    To: '$newSrch'\n";
	    $consRef->{ search } = $newSrch;
	  }
	}
      }


    }
  }

}
  
  
sub writeConfig( $$ ) {
  my $configFile = shift;
  my $statHashRef = shift;

  my %sysNames = ();
  my $sysNameId = 0;

  if ( $zipFiles ) {
    open DATAFILE, "| gzip -c >$configFile" or die "Can't open output file $configFile\n";
  } else {
    open DATAFILE, ">$configFile" or die "Can't open output file $configFile\n";
  }

  print "Writing stored configuration to $configFile \n";

  my $hostName = `hostname`;
  chomp $hostName;
  my $domainName = `domainname`;
  chomp $domainName;
  my $runDate = `date`;
  chomp $runDate;
  print  DATAFILE "#\t./base/migrate/upgrades/verify/dbver.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527\n";
  printf DATAFILE "#\tcreated: %s\n", $runDate;
  printf DATAFILE "#\thost = %s, domain = %s\n", $hostName, $domainName;
  printf DATAFILE "#\tsource = '%s' %s\n", $statHashRef->{ input }->{ type }, $statHashRef->{ input }->{ name };

  foreach my $tabName ( sort keys %{ $statHashRef->{ tables } } ) {
    my $tabRef = $statHashRef->{ tables }->{ $tabName };
    printf DATAFILE "TABLE\t%s\t%s\n", $tabName, $tabRef->{ type };
    if ( $tabRef->{ type } eq 'TABLE' ) {
      printf DATAFILE "TABINFO\t%s\t%s\t%s\t%s\t%s", $tabRef->{ tablespace }, $tabRef->{ monitoring }, $tabRef->{ partitioned }, $tabRef->{ iot }, $tabRef->{ temp };
      printf DATAFILE "\t%s\t%s\t%s\t%s\t%s\n", $tabRef->{ pct_free }, $tabRef->{ pct_used }, $tabRef->{ freelists }, $tabRef->{ freelist_groups }, $tabRef->{ duration };

      if ( exists $tabRef->{ part_info } ) {
	my $partTabRef = $tabRef->{ part_info };
	printf DATAFILE "TABPART\t%s\t%s\t%s", $partTabRef->{ tablespace }, $partTabRef->{ part_type }, $partTabRef->{ part_key_count };
	printf DATAFILE "\t%s\t%s\t%s\t%s", $partTabRef->{ pct_free }, $partTabRef->{ pct_used }, $partTabRef->{ freelists }, $partTabRef->{ freelist_groups };
	printf DATAFILE "\t%s\n", join(";", @{ $partTabRef->{ part_keys } } );
      }
    }
    elsif ( $tabRef->{ type } eq 'VIEW' ) {
      printf DATAFILE "VIEWTEXT START\n%s\nVIEWTEXT END\n", $tabRef->{ viewtext }   if exists $tabRef->{ viewtext };
    }

    foreach my $colName ( sort keys %{ $statHashRef->{ tables }->{ $tabName }->{ columns } } ) {
      my $colRef = $statHashRef->{ tables }->{ $tabName }->{ columns }->{ $colName };
      printf DATAFILE "COLUMN\t%s\t%s\t%s\t%s\t%s\t%s\t%i\t%s\n", $colName, $colRef->{ type },
             $colRef->{ len }, $colRef->{ prec }, $colRef->{ scale }, $colRef->{ nullable }, $colRef->{ ordinal }, $colRef->{ default };
    }

    foreach my $indName ( sort keys %{ $statHashRef->{ tables }->{ $tabName }->{ indexes } } ) {
      my $indRef = $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName };
      #print "IndKeys  $tabName.$indName = ", join(", ", keys %{  $statHashRef->{ tables }->{ $tabName }->{ indexes }->{ $indName } } ), "\n";
      defined $indRef->{ type } or print "Undefined type - $tabName.$indName\n";
      defined $indRef->{ uniqueness } or print "Undefined uniqueness - $tabName.$indName\n";
      defined $indRef->{ tablespace } or print "Undefined tablespace - $tabName.$indName\n";
      defined $indRef->{ compression } or print "Undefined compression - $tabName.$indName\n";
      if ( $replSysNames and ( $indName =~ m/^SYS_C\d*$/ ) ) {
	$sysNames{ $indName } = sprintf 'SYS_Cxxxx%05i', ++$sysNameId   unless exists $sysNames{ $indName };
	$indName = $sysNames{ $indName };
      }
      printf DATAFILE "INDEX\t%s\t%s\t%s\t%s\t%s", $indName, $indRef->{ type }, $indRef->{ uniqueness }, $indRef->{ tablespace }, $indRef->{ compression };
      printf DATAFILE "\t%s\t%s\t%s\n", $indRef->{ pct_free }, $indRef->{ freelists }, $indRef->{ freelist_groups };

      if ( exists $indRef->{ part_info } ) {
	my $partIndRef = $indRef->{ part_info };
	printf DATAFILE "INDPART\t%s\t%s\t%s", $partIndRef->{ tablespace }, $partIndRef->{ part_type }, $partIndRef->{ part_key_count };
	printf DATAFILE "\t%s\t%s\t%s", $partIndRef->{ pct_free }, $partIndRef->{ freelists }, $partIndRef->{ freelist_groups };
	printf DATAFILE "\t%s\n", join(";", @{ $partIndRef->{ part_keys } } );
      }

      my $indCols = $indRef->{ columns };
      foreach my $indColName ( @{ $indCols } ) {
	printf DATAFILE "INDCOL\t%s\n", $indColName;
      }
    }

    foreach my $consName ( sort keys %{ $statHashRef->{ tables }->{ $tabName }->{ constraints } } ) {
      my $consRef = $statHashRef->{ tables }->{ $tabName }->{ constraints }->{ $consName };
      if ( $replSysNames and ( $consName =~ m/^SYS_C\d*$/ ) ) {
	$sysNames{ $consName } = sprintf 'SYS_Cxxxx%05i', ++$sysNameId   unless exists $sysNames{ $consName };
	$consName = $sysNames{ $consName };
      }
      printf DATAFILE "CONSTRAINT\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", $consName, $consRef->{ type }, $consRef->{ status }, $consRef->{ refName }, 
      $consRef->{ delRule }, $consRef->{ deferrable }, $consRef->{ deferred }, $consRef->{ search };
      #my $consCols = $consRef->{ columns };
      foreach my $consColName ( sort { $consRef->{ columns }->{ $a }->{ position } <=>  $consRef->{ columns }->{ $b }->{ position }
				       or $a cmp $b
				     } keys %{ $consRef->{ columns } } ) {
	printf DATAFILE "CONSCOL\t%s\t%s\n", $consColName, $consRef->{ columns }->{ $consColName }->{ position };
      }
    }
  }

  foreach my $seqName ( sort keys %{ $statHashRef->{ sequences } } ) {
    my $seqRef = $statHashRef->{ sequences }->{ $seqName };
    printf DATAFILE "SEQUENCE\t%s\t%s\t%s\n", $seqName, $seqRef->{ cycleFlag }, $seqRef->{ orderFlag };
  }

  close DATAFILE;

  print "Completed writing stored configuration.\n";
}


sub runSQL ( $$ ) {
  my $connectString = shift;
  my $sqlStmt = shift;

#  print "runSQL - $sqlStmt\n";

  my @rawVal = `sqlplus -s $connectString <<EOF
SET HEAD OFF
SET FEEDBACK OFF
SET TAB OFF
SET PAGES 0
SET COLSEP '**'
SET LONG 1000
SET LINES 2000
COLUMN table_name FORMAT A30
COLUMN column_name FORMAT A40
COLUMN data_type FORMAT A106
COLUMN data_length FORMAT 99999
COLUMN data_precision FORMAT 99999
COLUMN data_scale FORMAT 99999
$sqlStmt;
EXIT;
EOF
`;

  #  printf "   %i lines returned\n", $#rawVal;

  my @returnVal;

  foreach my $rawLine ( @rawVal ) {
    chomp $rawLine;
    my @fields=split /\*\*/, $rawLine, -1;
    for ( my $fieldInd = 0; $fieldInd <= $#fields; $fieldInd++ ) {
      $fields[ $fieldInd ] =~ s/^\s+//;
      $fields[ $fieldInd ] =~ s/\s+$//;
    }
    my $returnLine = join("\t", @fields);
    push @returnVal, $returnLine;
  }

  #  printf"    %i lines to caller\n", $#returnVal;
  return @returnVal;
}


sub printMessage( $$@ ) {
  my $level = shift;
  my $msg = shift;
  my @params = @ARG;
  
  my @labels = ( "NOTE:", "INFO:", "WARNING:", "ERROR:" );

  $level = 0 if ( $level < 0 );
  $level = $#labels if ( $level > $#labels );

  ( $level >= $verbosity ) and printf "  %-8s $msg\n", $labels[ $level ], @params;
}
