#!/bin/ksh
###################################################################
# @(#) configure-smtp.sh 1.6.0 04/04/20 17:14:38
# @(#) 
#
# COPYRIGHT WatchMark Corp 2002-2004
###################################################################
#
# Registers SMTP entries in WM_SYSTEM_VARIABLE / WM_SYSTEM_VALUE
# tables.
#
# The values are used for sending email notification.
#
#Updated 10/02/2009 - valnt00077479
DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`

SCRIPTNAME=`basename $0`

RESPONSE=""
SMTP_MAIL_SERVER=""
SMTP_FROM_ADDRESS=""
SMTP_MAIL_PORT=""
SMTP_MAIL_PORT_DEFAULT=25
SMTP_USER=""
SMTP_PASSWORD=""

SMTP_FIREWALL_HOST=""
SMTP_FIREWALL_TYPE=""
SMTP_FIREWALL_TYPE_DEFAULT=0
SMTP_FIREWALL_USER=""
SMTP_FIREWALL_PASSWORD=""
SMTP_FIREWALL_PORT=""
EMPTY=""

usage()
{
    echo "$SCRIPTNAME   [-l | -v]"
    echo "$SCRIPTNAME        :: Interactive input"
    echo "$SCRIPTNAME   -l   :: List current SMTP values"
    echo "$SCRIPTNAME   -v   :: Show usage"
    exit 1
}

get_response()
{
    while true
    do
        echo "Enter $1"
        read RESPONSE
        if [ -n ${RESPONSE:-""} ]
        then
            break
        fi
    done

    echo "\n"
}

getYesNo()
{
    while true
    do
        echo "\nDo you want to $1? (y/n or q) [default: $2]"
        read RESPONSE
        if [ ${RESPONSE:-${2}} = y ]
        then
            RESPONSE=y
            break
        fi
        if [ ${RESPONSE:-${2}} = n ]
        then
            RESPONSE=n
            break
        fi
        if [ ${RESPONSE:-${2}} = q ]
        then
            RESPONSE=q
            break
        fi
    done
}

#Mail server values
get_smtp_required_values()
{
    get_response "SMTP Mail Server. Specify a fully qualified name."
    SMTP_MAIL_SERVER=$RESPONSE

    get_response "SMTP Mail From Address."
    SMTP_FROM_ADDRESS=$RESPONSE
}

# Optional values for SMTP.
get_smtp_optional_values()
{
    echo "Enter SMTP Mail Port. [default: $SMTP_MAIL_PORT_DEFAULT]"
    read RESULT
    SMTP_MAIL_PORT=${RESULT:-$SMTP_MAIL_PORT_DEFAULT}

    echo "Enter SMTP User. (Optional)"
    read SMTP_USER

    if [ -n ${SMTP_USER:-""} ]
    then
        get_response "SMTP Password."
        SMTP_PASSWORD=$RESPONSE
    fi
}

# Firewall values
get_firewall_values()
{
    echo "Enter SMTP Firewall Host. (Optional)"
    read SMTP_FIREWALL_HOST

    if [ -n ${SMTP_FIREWALL_HOST:-""} ]
    then
        get_response "SMTPFirewallPort" 
        SMTP_FIREWALL_PORT=$RESPONSE

        echo "SMTP Firewall Type. (Select from 0 - 3.) [default: $SMTP_FIREWALL_TYPE_DEFAULT]"
        read RESULT
        SMTP_FIREWALL_TYPE=${RESULT:-$SMTP_FIREWALL_TYPE_DEFAULT}

        echo "Enter SMTP Firewall User. (Optional)"
        read SMTP_FIREWALL_USER

        if [ -n ${SMTP_FIREWALL_USER:-""} ]
        then
            get_response "SMTP Firewall Password"
            SMTP_FIREWALL_PASSWORD=$RESPONSE
        fi
    fi
}

updateSMTP()
{
    echo "Updating SMTP."
    $ORACLE_HOME/bin/sqlplus  -s  $DB_CONNECT <<! 

     set feedback off

     EXEC wm_smtp.set_values( '$SMTP_MAIL_SERVER',     -
                              '$SMTP_FROM_ADDRESS',    -
                              '$SMTP_MAIL_PORT',       -
                              '$SMTP_USER',            -
                              '$SMTP_PASSWORD',        -
                              '$SMTP_FIREWALL_HOST',   -
                              '$SMTP_FIREWALL_PORT',   -
                              '$SMTP_FIREWALL_TYPE',   -
                              '$SMTP_FIREWALL_USER',   -
                              '$SMTP_FIREWALL_PASSWORD');

     COMMIT;
!
}

printSMTP()
{
    $ORACLE_HOME/bin/sqlplus  -s  $DB_CONNECT <<!
    set serveroutput on size 1000000
    set feedback off
    set lines 100 pages 50
    exec wm_smtp.print_values;
!
}

printValues()
{
    echo "SMTP Mail Server : ${SMTP_MAIL_SERVER}"
    echo "SMTP Mail From Address : ${SMTP_FROM_ADDRESS}"

    if [ -n ${SMTP_FIREWALL_HOST:-""} ]
    then
        echo "SMTP Firewall Host : ${SMTP_FIREWALL_HOST}"
        echo "SMTP Firewall Port : ${SMTP_FIREWALL_PORT}"
        echo "SMTP Firewall Type : ${SMTP_FIREWALL_TYPE}"
        echo "SMTP Firewall User : ${SMTP_FIREWALL_USER}"
        echo "SMTP Firewall Password : ${SMTP_FIREWALL_PASSWORD}"
    fi
}

# Process command arguments if any
while [ $# -gt 0 ]
do

    case ${1} in
        -l)
            printSMTP
	    exit
            ;;

        -v)
            usage
            ;;

        *)
            echo "$SCRIPTNAME: Invalid parameter ${1}"
            usage
            ;;
    esac

    shift

done

while true
do
    echo "Enter values for email notification.\n"
    get_smtp_required_values   
    get_smtp_optional_values
    get_firewall_values

    echo "You entered:"
    printValues

    echo "\nTo continue and update a database table, select y."
    echo "To re-enter values, select n." 
    echo "To quit without changing the database, select q." 
    getYesNo "Update a database table" y

    if [ ${RESPONSE} = y ] 
    then
	updateSMTP
        echo "\n\nYou must restart the eventhandler for these changes to"
        echo "take effect :"
        echo "\n             ps-mgr restart eh"
        break
    fi

    if [ ${RESPONSE} = q ]
    then
       echo "\n\nThe database has not been updated with your changes."
       break
    fi
done
