#!/usr/bin/env perl -w
#----------------------------------------------------------------------
# (#) ./base/pm/adm/create_users.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
# Copyright 2004 WatchMark Corp.
#----------------------------------------------------------------------
#


use strict;
use Getopt::Long;
use File::Basename;

BEGIN {
  unshift @INC, $ENV{ FLEXPM_HOME } . "/sw/perl/lib"  if   exists $ENV{ FLEXPM_HOME };
}

use DBI;

# Prototypes
sub usage($);
sub show_help();
sub get_base_folder();
sub acct_exists($);
sub create_acct();
sub parse_line($$);

# 
my $SCRIPT_ID = basename $0;

# Vars
my $acct;
my $dbh;
my $sth;
my $usercnt;
my $basefolder;
my $usage;
my $line_number;
my %userRecord;

# Options
my $dbConnect;
my $acctList;
my $help;


# Defaults
my $DELIMITER =",";
my $DEFAULT_PASSWORD = "~WxwH}n-pF";          # DO NOT CHANGE - "ChangeMe"
my $DEFAULT_TYPE = "N";                       # Type : N-normal/D-Data Admin/S-System Admin
my $DEFAULT_AUTOPURGEREQUIRED = "N";          # AutoPurgeRequired: Y-yes/N - no
my $DEFAULT_MAXUSAGE = 10;                    # MaxUsage MB: 0-99
my $DEFAULT_MAXPURGEINTERVAL = 999;           # MaxPurgeInterval
                                              #  =0 - Requires purge on download
                                              #  >0 - Specifies max days user may retain output after download



   GetOptions( 'connect=s' => \$dbConnect,
               'file=s' => \$acctList,
               'help' => \$help );

   # Do we need help?
   show_help() if defined $help;
   usage('ABORT') if not defined $dbConnect;
   usage('ABORT') if not defined $acctList;

   open(SOURCE, "< $acctList") or die "Couldn't open $acctList for reading: $!\n";

   $dbh = DBI->connect( 'dbi:Oracle:', $dbConnect, '' )
          or die "Error: Can't connect to '$dbConnect' ($DBI::errstr).\n";

# Get the ID for the /Users folder
   $basefolder = get_base_folder();
   #print "BASE: $basefolder\n";

# Prepare userRecord
   $userRecord{"folderid"}          = $basefolder;
   $userRecord{"password"}          = $DEFAULT_PASSWORD;
   $userRecord{"autopurgerequired"} = $DEFAULT_AUTOPURGEREQUIRED;
   $userRecord{"home"}              = "";  # will be set  in parse_line()
   $userRecord{"name"}              = "";  #
   $userRecord{"description"}       = "";  #
   $userRecord{"type"}              = "";  #
   $userRecord{"maxpurgeinterval"}  = "";  #
   $userRecord{"maxusage"}        = "";  #

# Loop over names
   $line_number=0;
   while (<SOURCE>) {

      chomp;
      $acct = $_;
      $line_number++;

      # Ignore comments and blank lines
      next if ( $acct =~ /^[ 	]*#/  || $acct =~ /^[ 	]*$/ );

      # Parse input and set userRecord. Skip line on error
      next if ( ! parse_line($line_number, $acct) );

      # Create account
      create_acct();
   }

   close(SOURCE);

   $dbh->disconnect;

exit;



#
# SUBS
#


#-----------------------------------------------------------------------------
sub usage ($)
#-----------------------------------------------------------------------------
{
   my $action = shift;

   print STDERR "usage: $SCRIPT_ID -connect <connectstring> -file <filename> \n";
   print STDERR "usage: $SCRIPT_ID -help\n";
   print STDERR "\n";

   exit(-1) if ( $action eq 'ABORT' );
}

#-----------------------------------------------------------------------------
sub get_base_folder()
#-----------------------------------------------------------------------------
{
   my $id;
   $sth = $dbh->prepare( " 
      SELECT id
        FROM PM_FOLDER
       WHERE UPPER(name) = 'USERS'
         AND folderid = ( SELECT id
                            FROM PM_FOLDER
                           WHERE folderid IS NULL
                             AND name = 'ROOT'
                        )
      " );

   $sth->execute();
   $sth->bind_columns(undef, \$id);

   while( $sth->fetch() ) {
   }

   return $id;

}

#-----------------------------------------------------------------------------
sub acct_exists($)
#-----------------------------------------------------------------------------
{
   my $name = shift;
   my $quote_name = $dbh->quote( $name );

   $sth = $dbh->prepare( " 
      SELECT count(*) usercnt
      FROM PM_USER
      WHERE UPPER(name) = UPPER($quote_name)
      " );

   $sth->execute();
   $sth->bind_columns(undef, \$usercnt);

   while( $sth->fetch() ) {
   }

   return $usercnt;
}

#-----------------------------------------------------------------------------
sub create_acct()
#-----------------------------------------------------------------------------
{

    $sth = $dbh->prepare( "BEGIN FS_CreateUser( ?, ?, ?, ?, ?, ?, ?, ?, ? ); END;" );
 
    $sth->bind_param(1, $userRecord{"folderid"} );
    $sth->bind_param(2, $userRecord{"home"} );
    $sth->bind_param(3, $userRecord{"name"} );
    $sth->bind_param(4, $userRecord{"description"} );
    $sth->bind_param(5, $userRecord{"password"} );
    $sth->bind_param(6, $userRecord{"type"} );
    $sth->bind_param(7, $userRecord{"autopurgerequired"} );
    $sth->bind_param(8, $userRecord{"maxpurgeinterval"} );
    $sth->bind_param(9, $userRecord{"maxusage"} );

    print STDOUT "CREATED: " . $userRecord{"name"} . "\n";
    $sth->execute();

}



#----------------------------------------------------------------------
sub parse_line($$)
#----------------------------------------------------------------------
{
# username,Description,Type (User/DataAdmin/SysAdmin),maxusage,purge limit
#
   my $lineNumber = shift;
   my $dataLine = shift;
   my $return_status = 1;
   my @arrayLine;

   $dataLine =~ s/[ 	]*,[ 	]*/,/g;
   $dataLine =~ s/^[ 	]*//g;
   $dataLine =~ s/[ 	]*$//g;
   $dataLine = $dataLine . $DELIMITER . $DELIMITER . $DELIMITER . $DELIMITER;
   @arrayLine = split( $DELIMITER, $dataLine, 6 );

   $userRecord{"name"} = $arrayLine[0];
   $userRecord{"description"} = $arrayLine[1];
   $userRecord{"type"} = $arrayLine[2];
   $userRecord{"maxusage"} = $arrayLine[3];
   $userRecord{"maxpurgeinterval"} = $arrayLine[4];
   $userRecord{"home"} = $userRecord{"name"};

   # Check for name syntax
   if ( !( $userRecord{"name"} =~ /^[A-Za-z0-9_]+$/ ) ) {
       print STDERR "INVALID USER NAME: " . $userRecord{"name"} . ", skipping\n";
       $return_status = 0;
   
   # Check for name existence
   } elsif ( acct_exists($userRecord{"name"}) > 0 ) {
       print STDERR "USER: " . $userRecord{"name"} . " exists, skipping\n";
       $return_status = 0;

   } else {

      if ( $userRecord{"description"} eq "" ) {
         $userRecord{"description"} = "User ". $userRecord{"name"};
      }

      if ( !($userRecord{"type"} =~ /^[NDS]$/) ) {
         print STDERR "INVALID USER TYPE: " . $userRecord{"type"} 
                    . ", using default (" . $DEFAULT_TYPE . ")\n";
         $userRecord{"type"} = $DEFAULT_TYPE;
      }

      if ( !($userRecord{"maxusage"}  =~ /^[0-9]+$/ ) || 
           ($userRecord{"maxusage"} < 0)            ||
           ($userRecord{"maxusage"} > 99)
         ) {
         print STDERR "INVALID MAX USAGE: " . $userRecord{"maxusage"} 
                    . ", using default (" . $DEFAULT_MAXUSAGE . ")\n";
         $userRecord{"maxusage"} = $DEFAULT_MAXUSAGE;
      }

      if ( !($userRecord{"maxpurgeinterval"}  =~ /^[0-9]+$/) ) {
         print STDERR "INVALID MAX PURGE INTERVAL: " . $userRecord{"maxpurgeinterval"} 
                    . ", using default (" . $DEFAULT_MAXPURGEINTERVAL . ")\n";
         $userRecord{"maxpurgeinterval"} = $DEFAULT_MAXPURGEINTERVAL;
      }

   }

   return $return_status;

}

#-----------------------------------------------------------------------------
sub show_help ()
#-----------------------------------------------------------------------------
{

usage('NOEXIT');
print STDERR qq~
#
# The create_users.pl is a perl script that takes a dbconnect string
# and an input file that contains user info.
#
# Input file is a list of lines with 5 fields of comma-separated values in
# the following order:
#   UserName   : alphanumeric plus underscore. Required
#   Description: text description of User. Commas not allowed. Optional
#   Type       : type of user. Single character (S/D/N). Optional (defaulted)
#                    S - System Administrator
#                    D - Data Administrator
#                    N - normal User
#   MaxUsage   : Maximum disk usage in MB (0-99). Optional (defaulted)
#   PurgeLimit : Maximum purge interval in days. Optional (defaulted)
#
# For default values, please see "# Defaults" section in the script.
# Blank lines and Comments (line starting with #) are ignored.
#
# The usernames are checked to see that they do not exist. Invalid usernames are
# skipped. Invalid values in the other fields are discarded and default values
# are substituted.
#
# If the user info is valid, the account is created with a default password.
#
# The script has several default values hard-coded in. These may be changed by editing
# the script (see "# Defaults" of the script. Be careful with the password. It must be
# pre-encrypted to be added to the database in this manner.
#

#---------------------------------------------------------------------------
# SAMPLE INPUT FILE (users.list)
#---------------------------------------------------------------------------
kryten,Generated Sys Admin,S,99,
kochanski,Generated Data Admin,D,50,500
starbug,Generated User


#---------------------------------------------------------------------------
# SAMPLE OUTPUT
#---------------------------------------------------------------------------
Sample output:

\$ create_users.pl -connect 'adminuser/adminpasswd\@oraclesid' -f ./users.list
INVALID MAX PURGE INTERVAL: , using default (999)
CREATED: kryten
USER: kochanski exists, skipping
USER: starbug exists, skipping
INVALID USER NAME: prospect-98, skipping
INVALID USER NAME: prospect 99, skipping

# End
~;
exit (0);


}

# End
