#!/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2009,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/pm/adm/createinstance.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
# Eoghan Shields  09/04/09
#
APPLICATION=`basename $0`
ENTITY_NAME="UNSET";
DEBUG="N"
KEY="UNSET";
RESULT_COUNT=0;
SQLPLUS_OPTS="
		SET ECHO OFF
		SET NEWPAGE 0
		SET SPACE 0
		SET PAGESIZE 0
		SET FEEDBACK OFF
		SET HEADING OFF
		SET TRIMSPOOL ON
		SET TAB OFF
		SET SERVEROUTPUT ON
		-- default set of options to be applied to all sqlplus activities
		"

#when we run an executeSQL our results are put here
SQLPLUS_RESULTS="";
export EXIT_STATE=1
EXIT_MSG="";
DB_CONNECT=`crypt_util.ksh -g $DBUSER -c`

function userhelp
{
	if [ ! -z $1 ]; then
		if [ $1 = "--help" ] || [ $1 = "-h" ]; then
			logHelp "[-entityname] A valid entity name on the system e.g. BTS"
			logHelp "[-key] The full key of the entity"
			logHelp "[-debug] Print out debug messages"
			logHelp "run $APPLICATION with -entityname <Name of Entity> -key <the full key>"
			exit 3;
		fi
	fi
}
#Handy function to log debug messages
function logDebug
{
	if [ $DEBUG = "Y" ]; then
		echo "`date` [HELP] - $1"
	fi
}
#Handy function to log information messages
function logInfo
{
	echo "`date` [INFO] - $1"
}
#Handy function to log warning messages
function logWarning
{
	echo "`date` [WARNING] - $1"
}
#Handy function to log error messages
function logError
{
	echo "`date` [ERROR] - $1"
}
#Handy function to log help messages
function logHelp
{
	echo "`date` [HELP] - $1"
}
#Tidy up running sql where possible
function executeSQL
{
	logDebug "$SQLPLUS_OPTS $1"
	SQLPLUS_RESULTS=`echo "$SQLPLUS_OPTS $1"|sqlplus -s $DB_CONNECT`
	RESULT_COUNT=`echo "$SQLPLUS_OPTS $1"|sqlplus -s $DB_CONNECT|wc -l`
	exitStatus $?
}
#Create a list of results from say a select for a particular colum on a table
function formatSQLResults
{
	if [ -z $1 ]; then
		echo $SQLPLUS_RESULTS|tr -s ' ' '\n';
	else
		echo $1|tr -s ' ' '\n';
	fi
}
#log exit status
function exitStatus
{
	EXIT_STATE=$1;
	setExitMsg
}
#set some exit messages
function setExitMsg
{
	if [ $EXIT_STATE -eq 1 ]; then
		EXIT_MSG="Exiting, Key already exists on system"
	elif [ $EXIT_STATE -eq 3 ]; then
		EXIT_MSG="Exiting, an error occured"
	elif [ $EXIT_STATE -eq 0 ]; then
		EXIT_MSG="Exiting, successful creation"
	else
		EXIT_MSG="Exiting with a $EXIT_STATE"
	fi
}
#parse our input arguments
function parseArgs
{
	#parse all our input arguments
	while [[ $1 = -* ]]; do
		case $1 in
			-entityname)
				ENTITY_NAME=$2
				shift;;

			-key)
				KEY=$2;
				shift;;

			-debug)
				DEBUG="Y";;
						
			*)	logError "$1 is not a valid argument in $APPLICATION"
				userhelp -h;
				exit 3;;#error exit
		esac
		shift
	done
	#validate that all required arguments have been set
	if [ $ENTITY_NAME = "UNSET" ] || [ $KEY = "UNSET" ] ; then
		logError "$APPLICATION expects an entityname and a full key\n";
		userhelp -h;
		exitStatus 3;
		exit $EXIT_STATE;#error exit
	fi
}
#ensure that this is a valid entity
function validateEntityName
{
	#get a list of entities from prospect
	executeSQL "select distinct(entity_name) from wmn_entity where upper(entity_name) like upper('%${ENTITY_NAME}%');"
	LIKE=`formatSQLResults`;
	#count results
	LIKE_RC=$RESULT_COUNT;
	#search for the specific one we are interested in
	executeSQL "select distinct(entity_name) from wmn_entity where upper(entity_name) = upper('${ENTITY_NAME}');"
	EQUAL=`formatSQLResults`;
	EQUAL_RC=$RESULT_COUNT;
	if [ $EQUAL_RC -gt 0 ]; then
		#We found a result
		logDebug "$RC result(s) found for $EQUAL"
		logInfo "$ENTITY_NAME is a valid entityname"
		EQUAL=`formatSQLResults`;
	else
		#We found no result
		logError "$ENTITY_NAME is not a valid entityname";
		logHelp "Did not find entity called \"$ENTITY_NAME\"."
		if [ $LIKE_RC -gt 0 ]; then
			#We found a results that are like what user inputted. handy for the help messages
			logHelp "Found these entities that contain $ENTITY_NAME:\n"
			TMPCOUNT=1
			LIKE=`echo $LIKE|sort`
			for i in $LIKE; do
				#just format these into a nice list
				printf "%30s\t" $i
				VAL=$(($TMPCOUNT % 3))
				if [ $VAL -eq 0 ]; then 
					printf "\n"
				fi
				TMPCOUNT=$((${TMPCOUNT}+1))
			done
			printf "\n"
		fi
		exitStatus 3;
		exit $EXIT_STATE;
	fi
}
#check that the key is valid and determine whether to create it 
function validateAndCreateKey
{
	sqlplus -S ${DB_CONNECT} <<EOF
${SQLPLUS_OPTS}
--prepend the default options
variable exitValue number;
declare
  vContextID integer;
  vEntityID integer;
  vResult integer;
  vParentEntityID integer;
  vFullkey varchar2(128);
  vParentCount integer;
  counter integer;
  keyindexr integer;
  keytest varchar2(128);
  keylen integer;
  lev integer;
begin
  :exitValue := 3;
  vResult := -1;
  --we always look for generic context, used for reporting etc
   select context_id into vContextID from wmn_context where upper(context_use)=upper('R');
  --get our entity id
  vEntityID := WM_Network.Get_Entity( '${ENTITY_NAME}' );
  --check does the instnace already exist
  vResult := WM_Network.Search_Instance(vContextId,vEntityID,'${KEY}');
  vParentCount := -1;
  keytest:='${KEY}';
  counter :=0;
  keyindexr := -1;
  keylen := 2;
  --Count the number of sections in the full key
  while keylen > 0 loop
       select instr(keytest,'-',-1) into keyindexr from dual;
       select substr(keytest,0,keyindexr-1) into keytest from dual;
       select length(keytest) into keylen from dual;
       IF '${DEBUG}' = 'Y' THEN
			dbms_output.put_line('`date` - [SQL] - Count of - '||counter||'   current index '||keyindexr||'  sub key '||keytest);
	   END IF;
       counter := counter+1;
  end loop;
  -- enabled if the debug flag is enabled
  IF '${DEBUG}' = 'Y' THEN
	dbms_output.put_line('`date` - [SQL] - Key length is '||counter||' for key ${KEY}');
  END IF;
  --check the level that our selected entity is at
  SELECT LEVEL into lev
  FROM wmn_entity_context wec,
       wmn_entity we,
       wmn_context wc
  WHERE wec.context_id = wc.context_id
        AND wec.entity_id = we.entity_id
        AND wc.context_use = 'R'
        AND upper(we.entity_name) = upper('${ENTITY_NAME}')
        START WITH wec.entity_id =
              (SELECT we.entity_id
              FROM wmn_entity we
              WHERE upper(we.entity_name) = 'SYSTEM')
              CONNECT BY PRIOR wec.entity_id = wec.parent_entity_id
              AND wec.context_id = PRIOR wec.context_id;
  --If our keylength is different to the system level of the entity we should exit
IF counter != lev THEN
	dbms_output.put_line('`date` - [SQL] - Key length incorrect for ${ENTITY_NAME}');
	:exitValue := 3;  --key length isnt correct we should exit
	IF '${DEBUG}' = 'Y' THEN
		dbms_output.put_line('`date` - [SQL] - Expected key length for a(n) ${ENTITY_NAME} is '||lev);
	END IF;
ELSE
    dbms_output.put_line('`date` - [SQL] - Key length is correct for entity ${ENTITY_NAME}');
	IF vResult is not null THEN
		dbms_output.put_line('`date` - [SQL] - Full Key already exists');
		:exitValue := 1;  --key already exists we are finished
	ELSE
		select PARENT_ENTITY_ID into vParentEntityID from wmn_entity_context where ENTITY_ID=vEntityID and CONTEXT_ID=vContextID;
    select CASE
           WHEN vEntityID = wm_network.get_entity('SYSTEM') THEN
            -1
           ELSE
            COUNT(instance_id)
         END into vParentCount 
         from wmn_instance_context 
         where entity_id=vParentEntityID 
         and CONTEXT_ID=vContextID and 
		     fullkey=substr('${KEY}',0,instr('${KEY}','-',-1)-1);
		--ensure that the parent exists
    IF vParentCount = 0 THEN
      dbms_output.put_line('`date` - [SQL] - Parent does not exist, please create this first');
      :exitValue := 3; -- no parent  exists  so we should exit 
    ELSE
      IF vParentCount < 0 THEN -- indicates this is at system level
        dbms_output.put_line('`date` - [SQL] - System Level entity no parent required');
      ELSE -- it is greater than 0 and we have a parent
        dbms_output.put_line('`date` - [SQL] - Parent exists');
      END IF;      
			vResult := WM_Network.Find_Instance(vContextID,vEntityID,'${KEY}');
			commit;
			select count(instance_id) into vResult from wmn_instance_context where CONTEXT_ID=vContextID and ENTITY_ID=vEntityID and FULLKEY='${KEY}';
			dbms_output.put_line('`date` - [SQL] - '||vResult || ' instance created');
			:exitValue := 0;  --successful creatuon of instance
		END IF;
	END IF;
END IF;
end;
/
EXIT :exitValue
EOF
exitStatus $?
}

#Make sure we get all  profiles environment variables
userhelp $@

#. $FLEXPM_HOME/.profile

parseArgs $@

validateEntityName

validateAndCreateKey
if [ "$DEBUG" = "Y" ]; then
	echo "$EXIT_MSG"
fi
exit $EXIT_STATE


