#!/usr/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/pm/adm/export_docs.sh BL_core_schema_8.0.7_8.0.7.0.10_20090721  102739
#
SCRIPTNAME=`basename $0`
#Updated 12/02/2009 - valnt00077479
DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`

show_title()
{
   echo Prospect User Document Export Utility.
   echo ----------------------------------------------------------------------
   echo @(#) export_docs.sh 1.11.0 07/08/20 15:53:18
   echo @(#) 
   echo Copyright 2003 IBM Corp.
   echo ----------------------------------------------------------------------
   echo
}

show_usage()
{
   (
   show_title
   echo "USAGE: " 
   echo
   echo "$SCRIPTNAME [-r] -p <prospect path> -f <datafile> -l <logfile>"
   echo "$SCRIPTNAME [-r] -d <report/template> -f <datafile> -l <logfile>"
   echo "$SCRIPTNAME -h "
   echo
   echo "Where:"
   echo "-p <prospect path>  : Export folder <prospect path> and its contents."
   echo "                      The full path must be specified."
   echo "-d <report/template>: Export the report or template specified by "
   echo "                      <report/template>.  The full path must be"
   echo "                      specified"
   echo "-f <datafile>       : Write exported data to file <datafile>"
   echo "-l <logfile>        : Write export logs to <logfile>"
   echo "-r                  : Include scenarios in export."
   echo "-h                  : Show usage"
   echo
   ) >&2
#   echo   verbosity    Volume of logging/debugging information.
#   echo     -a  Actions.
#   echo     -q  Dynamic SQL
#   echo     -c  Procedure and Function calls.
#   echo     -m  Performance Metrics.
#   echo     -n  Actions SQL.
#   echo     -b  Debug
    echo
}

if [ "${DB_CONNECT+SET}" != "SET" ]
then
    echo "Error: DB_CONNECT not set" >&2
    exit 2
fi

if [ "${FLEXPM_HOME+SET}" != "SET" ]
then
    echo "Error: FLEXPM_HOME not set" >&2
    exit 2
fi


# Set default values.
lv_actions=0
lv_dyna_sql=0
lv_procs=0
lv_perfs=0
lv_act_sql=0
lv_debug=0
lv_version=1
lv_mode="NONE"
lv_do_scenarios=0

# Process the arguments.
while getopts d:v:p:f:l:s:o:haqcmnrb arg
do
    case $arg in
    v) lv_version=$OPTARG;;
    r) lv_do_scenarios=1;;
    p) if [ "$lv_mode" != "NONE" ]
       then
          echo "$SCRIPTNAME: Options -p and -d cannot be both specified" >&2
          show_usage
       exit 2;
       fi
       lv_path=$OPTARG
       lv_mode='PATH'
       ;;
    d) if [ "$lv_mode" != "NONE" ]
       then
          echo "$SCRIPTNAME: Options -p and -d cannot be both specified" >&2
          show_usage
          exit 2;
       fi
       lv_path=$OPTARG
       lv_mode='DOCUMENT'
       ;;
    f) lv_datafile=$OPTARG;;
    l) lv_logfile=$OPTARG;;
    s) lv_filesystem=$OPTARG;;
    o) lv_owner=$OPTARG;;
    a) lv_actions=4;;
    q) lv_dyna_sql=8;;
    c) lv_procs=16;;
    m) lv_perfs=32;;
    n) lv_act_sql=64;;
    b) lv_debug=128;; 
    h) show_usage
       exit 0
       ;;
    \? | \:)
       show_usage
       exit 2
       ;;
   esac
done

# Specify required arguments?
if [ ! "$lv_path" ] || [ ! "$lv_datafile" ] || [ ! "$lv_logfile" ]
then
    echo "$SCRIPTNAME: Required arguments not passed" >&2
    show_usage
    exit 2
fi

# Log file writable?
> $lv_logfile
if [ $? -ne 0 ]
then
    echo "$SCRIPTNAME: Cannot write to $lv_logfile" >&2
    exit 2
fi

lv_logspool=${lv_logfile}_$$

# Datafile writable?
lv_datadir=$(dirname $lv_datafile)
> $lv_datafile
if [ $? -ne 0 ]
then
    echo "$SCRIPTNAME: Cannot write to $lv_datafile" >&2
    exit 2
fi
rm $lv_datafile

# Default verbosity (2).
lv_verbosity=2
lv_verbosity=$(( $lv_verbosity + $lv_actions + $lv_dyna_sql + \
                 $lv_procs + $lv_perfs + $lv_act_sql + $lv_debug))

#---------------------------------------------------------------------------
# STARTUP
#---------------------------------------------------------------------------
#
(
show_title
echo "Export Process STARTED: $(date)"
echo "COMMAND OPTIONS: $*"
echo "======================================================" 
) | tee -a $lv_logfile

#---------------------------------------------------------------------------
# EXPORT
#---------------------------------------------------------------------------
#
(
sqlplus -S /NOLOG <<EOF
  WHENEVER SQLERROR EXIT FAILURE ROLLBACK;

  PROMPT Connecting to Oracle...
  CONNECT $DB_CONNECT

  SET PAGESIZE 0
  SET LONG 2000000
  SET LONGCHUNKSIZE 2000000
  SET WRAP OFF
  SET FEEDBACK OFF
  SET COLSEP '	'
  SET TRIMSPOOL ON
  SET TAB OFF
  SET HEADING OFF
  SET SERVEROUTPUT ON SIZE 1000000
  SET LINESIZE 128

  begin
     dbms_output.put_line('***');
     dbms_output.put_line('*** Export Package        : ' || export2.version());
     dbms_output.put_line('*** Custom Export Package : ' || exp_custom.version());
     dbms_output.put_line('***');
  end;
/

  PROMPT Configuring...
  exec export2.gv_version := $lv_version;

  PROMPT Activate/deactivate scenario export...
  exec exp_custom.g_do_scenarios := $lv_do_scenarios

  PROMPT Setting verbosity..
  exec export2.gv_verbosity := $lv_verbosity;

  PROMPT Clearing old exports...
  TRUNCATE TABLE exp_des;
  TRUNCATE TABLE exp_msg;
  TRUNCATE TABLE exp_query_text;

  SET SERVEROUTPUT ON SIZE 1000000
  PROMPT Exporting...
  declare
     table_name varchar2(100);
     criteria   varchar2(512);
     no_recs_found EXCEPTION;
     PRAGMA EXCEPTION_INIT (no_recs_found, -20503);
  begin

     if '$lv_mode' = 'DOCUMENT'
     then
        exp_custom.decode_document('$lv_path', table_name, criteria);
        export2.export(table_name, criteria);
     else
	begin  
           export2.export( 'EXP_WM_LOCALES', '1=1' );
	exception
	   when no_recs_found then
	      DBMS_OUTPUT.PUT_LINE('Warn: '|| substr(SQLERRM,11));
           when others then 
	      raise_application_error(sqlcode,sqlerrm);
	end; 
	begin
           export2.export( 'EXP_WM_DST_RULES', '1=1' );
	exception
	   when no_recs_found then
	      DBMS_OUTPUT.PUT_LINE('Warn: '|| substr(SQLERRM,11));
           when others then 
	      raise_application_error(sqlcode,sqlerrm);
	end;

        export2.export( exp_custom.CONST_FOLDER_TABLE
	              , 'ID=' || TO_CHAR( exp_custom.GET_FOLDER_ID_BY_PATH( '$lv_path' ) ) );
     end if;

  end;
/
  COMMIT;
  EXIT;
EOF
if [ $? -ne 0 ]
then
   echo "FAILURE"
else
   echo "OK"
fi
) | tee -a $lv_logspool
EXPORT_STATUS=$(tail -1 $lv_logspool)
cat $lv_logspool >> $lv_logfile

#---------------------------------------------------------------------------
# WRITE TO EXPORT FILE
#---------------------------------------------------------------------------
#
if [ $EXPORT_STATUS = "OK" ]
then

echo "Writing to $lv_datafile ..." >> $lv_logfile
sqlplus -S /NOLOG <<EOF
  WHENEVER SQLERROR EXIT FAILURE ROLLBACK;

  PROMPT Connecting to Oracle...
  CONNECT $DB_CONNECT

  COL TAB# NEW_VALUE TAB NOPRINT
  SELECT CHR(9) TAB# FROM DUAL;
  SET COLSEP "&TAB"

  SET SERVEROUTPUT OFF
  SET PAGESIZE 0
  SET LINESIZE 32767
  SET LONG 2000000
  SET LONGCHUNKSIZE 2000000
  SET WRAP OFF
  SET FEEDBACK OFF
  
  SET TRIMSPOOL ON
  SET TAB OFF
  SET HEADING OFF

  PROMPT Writing data file...

  COLUMN STUFF FORMAT a40
  SPOOL $lv_datafile
    SELECT 'R' || CHR(9) || TABLE_NAME || CHR(9) || COLUMN_NAME || CHR(9) || TO_CHAR( ORDINAL )
        FROM exp_col
        WHERE VERSION = $lv_version;
      SELECT 'D' || CHR(9) || ID || CHR(9) || P_ID || CHR(9) || RTRIM(TABLE_NAME) , DATA
      FROM exp_des;

  SPOOL OFF
  EXIT;
EOF

fi
echo "======================================================" | tee -a $lv_logfile

  
#---------------------------------------------------------------------------
# MESSAGES
#---------------------------------------------------------------------------
#
sqlplus -S /NOLOG <<EOF
  WHENEVER SQLERROR EXIT FAILURE ROLLBACK;

  PROMPT Connecting to Oracle...
  CONNECT $DB_CONNECT

  COL TAB# NEW_VALUE TAB NOPRINT
  SELECT CHR(9) TAB# FROM DUAL;
  SET COLSEP "&TAB"


  SET SERVEROUTPUT OFF
  SET PAGESIZE 0
  SET LINESIZE 32767
  SET LONG 2000000
  SET LONGCHUNKSIZE 2000000
  SET WRAP OFF
  SET FEEDBACK OFF
  SET TRIMSPOOL ON

  SPOOL $lv_logspool
    PROMPT MESSAGES:
    PROMPT 
    SELECT * FROM exp_msg order by id;
  SPOOL OFF

  ROLLBACK;
  EXIT;

EOF

cat $lv_logspool >> $lv_logfile
rm $lv_logspool 2>/dev/null
  
#---------------------------------------------------------------------------
# END PROCESS
#---------------------------------------------------------------------------
(
echo "======================================================"
echo "Export Process FINISHED: $(date)"
echo "STATUS: $EXPORT_STATUS"
) | tee -a $lv_logfile
echo Done.

if [ "$EXPORT_STATUS" = "FAILURE" ]
then
   exit 9
fi
