#!/usr/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/pm/adm/import_docs.sh BL_core_schema_8.0.7_8.0.7.1.01_20091015  045443
#
SCRIPTNAME=`basename $0`
#Updated 12/02/2009 - valnt00077479
DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`

show_title()
{
   echo Prospect User Document Import Utility.
   echo ----------------------------------------------------------------------
   echo @(#) import_docs.sh 1.9.0 07/06/25 16:12:51
   echo @(#) 
   echo Copyright 2003 IBM Corp.
   echo ----------------------------------------------------------------------
   echo
}

show_usage()
{
   (
   show_title
   echo "USAGE: "
   echo
   echo "$SCRIPTNAME [-r] -p <prospect path> -f <datafile> -l <logfile>"
   echo "$SCRIPTNAME -h "
   echo
   echo "Where:"
   echo "-p <prospect path>  : Import <datafile> into the prospect folder"
   echo "                      specified by <prospect path>. The full path"
   echo "                      must be specified."
   echo "-f <datafile>       : Import exported data file <datafile>"
   echo "-l <logfile>        : Write import logs to <logfile>"
   echo "-r                  : Import scenarios. Valid only for export files"
   echo "                      containing exported scenarios."
   echo "-h                  : Show usage"
   echo
   ) >&2
#   echo   verbosity    Volume of logging/debugging information.
#   echo     -a Actions.
#   echo     -q Dynamic SQL
#   echo     -c Procedure and Function calls.
#   echo     -m Performance Metrics.
#   echo     -n Actions SQL.
    echo     -b Debug
}

if [ "${DB_CONNECT+SET}" != "SET" ]
then
    echo "Error: DB_CONNECT not set" >&2
    exit 2
fi

if [ "${FLEXPM_HOME+SET}" != "SET" ]
then
    echo "Error: FLEXPM_HOME not set" >&2
    exit 2
fi

# Set default values.
lv_actions=0
lv_dyna_sql=0
lv_procs=0
lv_perfs=0
lv_act_sql=0
lv_debug=0
lv_version=1
lv_do_scenarios=0

# Process the arguments.
while getopts v:p:f:l:s:o:haqcmnrb arg
do
    case $arg in
    v) lv_version=$OPTARG;;
    r) lv_do_scenarios=1;;
    p) lv_path=$OPTARG;;
    f) lv_datafile=$OPTARG;;
    l) lv_logfile=$OPTARG;;
    s) lv_filesystem=$OPTARG;;
    o) lv_owner=$OPTARG;;
    a) lv_actions=4;;
    q) lv_dyna_sql=8;;
    c) lv_procs=16;;
    m) lv_perfs=32;;
    n) lv_act_sql=64;;
    b) lv_debug=128;;
    h) show_usage
       exit 0
       ;;
    \? | \:)
       show_usage
       exit 2;;
    esac;
done

# Specify required arguments?
if [ ! "$lv_path" ] || [ ! "$lv_datafile" ] || [ ! "$lv_logfile" ]
then
    echo "$SCRIPTNAME: Required arguments not passed." >&2
    show_usage
    exit 2
fi

# Log file writable?
> $lv_logfile
if [ $? -ne 0 ]
then
    echo "$SCRIPTNAME: Cannot write to $lv_logfile" >&2
    exit 2
fi
lv_logdir=`dirname $lv_logfile`

lv_logspool=${lv_logfile}_$$

# Datafile readable?
if [ ! -r "$lv_datafile" ]
then
    echo "$SCRIPTNAME: Cannot read $lv_datafile" >&2
    exit 2
fi

# Default verbosity (2).
lv_verbosity=2
lv_verbosity=$(( $lv_verbosity + $lv_actions + $lv_dyna_sql + \
                 $lv_procs + $lv_perfs + $lv_act_sql + $lv_debug ))

# clean up the WMD_UDC_DEF.CALC_SOURCE field
lv_datafile_tmp=$lv_datafile.$$.tmp
> $lv_datafile_tmp

if [ $? -ne 0 ]
then
    echo "$SCRIPTNAME: Cannot write to $lv_datafile_tmp" >&2
    exit 2
fi
rm $lv_datafile_tmp

# strip off the CALC_SOURCE field
${FLEXPM_HOME}/pm/adm/exp_udc_convert.pl $lv_datafile > $lv_datafile_tmp 

#---------------------------------------------------------------------------
# STARTUP
#---------------------------------------------------------------------------
#
(
show_title
echo "Import Process STARTED: $(date)"
echo "COMMAND OPTIONS: $*"
echo "======================================================"
) | tee -a $lv_logfile

#---------------------------------------------------------------------------
# SQL LOADER
#---------------------------------------------------------------------------
#
echo "Reading data file..." | tee -a $lv_logfile

sqlldr \
    silent=\(HEADER, FEEDBACK\) \
    userid=$DB_CONNECT \
    data=$lv_datafile_tmp \
    control=$FLEXPM_HOME/db/Oracle8i/export2/export2.ctl \
    log=$lv_logdir/loader.log \
    bad=$lv_logdir/loader.bad \
    discard=$lv_logdir/loader.dsc

LOADER_STATUS=$?

if [ -f $lv_logdir/loader.log ]
then
  echo # Loader log ######### >> $lv_logfile
  cat $lv_logdir/loader.log >> $lv_logfile
  rm $lv_logdir/loader.log 2> /dev/null
fi

if [ -f $lv_logdir/loader.bad ]
then
  echo # Bad records ######### >> $lv_logfile
  cat $lv_logdir/loader.bad >> $lv_logfile
  rm $lv_logdir/loader.bad 2> /dev/null
fi

if [ -f $lv_logdir/loader.dsc ]
then
  echo # Disarded records ######### >> $lv_logfile
  cat $lv_logdir/loader.dsc >> $lv_logfile
  rm $lv_logdir/loader.dsc 2> /dev/null
fi

rm -f $lv_datafile_tmp

case $LOADER_STATUS in
    1 | 3) echo "SQL*Loader: Fatal error reading data file." >&2 | tee -a $lv_logfile
	exit 1;;
    2) echo "SQL*Loader: All/some rows discarded/rejected." >&2 | tee -a $lv_logfile
        exit 1;;
    0) echo "SQL*Loader: All data files rows read." | tee -a $lv_logfile
esac
echo "======================================================" | tee -a $lv_logfile

#---------------------------------------------------------------------------
# IMPORT
#---------------------------------------------------------------------------
#
(
sqlplus -S /NOLOG <<EOF 
  WHENEVER SQLERROR EXIT FAILURE ROLLBACK

  PROMPT Connecting to Oracle...
  CONNECT $DB_CONNECT;

  SET FEEDBACK OFF
  SET SERVEROUTPUT ON SIZE 1000000
  SET LINESIZE 128
  begin
     dbms_output.put_line('***');
     dbms_output.put_line('*** Import Package        : ' || export2.version());
     dbms_output.put_line('*** Custom Import Package : ' || exp_custom.version());
     dbms_output.put_line('***');
  end;
/

  PROMPT Configuring environment...
  exec export2.gv_version := $lv_version;

  PROMPT Checking compatibility...
  exec exp_custom.compatibility_fixes;

  PROMPT Activate/deactivate scenario import...
  exec exp_custom.g_do_scenarios := $lv_do_scenarios; 

  PROMPT Setting verbosity..
  exec export2.gv_verbosity := $lv_verbosity;

  PROMPT Clearing old exports...
  TRUNCATE TABLE exp_query_text;
  truncate table exp_msg;

  PROMPT Importing...
  begin
     if '$lv_path' = '/' or '$lv_path' = 'ROOT'
     then
        export2.import_full();
     else
        export2.gv_folder_id := exp_custom.GET_FOLDER_ID_BY_PATH( '$lv_path' );
        export2.import( EXP_CUSTOM.CONST_FOLDER_TABLE
                      , 'ID=' || TO_CHAR(export2.gv_folder_id),0 );
     end if;
     if ( exp_custom.do_scenarios = 1 ) then
       FOR c1 IN ( SELECT DISTINCT query_id FROM exp_query_text ) LOOP
         exp_scenario.read_exp_query ( c1.query_id );
         exp_scenario.translate( exp_scenario.c_unmangle );
         exp_scenario.write_query( c1.query_id );
       END LOOP; 
    --   exp_custom.fix_imported_system_scenarios;
     end if;
  end;
/

  PROMPT Saving changes...
  COMMIT;
  EXIT;
EOF
if [ $? -ne 0 ]
then
   echo "FAILURE"
else
   echo "OK"
fi
) | tee -a $lv_logspool
IMPORT_STATUS=$(tail -1 $lv_logspool)
cat $lv_logspool >> $lv_logfile
echo "======================================================" | tee -a $lv_logfile

#---------------------------------------------------------------------------
# MESSAGES
#---------------------------------------------------------------------------
#
sqlplus -S /NOLOG <<EOF
  WHENEVER SQLERROR EXIT FAILURE ROLLBACK

  PROMPT Connecting to Oracle...
  CONNECT $DB_CONNECT;

  COL TAB# NEW_VALUE TAB NOPRINT
  SELECT CHR(9) TAB# FROM DUAL;
  SET COLSEP "&TAB"

  SET SERVEROUTPUT OFF
  SET PAGESIZE 0
  SET LINESIZE 32767
  SET LONG 2000000
  SET LONGCHUNKSIZE 2000000
  SET WRAP OFF
  SET FEEDBACK OFF
  SET TRIMSPOOL ON

  SPOOL $lv_logspool
    PROMPT MESSAGES:
    PROMPT 
    SELECT * FROM exp_msg order by id;
  SPOOL OFF

  COMMIT;
  EXIT;
EOF

cat $lv_logspool >> $lv_logfile
rm $lv_logspool 2> /dev/null


#---------------------------------------------------------------------------
# END PROCESS
#---------------------------------------------------------------------------
#
(
echo "======================================================"
echo "Import Process FINISHED: $(date)"
echo "STATUS: $IMPORT_STATUS"
) | tee -a $lv_logfile
echo Done.

if [ "$IMPORT_STATUS" = "FAILURE" ]
then
   exit 9
fi
