#!/usr/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/pm/adm/import_soft_alarms.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
#
SCRIPTNAME=`basename $0`
#Updated 12/02/2009 - valnt00077479
DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`


show_title()
{
   echo Prospect User Document Import Soft Alarms Utility.
   echo ----------------------------------------------------------------------
   echo @(#) import_soft_alarms.sh 1.9.0 07/06/25 16:12:51
   echo @(#) 
   echo Copyright 2003 IBM Corp.
   echo ----------------------------------------------------------------------
   echo
}
usage() {
    show_title
    echo "USAGE: "
    echo
    echo "$SCRIPTNAME -f <data_filename> -a <FAIL | IGNORE | OVERWRITE > -u <username> [-l <log_filename> ] [ -sn <scenario_name>] [-p <path>] "
    echo    
    echo "Where:"
    echo "-f  | -file <data_filename> :  The name of the file to import the data from. Mandatory. "
    echo "-a  | -action <switch>      :  Allows FAIL, IGNORE or OVERWRITE actions to be specified on soft alarm import. Mandatory. "
    echo "-u  | -username <username>  :  The name of an existing user on target system to which Soft Alarm configuration data "
    echo "                               will be associated. Importing the SA to specific user means -p option also required if "
    echo "                               the user as specified in the import data does not exist. Mandatory. "
    echo "-l  | -log <log_filename>   :  The name of a log file to capture the logging information to. Optional. "
    echo "-sn | -scenario_name <name> :  The name of the existing scenario to associate the imported soft alarm data with. Optional. "
    echo "-p  | -path <path>          :  An existing path on the target system to import all Soft Alarm configuration data to. Optional. "
    echo "-h  | -help                 :  Displays a usage message. "
    exit 1
}
# for internal debugging only
#   echo     -da Actions.
#   echo     -dq Dynamic SQL
#   echo     -dc Procedure and Function calls.
#   echo     -dm Performance Metrics.
#   echo     -dn Actions SQL.
#   echo     -dd Debug.



pv_log_file=NULL

# Version is 2 for soft alarms import/export
lv_version=2
# Default verbosity (2 - Warnings).
lv_verbosity=2 

# different levels of verbosity
lv_actions=0
lv_dyna_sql=0
lv_procs=0
lv_perfs=0
lv_act_sql=0
lv_debug=0

pv_scenario_name="NULL"
pv_action="NULL"
pv_path="NULL"

if [ $# -eq 0 ]
then
    usage
fi

while [ $# -gt 0 ]
do

    case $1 in
        -f)
            shift
            pv_data_file=$1
            ;;

        -file)
            shift
            pv_data_file=$1
            ;;
        -l)
            shift
            pv_log_file=$1
            ;;

        -log)
            shift
            pv_log_file=$1
            ;;
        -sn)
            shift
            pv_scenario_name="'"$1"'"
            ;;
        -scenario_name)
            shift
            pv_scenario_name="'"$1"'"
            ;;
        -p)
            shift
            pv_path="'"$1"'"
            ;;
        -path)
            shift
            pv_path="'"$1"'"
            ;;
        -u)
            shift
            pv_username="'"$1"'"
            ;;
        -username)
            shift
            pv_username="'"$1"'"
            ;;
        -a)
            shift
            pv_action="'"$1"'"
            ;;
        -action)
            shift
            pv_action="'"$1"'"
            ;;
        -h)
            usage
            ;;
        -help)
            usage
            ;;
				-da) 
				  	lv_actions=4
						;;
        -dq) 
            lv_dyna_sql=8
            ;;
        -dc) 
            lv_procs=16
            ;;
        -dm) 
            lv_perfs=32
            ;;
        -dn) 
            lv_act_sql=64
            ;;              
        -dd) 
            lv_debug=128
            ;;  
        *)
            echo "$SCRIPTNAME: Invalid parameter $1"
            usage
            ;;
    esac
    shift
done

# Check mandatory arguments: File & Action
if [ ! "$pv_data_file" ] || [ ! "$pv_action" ] || [ ! "$pv_username" ] 
then
    echo "$SCRIPTNAME: Required arguments not passed" >&2
    usage
fi 

# Check Action is a valid
if  [ "$pv_action" != "'FAIL'" ] &&  [ "$pv_action" != "'IGNORE'" ] &&  [ "$pv_action" != "'OVERWRITE'" ] 
then
    echo "$SCRIPTNAME: Required action arguments not passed" >&2
    usage
fi 

# output passed args

echo "$SCRIPTNAME: Arguments passed: FILE = $pv_data_file ; LOG = $pv_log_file ; SCENARIO NAME = $pv_scenario_name ; ACTION = $pv_action"

# Log file writable?
> $pv_log_file
if [ $? -ne 0 ]
then
    echo "$SCRIPTNAME: Cannot write to $pv_log_file" >&2
    exit 2
fi
lv_logdir=`dirname $pv_log_file`

lv_logspool=${pv_log_file}_$$

# Datafile readable?
if [ ! -r "$pv_data_file" ]
then
    echo "$SCRIPTNAME: Cannot read $pv_data_file" >&2
    exit 2
fi

# Default verbosity (2).
lv_verbosity=$(( $lv_verbosity + $lv_actions + $lv_dyna_sql + $lv_procs + $lv_perfs + $lv_act_sql + $lv_debug ))

#---------------------------------------------------------------------------
# STARTUP
#---------------------------------------------------------------------------
#
(
echo "Import Process STARTED: $(date)"
echo "COMMAND OPTIONS: $*"
echo "======================================================"
) | tee -a $pv_log_file

#---------------------------------------------------------------------------
# SQL LOADER
#---------------------------------------------------------------------------
#
echo "Reading data file..." | tee -a $pv_log_file

sqlldr \
    silent=\(HEADER, FEEDBACK\) \
    userid=$DB_CONNECT \
    data=$pv_data_file \
    control=$FLEXPM_HOME/db/Oracle8i/export2/export2.ctl \
    log=$lv_logdir/loader.log \
    bad=$lv_logdir/loader.bad \
    discard=$lv_logdir/loader.dsc

LOADER_STATUS=$?

if [ -f $lv_logdir/loader.log ]
then
  echo # Loader log ######### >> $pv_log_file
  cat $lv_logdir/loader.log >> $pv_log_file
  rm $lv_logdir/loader.log 2> /dev/null
fi

if [ -f $lv_logdir/loader.bad ]
then
  echo # Bad records ######### >> $pv_log_file
  cat $lv_logdir/loader.bad >> $pv_log_file
  rm $lv_logdir/loader.bad 2> /dev/null
fi

if [ -f $lv_logdir/loader.dsc ]
then
  echo # Disarded records ######### >> $pv_log_file
  cat $lv_logdir/loader.dsc >> $pv_log_file
  rm $lv_logdir/loader.dsc 2> /dev/null
fi

case $LOADER_STATUS in
    1 | 3) echo "SQL*Loader: Fatal error reading data file." >&2 | tee -a $pv_log_file
   exit 1;;
    2) echo "SQL*Loader: All/some rows discarded/rejected." >&2 | tee -a $pv_log_file
        exit 1;;
    0) echo "SQL*Loader: All data files rows read." | tee -a $pv_log_file
esac
echo "======================================================" | tee -a $pv_log_file

#---------------------------------------------------------------------------
# IMPORT
#---------------------------------------------------------------------------
#
(
sqlplus -S /NOLOG <<EOF 
  WHENEVER SQLERROR EXIT FAILURE ROLLBACK

  PROMPT Connecting to Oracle...
  CONNECT $DB_CONNECT;

  SET FEEDBACK OFF
  SET SERVEROUTPUT ON SIZE 1000000
  SET LINESIZE 128
  begin
     dbms_output.put_line('***');
     dbms_output.put_line('*** Import Soft Alarms');
     dbms_output.put_line('***');
  end;
/

  PROMPT Configuring...
  exec export2.gv_version := $lv_version;

  PROMPT Setting verbosity..
  exec export2.gv_verbosity := $lv_verbosity;

  PROMPT Clearing old exports...
  TRUNCATE TABLE exp_query_text;
  TRUNCATE TABLE exp_msg;

  PROMPT Setting scenario name...
  exec  exp_custom.gv_scenario_name := $pv_scenario_name

  PROMPT Setting action...
  exec exp_custom.gv_action := $pv_action  

  PROMPT Checking compatibility...
  exec exp_custom.compatibility_fixes;

	PROMPT Setting default user..
  exec exp_custom.gv_default_user_id := exp_custom.get_user_id_from_name( $pv_username );
  
  PROMPT Setting path...
  exec exp_custom.gv_default_path  := $pv_path;

	PROMPT Importing...
  begin
    exp_custom.import_sa;
  end;

/

  PROMPT Saving changes...
  COMMIT;
  EXIT;
EOF
if [ $? -ne 0 ]
then
   echo "FAILURE"
else
   echo "OK"
fi
) | tee -a $lv_logspool
IMPORT_STATUS=$(tail -1 $lv_logspool)
cat $lv_logspool >> $pv_log_file
echo "======================================================" | tee -a $pv_log_file

#---------------------------------------------------------------------------
# MESSAGES
#---------------------------------------------------------------------------
#
sqlplus -S /NOLOG <<EOF
  WHENEVER SQLERROR EXIT FAILURE ROLLBACK

  PROMPT Connecting to Oracle...
  CONNECT $DB_CONNECT;

  COL TAB# NEW_VALUE TAB NOPRINT
  SELECT CHR(9) TAB# FROM DUAL;
  SET COLSEP "&TAB"

  SET SERVEROUTPUT OFF
  SET PAGESIZE 0
  SET LINESIZE 32767
  SET LONG 2000000
  SET LONGCHUNKSIZE 2000000
  SET WRAP OFF
  SET FEEDBACK OFF  
  SET TRIMSPOOL ON

  SPOOL $lv_logspool
    PROMPT MESSAGES:
    PROMPT 
    SELECT * FROM exp_msg ORDER BY id;
  SPOOL OFF

  COMMIT;
  EXIT;
EOF

cat $lv_logspool >> $pv_log_file
rm $lv_logspool 2> /dev/null


#---------------------------------------------------------------------------
# END PROCESS
#---------------------------------------------------------------------------
#
(
echo "======================================================"
echo "Import Process FINISHED: $(date)"
echo "STATUS: $IMPORT_STATUS"
) | tee -a $pv_log_file
echo Done.

if [ "$IMPORT_STATUS" = "FAILURE" ]
then
   exit 9
fi
