#!/bin/ksh
#-- -------------------------------------
#-- (#) ./base/pm/adm/sa_snmp_manager.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
#-- (#) 
#-- -------------------------------------
#
SCRIPTNAME=`basename $0`
#Updated 10/02/2009 - valnt00077479
DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`

display_usage()
{
   (\
   echo "$SCRIPTNAME"
   echo $SCRIPTNAME | sed "s/./-/g"
   echo "This script allows the setup of SNMP manager parameters for"
   echo "the softalarmer (sa) program."
   echo
   echo "Note: Make sure DB_CONNECT is properly set."
   echo
   echo "Usage: $SCRIPTNAME [ -h | -l | -a <host port version> | -r <mgr_id>]"
   echo "where:"
   echo "  -h    : show the usage information"
   echo "  -l    : display the currently registered SA SNMP managers"
   echo "  -a    : add a new SNMP manager with host, port and version"
   echo "  -r    : remove SNMP manager with id <mgr_id>"
   ) >&2
}



list_snmp_managers()
{

    sqlplus -s << !
        $DB_CONNECT
        set feedback off
        set serveroutput on size 1000000
        variable vCur refcursor;
        column host format a20;
        column version format a4;
        column time_format format a3;
        column port format 99999;
        execute :vCur := sa_client.list_snmp_managers;
        print vCur
        exit        
!
}

add_snmp_manager()
{
    echo "add_snmp_manager: $1 $2 $3"
    if [ $# -ne 3 ] 
    then
        display_usage
        echo
        echo "ERROR: not enough arguments for -a option"
        echo
        exit 1
    fi

    nodigits=`echo $2 | sed "s/[0-9]//g"`
    
    if [ "$nodigits" != "" ]
    then
        display_usage
        echo
        echo "ERROR: port must be a number. You entered: $2"
        echo
        exit 1
    fi

    if [ "$3" != "1" ] && [ "$3" != "2" ]
    then
        display_usage
        echo 
        echo "ERROR: SNMP version can be either 1 or 2. You entered: $3"
        echo
        exit 1
    fi        
         
    sqlplus -s << !
        $DB_CONNECT
        set feedback off
        set serveroutput on size 100000
        variable mgrId number;
        execute :mgrId := sa_client.add_snmp_manager('$1', $2, '$3');
        select :mgrId as snmp_manager_id from dual;
        exit
!
}

remove_snmp_manager()
{
    echo "remove_snmp_manager: $1"
    if [ $# -ne 1 ]
    then
        display_usage
        exit 1
    fi

    sqlplus -s << !
        $DB_CONNECT
        set feedback off
        set serveroutput on size 10000
        execute sa_client.remove_snmp_manager($1);
        exit
!
}

if [ -z "$DB_CONNECT" ]
then
   echo "$SCRIPTNAME: The environment variable DB_CONNECT is not set."
   exit 9
fi

while getopts hla:r: opts
    do
        case $opts in
        l ) list_snmp_managers
            exit 0;;
        a ) host=$OPTARG
            port=$(eval "print $"$OPTIND)
            nextindex=`expr $OPTIND + 1`
            version=$(eval "print $"$nextindex)
            add_snmp_manager $host $port $version
            exit 0;;
        r ) remove_snmp_manager $OPTARG
            exit 0;;
        h ) display_usage
            exit 0;;
        \? ) display_usage
             exit 1;;
    esac
done        

display_usage;
exit 2





