#! /bin/ksh
#
# (#) ./base/pm/adm/timezoneregion.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
# COPYRIGHT WatchMark Corp 2004
#
#Updated 10/02/2009 - valnt00077479
DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`

SCRIPTNAME=`basename $0`

usage() {
    echo "${SCRIPTNAME} [ -n name -g gmt_offset [ -r dst_id ] ] | [ -N name -R dst_id ] | [ -t [name] ] | [ -x name ]"
    echo "   -n name : Name of the time zone region (<= 64 characters)"
    echo "   -g gmt_offset : GMT offset in minutes (-720 to 720)"
    echo "   -r dst_id : DST Rule ID"
    echo "OR "
    echo "   -N name : Name of the time zone region (<= 64 characters)"
    echo "   -R dst_id : DST Rule ID"
    echo "OR "
    echo "   -t [name] : Display the TZR and it's DST information"
    echo "OR "
    echo "   -x name : Delete the time zone region"
    
    exit 1
}

# Create options
XNAME=""
XGMTOFFSET=""
XDSTID=""
XFLAG=0

# Link options
XNAME2=""
XDSTID2=""
XLINKFLAG=0

# TZR list options
XTZR=""
XTZRFLAG=0

# Delete options
XDELETE=""
XDELETEFLAG=0

# Check for no options
if [[ $# -eq 0 ]] then
   usage
fi

while [ $# -gt 0 ]
do

    case ${1} in
        -g) 
            shift
            if [[ $# -gt 0 ]] then
               XGMTOFFSET=${1}
               XFLAG=1
               shift
            fi
            ;;

        -n) 
            shift
            if [[ $# -gt 0 ]] then
               XNAME=${1}
               XFLAG=1
               shift
            fi
            ;;

        -N) 
            shift
            if [[ $# -gt 0 ]] then
               XNAME2=${1}
               XLINKFLAG=1
               shift
            fi
            ;;

        -r)
            shift
            if [[ $# -gt 0 ]] then
               XDSTID=${1}
               XFLAG=1
               shift
            fi
            ;;

        -R)
            shift
            if [[ $# -gt 0 ]] then
               XDSTID2=${1}
               XLINKFLAG=1
               shift
            fi
            ;;

        -t)
            shift
            if [[ $# -gt 0 ]] then
               XTZR=${1}
               shift
            fi

            # value is optional
            XTZRFLAG=1
            ;;
        -x)
            shift
            if [[ $# -gt 0 ]] then
               XDELETE=${1}
               XDELETEFLAG=1
               shift
            fi
            ;;

        *)
            echo "$0: Invalid parameter ${1}"
            usage
            ;;
    esac

done

# exclusivity
if [[ $(( $XFLAG + $XLINKFLAG + $XTZRFLAG + $XDELETEFLAG )) -ne 1 ]] then
   usage
fi

#
# Allow the database to handle the parameter verification
#
sqlplus -s /NOLOG << !
set feed off
set lines 200
set serveroutput on size 1000000
connect $DB_CONNECT
VARIABLE return_status NUMBER;
DECLARE
   p_id       INTEGER;
   pErrorCode INTEGER;
   pErrorMessage VARCHAR2(512);
BEGIN

    IF ( $XTZRFLAG = 1 ) THEN

       wm_time.list_tzr( '$XTZR',
                         pErrorCode,
                         pErrorMessage );

    ELSIF ( $XDELETEFLAG = 1 ) THEN

       wm_time.delete_tzr( '$XDELETE',
                           pErrorCode,
                           pErrorMessage );

    ELSIF ( $XLINKFLAG = 1 ) THEN

       wm_time.link_tzr( '$XNAME2',
                         '$XDSTID2',
                         pErrorCode,
                         pErrorMessage );
    ELSE

       wm_time.create_tzr( '$XNAME',
                           '$XGMTOFFSET',
                           '$XDSTID',
                           pErrorCode, 
                           pErrorMessage );
   END IF;

   IF ( pErrorCode != 0 ) THEN
      :return_status := 1;

      wm_print.print(pErrorMessage, LENGTH( pErrorMessage ) );
   ELSE
      :return_status := 0;

   END IF;

   COMMIT;


END;
/
EXIT :return_status
!
XRETURN=$?

exit $XRETURN
