#!/usr/bin/env perl

###################################################################
#  Copyright  2003-2004 WatchMark Corp. All rights reserved      #
###################################################################
# Script   : validate_java_version.pl                             #
# Function : Compare the value associated with the WM_JVM_VERSION #
#            against the system java version and return a         #
#            comparison of 0 if they are different or a 1 if they #
#            are the same.  This script is called at login time   #
#            in order to set the value for WM_PROSPECT_APP_SERVER #
#            for the JDDSQL middleware.                           #
###################################################################
# @(#) ./base/pm/adm/validate_java_version.pl ; BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527

use Env;

###################################################################
# Main Processing :  All logic starts and stops here              #
###################################################################

my ($environ_value) = $ENV{'WM_JVM_VERSION'} || "0.0.0";
my ($java_home)     = $ENV{'JAVA_HOME'}      || "/usr/java";

my @java_version;
my @environ_version;

sub compare_arrays {
########################################################################
# compare_arrays:  string'ize the two arrays and determine if they are
#                  less than, equal to, or greater than each other
########################################################################

    # make both arrays have the same number of elements so that we can compare them

    if ( $#java_version > $#environ_version ) {
        for ( $x = $#environ_version ; $#java_version > $x ; $x++ ) {
            ++$#environ_version;
            $environ_version[$#environ_version] = "0";
        }
    }

    if ( $#java_version < $#environ_version ) {
        for ( $x = $#java_version ; $#environ_version > $x ; $x++ ) {
            ++$#java_version;
            $java_version[$#java_version] = "0";
        }
    }

    # make each element of array 1 equal in length to each element in array 2 (and vice versa)

    for ( $ctr = 0 ; $#java_version >= $ctr ; ++$ctr ) {
        if ( length( $java_version[$ctr] ) != length( $environ_version[$ctr] ) )
        {
            if (
                length( $java_version[$ctr] ) >
                length( $environ_version[$ctr] ) )
            {
                for ( $loop = length( $environ_version[$ctr] ) ;
                    length( $java_version[$ctr] ) > $loop ; $loop++ )
                {
                    $environ_version[$ctr] = "0" . $environ_version[$ctr];
                }
            }
            else {
                for ( $loop = length( $java_version[$ctr] ) ;
                    length( $environ_version[$ctr] ) > $loop ; $loop++ )
                {
                    $java_version[$ctr] = "0" . $java_version[$ctr];
                }
            }
        }
    }

    for ( $x = 0 ; $#java_version >= $x ; $x++ ) {
        $java_string .= $java_version[$x];
        $environ_string .= $environ_version[$x];
    }

    return ( $java_string <=> $environ_string );

}

sub return_compare_string {

    my ($compare_value) = @_;

    if ( $compare_value == 0 ) {
      if ( -f "$java_home/lib/tools.jar" ) {
           print STDOUT "1";
	 }
      else
	{
          print STDERR "JAVA appears to be the JRE and not the SDK ($java_home)\n";
          print STDOUT "0";
	}
    }
    else {
        print STDOUT "0";
    }
}

#################################################################################
# Pseudo Main                                                                   #
#################################################################################

my ($cmd) = join "/", $java_home, "bin/java";

open JAVA_PIPE, "$cmd -version 2>&1 |"
  or die "Unable to open pipe for java -version system call :$!";

while (<JAVA_PIPE>) {
    chomp;
    my ($test_line) = $_;
    next if ( $test_line !~ 'java version \"' );
    my ( $dummy, $version ) = split ( 'java version \"', $test_line );
    chop($version);
    my ( $my_java_version, $dummy1 ) = split ( "\_", $version );
    @java_version    = split ( /\./, $my_java_version );
    @environ_version = split ( /\./, $environ_value );
}

close JAVA_PIPE;

my ($compare_value) = &compare_arrays;

&return_compare_string($compare_value);

