/*
 *=============================================================================
 *                      COPYRIGHT WatchMark Inc 2004
 *=============================================================================
 * 
 * Created on Feb 23, 2004
 * 
 */
package com.watchmark.loader.framework;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import org.w3c.dom.Element;
import org.apache.regexp.*;

public abstract class ArchiveReader implements FileReader
{
  private String _name;
  private List _patterns = new ArrayList();
  private Controller _controller = null;
  
  public ArchiveReader(String name)
  {
    _name = name;
  }

  /**
   * Returns the name of this reader.
   * @return 
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Configure the Processor using data it finds under this element. Usually only
   * things specific to this instance.
   * @param e
   */
  public void configure(Element e)
  {
    
  }
  
  public void setController(Controller c)
  {
    _controller = c;
  }
  
  protected Controller getController()
  {
    return _controller;
  }

  /**
   * Adds a pattern that can be used to match filenames
   * @param p
   */
  public void addPattern(String ptn)
  {
    try
    {
      RE r = new RE(ptn);
      _patterns.add(r);
    }
    catch (RESyntaxException e)
    {
      System.err.println("Invalid pattern for reader " + _name + ": \"" + ptn
        + "\", " + e.getMessage());
    }
  }

  /**
   * 
   * @param filename
   * @param readerName
   * @return true if this FileReader should handle the given file. 
   */
  public boolean accept(String filename)
  {
    boolean result = false;

    Iterator it = _patterns.iterator();
    while (!result && it.hasNext())
    {
      RE r = (RE)it.next();
      result = r.match(filename);
    }
    
    return result;
  }
  
  public void processEntry(InputStream is, String entryName)
    throws IOException
  {
    // Find a processor that matches the name of this entry.
    Processor p = getController().findProcessor(entryName, getName());
    if (p != null)
    {
      long startTime = System.currentTimeMillis();
      p.process(is, entryName);
      long endTime = System.currentTimeMillis();
      double delta = (endTime - startTime)/1000.0;

        System.err.println("  ENTRY_TIME: " + delta + "s, " + entryName);
    }
    else
    {
      System.err.println("No processor found to accept entry: " + entryName);
    }
  }
}