/*
 *=============================================================================
 *                      COPYRIGHT WatchMark Inc 2004
 *=============================================================================
 * 
 * Created on Feb 23, 2004
 * 
 */
package com.watchmark.loader.framework;

import java.io.IOException;
import java.text.ParseException;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

public class XmlUtil
{
    public XmlUtil()
    {
    }

    static public String getText(Node n)
    {
      String result = "";
      
      NodeList children = n.getChildNodes();
      for (int i=0; i<children.getLength(); i++)
      {
        Node c = children.item(i);
        if (c.getNodeType() == Node.TEXT_NODE)
          result = c.getNodeValue();
      }
      
      return result;
    }

    static public String getValueByName(Node n, String name)
    {
      String result = "";

      Node c = getFirstNodeByName(n, name);
      if (c != null)
        result = getText(c);
      
      return result;
    }
    
    static public Node getFirstNodeByName(Node n, String name)
    {
      Node result = null;
      
      if (n.getNodeType() == n.ELEMENT_NODE)
      {
        Element e = (Element)n;
        NodeList nl = e.getElementsByTagName(name);
        if (nl.getLength() > 0)
        {
          result = nl.item(0);
        }
      }
      
      return result;
    }
    
    static public NodeList getNamedChildren(Node n, String name)
    {
      NodeList result = null;
      
      if (n.getNodeType() == n.ELEMENT_NODE)
      {
        Element e = (Element)n;
        result = e.getElementsByTagName(name);
      }

      return result;
    }
    
    static public String getAttributeValue(Node n, String name)
    {
      String result = "";

      NamedNodeMap m = n.getAttributes();
      Node attr = m.getNamedItem(name);
      if (attr != null)
        result = attr.getNodeValue();
        
      return result;
    }
    
  static public Element getDocumentElement(String filename)
  {
    Element result = null;
    try 
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document document = builder.parse(filename);
      result = document.getDocumentElement();
    }
    catch (FactoryConfigurationError e) 
    {
      // unable to get a document builder factory
      System.err.println("Unable to get a document builder factory: " +
        e.getMessage());
    }
    catch (ParserConfigurationException e) 
    {
      // parser was unable to be configured
      System.err.println("Unable to configure parser: " +
        e.getMessage());
    }
    catch (SAXException sx) 
    {
      // parsing error
      System.err.println("XML (SAX) error: " + sx.getMessage());
      sx.printStackTrace();

      Throwable t = sx.getCause();
      if (t != null)
        t.printStackTrace();
    } 
    catch (IOException iox) 
    {
      // i/o error
      System.err.println("IO error: " + iox.getMessage());
    }
    
    return result;
  }
}
