/*
 *=============================================================================
 *                      COPYRIGHT WatchMark Inc 2004
 *=============================================================================
 * 
 * Created on Feb 23, 2004
 * 
 */
package com.watchmark.loader.xml.model;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.Reader;

import java.io.Writer;
import java.text.ParseException;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;
import java.sql.Connection;

import org.apache.log4j.Logger;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

public class XMLController extends DefaultHandler implements Controller
{
    private String                 _elementName;
    private Stack                  _stack;
    private List                   _errors;
    private final Logger _logger = Logger.getLogger(getClass());
    private Connection             _connection;
    private PrintWriter            _out = null;

    /**
     * constructor 
     * initializes creates Stack, List and WmReportDefValueHelper
     * sets state to START and pushes onto stack
     */
     public XMLController(Connection c)
     {
        _stack = new Stack();  
        _errors = new ArrayList();
        _connection = c;
     }

    /**
     * Set to true if any LocalizedExceptions are caught from WmReportDefValueHelper
     * @returns boolean
     */
    public boolean getErrorFound()
    {
        return _errors.size() > 0;
    }

    /**
     * A list of all the errors caught from WmReportDefValueHelper
     * @returns List
     */
    public List getErrors()
    {
        return _errors;
    }

    public void saveError(Exception e)
    {
      _errors.add(e.getLocalizedMessage());
    }
        
    /**
     * begin parsing the xml string. create an XMLReader and call parse 
     * 
     * @param String xml
     * @throws SAXEceptions, IOException
     */
    public void parse(Reader reader, PrintWriter out)
      throws SAXException, IOException
    {
        String proc = "parse: ";

        _out = out;        
            
        // Parse the input. We like xerces, so code it in.
        XMLReader xr = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
        xr.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
        xr.setContentHandler(this);
        xr.parse(new InputSource(reader));
    }

    public void parse(String xmlString)
      throws SAXException, IOException
    {
        StringReader reader = new StringReader(xmlString);
        parse(reader, null);
    }
    
    public Logger getLogger()
    {
      return _logger;
    }

    /**
     * logs the start of the xml document
     * 
     * @throws SAXException
     */
    public void startDocument() throws SAXException
    {
      // who cares?
    }

    /**
     * logs the end of the xml document
     * 
     * @throws SAXException
     */
    public void endDocument() throws SAXException
    {
      // who cares?
    }

    /**
     * changes states and initializes objects for the report definition
     * 
     * @param String uri
     * @param String localName
     * @param String qName - using this parameter for the element name
     * @param String Attributes
     * @throws SAXException
     * 
     */
    public void startElement(String uri, String localName, String qName,
                             Attributes attributes) throws SAXException
    {
//        _logger.info("<" + qName + ">");
        // Some adapters may create a new sub-adapter, some may just return
        // themselves if they're just setting attributes on a bean.
        try
        {
          _elementName = qName; // remember the latest element we hit.
          Adapter a = peek();

          if (a == null)
          {
            // bootstrap an initial Adapter.
            a = create(qName, attributes);
            push(a);
          }

          push(a.create(qName, attributes));
        }
        catch (Exception le)
        {
          saveError(le);
        }
    }

    /**
     * changes states and creates report definition
     * @param String uri
     * @param String localName
     * @param String qName - using this parameter for the element name
     * @param String Attributes
     * @throws SAXException
     * 
     */
    public void endElement(String uri, String localName, String qName) 
      throws SAXException
    {
//        _logger.info("</" + qName + ">");
        _elementName = null;
        Adapter a = pop();

        try
        {
          // Don't close it until we're the last instance of this object on the
          // stack (an Adapter can push itself on the stack many times when
          // processing attributes. An Adapter should only be closed when popping
          // of the very first instance to be pushed on. Note that object equivalence
          // is correct here.
          if (_stack.size() == 0 || a != peek())
            a.close(); // we're done with you.
        }
        catch (Exception e)
        {
          saveError(e);
        }
    }

    /**
     * reads the character data between the start and end element
     * 
     * @param char ch[] - the data
     * @param int start - where to begin reading data
     * @param int length - where to stop reading data
     * @throws SAXException
     */
    public void characters(char ch[], int start, int length) throws SAXException
    {
      String characters;
      if (length > 0)
        characters = new String(ch, start, length);
      else
        characters = "";

      Adapter a = peek();
      if (characters != null && characters.length() > 0 &&
        _elementName != null && _elementName.length() > 0)
      {
        try
        {
          a.text(_elementName, characters);
        }
        catch (Exception le)
        {
          saveError(le);
        }
      }
    }


  /**
   * Manipulate the Adapter stack.
   * @see #aaa
   * @param paramName comments
   * @exception XxxxxxException if ...
   * @return comments
   */
  public void push(Adapter a)
  {
    _stack.push(a);
  }

  /**
   * Manipulate the Adapter stack.
   * @see #aaa
   * @param paramName comments
   * @exception XxxxxxException if ...
   * @return comments
   */
  public Adapter pop()
  {
    Adapter a = (Adapter)_stack.pop();
    return a;
  }

  /**
   * Peek at the top of the Adapter stack. Does not modify the stack.
   * Returns null if the stack is empty.
   * @see #aaa
   * @param paramName comments
   * @exception XxxxxxException if ...
   * @return Adapter on top of stack, or null if the stack is empty.
   */
  public Adapter peek()
  {
    Adapter result = null;

    // need to avoid a possible stack underflow.
    if (_stack.size() > 0)
      result = (Adapter)_stack.peek();

    return result;
  }

  public Adapter create(String elementName, Attributes attributes) 
    throws Exception
  {
    return new XMLBaseAdapter(this, _connection);
  }
  
  public void write(String s) throws IOException
  {
    if (_out != null)
      _out.print(s);
    else
      System.out.print(s);
  }
  
  public void endLine() throws IOException
  {
    if (_out != null)
      _out.println();
    else
      System.out.println();
  }
}