/*
 *=============================================================================
 *                      COPYRIGHT WatchMark Inc 2004
 *=============================================================================
 * 
 * Created on Feb 23, 2004
 * 
 */
package com.watchmark.loader.zipstream;

import com.watchmark.loader.framework.*;

import java.io.IOException;
import java.text.ParseException;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.w3c.dom.*;

public class ConfigHandler
{
    private final Logger _logger = Logger.getLogger(getClass());
    private ZipStream              _zipStream;

     public ConfigHandler(ZipStream zs)
     {
        _zipStream = zs;
     }
        
    public void process(Document d)
    {
      // Dig into the DOM and find add entries for readers and processors.
      Element top = d.getDocumentElement();
      NodeList readers = top.getElementsByTagName("reader");
      NodeList processors = top.getElementsByTagName("processor");
      
      for (int i=0; i<readers.getLength(); i++)
      {
        Node c = readers.item(i);
        processReader(c);
      }

      for (int i=0; i<processors.getLength(); i++)
      {
        Node c = processors.item(i);
        processProcessor(c);
      }
    }
    
    public void processReader(Node n)
    {
      // Need to find the className for this reader's factory.
      String readerName = getValueByName(n, "name");
      Node factory = getFirstNodeByName(n, "factory");
      if (factory != null)
      {
        String className = getValueByName(factory, "class");
        FileReader reader = getFileReader(className, readerName);
        if (reader != null)
        {
          reader.setController(_zipStream);
          NodeList patterns = getNamedChildren(n, "pattern");
          for (int i=0; i<patterns.getLength(); i++)
          {
            String pattern = getText(patterns.item(i));
            if (pattern != null && pattern.length() > 0)
              reader.addPattern(pattern);
          }

          // Hand in the factory element, so that the reader can load any
          // specific configuration data it might want or need.
          Node properties = getFirstNodeByName(factory, "properties");
          if (properties != null && properties.getNodeType() == n.ELEMENT_NODE)
            reader.configure((Element)properties);
          _zipStream.addReader(reader);
        }
      }
      
    }
    
    public void processProcessor(Node n)
    {
      // Need to find the className for this processor's factory.
      String processorName = getValueByName(n, "name");
      Node factory = getFirstNodeByName(n, "factory");
      if (factory != null)
      {
        String className = getValueByName(factory, "class");
        Processor processor = getProcessor(className, processorName);
        if (processor != null)
        {
          NodeList patterns = getNamedChildren(n, "pattern");
          for (int i=0; i<patterns.getLength(); i++)
          {
            String pattern = getText(patterns.item(i));
            if (pattern != null && pattern.length() > 0)
              processor.addPattern(pattern);
          }

          // Hand in the factory element, so that the reader can load any
          // specific configuration data it might want or need.
          Node properties = getFirstNodeByName(factory, "properties");
          if (properties != null && properties.getNodeType() == n.ELEMENT_NODE)
            processor.configure((Element)properties);
          _zipStream.addProcessor(processor);
        }
      }
    }

    
    public Logger getLogger()
    {
      return _logger;
    }
    
    public FileReader getFileReader(String className, String readerName)
    {
      FileReader result = null;
     
      try
      {
        Class c = Class.forName(className);
        FileReaderFactory f = (FileReaderFactory)c.newInstance();
      
        result = f.create(readerName);
      }
      catch (ClassNotFoundException ex )
      {
        System.err.println( ex + " " + className + " class must be in class path.");
      }
      catch(InstantiationException ex )
      {
        System.err.println( ex + " " + className + " class must be concrete.");
      }
      catch(IllegalAccessException ex )
      {
        System.err.println( ex + " " + className + " class must have a no-arg constructor.");
      }
      
      if (result == null)
        throw new IllegalArgumentException("Configuration is invalid.");

      return result;
    }
    
    public Processor getProcessor(String className, String processorName)
    {
      Processor result = null;
      
      try
      {
        Class c = Class.forName(className);
        ProcessorFactory f = (ProcessorFactory)c.newInstance();
      
        result = f.create(processorName);
      }
      catch (ClassNotFoundException ex )
      {
        System.err.println( ex + " " + className + " class must be in class path.");
      }
      catch(InstantiationException ex )
      {
        System.err.println( ex + " " + className + " class must be concrete.");
      }
      catch(IllegalAccessException ex )
      {
        System.err.println( ex + " " + className + " class must have a no-arg constructor.");
      }

      if (result == null)
        throw new IllegalArgumentException("Configuration is invalid.");
      
      return result;
    }
    
    static public String getText(Node n)
    {
      String result = "";
      
      NodeList children = n.getChildNodes();
      for (int i=0; i<children.getLength(); i++)
      {
        Node c = children.item(i);
        if (c.getNodeType() == Node.TEXT_NODE)
          result = c.getNodeValue();
      }
      
      return result;
    }

    static public String getValueByName(Node n, String name)
    {
      String result = "";

      Node c = getFirstNodeByName(n, name);
      if (c != null)
        result = getText(c);
      
      return result;
    }
    
    static public Node getFirstNodeByName(Node n, String name)
    {
      Node result = null;
      
      if (n.getNodeType() == n.ELEMENT_NODE)
      {
        Element e = (Element)n;
        NodeList nl = e.getElementsByTagName(name);
        if (nl.getLength() > 0)
        {
          result = nl.item(0);
        }
      }
      
      return result;
    }
    
    static public NodeList getNamedChildren(Node n, String name)
    {
      NodeList result = null;
      
      if (n.getNodeType() == n.ELEMENT_NODE)
      {
        Element e = (Element)n;
        result = e.getElementsByTagName(name);
      }

      return result;
    }
}
