#!/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2006,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./jloader/bin/jloader.ksh BL_core_loader_8.0.7_8.0.7.5.02_20100605  060808
# (#) 
#
#-- ----------------------------------------------------------------------
#-- jloader.ksh
#--
#-- This script acts as an entry point to the JLoader. This is a standalone
#-- loader which can be configured to process archive files (tar/gzip/zip)
#-- at multiple levels.
#-- These are termed the Readers
#-- It also supports the use of per file type processors which process
#-- the file input
#--
#-- Author: Ollie Fagan
#-- Date: 25Sep06
#-- ----------------------------------------------------------------------



if [ -z	"$JAVA_HOME" ];	then
  echo "ERROR: JAVA_HOME not found in your environment."
  echo ""
  echo "Please,	set the	JAVA_HOME environment variable to match	the"
  echo "root directory of the Java Virtual Machine you want to use."
  echo ""
  exit 1;
else
  echo PATH | grep $JAVA_HOME || PATH=$JAVA_HOME/bin:$PATH
fi

if [ -z "$JAVA_HOME/lib/tools.jar" ]; then
  echo "ERROR: Either your JDK is too old, or I was unable to find"
  echo "lib/tools.jar in your JAVA_HOME directory.  Please make"
  echo "sure you have a Java2-compliant compiler and JVM."
  exit 2;
else
  CLASSPATH="$JAVA_HOME/lib/tools.jar"
fi

JAVA_SHARED="/usr/share/java"

JLOADER_HOME=$FLEXPM_HOME/pm/jloader
JLOADER_LIB=$JLOADER_HOME/lib

# set the class path
UNIXCP=""

for jarfile in `find $JLOADER_LIB -name '*.jar' -print 2>/dev/null`
do
  UNIXCP="$UNIXCP:$jarfile"
done

# Now tack on the plugins to the classpath. All the Application Engineers have to do is supply
# the appropriate jar file and drop it into the plugins directory.
for jarfile in `find $JLOADER_LIB/plugins -name '*.jar' -print 2>/dev/null`
do
  UNIXCP="$UNIXCP:$jarfile"
done

if [ "x$LOG_DIR" = "x" ]; then
   LOG_DIR="$JLOADER_HOME/log"
fi

# set the arguments

CLI_ARGS="$@"

# Set the default values if the variable are not set 
if [ "x$DBHOST" = "x" ]; then
   DBHOST=localhost
fi
if [ "x$DBPORT" = "x" ]; then
   DBPORT=1521
fi

# database properties
DB_PROPERTIES="-Ddb.name=$DBSID -Ddb.user=$DBUSER -Ddb.pass=$DBPASSWD -Ddb.host=$DBHOST -Ddb.listener.port=$DBPORT"

#get current date and time for main log file
TIME_STAMP=`date '+20%y%m%d_%H./jloader/bin/jloader.kshS'`

# other properties required by jloader
NONDB_PROPERTIES="-Dlog.dir=$LOG_DIR -Dtime.stamp=$TIME_STAMP -DFLEXPM_HOME=$FLEXPM_HOME -Djloader.home=$JLOADER_HOME $VENDOR_PROPERTIES"

echo ""
echo "JLoader> classpath: $UNIXCP"
echo ""
echo "JLoader> passing arguments: $CLI_ARGS"
echo ""
ARGS="$CLI_ARGS"

# the main class
MAINCLASS="com.watchmark.jloader.main.JLoader"

# Performance tuning attributes. The "client" JVM is used because
# we're running single-threaded (there being many instances of
# JLoader, and each instance is relatively short-lived.
# Java 1.5.0_12+ recommended
heap="-Xmx256M"
#GC_PROPS="-client -XX:+PrintGCDetails -XX:+UseSerialGC"
GC_PROPS="-client -XX:+UseSerialGC"

PATH="$JAVA_HOME/bin:$PATH"

exec $JAVA_HOME/bin/java $GC_PROPS $heap -classpath $UNIXCP $DB_PROPERTIES $NONDB_PROPERTIES $MAINCLASS $CLI_ARGS

