#! /bin/ksh
#
# @(#) ./base/pm/run/bhupdate.sh ; BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
#
#Updated 10/02/2009 - valnt00077479
DBPASSWD=`crypt_util.ksh -g ${DBUSER}`

SCRIPTNAME=`basename $0`

usage() {
    echo "$SCRIPTNAME [ -f | -b | -d firstdate -D lastdate [ -o ] ]  [ [-e entity ] [ -n name ] | [-i BH_ID ] ] [ -a ] ]"
    echo " -e : Run only those busy hours at the specified entity level"
    echo " -f : Only run on the forward gap"
    echo " -b : Only run on the backward gap"
    echo " -d : Start date, must also specify end date in the format YYYYMMDD"
    echo " -D : End date, must also specify start date in the format YYYYMMDD"
    echo " -a : Process only those dates for which all 24 hours are present"
    echo " -o : Override the default behaviour of calculating all SBH for a new BHD"
    echo " -n : busyhour name"
    echo " -i : busyhour ID"
    exit 1
}

if test "${VENDOR:=not_defined}" = "not_defined"
 then
  echo '"The bhupdate.sh operation is not supported on an Enterprise system. "'
  echo '" Please execute the command on a Remote Server."'
  exit 0
fi

ENTITY_NAME=N
FORWARD_FLAG=N
START_DATE=N
END_DATE=N
BACKWARD_FLAG=N
SKIP_FLAG=N
OVERRIDE=N
BH_NAME=NULL
BH_ID=NULL
SPAN=NULL
TAB_TYP=NULL

while [ $# -gt 0 ]
do

    case $1 in
	-e) 
	    shift
	    ENTITY_NAME=$1
	    ;;

	-f)
	    FORWARD_FLAG=Y
	    ;;

	-b)
	    BACKWARD_FLAG=Y
	    ;;

	-d)
	    shift
	    START_DATE=$1
	    ;;

	-D)
	    shift
	    END_DATE=$1
	    ;;

	-a)
	    SKIP_FLAG=Y
	    ;;
        -o) 
            OVERRIDE=Y
            ;;
        -n) 
	    shift
            BH_NAME="'$1'"
            ;;
        -i) 
	    shift
            BH_ID=$1
            ;;
        -v)
           usage
           ;;

	*)
	    echo "$SCRIPTNAME: Invalid parameter $1"
	    usage
	    ;;
    esac

    shift

done

## Following block calls stored proc. to validate the input
## args. passed and schedules the job.

sqlplus -s /NOLOG << !
set feed off
set serveroutput on
set lin 120
connect $DBUSER/$DBPASSWD@$DBSID
variable feedback_id_sh NUMBER
DECLARE
    feedback_id_sql NUMBER;
BEGIN
    bh_update.validate_and_insert_bh_params(
	'$FORWARD_FLAG','$BACKWARD_FLAG','$SKIP_FLAG',
	'$START_DATE','$END_DATE','$ENTITY_NAME','$OVERRIDE',feedback_id_sql,
        'Y', ${BH_NAME}, ${BH_ID});
:feedback_id_sh := feedback_id_sql;
END;
/
EXIT :feedback_id_sh
!
feedback_id=$?

## Following block traps any kill command schedules a cleanup job
trap '
sqlplus -s /NOLOG <<EOF
set feed off
connect $DBUSER/$DBPASSWD@$DBSID
exec bh_update.ins_and_del($feedback_id)
EOF
echo "******** bhupdate killed ********** "
exit 1' 1 2 3 9 15

## The following block calls stored proc. which polls
## task_status table to display the status message of
## bhupdate run as it progresses 
STATUS_CODE=0
if [ $feedback_id -ne 0 ]
then
   while [ "$STATUS_CODE" -ne 5 -a "$STATUS_CODE" -ne 4 \
           -a "$STATUS_CODE" -ne 6 -a "$STATUS_CODE" -ne 255 ]
   do
      sqlplus -s /NOLOG <<!
      set serveroutput on
      set feed off
      connect $DBUSER/$DBPASSWD@$DBSID
      variable status_code_sh NUMBER
      DECLARE
         status_code_sql NUMBER;
      BEGIN
         bh_update.poll($feedback_id,status_code_sql);
         :status_code_sh := status_code_sql;
      END;
/
EXIT :status_code_sh
!
     STATUS_CODE=$?
     sleep 5
  done
fi
