#! /bin/ksh
#
# @(#) ./base/pm/run/smupdate.sh ; BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
#
#Updated 10/02/2009 - valnt00077479
DBPASSWD=`crypt_util.ksh -g ${DBUSER}`

SCRIPTNAME=`basename $0`
usage() {
    echo "$SCRIPTNAME [[-e Entity] [-s span] [-n SM_name] | [-i SM_ID]] [ [-f] | [-b] | [-d FirstDate -D LastDate [-o] ]"
    echo " -e : Run only those summaries at the specified entity level"
    echo " -f : Only run on the forward gap"
    echo " -b : Only run on the backward gap"
    echo " -d : Start date, must also specify end date in the format YYYYMMDD"
    echo " -D : End date, must also specify start date in the format YYYYMMDD"
    echo " -o : Override the default behaviour of calculating all dates for a new summary"
    echo " -n : Summary name"
    echo " -i : Summary ID"
    echo " -s : span can be one of D(aily), W(eekly) and M(onthly)"
    exit 1
}

if test "${VENDOR:=not_defined}" = "not_defined"
 then
  echo '"The smupdate.sh operation is not supported on an Enterprise system. "'
  echo '" Please execute the command on a Remote Server."'
  exit 0
fi

ENTITY_NAME=N
FORWARD_FLAG=N
START_DATE=N
END_DATE=N
BACKWARD_FLAG=N
SKIP_FLAG=N
OVERRIDE=N
SM_NAME=NULL
SM_ID=NULL
SPAN=NULL
TAB_TYP=NULL

while [ $# -gt 0 ]
do

    case $1 in
	-e) 
	    shift
	    ENTITY_NAME=$1
	    ;;

	-f)
	    FORWARD_FLAG=Y
	    ;;

	-b)
	    BACKWARD_FLAG=Y
	    ;;

	-d)
	    shift
	    START_DATE=$1
	    ;;

	-D)
	    shift
	    END_DATE=$1
	    ;;

        -o) 
            OVERRIDE=Y
            ;;
        -n)
            shift
            SM_NAME="'$1'"
            ;;
        -i)
            shift
            SM_ID=$1
            ;;
        -s)
            shift
            SPAN="'$1'"
            ;;
        -v)
           usage
           ;;

	*)
	    echo "$SCRIPTNAME: Invalid parameter $1"
	    usage
	    ;;
    esac

    shift

done


sqlplus -s /NOLOG << !
set feed off
set serveroutput on
connect $DBUSER/$DBPASSWD@$DBSID
variable feedback_id_sh NUMBER
DECLARE
    feedback_id_sql NUMBER;
BEGIN
    bh_update.validate_and_insert_bh_params(
        '$FORWARD_FLAG','$BACKWARD_FLAG','$SKIP_FLAG',
        '$START_DATE','$END_DATE','$ENTITY_NAME','$OVERRIDE',feedback_id_sql,
        'N',${SM_NAME}, ${SM_ID}, ${SPAN});
:feedback_id_sh := feedback_id_sql;
END;
/
EXIT :feedback_id_sh
!
feedback_id=$?

export N=N

trap '
sqlplus -s /NOLOG <<EOF
set feed off
connect $DBUSER/$DBPASSWD@$DBSID
exec bh_update.ins_and_del_sm($feedback_id)
EOF
echo "******** smupdate killed ********** "
exit 1' 1 2 3 9 15

STATUS_CODE=0
## The following block calls stored proc. which polls
## task_status table to display the status message of
## bhupdate run as it progresses 
if [ $feedback_id -ne 0 ]
then
   while [ "$STATUS_CODE" -ne 5 -a "$STATUS_CODE" -ne 4 \
          -a "$STATUS_CODE" -ne 6 -a "$STATUS_CODE" -ne 255 ]
   do
      sqlplus -s /NOLOG <<!
      set serveroutput on
      set feed off
      connect $DBUSER/$DBPASSWD@$DBSID
      variable status_code_sh NUMBER
      DECLARE
        status_code_sql NUMBER;
      BEGIN
         bh_update.poll($feedback_id,status_code_sql);
         :status_code_sh := status_code_sql;
      END;
/
EXIT :status_code_sh
!
STATUS_CODE=$?
sleep 1
done
fi
