#! /bin/ksh

flag=$1
function get_count_of_lock_rows {

log_cnt=`sqlplus -s $DB_CONNECT <<EOF
set head off
set echo off
set feed off
select count(*) from 
(
select /*+ ordered */ w1.sid  waiting_session,
	h1.sid  holding_session,
	w.kgllktype lock_or_pin,
        w.kgllkhdl address,
	decode(h.kgllkmod,  0, 'None', 1, 'Null', 2, 'Share', 3, 'Exclusive',
	   'Unknown') mode_held, 
	decode(w.kgllkreq,  0, 'None', 1, 'Null', 2, 'Share', 3, 'Exclusive',
	   'Unknown') mode_requested
  from dba_kgllock w, dba_kgllock h, v\\$session w1, v\\$session h1
 where
  (((h.kgllkmod != 0) and (h.kgllkmod != 1)
     and ((h.kgllkreq = 0) or (h.kgllkreq = 1)))
   and
     (((w.kgllkmod = 0) or (w.kgllkmod= 1))
     and ((w.kgllkreq != 0) and (w.kgllkreq != 1))))
  and  w.kgllktype	 =  h.kgllktype
  and  w.kgllkhdl	 =  h.kgllkhdl
  and  w.kgllkuse     =   w1.saddr
  and  h.kgllkuse     =   h1.saddr
)
/
EOF
`
return $log_cnt
}

function get_count_of_lock_rows1 {

log_cnt1=`sqlplus -s $DB_CONNECT <<EOF
set head off
set echo off
set feed off
select count(*) from (
SELECT o.object_name,
       v.session_id||','||s.serial# sid,
       v.oracle_username,
       decode(l.type,'TM', 'DML enqueue (TM)', 'TX', 'Transaction enqueue (TX)', 'UL', 'User supplied (UL)', l.type) lock_type,
       decode(l.lmode,0,'None',1,'Null',2,'Row Share',3,'Row Excl.',4,'Share',5,'S_Row Excl.',6,'Exclusive',to_char(l.lmode,'990')) holding_session,
decode(l.request,0,'None',1,'Null',2,'Row Share',3,'Row Excl.',4,'Share',5,'S_Row Excl.',6,'Exclusive',to_char(l.request, '990')) wanting_session,
       s.terminal,
       s.machine, v.process unix_pid
FROM sys.v_\\$locked_object v,
     sys.v_\\$lock l,
     sys.dba_objects o,
     sys.v_\\$session s
WHERE l.sid = v.session_id
AND   s.sid = v.session_id
AND   v.object_id = o.object_id
AND  (v.oracle_username = user OR l.ctime > 30)
)
/
EOF
`
return $log_cnt1
}

function get_count_of_sessions {

session_cnt=`sqlplus -s $DB_CONNECT <<EOF
set head off
set echo off
set feed off
select count(*) from (
SELECT sid, serial#, program, osuser, machine, process
  FROM v\\$session s
 WHERE s.USERNAME = USER
   AND s.sid != (SELECT sid FROM v\\$mystat WHERE rownum = 1)
)
/
EOF
`
return $session_cnt
}


function get_actual_lock_rows
{

sqlplus -s $DB_CONNECT<<EOF
set linesize 100
set lines 130 pages 999
col object_name form a40
col sid         form a15
col oracle_user  form a15
col lock_type    form a15
col holding_session form a15
col wanting_session form a15
col sid_and_Serial form a15
col program form a20
col osuser form a15
col machine form a15
select /*+ ordered */ w1.sid  waiting_session,
	h1.sid  holding_session,
	w.kgllktype lock_or_pin,
        w.kgllkhdl address,
	decode(h.kgllkmod,  0, 'None', 1, 'Null', 2, 'Share', 3, 'Exclusive',
	   'Unknown') mode_held, 
	decode(w.kgllkreq,  0, 'None', 1, 'Null', 2, 'Share', 3, 'Exclusive',
	   'Unknown') mode_requested
  from dba_kgllock w, dba_kgllock h, v\$session w1, v\$session h1
 where
  (((h.kgllkmod != 0) and (h.kgllkmod != 1)
     and ((h.kgllkreq = 0) or (h.kgllkreq = 1)))
   and
     (((w.kgllkmod = 0) or (w.kgllkmod= 1))
     and ((w.kgllkreq != 0) and (w.kgllkreq != 1))))
  and  w.kgllktype	 =  h.kgllktype
  and  w.kgllkhdl	 =  h.kgllkhdl
  and  w.kgllkuse     =   w1.saddr
  and  h.kgllkuse     =   h1.saddr
/
SELECT o.object_name,
       v.session_id||','||s.serial# sid_and_Serial,
       v.oracle_username,
       decode(l.type,'TM', 'DML enqueue (TM)', 'TX', 'Transaction enqueue (TX)', 'UL', 'User supplied (UL)', l.type) lock_type,
       decode(l.lmode,0,'None',1,'Null',2,'Row Share',3,'Row Excl.',4,'Share',5,'S/Row Excl.',6,'Exclusive',to_char(l.lmode,'990')) holding_session,
decode(l.request,0,'None',1,'Null',2,'Row Share',3,'Row Excl.',4,'Share',5,'S/Row Excl.',6,'Exclusive',to_char(l.request, '990')) wanting_session,
       s.terminal,
       s.machine, v.process unix_pid
FROM sys.v_\$locked_object v,
     sys.v_\$lock l,
     sys.dba_objects o,
     sys.v_\$session s
WHERE l.sid = v.session_id
AND   s.sid = v.session_id
AND   v.object_id = o.object_id
AND  (v.oracle_username = user OR l.ctime > 30)
/
SELECT s.sid||','||s.serial# sid_and_Serial, program, osuser, machine, process
  FROM v\$session s
 WHERE s.USERNAME = USER
   AND s.sid != (SELECT sid FROM v\$mystat WHERE rownum = 1)
/
EOF
}

get_count_of_lock_rows
get_count_of_lock_rows1
get_count_of_sessions
if [ $log_cnt -gt 0 ] || [ $log_cnt1 -gt 0 ] || [ $session_cnt -gt 0 ]
then
	echo "Found locks"
	get_actual_lock_rows
	return 2;
else
	return 0;
fi
