#!/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2009,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./scripts/gatherHelpFiles.sh BL_wminstall_8.0.7_8.0.7.5.02_20100605  065855
# (#) 
#
# Eoghan Shields  18/08/09
#

EXIT_OKAY=0
EXIT_FAIL=2
# Initialise HELPFILES_DIR to default that wminstall has added to the environment
[ -n "${WMINSTALL_DIR}" -a -d ${WMINSTALL_DIR}/HelpFiles ] && HELPFILES_DIR=${WMINSTALL_DIR}/HelpFiles

# valnt00093821 -- resource environment before running crypt_util -g
cd $FLEXPM_HOME
. ./.profile
cd -

DB_CONNECT=`$FLEXPM_HOME/pm/adm/crypt_util.ksh -g $DBUSER -c`
APPLICATION=`basename $0`
ZER0=$0;
RESULT_COUNT=0;
SQLPLUS_RESULTS="";
DEBUG='N';
export EXIT_STATE=1;
# sql options
SQLPLUS_OPTS="
		SET ECHO OFF 
		SET NEWPAGE 0 
		SET SPACE 0 
		SET PAGESIZE 0 
		SET FEEDBACK OFF 
		SET HEADING OFF 
		SET TRIMSPOOL ON 
		SET TAB OFF 
		SET SERVEROUTPUT ON 
		-- default set of options to be applied to all sqlplus activities 
		";
# execute sql
function executeSQL
{
	logDebug "$SQLPLUS_OPTS $1"
	SQLPLUS_RESULTS=`echo "$SQLPLUS_OPTS \n $1"|sqlplus -s $DB_CONNECT`
	RESULT_COUNT=`echo "$SQLPLUS_OPTS \n $1"|sqlplus -s $DB_CONNECT|wc -l`
	exitStatus $?
}
#Create a list of results from say a select for a particular colum on a table
function formatSQLResults
{
	if [ -z $1 ]; then
		print ${SQLPLUS_RESULTS}|tr -s " " "\n";
	else
		print $1|tr -s " " "\n";
	fi
}
#Handy function to log debug messages
function logDebug
{
	if [ $DEBUG = "Y" ]; then
		echo "`date` [DEBUG] - $1"
	fi
}
#Handy function to log information messages
function logInfo
{
	echo "`date` [INFO] - $1"
}
function validateInputDir
{
	if [[ ${HELPFILES_DIR} = /* ]]; then
		logDebug "$HELPFILES_DIR is an absolute path"
	else
		logDebug "$HELPFILES_DIR is a relative path"
		CWD=`pwd`;
		HELPFILES_DIR=`echo "${CWD}/$HELPFILES_DIR"`;
	fi
}
# print help 
function userhelp
{
	echo "`date` [HELP] - A single argument, containing the directory location of the help files,\nshould be passed into ${APPLICATION}";
}
#log exit status
function exitStatus
{
	EXIT_STATE=$1;
	if [ $DEBUG = "Y" ]; then
		logDebug "Exit status was [${EXIT_STATE}]";
	fi
}

#parse all our input arguments
while [ $# -gt 0 ]
do
	case $1 in
		-location)
      shift;
			HELPFILES_DIR=$1;;

			-debug)
			DEBUG="Y";;
					
		*)	logInfo "$1 is not a valid argument in $APPLICATION"
			userhelp;
			exit $EXIT_FAIL ;;
	esac
	shift
done

validateInputDir
# only going to update the core woend files (no PDR here)
executeSQL "
SELECT server_path
  FROM pm_help_file
 WHERE help_file_type IN
       (SELECT id
          FROM pm_help_file_type t
         WHERE t.NAME IN
               ('Administration Guide', 'User Guide', 'Expressions Guide'));"


logDebug "Help Files location is $HELPFILES_DIR"

for  i in `formatSQLResults "${SQLPLUS_RESULTS}"`; do
	DIR=`dirname $i`
	FILE=`basename $i`
	FILE_EXISTS=0;
	DIR=`echo ${DIR}|sed "s!FLEXPM_HOME!${FLEXPM_HOME}!"|tr -d '$'`
	logDebug "Checking directory $DIR"
	if [ -d "${DIR}" ]; then
		cd ${DIR}
		if [ -f ${FILE} ]; then
			logDebug "[${FILE}] Already exists"
		else
			logDebug "[${FILE}] Does not exist"
		fi
		cd ${HELPFILES_DIR}
		if [ -f ${FILE} ]; then
			logInfo "Found ${FILE} in ${HELPFILES_DIR}, copying..."
			cp -f ${FILE} ${DIR}/${FILE}
		else
			logInfo "${HELPFILES_DIR}/${FILE} was not found , skipping file..."
		fi
		
	else
		echo "[${DIR}] Does not exist";
		exit $EXIT_FAIL ;
	fi
  
done
# finally update the version to last installed core version to ensure the client picks up the new files
executeSQL "
UPDATE pm_help_file hf
   SET hf.version = (SELECT substr(rq.rev, 0, 10)
                       FROM (SELECT cv.revision rev,
                                    MAX(ch.action_date) dat
                               FROM component         c,
                                    component_history ch,
                                    component_version cv
                              WHERE cv.component_id = c.component_id
                                AND ch.component_version_id =
                                    cv.component_version_id
                                AND c.component_type = 'CORE'
                              GROUP BY cv.revision) rq
                      WHERE rq.dat =
                            (SELECT MAX(ch.action_date) dat
                               FROM component         c,
                                    component_history ch,
                                    component_version cv
                              WHERE cv.component_id = c.component_id
                                AND ch.component_version_id =
                                    cv.component_version_id
                                AND c.component_type = 'CORE'))
 WHERE hf.help_file_type IN
       (SELECT id
          FROM pm_help_file_type t
         WHERE t.NAME IN
               ('Administration Guide', 'User Guide', 'Expressions Guide'));
COMMIT;"

