---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2008. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/sw/oracle/monitor-sizes.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
--

    set pagesize 200
    set linesize 200
    set feedback off

    column "Tablespace Name" format a20 trunc;
    column "# Bytes"         format 99,999,999,999;
    column "File Name"       format a50;

    prompt
    prompt List of Tablespace Datafiles

    select 
       (b.tablespace_name)     "Tablespace Name",
       (b.bytes)               "# Bytes"   ,
       (b.file_name)           "File Name"  
    from sys.dba_data_files b
    order by b.tablespace_name;

    prompt
    prompt Summary of Tablespace Utilization

    column tablespace_name format a20 trunc           heading "Tablespace Name";
    column total_mb        format 9,999,999.999 trunc heading "Tablespace Size"
    column total_avail_mb  format 9,999,999.999 trunc heading "Available Size"
    column pct_free        format 999.999             heading "% Free"
    column over_flag       format a6                  heading "< 10%"
    column free_mb         format 9,999,999.999 trunc heading "MB Free"
    column used_mb         format 9,999,999.999 trunc heading "MB Used"

    break on report;

    compute sum label "TOTAL" of total_mb  on report;
    compute sum label "TOTAL" of total_avail_mb   on report;
    compute sum label "TOTAL" of used_mb   on report;
    compute sum label "TOTAL" of free_mb   on report;

    SELECT ts.tablespace_name         tablespace_name,
           ts.ts_size/(1024*1024)     total_mb,
           ts.ts_maxsize/(1024*1024)  total_avail_mb,
           ((((ts.ts_maxsize - ts.ts_size) + free.ts_free)/ts.ts_maxsize) * 100 ) pct_free,
           decode(sign(((((ts.ts_maxsize - ts.ts_size) + free.ts_free)/ts.ts_maxsize) * 100 - 10)), 1, ' ', '<**>')  over_flag,
           (ts.ts_size - free.ts_free)/(1024*1024)      used_mb,
           (ts_maxsize - (ts.ts_size - free.ts_free))/(1024*1024)      free_mb
    FROM
    (SELECT tablespace_name, sum(bytes) ts_size, sum(CASE  WHEN maxbytes = 0 THEN bytes WHEN maxbytes < bytes THEN bytes ELSE maxbytes END) ts_maxsize
     FROM sys.dba_data_files
     GROUP BY tablespace_name) ts,
    (SELECT df.tablespace_name, sum(nvl(fs.bytes, 0)) ts_free
     FROM sys.dba_free_space fs, sys.dba_data_files df
     WHERE fs.file_id (+)= df.file_id
     GROUP BY df.tablespace_name) free
    WHERE
          ts.tablespace_name = free.tablespace_name
    ORDER BY ts.tablespace_name;

    prompt

    exit;



