#!/usr/bin/env perl -w
#
# (#) ./base/sw/parser/log_parser.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
# COPYRIGHT WatchMark Corp 2005
#

#
# Parse a loader log for key timings
#   Parse - time spent parsing the file
#   TabGen - time spent building tables from the parse gateway
#   Load - time spent doing Oracle table load
#   Total - total loading time
#   
#

sub process_file;
sub register;
sub unregister;

## #######################################################
## MAIN
if (@ARGV) {
  printf("%s%s\t%s\t%s\t%s\t%s\n",
	 "#File", 
	 " " x (length($ARGV[0])-10),
	 "Parse",
	 "TabGen",
	 "Load",
	 "Total");

  foreach (@ARGV) {
    process_file($_);
  }
}

## #######################################################
sub process_file {
  my $filename = shift;
  
  unless (open(INFILE, $filename)) {
    die("Can't open $filename\n");
  }
  
  unregister;
  while (<INFILE>) {  
    if    (/Total file processing/)  { register("TOTAL", $_); }
    elsif (/\#\# sectionParse: /)    { register("PARSE", $_); }
    elsif (/\#\# tableGeneration: /) { register("TABLEGEN", $_); }
    elsif (/\#\# sectionLoad: /)     { register("LOAD", $_); }
  }  

  printf("%s\t%.1f\t%.1f\t%.1f\t%.1f\n", 
	 $filename,
	 $stats{"PARSE"},
	 $stats{"TABLEGEN"},
	 $stats{"LOAD"},
	 $stats{"TOTAL"});
  close(INFILE);
}

## #######################################################
sub register {
  my $statType = shift;
  my $line = shift;

  if (defined $stats{$statType}) {
    printf("%s COLLISION %s\n", $filename, $line);
  }
  
  $line =~ s/(.+?)(\d+\.?\d+)(.+\n)/$2/;
  #$line += 0;
  $stats{$statType} = $line;
}

## #######################################################
sub unregister {
  undef %stats;
}
