#!/usr/bin/env perl -w

# (#) ./base/sw/peik_tools/pop_data.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 

#Updated 12/02/2009 - valnt00077479
use pass_crypt;

use strict;
use English;
use Getopt::Long;
use File::Copy;

BEGIN {
  unshift @INC, $ENV{ FLEXPM_HOME } . "/sw/perl/lib"  if exists $ENV{ FLEXPM_HOME };
}

use DBI;

sub showUsage($);
sub get_valid_fullkey($);
sub get_valid_file_type($);
sub pre_install($);
sub post_deinstall($);
sub insert_data_in_DB($$$$$$$);
sub prepare_data_for_xml($$$$$$$);

sub output_xml_data($$$$$$$$);
sub output_entities($$$);
sub output_entity($$$$$);
sub output_instances($$$$);
sub output_ancestorINSs($$$$);
sub get_fakeValue;
sub xml_dump($$);
sub xml_header($);

#######################################
#  START
#######################################
  my $dbh;
  my $dbConnect;
  my $dbUser;
  my $dbPassword;
  my $dbTnsname;
  my $path;
  my $showHelp;
  my $entity_id;
  my $entity_name;
  my $file_name;
  my $file_type;
  my ($db_file_type, $db_file_type_id, $db_fullkey, $db_entity_id, $db_freq_id);
  my ($startDate, $endDate);
  my $file_type_list;
  my $fullkey;
  my $output;
  my ($vendor, $version, $compat, $numOfField);

  &GetOptions( 'connect=s' => \$dbConnect, 'help' => \$showHelp,
               'filetype=s' => \$file_type, 'fullkey=s' => \$fullkey,
               'startDate=s' => \$startDate, 'endDate=s' => \$endDate,
               'output=s' => \$output, 'path=s' => \$path,
               'vendor=s' => \$vendor, 'numOfField=s' => \$numOfField);

  #set default $dbConnect
  if ( not defined $dbConnect ) {
      $dbConnect = pass_crypt::getConnectForUser($ENV{DBUSER});
  }

  #set default $output
  (defined $output) || ( $output="DB" );

  if ( defined $showHelp )
  {
    showUsage($dbConnect); 
  }
  if ( not defined $file_type )
  {
    print "Error: no filetype specified.\n\n";
    showUsage($dbConnect);
  }
  if ( not defined $fullkey )
  {
    print "Error: no fullkey specified.\n\n";
    showUsage($dbConnect);
  }
  if ( not defined $startDate )
  {
    print "Error: no startDate specified.\n\n";
    showUsage($dbConnect);
  }
  if ( not defined $endDate )
  {
    print "Error: no endDate specified.\n\n";
    showUsage($dbConnect);
  }

  # set default $path
  if ( not defined $path ) {
    if ( exists $ENV{ FLEXPM_HOME } ) {
      $path = $ENV{ FLEXPM_HOME } . "/";
    }
    else {
      $path = "./" ;
    }
  }
  else {
    $path=$path . "/";
  }
  # set default $vendor
  (defined $vendor) || ($vendor="None");
  # set default number of field outputing for each entity
  (defined $numOfField) || ($numOfField=4);

  #
  # get file type(s) to process
  #
  my $ftypes = "";
  if ( $file_type eq 'ALL' || $file_type eq 'All' || $file_type eq 'all' )
  {
     $dbh = DBI->connect( 'dbi:Oracle:', $dbConnect, '',
                             { PrintError => 0, ora_module_name => "SchemaInf.pm" } )
        or die "Error: Can't connect to '$dbConnect' ($DBI::errstr).\n";

     my $ft_type;
     my $sth_ft = $dbh->prepare( "select distinct file_type
                                    from wm_file_control a,
                                         wm_file_table_map b
                                   where a.file_type_id=b.file_type_id
                                     and exists (select 1 from wmd_dict_table c
                                                  where  b.table_id=c.table_id
                                                    and  IS_TIME_SERIES='Y'
                                                    and SUMMARIZE>0)
                                 " );
     $sth_ft->execute;
     $sth_ft->bind_columns(undef, \$ft_type);
     my $ft_count = 0;
     while ( $sth_ft->fetch() )
     {
        if ( $ft_count > 0 )
        {
           $ftypes = $ftypes . ",";
        }
        $ftypes = $ftypes . $ft_type;
        $ft_count++;
     }
     $sth_ft->finish();
     my $rx = $dbh->disconnect;

  }
  else
  {
     $ftypes = $file_type;
  }


#
# process file type(s) requested
#
my @xftypes = split(",",$ftypes);
foreach $file_type ( @xftypes )
{
 
 $dbh = DBI->connect( 'dbi:Oracle:', $dbConnect, '',
                         { PrintError => 0, ora_module_name => "SchemaInf.pm" } )
    or die "Error: Can't connect to '$dbConnect' ($DBI::errstr).\n";


 my $sth = $dbh->prepare( "select distinct file_type,a.file_type_id, fullkey, frequency_id, a.entity_id
                             from wm_file_control a,
                                  wm_file_table_map b,
                                  wmn_instance_context c
                            where a.file_type_id=b.file_type_id
                              and a.file_type = ?
                              and c.fullkey = ?
                              and a.entity_id = c.entity_id
                              and exists (select 1 from wmd_dict_table c
                                           where  b.table_id=c.table_id
                                             and  IS_TIME_SERIES='Y'
                                             and SUMMARIZE>0)
                          " );
 $sth->bind_param(1, $file_type);
 $sth->bind_param(2, $fullkey);
 $sth->execute;
 $sth->bind_columns(undef, \$db_file_type, \$db_file_type_id,
                    \$db_fullkey, \$db_freq_id, \$db_entity_id);
 $sth->fetch();
  if (not (defined $db_file_type && defined $db_file_type) )
  {
    print "No data for file type($file_type) and fullkey($fullkey)\n";
    next;
    ### showUsage($dbConnect);
  }
 $sth->finish();

 print "Generating \"$file_type\" data for \"$fullkey\" in $output \n";
 pre_install($dbConnect);

 if ( $output eq "DB" )
 {
   # do DB
   print "Generating data in DB ...\n";
   insert_data_in_DB($dbConnect,
                     $db_entity_id,
                     $db_file_type_id,
                     $db_fullkey,
                     $startDate,
                     $endDate,
                     $output 
                    );
   print "Finished populating data DB.\n";
 }else {
   # do XML
   print "Preparing data in DB for XML file ...\n";
   prepare_data_for_xml($dbConnect,
                       $db_entity_id,
                       $db_file_type_id,
                       $db_fullkey,
                       $startDate,
                       $endDate,
                       'XML'
                       );
   print "Outputing to XML file ...\n";
   output_xml_data($dbConnect,
                   $db_entity_id,
                   $db_file_type,
                   $db_file_type_id,
                   $db_fullkey,
                   $db_freq_id,
                   $path,
                   $vendor);
   print "Finished Outputing data in XML file.\n";
 }
# post_deinstall($dbConnect);


 my $rc = $dbh->disconnect;
}
 

sub showUsage($)
{
 my $dbConnect=shift;
 my $file_type;
 print "usage: pop_data.pl -filetype <file type>  -fullkey <fullkey> \n";
 print "                   -startDate <YYYYMMDDHHMI>  -endDate <YYYYMMDDHHMI>\n";
 print "                   [ -connect <connectstring> ]\n";
 print "                   [ -output <DB|XML> ] [ -path <output file path> ]\n";
 print "                   [ -vendor <vendor name> ]\n";
 print "                   [ -numOfField < No. of field outputing for each entity> ]\n";
 print "       pop_data.pl -help\n\n";
 get_valid_file_type($dbConnect);
 print "\n";
 get_valid_fullkey($dbConnect);
 die "\n";
}

sub get_valid_file_type($)
{
 my $dbConnect=shift;
 my $file_type;
 $dbh = DBI->connect( 'dbi:Oracle:', $dbConnect, '',
                         { PrintError => 0, ora_module_name => "SchemaInf.pm" } )
    or die "Error: Can't connect to '$dbConnect' ($DBI::errstr).\n";

 my $sth = $dbh->prepare( "select distinct file_type
                             from wm_file_control a,
                                  wm_file_table_map b
                            where a.file_type_id=b.file_type_id
                              and exists (select 1 from wmd_dict_table c
                                           where  b.table_id=c.table_id
                                             and  IS_TIME_SERIES='Y'
                                             and SUMMARIZE>0)
                           union select 'ALL' from dual
                          " );

 $sth->execute;
 $sth->bind_columns(undef, \$file_type);
 print "The valid file types are:\n";
 while( $sth->fetch() ) {
  print "$file_type ";
 }
 $sth->finish();
 print "\n";
 my $rc = $dbh->disconnect;
}

sub get_valid_fullkey($)
{
 my $dbConnect=shift;
 my $file_type_id;
 my $file_type;
 my $fullkey;
 my ($sth, $sth_fullkey);
 $dbh = DBI->connect( 'dbi:Oracle:', $dbConnect, '',
                         { PrintError => 0, ora_module_name => "SchemaInf.pm" } )
    or die "Error: Can't connect to '$dbConnect' ($DBI::errstr).\n";

 $sth = $dbh->prepare( "select distinct file_type, a.file_type_id
                          from wm_file_control a,
                               wm_file_table_map b
                         where a.file_type_id=b.file_type_id
                           and exists (select 1 from wmd_dict_table c
                                        where  b.table_id=c.table_id
                                          and  IS_TIME_SERIES='Y'
                                          and SUMMARIZE>0)
                       " );
 $sth->execute;
 $sth->bind_columns(undef, \$file_type, \$file_type_id);
 print "The valid file types and fullkeys are:\n";
 while( $sth->fetch() ) {
  print "$file_type:\n";
  $sth_fullkey = $dbh->prepare("select fullkey
                                     from wm_file_control a,
                                          wmn_instance_context b
                                    where a.entity_id=b.entity_id
                                      and b.context_id in (select context_id
                                                             from wmn_context
                                                            where context_use='R')
                                      and a.file_type_id=? ");
  $sth_fullkey->bind_param(1, $file_type_id);
  $sth_fullkey->execute;
  $sth_fullkey->bind_columns(undef, \$fullkey); 
  while( $sth_fullkey->fetch() ) {
    print "\"$fullkey\" ";
  }
  $sth_fullkey->finish();
  print "\n";
 }
 $sth->finish();
 print "\n";
 my $rc = $dbh->disconnect;
}

sub pre_install($)
{
  my $db_connect=shift;
`  sqlplus -s /NOLOG << !
whenever sqlerror exit sql.sqlcode
set feed off
set serveroutput on size 1000000
connect $db_connect
declare
 vSQL varchar2(400);
 table_not_exists exception;
 object_exists exception;
 seq_not_exists exception;
 PRAGMA EXCEPTION_INIT(table_not_exists, -942);
 PRAGMA EXCEPTION_INIT(object_exists, -955);
 PRAGMA EXCEPTION_INIT(seq_not_exists, -2289);
begin
 begin -- drop table pop_ent_data_ins_tmp
   vSQL := 'drop table pop_ent_data_ins_tmp';
   execute immediate vSQL;
 exception
   when table_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_ent_data_ins_tmp

 begin -- drop table pop_time_slice_tmp
   vSQL := 'drop table pop_time_slice_tmp';
   execute immediate vSQL;
 exception
   when table_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_time_slice_tmp

 begin -- create table pop_ent_data_ins_tmp
   vSQL := 'create table pop_ent_data_ins_tmp '||
           '(instance_id integer,'||
           'entity_id integer,'||
           'fullkey   varchar2(360),'||
           'localkey varchar2(40),'||
           'entity_name varchar2(30))';
   execute immediate vSQL;
 exception
   when object_exists then
    null; 
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- create table pop_ent_data_ins_tmp

 begin -- create table pop_time_slice_tmp
   vSQL := 'create table pop_time_slice_tmp '||
           '(tstamp date )';
   execute immediate vSQL;
 exception
   when object_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- create table pop_time_slice_tmp

 begin -- drop sequence sq_popInsData
   vSQL := 'drop sequence sq_popInsData';
   execute immediate vSQL;
 exception
   when seq_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_time_slice_tmp

 begin -- create sequence sq_popInsData
   vSQL := 'create sequence sq_popInsData start with 1 maxvalue 99 cycle';
   execute immediate vSQL;
 exception
   when object_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- create table pop_time_slice_tmp

end;
/
EXIT 0
!
`;

`  sqlplus -s /NOLOG << !
connect $db_connect
start popInsData_pkg.sql
EXIT 0
!
`;

`  sqlplus -s /NOLOG << !
connect $db_connect
start popInsData_pkgb.sql
EXIT 0
!
`;

}

sub post_deinstall($)
{
 my $db_connect=shift;

` sqlplus -s /NOLOG << !
whenever sqlerror exit sql.sqlcode
set feed off
set serveroutput on size 1000000
connect $db_connect
declare
 vSQL varchar2(400);
 table_not_exists exception;
 object_exists exception;
 seq_not_exists exception;
 PRAGMA EXCEPTION_INIT(table_not_exists, -942);
 PRAGMA EXCEPTION_INIT(seq_not_exists, -2289);
begin
 begin -- drop table pop_ent_data_ins_tmp
   vSQL := 'drop table pop_ent_data_ins_tmp';
   execute immediate vSQL;
 exception
   when table_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_ent_data_ins_tmp

 begin -- drop table pop_time_slice_tmp
   vSQL := 'drop table pop_time_slice_tmp';
   execute immediate vSQL;
 exception
   when table_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_time_slice_tmp

 begin -- drop sequence sq_popInsData
   vSQL := 'drop sequence sq_popInsData';
   execute immediate vSQL;
 exception
   when seq_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_time_slice_tmp

 begin -- drop package populate_entity
   vSQL := 'drop package populate_entity';
   execute immediate vSQL;
 exception
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop package populate_entity
end;
/
EXIT 0
!
`;

}

sub insert_data_in_DB($$$$$$$)
{
 my $db_connect=shift;
 my $entityID=shift;
 my $fileTypeID=shift;
 my $FullKey=shift;
 my $StartDate=shift;
 my $EndDate=shift;
 my $outPut=shift;
my @ret= `  sqlplus -s /NOLOG << !
connect $db_connect
set echo on
set serveroutput on size 1000000
exec populate_InsData.pop_ins_data($entityID, $fileTypeID, \'$FullKey\', to_date($StartDate,'YYYYMMDDHH24MI'), to_date($EndDate,'YYYYMMDDHH24MI'), \'$outPut\');
EXIT 0
!
`;
print "@ret";
}
sub prepare_data_for_xml($$$$$$$)
{
 my $db_connect=shift;
 my $entityID=shift;
 my $fileTypeID=shift;
 my $FullKey=shift;
 my $StartDate=shift;
 my $EndDate=shift;
 my $outPut=shift;
my @ret=`  sqlplus -s /NOLOG << !
connect $db_connect
set echo on
set serveroutput on size 1000000
exec populate_InsData.pop_ins_data($entityID, $fileTypeID, \'$FullKey\', to_date($StartDate,'YYYYMMDDHH24MI'), to_date($EndDate,'YYYYMMDDHH24MI'), \'$outPut\');
EXIT 0
!
`;
print "@ret";
}

sub output_xml_data($$$$$$$$)
{
  my $db_connect=shift;
  my $file_entity_id=shift;
  my $file_type_name=shift;
  my $file_type=shift;
  my $fullkey=shift;
  my $Frequency_id=shift;
  my $path=shift;
  my $vendor=shift;
  my $file_entity_name;
  my $timesliceStart;
  my $timesliceEnd;
  my $granularitySecond;
  
  # get the entity_name with the given file type
  my $sth_Ent = $dbh->prepare("select entity_name
                                from wmn_entity
                               where entity_id=?");

  $sth_Ent->bind_param(1, $file_entity_id);
  $sth_Ent->execute;
  $sth_Ent->bind_columns(undef, \$entity_name);
  $sth_Ent->fetch();
  $sth_Ent->finish();

  my $sth_TS = $dbh->prepare("select to_char(tstamp,'YYYYMMDDHH24MISS'),
                              to_char(
                              populate_InsData.get_next_tstamp(tstamp, $Frequency_id)
                              ,'YYYYMMDDHH24MISS'),
                              populate_InsData.get_time_granularity($Frequency_id)
                              from pop_time_slice_tmp
                              order by tstamp ");
  $sth_TS->execute;
  $sth_TS->bind_columns(undef, \$timesliceStart,
                        \$timesliceEnd, \$granularitySecond);
  while($sth_TS->fetch()){
   #openfile;
   print "Writing to: $path"."$file_type_name:$fullkey:$timesliceStart.xml\n";
   open(XMLFILE, "> $path"."$file_type_name:$fullkey:$timesliceStart.xml");

   #outputxml file header;
   xml_header(\*XMLFILE);
   xml_dump(\*XMLFILE, "<mfh>\n");
   xml_dump(\*XMLFILE, "<ffv>2</ffv>\n");
   xml_dump(\*XMLFILE, "<sn></sn>\n");
   xml_dump(\*XMLFILE, "<vn>$vendor</vn>\n");
   xml_dump(\*XMLFILE, "<cbt>$timesliceStart</cbt>\n");
   xml_dump(\*XMLFILE, "</mfh>\n");
   xml_dump(\*XMLFILE, "<md>\n");
   xml_dump(\*XMLFILE, "<neid>\n");
   xml_dump(\*XMLFILE, "<neun>$fullkey</neun>\n");
   xml_dump(\*XMLFILE, "<nedn></nedn>\n");
   xml_dump(\*XMLFILE, "</neid>\n");
#   xml_dump(\*XMLFILE, "<mi>\n");
#   xml_dump(\*XMLFILE, "<mts>$timesliceStart</mts>\n");
#   xml_dump(\*XMLFILE, "<gp>$granularitySecond</gp>\n"); 
    
   output_entities(\*XMLFILE, $timesliceStart, $granularitySecond);

   #output file foot;
#   xml_dump(\*XMLFILE, "</mi>\n");
   xml_dump(\*XMLFILE, "</md>\n");
   xml_dump(\*XMLFILE, "<mff>\n");
   xml_dump(\*XMLFILE, "<ts>$timesliceEnd</ts>\n");
   xml_dump(\*XMLFILE, "</mff>\n");
   xml_dump(\*XMLFILE, "</mdc>\n");
   close(XMLFILE);
  }
 $sth_TS->finish();
} 

sub output_entities($$$)
{
  my $FILE=shift;
  my $timesliceStart=shift;
  my $granularitySecond=shift;
  my ($entity_id, $entity_name);
  my $sth_ENTs = $dbh->prepare(" select distinct tmp.entity_id,
					decode(mo_class, null, entity_name, mo_class) entity_name
                                   from pop_ent_data_ins_tmp tmp, wmd_dict_table dt
                                  where tmp.entity_id=dt.entity_id
                                    and is_time_series='Y' and summarize>0
                                  order by 2");
  $sth_ENTs->execute;
  $sth_ENTs->bind_columns(undef, \$entity_id, \$entity_name);
  while($sth_ENTs->fetch()){
   output_entity(\*$FILE, $entity_id, $entity_name, $timesliceStart, $granularitySecond);
  }
 $sth_ENTs->finish();
}

sub output_entity($$$$$)
{
  my $FILE=shift;
  my $entity_id=shift;
  my $entity_name=shift;
  my $timesliceStart=shift;
  my $granularitySecond=shift;
  my $field_name;
  my $pegCount;

  xml_dump(\*XMLFILE, "<mi>\n");
  xml_dump(\*XMLFILE, "<mts>$timesliceStart</mts>\n");
  xml_dump(\*XMLFILE, "<gp>$granularitySecond</gp>\n"); 

  my $sth_field = $dbh->prepare("select distinct fp.value
				   from wmd_dict_name dn,
					peik_extra_fld_property fp,
                                        wmd_dict_table dt
                                  where dn.entity_id=?
                                    and dn.entity_id=dt.entity_id
				    and dn.field_id=fp.field_id
				    and nvl(dt.mo_class, ?) = ?
                                    and namespace_id=0
                                    and rownum<=? ");
  $sth_field->bind_param(1, $entity_id);
  $sth_field->bind_param(2, $entity_name);
  $sth_field->bind_param(3, $entity_name);
  $sth_field->bind_param(4, $numOfField);
  $sth_field->execute;
  $sth_field->bind_columns(undef, \$field_name);
  $pegCount = 0;
  while($sth_field->fetch()){
  #output entity fields 
    $pegCount = $pegCount + 1;
    xml_dump(\*$FILE, "<mt p=\"$pegCount\">$field_name</mt>\n");
  }
 $sth_field->finish();
 #output_fieldName;
 ($pegCount != 0 ) && output_instances(\*$FILE, $entity_id, $entity_name, $pegCount);
  xml_dump(\*XMLFILE, "</mi>\n");
}

sub output_instances($$$$)
{
  my $FILE=shift;
  my $entity_id=shift;
  my $entity_name=shift;
  my $pegCount=shift;
  my $localkey;
  my $instance_id;
  my $fakeValue;
  my $vCount;
  my $sth_INSTs = $dbh->prepare("select instance_id, localkey
                                   from pop_ent_data_ins_tmp
                                  where entity_id=?");
  $sth_INSTs->bind_param(1, $entity_id);
  $sth_INSTs->execute;
  $sth_INSTs->bind_columns(undef, \$instance_id, \$localkey);
  while($sth_INSTs->fetch()){
   #output_instance;
   xml_dump(\*$FILE, "<mv>\n");
   xml_dump(\*$FILE, "<moid>");
   # get ancestor instances' list
   output_ancestorINSs(\*$FILE, $instance_id, $entity_id, $entity_name);
   xml_dump(\*$FILE, "</moid>\n");
   $vCount=1;
   while($vCount <= $pegCount) {
     $fakeValue=get_fakeValue;
     xml_dump(\*$FILE, "<r p=\"$vCount\">$fakeValue</r>\n");
     $vCount = $vCount + 1;
   }
   xml_dump(\*$FILE, "</mv>\n"); 
  }
  $sth_INSTs->finish();
}

sub output_ancestorINSs($$$$)
{
  my $FILE=shift;
  my $instance_id=shift;
  my $ins_entity_id=shift;
  my $ins_entity_name=shift;
  my $GenContextId;
  my $entity_id;
  my $level_count;
  my ($entity_name, $localkey);
  my $sth_GenContext = $dbh->prepare("select context_id
                                      from wmn_context
                                      where context_use=\'R\' ");
  $sth_GenContext->execute;
  $sth_GenContext->bind_columns(undef, \$GenContextId);
  $sth_GenContext->fetch();
  $sth_GenContext->finish();
 
  my $sth_INSs=$dbh->prepare("
              select entity_name, localkey, b.entity_id
                from 
                    (select level lev, ic.*
                       from wmn_instance_context ic
                      start with instance_id=? and context_id=?
                    connect by PRIOR parentinstance_id=instance_id
                                 and context_id=?) a,
                    wmn_entity b
               where a.entity_id=b.entity_id
               order by lev desc ");
  $sth_INSs->bind_param(1, $instance_id);
  $sth_INSs->bind_param(2, $GenContextId);
  $sth_INSs->bind_param(3, $GenContextId);
  $sth_INSs->execute;
  $sth_INSs->bind_columns(undef, \$entity_name, \$localkey, \$entity_id);
  $level_count=1;
  while($sth_INSs->fetch()){
     if ($level_count == 3){
       if ($ins_entity_id == $entity_id){
	 xml_dump(\*$FILE, "$ins_entity_name=$localkey");
       } else {
         xml_dump(\*$FILE, "$entity_name=$localkey");
       }
     }
     if ($level_count > 3){
       if ($ins_entity_id == $entity_id){
	 xml_dump(\*$FILE, ", $ins_entity_name=$localkey");
       } else {
         xml_dump(\*$FILE, ", $entity_name=$localkey");
       }
     } 
     $level_count = $level_count + 1;
  }
  $sth_INSs->finish();
}

sub get_fakeValue
{
  my $fakevalue;
  my $sth_seq=$dbh->prepare("select sq_popInsData.nextval from dual");
  $sth_seq->execute;
  $sth_seq->bind_columns(undef, \$fakevalue);
  $sth_seq->fetch();
  $sth_seq->finish();
  return $fakevalue;
}

sub xml_dump($$)
{
 my $FILE=shift;
 my $xml_string = shift;
 print $FILE "$xml_string";
}

sub xml_header($)
{
 my $FILE=shift;
 print $FILE "<?xml version='1.0'?>\n";
 print $FILE "<?xml-stylesheet type=\"text/xsl\" href=\"MeasDataCollection.xsl\" ?>\n";
 print $FILE "<!DOCTYPE mdc SYSTEM \"MeasDataCollection.dtd\">\n";
 print $FILE "<mdc xmlns:HTML=\"http://www.w3.org/TR/REC-xml\">\n";
}

