#!/usr/bin/env perl -w

# @(#) SchemaInf.pm 1.1.0 03/02/12 18:49:30
# @(#) 

use strict;
use English;
use DBI;

#require 5.005_03;

my @ISA;
my @EXPORT;
my @EXPORT_OK;

BEGIN {
  $SchemaInf::VERSION = "1.1.0";
}

{
  package SchemaInf;
  
  #  use DynaLoader ();
  use Exporter ();
  use English;

  my $VERSION;

  BEGIN {
    #@ISA = qw(Exporter DynaLoader);
    @ISA = qw(Exporter);
    
    # package export info
    @EXPORT    = qw( readDB readFile writeFile );
    @EXPORT_OK = qw( ); 
    
    #bootstrap SchemaInf;
  }
  
  return 1;


  sub readDB {
    my $class = shift;
    my $connectString = shift;

    my $self = {};
    bless $self;
    
    my $dbName = $connectString;
    $dbName =~ s/\/.*\@/\@/;
    
    $self->{ input } = { type => 'db', 
			 connect => $connectString,
			 name => $dbName
		       };
    
    my %indexList;
    
    my $dbh = DBI->connect( 'dbi:Oracle:', $connectString, '', { PrintError => 0, ora_module_name => "SchemaInf.pm" } )  
      or die "Error: Can't connect to '$connectString' ($DBI::errstr).\n";
    
    print "Reading database configuration from $dbName .. \n";
    
    print "  Reading tables from database .. \n";

    {
      my $statement = "SELECT table_name, tablespace_name, monitoring, partitioned, iot_type, temporary, " .
	"pct_free, pct_used, pct_increase, initial_extent, next_extent, min_extents, max_extents, " .
	  "ini_trans, max_trans, freelists, freelist_groups, logging, duration " .
	    "FROM USER_TABLES";
      
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } ) {
	  print "** Return Row with undefined Table Name\n";
	  next;
	}
	my $tabName = $returnRowRef->{ TABLE_NAME };

	$self->{ tables }->{ $tabName } = {}  unless exists $self->{ tables }->{ $tabName };
	my $tabRef = $self->{ tables }->{ $tabName };

	foreach my $colKey ( keys %{ $returnRowRef } ) {
	  $returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	}

	$tabRef->{ type } = 'TABLE';
	$tabRef->{ tablespace } = $returnRowRef->{ TABLESPACE_NAME };
	$tabRef->{ monitoring } = $returnRowRef->{ MONITORING };
	$tabRef->{ partitioned } = $returnRowRef->{ PARTITIONED };
	$tabRef->{ iot } = $returnRowRef->{ IOT_TYPE };
	$tabRef->{ temp } = $returnRowRef->{ TEMPORARY };
	$tabRef->{ pct_free } = $returnRowRef->{ PCT_FREE };
	$tabRef->{ pct_used } = $returnRowRef->{ PCT_USED };
	$tabRef->{ freelists } = $returnRowRef->{ FREELISTS };
	$tabRef->{ freelist_groups } = $returnRowRef->{ FREELIST_GROUPS };
	$tabRef->{ duration } = $returnRowRef->{ DURATION };
	$tabRef->{ columns } = {};
      }
      
    }

    print "  Reading views from database .. \n";
    {
      my $statement = "SELECT view_name FROM USER_VIEWS";
      
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });

      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	unless ( exists $returnRowRef->{ VIEW_NAME } and defined $returnRowRef->{ VIEW_NAME } ) {
	  print "** Return Row with undefined View Name\n";
	  next;
	}
	my $viewName = $returnRowRef->{ VIEW_NAME };
	$self->{ tables }->{ $viewName }->{ type } = 'VIEW';
      }
    }
    
    print "  Reading table/view columns from database .. \n";
    {
      my $statement = "SELECT table_name, column_name, data_type, data_length, data_precision, data_scale, nullable, column_id, data_default FROM USER_TAB_COLUMNS ORDER BY table_name, column_id";
      
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
      
	unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } ) {
	  print "** Return Row with undefined Table Name\n";
	  next;
	}
	my $tabName = $returnRowRef->{ TABLE_NAME };

	unless ( exists $returnRowRef->{ COLUMN_NAME } and defined $returnRowRef->{ COLUMN_NAME } ) {
	  print "** Return Row with undefined Column Name for table $tabName\n";
	  next;
	}
	my $colName = $returnRowRef->{ COLUMN_NAME };

	if ( not exists $self->{ tables }->{ $tabName } ) {
	  print "** Table does not exist - $tabName for column $colName\n";
	  next;
	}
	if ( not defined $returnRowRef->{ COLUMN_ID } ) {
	  print "Column Ordinal not defined for $tabName.$colName\n";
	  next;
	}

	#printf "%-40s\t%s\n", $tabName, $colName   if $tabName eq 'WMN_INSTANCE';

	foreach my $colKey ( keys %{ $returnRowRef } ) {
	  $returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	}

	$self->{ tables }->{ $tabName }->{ columns }->{ $colName } = {}  unless exists $self->{ tables }->{ $tabName }->{ columns }->{ $colName };
	my $colRef = $self->{ tables }->{ $tabName }->{ columns }->{ $colName };

	$colRef->{ type }        = $returnRowRef->{ DATA_TYPE };
	$colRef->{ len }         = $returnRowRef->{ DATA_LENGTH };
	$colRef->{ prec }        = $returnRowRef->{ DATA_PRECISION };
	$colRef->{ scale }       = $returnRowRef->{ DATA_SCALE };
	$colRef->{ nullable }    = $returnRowRef->{ NULLABLE };
	$colRef->{ ordinal }     = $returnRowRef->{ COLUMN_ID };
	$colRef->{ default }     = $returnRowRef->{ DATA_DEFAULT };
	$colRef->{ constraints } = [];
	$colRef->{ default } =~ s/\s+$//g;
      } 
    }

    print "  Reading indexes from database .. \n";
    {
      my $statement = "SELECT index_name, index_type, table_name, uniqueness, tablespace_name, compression, pct_free, freelists, freelist_groups FROM USER_INDEXES";
      
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	print STDERR "Program Error: Return Row with undefined Table Name\n"
	  and next
	    unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
	my $tabName = $returnRowRef->{ TABLE_NAME };

	print STDERR "Program Error: Return Row with undefined Index Name for table $tabName\n" 
	  and next
	    unless ( exists $returnRowRef->{ INDEX_NAME } and defined $returnRowRef->{ INDEX_NAME } );
	my $indName = $returnRowRef->{ INDEX_NAME };

	print STDERR "Program Error: Table does not exist - $tabName for index $indName\n"
	  and next
	    if ( not exists $self->{ tables }->{ $tabName } );

	$self->{ tables }->{ $tabName } = {}  unless exists $self->{ tables }->{ $tabName };
	my $tabRef =$self->{ tables }->{ $tabName };
	$tabRef->{ indexes }->{ $indName } = {}  unless exists $tabRef->{ indexes }->{ $indName };
	my $indRef =$tabRef->{ indexes }->{ $indName };

	$indexList{ $indName } = $tabName;
	  
	foreach my $colKey ( keys %{ $returnRowRef } ) {
	  $returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	}

	$returnRowRef->{ INDEX_TYPE } =~ s/\s+//g   if exists $returnRowRef->{ INDEX_TYPE } and defined $returnRowRef->{ INDEX_TYPE };
	my $indType = $returnRowRef->{ INDEX_TYPE };

	$indRef->{ type } = $indType;
	$indRef->{ uniqueness } = $returnRowRef->{ UNIQUENESS };
	$indRef->{ tablespace } = $returnRowRef->{ TABLESPACE_NAME };
	$indRef->{ compression } = $returnRowRef->{ COMPRESSION };
	$indRef->{ pct_free } = $returnRowRef->{ PCT_FREE };
	$indRef->{ freelists } = $returnRowRef->{ FREELISTS };
	$indRef->{ freelist_groups } = $returnRowRef->{ FREELIST_GROUPS };
	$indRef->{ columns }= [];
	
	if ( $indType eq 'IOT-TOP' ) {
	  $tabRef->{ iot_index } = $indName;
	}
      }
    }
    
    print "  Reading index columns from database .. \n";
    {
      my $statement = "SELECT ic.index_name, ic.table_name, ic.column_name, ie.column_expression  FROM USER_IND_COLUMNS ic, USER_IND_EXPRESSIONS ie " .
	"WHERE ic.index_name = ie.index_name (+)  AND ic.table_name = ie.table_name (+)  AND ic.column_position = ie.column_position (+) " . 
	  "ORDER BY ic.index_name, ic.table_name, ic.column_position";
      
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	print STDERR "Program Error:  Return Row with undefined Table Name\n"
	  and next
	    unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
	my $tabName = $returnRowRef->{ TABLE_NAME };

	print STDERR "Program Error:  Return Row with undefined Index Name for table $tabName\n"
	  and next
	    unless ( exists $returnRowRef->{ INDEX_NAME } and defined $returnRowRef->{ INDEX_NAME } );
	my $indName = $returnRowRef->{ INDEX_NAME };

	print STDERR "Program Error:  Return Row with undefined Column Name for table $tabName, index $indName\n"
	  and next
	    unless ( exists $returnRowRef->{ COLUMN_NAME } and defined $returnRowRef->{ COLUMN_NAME } );
	my $colName = $returnRowRef->{ COLUMN_NAME };

	print STDERR "Program Error:  Table does not exist - $tabName for index $indName, column $colName\n"
	  and next 
	    if ( not exists $self->{ tables }->{ $tabName } );

	print STDERR "Program Error: Index does not exist - $indName for table $tabName, column $colName\n"
	  and next
	    if ( not exists $self->{ tables }->{ $tabName }->{ indexes }->{ $indName } );
	print STDERR "Program Error: column tag does not exist - $indName for table $tabName, column $colName\n"
	  and next
	    if ( not exists $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ columns } );

	$self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ columns } = []  unless exists $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ columns };
	my $colArrRef = $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ columns };
	if ( exists $returnRowRef->{ COLUMN_EXPRESSION } and defined $returnRowRef->{ COLUMN_EXPRESSION } ) {
	  push @{ $colArrRef }, $returnRowRef->{ COLUMN_EXPRESSION };
	}
	else {
	  push @{ $colArrRef }, $colName;
	}
      }
    }

    print "  Reading partition table information from database .. \n";
    {
      my $statement = "SELECT table_name, partitioning_type, partitioning_key_count, def_tablespace_name, " .
	"def_pct_free, def_pct_used, def_freelists, def_freelist_groups " .
	  "FROM USER_PART_TABLES";
      
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	print STDERR "Program Error:  Return Row with undefined Table Name\n"
	  and next
	    unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
	my $tabName = $returnRowRef->{ TABLE_NAME };

	print STDERR "Program Error:  Table does not exist - $tabName for partition table info\n"
	  and next 
	    if ( not exists $self->{ tables }->{ $tabName } );

	foreach my $colKey ( keys %{ $returnRowRef } ) {
	  $returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	}

	$self->{ tables }->{ $tabName }->{ part_info } = {}  unless exists $self->{ tables }->{ $tabName }->{ part_info };
	my $partRef = $self->{ tables }->{ $tabName }->{ part_info };

	$partRef->{ part_type } = $returnRowRef->{ PARTITIONING_TYPE };
	$partRef->{ part_key_count } = $returnRowRef->{ PARTITIONING_KEY_COUNT };
	$partRef->{ part_keys } = [];
	$partRef->{ tablespace } = $returnRowRef->{ DEF_TABLESPACE_NAME };
	$partRef->{ pct_free } = $returnRowRef->{ DEF_PCT_FREE };
	$partRef->{ pct_used } = $returnRowRef->{ DEF_PCT_USED };
	$partRef->{ freelists } = $returnRowRef->{ DEF_FREELISTS };
	$partRef->{ freelist_groups } = $returnRowRef->{ DEF_FREELIST_GROUPS };
      }
    }
    
    print "  Reading partition index information from database .. \n";
    {
      my $statement = "SELECT table_name, index_name, partitioning_type, partitioning_key_count, def_tablespace_name, " .
	"def_pct_free, def_freelists, def_freelist_groups " .
	  "FROM USER_PART_INDEXES";
      
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	print STDERR "Program Error:  Return Row with undefined Table Name\n"
	  and next
	    unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
	my $tabName = $returnRowRef->{ TABLE_NAME };

	print STDERR "Program Error:  Return Row with undefined Index Name for table $tabName\n"
	  and next
	    unless ( exists $returnRowRef->{ INDEX_NAME } and defined $returnRowRef->{ INDEX_NAME } );
	my $indName = $returnRowRef->{ INDEX_NAME };

	print STDERR "Program Error:  Table does not exist - $tabName for partition table index\n"
	  and next 
	    if ( not exists $self->{ tables }->{ $tabName } );

	print STDERR "Program Error: Index does not exist - $tabName.$indName for partition index\n"
	  and next
	    if ( not exists $self->{ tables }->{ $tabName }->{ indexes }->{ $indName } );

	foreach my $colKey ( keys %{ $returnRowRef } ) {
	  $returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	}

	$self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ part_info } = {}  unless exists $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ part_info };
	my $indPartRef = $self->{ tables }->{ $tabName }->{ indexes }->{ $indName }->{ part_info };

	$indPartRef->{ part_type } = $returnRowRef->{ PARTITIONING_TYPE };
	$indPartRef->{ part_key_count } = $returnRowRef->{ PARTITIONING_KEY_COUNT };
	$indPartRef->{ part_keys } = [];
	$indPartRef->{ tablespace } = $returnRowRef->{ DEF_TABLESPACE_NAME };
	$indPartRef->{ pct_free } = $returnRowRef->{ DEF_PCT_FREE };
	$indPartRef->{ freelists } = $returnRowRef->{ DEF_FREELISTS };
	$indPartRef->{ freelist_groups } = $returnRowRef->{ DEF_FREELIST_GROUPS };
      }
    }
    
    print "  Reading partition key column information from database .. \n";
    {
      my $statement = "SELECT name, object_type, column_name " .
	"FROM USER_PART_KEY_COLUMNS ORDER BY column_position";

      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	# for each partition_key, add to table or index part_info
	print STDERR "Program Error:  Return Row with undefined Object Name\n"
	  and next
	    unless ( exists $returnRowRef->{ NAME } and defined $returnRowRef->{ NAME } );
	my $objName = $returnRowRef->{ NAME };

	print STDERR "Program Error:  Return Row with undefined Object Type for object $objName\n"
	  and next
	    unless ( exists $returnRowRef->{ OBJECT_TYPE } and defined $returnRowRef->{ OBJECT_TYPE } );
	my $objType = uc $returnRowRef->{ OBJECT_TYPE };
	$objType =~ s/\s+//g;

	print STDERR "Program Error:  Return Row with undefined Column Name for $objType object $objName\n"
	  and next
	    unless ( exists $returnRowRef->{ COLUMN_NAME } and defined $returnRowRef->{ COLUMN_NAME } );
	my $colName = $returnRowRef->{ COLUMN_NAME };

	print STDERR "Program Error:  Unknown Object Type $objType for object $objName\n"
	  and next
	    unless ( $objType eq 'INDEX' or $objType eq 'TABLE' );
	

	#print STDERR "\t$objType\t$objName\t$colName\n";
	if ( $objType eq 'INDEX' ) {
	  my $tabName = $indexList{ $objName };
	  print STDERR "Program Error: Index does not exist - $objName for partition key $colName\n"
	    and next
	      unless defined $tabName and exists $self->{ tables }->{ $tabName }->{ indexes }->{ $objName };
	  print STDERR "Program Error: PartKeys does not exist -  $tabName.$objName - $colName\n"
	    and next
	      unless exists $self->{ tables }->{ $tabName }->{ indexes }->{ $objName }->{ part_info }->{ part_keys };

	  push @{ $self->{ tables }->{ $tabName }->{ indexes }->{ $objName }->{ part_info }->{ part_keys } }, $colName;
	}
	elsif ( uc $objType eq 'TABLE' ) {
	  print STDERR "Program Error: Table does not exist - $objName for partition key $colName\n"
	    and next
	      unless exists $self->{ tables }->{ $objName };
	  print STDERR "Program Error: PartKeys does not exist -  $objName - $colName\n"
	    and next
	      unless exists $self->{ tables }->{ $objName }->{ part_info }->{ part_keys };

	  push @{ $self->{ tables }->{ $objName }->{ part_info }->{ part_keys } }, $colName;
	}
      }
    }

    print "  Reading constraints from database .. \n";
    {
      my $statement = "SELECT Constraint_Name, Constraint_Type, Table_Name, R_Constraint_Name, Delete_Rule, Status, Deferrable, Deferred, Search_Condition " .
	"FROM USER_CONSTRAINTS ORDER BY Constraint_Name";
      
      $dbh->{LongReadLen} = 2000;
      $dbh->{LongTruncOk} = 1;
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      print "Constraint array size = ", scalar( @{ $returnSetRef } ), "\n";
      my $consCount = 0;
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	print STDERR "Program Error:  Return Row with undefined Constraint Name\n"
	  and next
	    unless ( exists $returnRowRef->{ CONSTRAINT_NAME } and defined $returnRowRef->{ CONSTRAINT_NAME } );
	my $consName = $returnRowRef->{ CONSTRAINT_NAME };
	#printf "%5i\t%s\n", ++$consCount, $consName;
	print STDERR "Program Error:  Return Row with undefined Table Name for constraint $consName\n"
	  and next
	    unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
	my $tabName = $returnRowRef->{ TABLE_NAME };

	print STDERR "Program Error: Table does not exist - $tabName for constraint $consName\n"
	  and next
	    unless exists $self->{ tables }->{ $tabName };
	
	#print STDERR "Constraint $consName\t$tabName\n";
	
	foreach my $colKey ( keys %{ $returnRowRef } ) {
#	  $returnRowRef->{ $colKey } = '<NULL>'    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	  $returnRowRef->{ $colKey } = ''    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	}

	$self->{ tables }->{ $tabName }->{ constraints }->{ $consName } = {}  unless exists $self->{ tables }->{ $tabName }->{ constraints }->{ $consName };
	my $consRef = $self->{ tables }->{ $tabName }->{ constraints }->{ $consName };

	$consRef->{ type } = $returnRowRef->{ CONSTRAINT_TYPE };
	$consRef->{ status } = $returnRowRef->{ STATUS };
	$consRef->{ refName } = $returnRowRef->{ R_CONSTRAINT_NAME };
	$consRef->{ delRule } = $returnRowRef->{ DELETE_RULE };
	$consRef->{ deferrable } = $returnRowRef->{ DEFERRABLE };
	$consRef->{ deferred } = $returnRowRef->{ DEFERRED };
	$consRef->{ search } = $returnRowRef->{ SEARCH_CONDITION };
	$consRef->{ search } =~ s/^\s+//g;
	$consRef->{ search } =~ s/\s+$//g;
	$consRef->{ search } =~ s/\s*\n\s*/ /g;
	$consRef->{ search } =~ s/\s+/ /g;
	$consRef->{ columns } = {};

	$self->{ constraints }->{ $consName }->{ table } = $tabName;
      }
    }
    
    print "  Reading constraint columns from database .. \n";
    {
      my $statement = "SELECT Constraint_Name, Table_Name, Column_Name, Position FROM USER_CONS_COLUMNS ORDER BY Constraint_Name, Position";

      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	print STDERR "Program Error:  Return Row with undefined Constraint Name\n"
	  and next
	    unless ( exists $returnRowRef->{ CONSTRAINT_NAME } and defined $returnRowRef->{ CONSTRAINT_NAME } );
	my $consName = $returnRowRef->{ CONSTRAINT_NAME };

	print STDERR "Program Error:  Return Row with undefined Table Name for constraint $consName\n"
	  and next
	    unless ( exists $returnRowRef->{ TABLE_NAME } and defined $returnRowRef->{ TABLE_NAME } );
	my $tabName = $returnRowRef->{ TABLE_NAME };

	print STDERR "Program Error:  Return Row with undefined Column Name for constraint $consName, table $tabName\n"
	  and next
	    unless ( exists $returnRowRef->{ COLUMN_NAME } and defined $returnRowRef->{ COLUMN_NAME } );
	my $colName = $returnRowRef->{ COLUMN_NAME };

	print STDERR "Program Error:  Table does not exist - $tabName for constraint $consName, column $colName\n"
	  and next
	    unless exists $self->{ tables }->{ $tabName };

	print STDERR "Program Error: Constraint does not exist - $consName for table $tabName, column $colName\n"
	  and next
	    unless exists $self->{ tables }->{ $tabName }->{ constraints }->{ $consName };

	print STDERR "Program Error: Constraint Column does not exist - $colName for constraint $consName, table $tabName\n"
	  and next
	    unless exists $self->{ tables }->{ $tabName }->{ columns }->{ $colName };
    
	unless ( exists $returnRowRef->{ POSITION } and defined $returnRowRef->{ POSITION } ) {
	  my $colRef = $self->{ tables }->{ $tabName }->{ columns }->{ $colName };
	  push @{ $colRef->{ constraints } }, $consName;
	  my $consRef = $self->{ tables }->{ $tabName }->{ constraints }->{ $consName };
	  if ( $consRef->{ type } eq 'C'  and 
	       $consRef->{ search } =~ m/^\"?$colName\"\s+IS\s+NOT\s+NULL\s*/i and
	       $colRef->{ nullable } = 'N' ) {
	    $colRef->{ notNull } = $consName;
	  }
	  $self->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ columns }->{ $colName }->{ position } = 0;
	}
	else {
	  $self->{ tables }->{ $tabName }->{ constraints }->{ $consName }->{ columns }->{ $colName }->{ position } =  $returnRowRef->{ POSITION };
	}
      }
    }
    
    print "  Reading sequences from database .. \n";
    {
      my $statement = "SELECT Sequence_Name, Cycle_Flag, Order_Flag FROM USER_SEQUENCES";
    
      my $returnSetRef = $dbh->selectall_arrayref($statement, { Slice => {} });
      foreach my $returnRowRef ( @{ $returnSetRef } ) {
	print STDERR "Program Error:  Return Row with undefined Sequence Name\n"
	  and next
	    unless ( exists $returnRowRef->{ SEQUENCE_NAME } and defined $returnRowRef->{ SEQUENCE_NAME } );
	my $seqName = $returnRowRef->{ SEQUENCE_NAME };

	foreach my $colKey ( keys %{ $returnRowRef } ) {
	  $returnRowRef->{ $colKey } = '<NULL>'    unless exists $returnRowRef->{ $colKey } and defined $returnRowRef->{ $colKey };
	}

	$self->{ sequences }->{ $seqName } = {}  unless exists $self->{ sequences }->{ $seqName };
	my $seqRef = $self->{ sequences }->{ $seqName };

	$seqRef->{ cycleFlag } = $returnRowRef->{ CYCLE_FLAG };
	$seqRef->{ orderFlag } = $returnRowRef->{ ORDER_FLAG };
      }
    }	
    
    my $rc = $dbh->disconnect;
    print "Completed reading database configuration.\n";
    
    return $self;
  }


  sub readFile {
    my $class = shift;
    my $configFile = shift;

    my $self = {};
    bless $self;

    $self->{ input } = { type => 'file', 
			 name => $configFile 
		       };

    my $fType = `file $configFile`;
    if ( $fType =~ m/gzip compressed data/ ) {
      open DATAFILE, "gzip -d -c <$configFile |" or die "Can't open input file $configFile\n";
    }
    else {
      open DATAFILE, "<$configFile" or die "Can't open input file $configFile\n";
    }

    print "Reading stored configuration from $configFile \n";

    my $tabName;
    my $tabRef;
    my $indName;
    my $indRef;
    my $consName;
    my $consRef;
    
    while ( <DATAFILE> ) {
      chomp;
      my @inArgs = split /\t/, $_, -1;
      #printf "size = %3i\t\$inArgs[0] = '%s'\n", $#inArgs, $inArgs[0];
      my $argKey = shift @inArgs;
      if ( not defined $argKey or substr($argKey,0,7) eq 'COMMENT' or substr($argKey,0,1) eq '#' ) {
	# This is either a blank line or a comment line, ignore it
      }
      elsif ( $argKey eq 'TABLE' ) {
	$tabName = shift @inArgs;
	if ( defined $tabName ) {
	  $self->{ tables }->{ $tabName } = { } if not exists $self->{ tables }->{ $tabName };
	  #printf "Read table %s\n", $tabName;
	  $tabRef = $self->{ tables }->{ $tabName };
	  undef $indName;
	  undef $indRef;
	  undef $consName;
	  undef $consRef;
	  $tabRef->{ type } = shift @inArgs;
	  $tabRef->{ columns } = {};
	}
	else {
	  print " ** Discarding TABLE line - table name not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'TABINFO' ) {
	if ( defined $tabRef ) {
	  $tabRef->{ tablespace } = shift @inArgs;
	  $tabRef->{ monitoring } = shift @inArgs;
	  $tabRef->{ partitioned } = shift @inArgs;
	  $tabRef->{ iot } = shift @inArgs;
	  $tabRef->{ temp } = shift @inArgs;
	  $tabRef->{ pct_free } = shift @inArgs;
	  $tabRef->{ pct_used } = shift @inArgs; 
	  $tabRef->{ freelists } = shift @inArgs;
	  $tabRef->{ freelist_groups } = shift @inArgs;
	  $tabRef->{ duration } = shift @inArgs;
	}
	else {
	  print " ** Discarding TABINFO line - table not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'TABPART' ) {
	if ( defined $tabRef ) {
	  $tabRef->{ part_info }->{ tablespace } = shift @inArgs;
	  $tabRef->{ part_info }->{ part_type } = shift @inArgs;
	  $tabRef->{ part_info }->{ part_key_count } = shift @inArgs;
	  $tabRef->{ part_info }->{ pct_free } = shift @inArgs;
	  $tabRef->{ part_info }->{ pct_used } = shift @inArgs;
	  $tabRef->{ part_info }->{ freelists } = shift @inArgs;
	  $tabRef->{ part_info }->{ freelist_groups } = shift @inArgs;
	  my $keyString = shift @inArgs;
	  if ( defined $keyString ) {
	    my @keyList = split /\;/, $keyString, -1;
	    $tabRef->{ part_info }->{ part_keys } = [ @keyList ];
	  }
	}
	else {
	  print " ** Discarding TABPART line - table not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'COLUMN' ) {
	my $colName = shift @inArgs;
	if ( defined $tabRef and defined $colName ) {
	  $tabRef->{ columns }->{ $colName } = {} if not exists $tabRef->{ columns }->{ $colName };
	  my $colRef = $tabRef->{ columns }->{ $colName };
	  $colRef->{ type } = shift @inArgs;
	  $colRef->{ len } = shift @inArgs;
	  $colRef->{ prec } = shift @inArgs;
	  $colRef->{ scale } = shift @inArgs;
	  $colRef->{ nullable } = shift @inArgs;
	  $colRef->{ ordinal } = shift @inArgs;
	  $colRef->{ default } = shift @inArgs;
	  $colRef->{ constraints } = [];
	}
	elsif ( not defined $tabRef ) {
	  print " ** Discarding COLUMN line - table not defined at $INPUT_LINE_NUMBER\n";
	}
	else {
	  print " ** Discarding COLUMN line - column name not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'INDEX' ) {
	$indName = shift @inArgs;
	if ( defined $tabRef and defined $indName ) {
	  $tabRef->{ indexes }->{ $indName } = {} if not exists $tabRef->{ indexes }->{ $indName };
	  $indRef = $tabRef->{ indexes }->{ $indName };
	  $indRef->{ type } = shift @inArgs;
	  if ( defined $indRef->{ type } and $indRef->{ type } eq 'IOT-TOP' ) {
	    $tabRef->{ iot_index } = $indName;
	  }
	  $indRef->{ uniqueness } = shift @inArgs;
	  $indRef->{ tablespace } = shift @inArgs;
	  $indRef->{ compression } = shift @inArgs;
	  $indRef->{ pct_free } =shift @inArgs;
	  $indRef->{ freelists } = shift @inArgs;
	  $indRef->{ freelist_groups } = shift @inArgs;
	  $indRef->{ columns }= [];
	}
	elsif ( not defined $tabRef ) {
	  print " ** Discarding INDEX line - table not defined at $INPUT_LINE_NUMBER\n";
	}
	else {
	  print " ** Discarding INDEX line - index name not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'INDPART' ) {
	if ( defined $indRef ) {
	  $indRef->{ part_info }->{ tablespace } = shift @inArgs;
	  $indRef->{ part_info }->{ part_type } = shift @inArgs;
	  $indRef->{ part_info }->{ part_key_count } = shift @inArgs;
	  $indRef->{ part_info }->{ pct_free } = shift @inArgs;
	  $indRef->{ part_info }->{ freelists } = shift @inArgs;
	  $indRef->{ part_info }->{ freelist_groups } = shift @inArgs;
	  my $keyString = shift @inArgs;
	  if ( defined $keyString ) {
	    my @keyList = split /\;/, $keyString, -1;
	    $indRef->{ part_info }->{ part_keys } = [ @keyList ];
	  }
	}
	else {
	  print " ** Discarding INDPART line - index not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'INDCOL') {
	my $colName = shift @inArgs;
	if ( defined $indRef and defined $colName ) {
	  push @{ $indRef->{ columns } }, $colName;
	}
	elsif ( not defined $indRef ) {
	  print " ** Discarding INDCOL line - index not defined at $INPUT_LINE_NUMBER\n";
	}
	else {
	  print " ** Discarding INDCOL line - column name not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'CONSTRAINT' ) {
	$consName = shift @inArgs;
	if ( defined $tabRef and defined $consName ) {
	  $tabRef->{ constraints }->{ $consName } = {} if not exists $tabRef->{ constraints }->{ $consName };
	  $consRef = $tabRef->{ constraints }->{ $consName };
	  $consRef->{ type } = shift @inArgs;
	  $consRef->{ status } = shift @inArgs;
	  $consRef->{ refName } = shift @inArgs;
	  $consRef->{ delRule } = shift @inArgs;
	  $consRef->{ deferrable } = shift @inArgs;
	  $consRef->{ deferred } = shift @inArgs;
	  $consRef->{ search } =shift @inArgs;
	  $consRef->{ columns }= {};
	  $self->{ constraints }->{ $consName }->{ table } = $tabName;
	}
	elsif ( not defined $tabRef ) {
	  print " ** Discarding CONSTRAINT line - table not defined at $INPUT_LINE_NUMBER\n";
	}
	else {
	  print " ** Discarding CONSTRAINT line - constraint name not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'CONSCOL') {
	my $colName = shift @inArgs;
	if ( defined $consRef and defined $colName ) {
	  $consRef->{ columns }->{ $colName } = {} if not exists $consRef->{ columns }->{ $colName };
	  my $colRef = $consRef->{ columns }->{ $colName };
	  my $tabColRef = $tabRef->{ columns }->{ $colName };
	  my $position = shift @inArgs;
	  $position = $#{ $tabColRef->{ constraints } } + 2 unless defined $position;
	  if ( defined $position and $position == 0 ) {
	    push @{ $tabColRef->{ constraints } }, $consName;
	    
	    # Check for a 'NOT NULL' constraint
	    if ( $consRef->{ type } eq 'C'  and 
		 $consRef->{ search } =~ m/^\"?$colName\"\s+IS\s+NOT\s+NULL\s*/i and
		 $tabColRef->{ nullable } = 'N' ) {
	      $tabColRef->{ notNull } = $consName;
	    }
	  }
	  $colRef->{ position } = $position;
	}
	elsif ( not defined $consRef ) {
	  print " ** Discarding CONSCOL line - constraint not defined at $INPUT_LINE_NUMBER\n";
	}
	else {
	  print " ** Discarding CONSCOL line - column name not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      elsif ( $argKey eq 'SEQUENCE') {
	my $seqName = shift @inArgs;
	if ( defined $seqName ) {
	  $self->{ sequences }->{ $seqName } = { } if not exists $self->{ sequences }->{ $seqName };
	  my $seqRef = $self->{ sequences }->{ $seqName };
	  $seqRef->{ cycleFlag } = shift @inArgs;
	  $seqRef->{ orderFlag } = shift @inArgs;
	}
	else {
	  print " ** Discarding SEQUENCE line - sequence name not defined at $INPUT_LINE_NUMBER\n";
	}
      }
      else {
	
	print " ** Discarding $argKey line - unrecognized key at $INPUT_LINE_NUMBER\n";
      }
    }
    
    print "Completed reading stored configuration.\n";

    close DATAFILE;
    
    return $self;
  }


  sub writeFile {
    my $self = shift;
    my $configFile = shift;
    my $attrRef = shift;

    die 'Usage: $h->writeFile( $fileName [,\%attr])'  unless defined $configFile and $configFile and ( not $attrRef or ( ref $attrRef and ( ref $attrRef eq 'HASH' ) ) );

    # Initialize attributes so we don't have to check them too carefully later.
    $attrRef = { } unless defined $attrRef;
    $attrRef->{ ZipFile } = 0  unless exists $attrRef->{ ZipFile };
    $attrRef->{ ReplaceSysNames } = 0  unless exists $attrRef->{ ReplaceSysNames };

    my %sysNames = ();
    my $sysNameId = 0;

    if ( $attrRef->{ Zipfile }) {
      open DATAFILE, "| gzip -c >$configFile" or die "Can't open output file $configFile\n";
    } else {
      open DATAFILE, ">$configFile" or die "Can't open output file $configFile\n";
    }

    print "Writing stored configuration to $configFile \n";

    my $hostName = `hostname`;
    chomp $hostName;
    my $domainName = `domainname`;
    chomp $domainName;
    my $runDate = `date`;
    chomp $runDate;
    print  DATAFILE "#\tSchemaInf.pm 1.1.0 03/02/12 18:49:30\n";
    printf DATAFILE "#\tcreated: %s\n", $runDate;
    printf DATAFILE "#\thost = %s, domain = %s\n", $hostName, $domainName;
    printf DATAFILE "#\tsource = '%s' %s\n", $self->{ input }->{ type }, $self->{ input }->{ name };

    foreach my $tabName ( sort keys %{ $self->{ tables } } ) {
      my $tabRef = $self->{ tables }->{ $tabName };
      printf DATAFILE "TABLE\t%s\t%s\n", $tabName, $tabRef->{ type };
      if ( $tabRef->{ type } eq 'TABLE' ) {
	printf DATAFILE "TABINFO\t%s\t%s\t%s\t%s\t%s", $tabRef->{ tablespace }, $tabRef->{ monitoring }, $tabRef->{ partitioned }, $tabRef->{ iot }, $tabRef->{ temp };
	printf DATAFILE "\t%s\t%s\t%s\t%s\t%s\n", $tabRef->{ pct_free }, $tabRef->{ pct_used }, $tabRef->{ freelists }, $tabRef->{ freelist_groups }, $tabRef->{ duration };
	
	if ( exists $tabRef->{ part_info } ) {
	  my $partTabRef = $tabRef->{ part_info };
	  printf DATAFILE "TABPART\t%s\t%s\t%s", $partTabRef->{ tablespace }, $partTabRef->{ part_type }, $partTabRef->{ part_key_count };
	  printf DATAFILE "\t%s\t%s\t%s\t%s", $partTabRef->{ pct_free }, $partTabRef->{ pct_used }, $partTabRef->{ freelists }, $partTabRef->{ freelist_groups };
	  printf DATAFILE "\t%s\n", join(";", @{ $partTabRef->{ part_keys } } );
	}
      }

      #printf "%-40s\t%5i\n", $tabName, scalar( keys %{ $self->{ tables }->{ $tabName }->{ columns } } )  if $tabName eq 'WMN_INSTANCE';
      foreach my $colName ( sort keys %{ $self->{ tables }->{ $tabName }->{ columns } } ) {
	my $colRef = $self->{ tables }->{ $tabName }->{ columns }->{ $colName };
	printf DATAFILE "COLUMN\t%s\t%s\t%s\t%s\t%s\t%s\t%i\t%s\n", $colName, $colRef->{ type },
	$colRef->{ len }, $colRef->{ prec }, $colRef->{ scale }, $colRef->{ nullable }, $colRef->{ ordinal }, $colRef->{ default };
      }
      
      foreach my $indName ( sort keys %{ $self->{ tables }->{ $tabName }->{ indexes } } ) {
	my $indRef = $self->{ tables }->{ $tabName }->{ indexes }->{ $indName };
	#print "IndKeys  $tabName.$indName = ", join(", ", keys %{  $self->{ tables }->{ $tabName }->{ indexes }->{ $indName } } ), "\n";
	defined $indRef->{ type } or print "Undefined type - $tabName.$indName\n";
	defined $indRef->{ uniqueness } or print "Undefined uniqueness - $tabName.$indName\n";
	defined $indRef->{ tablespace } or print "Undefined tablespace - $tabName.$indName\n";
	defined $indRef->{ compression } or print "Undefined compression - $tabName.$indName\n";
	if ( $attrRef->{ ReplaceSysNames } and ( $indName =~ m/^SYS_C\d*$/ ) ) {
	  $sysNames{ $indName } = sprintf 'SYS_Cxxxx%05i', ++$sysNameId   unless exists $sysNames{ $indName };
	  $indName = $sysNames{ $indName };
	}
	printf DATAFILE "INDEX\t%s\t%s\t%s\t%s\t%s", $indName, $indRef->{ type }, $indRef->{ uniqueness }, $indRef->{ tablespace }, $indRef->{ compression };
	printf DATAFILE "\t%s\t%s\t%s\n", $indRef->{ pct_free }, $indRef->{ freelists }, $indRef->{ freelist_groups };
	
	if ( exists $indRef->{ part_info } ) {
	  my $partIndRef = $indRef->{ part_info };
	  printf DATAFILE "INDPART\t%s\t%s\t%s", $partIndRef->{ tablespace }, $partIndRef->{ part_type }, $partIndRef->{ part_key_count };
	  printf DATAFILE "\t%s\t%s\t%s", $partIndRef->{ pct_free }, $partIndRef->{ freelists }, $partIndRef->{ freelist_groups };
	  printf DATAFILE "\t%s\n", join(";", @{ $partIndRef->{ part_keys } } );
	}
	
	my $indCols = $indRef->{ columns };
	foreach my $indColName ( @{ $indCols } ) {
	  printf DATAFILE "INDCOL\t%s\n", $indColName;
	}
      }
      
      foreach my $consName ( sort keys %{ $self->{ tables }->{ $tabName }->{ constraints } } ) {
	my $consRef = $self->{ tables }->{ $tabName }->{ constraints }->{ $consName };
	if ( $attrRef->{ ReplaceSysNames } and ( $consName =~ m/^SYS_C\d*$/ ) ) {
	  $sysNames{ $consName } = sprintf 'SYS_Cxxxx%05i', ++$sysNameId   unless exists $sysNames{ $consName };
	  $consName = $sysNames{ $consName };
	}
	printf DATAFILE "CONSTRAINT\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", $consName, $consRef->{ type }, $consRef->{ status }, $consRef->{ refName }, 
	$consRef->{ delRule }, $consRef->{ deferrable }, $consRef->{ deferred }, $consRef->{ search };
	#my $consCols = $consRef->{ columns };
	foreach my $consColName ( sort { $consRef->{ columns }->{ $a }->{ position } <=>  $consRef->{ columns }->{ $b }->{ position }  } keys %{ $consRef->{ columns } } ) {
	  printf DATAFILE "CONSCOL\t%s\t%s\n", $consColName, $consRef->{ columns }->{ $consColName }->{ position };
	}
      }
    }
    
    foreach my $seqName ( sort keys %{ $self->{ sequences } } ) {
      my $seqRef = $self->{ sequences }->{ $seqName };
      printf DATAFILE "SEQUENCE\t%s\t%s\t%s\n", $seqName, $seqRef->{ cycleFlag }, $seqRef->{ orderFlag };
    }
    
    close DATAFILE;
    
    print "Completed writing stored configuration.\n";
  }
  
}


