###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2008,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/sw/perl/lib/pass_crypt.pm BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
###################################################################
# Module   : pass_crypt                                           #
# Function : Provides perl API to retieve information from        #
#            crypt_util.ksh CLI                                   #
###################################################################

package pass_crypt;

use strict;
require Exporter;



use vars qw(@ISA @EXPORT @EXPORT_OK );

@ISA               = qw(Exporter);
@EXPORT            = qw(getValue getEncryptedValue getValueWithDefault isValidDbConnect getConnectForUser getDBConnect getDBPasswd);
@EXPORT_OK         = qw(getValue getEncryptedValue getValueWithDefault isValidDbConnect getConnectForUser getDBConnect getDBPasswd);

###################################################################
# Moto subroutine: uses -g option to retrieve plaintext value     #
###################################################################

sub getValue ( $ ) {

    my ( $variableName ) = @_;
    my $variableValue;
    
    $variableValue = `crypt_util.ksh -g $variableName`;
    chomp $variableValue;
    
    return $variableValue;
    
}

###################################################################
# Moto subroutine: uses -e option to retrieve encrypted value     #
###################################################################

sub getEncryptedValue ( $ ) {
    my ( $variableName ) = @_;
    my $variableValue;
    
    $variableValue = `crypt_util.ksh -e $variableName`;
    chomp $variableValue;
    
    return $variableValue;
    
}

###################################################################
# Moto subroutine: uses -g option to retrieve plaintext value     #
# If this value is not defined then returns the default value     #
###################################################################

sub getValueWithDefault ( $$ ) {

    my ( $variableName, $defaultValue ) = @_;
    my $variableValue;
    
    $variableValue = getValue( $variableName );
    
    $variableValue = $defaultValue unless ($variableValue);
    
    return $variableValue;
    
}


###################################################################
# Moto subroutine: validates format of DB_CONNECT string          #
###################################################################

sub isValidDbConnect ( $ ) {

    my ( $dbConnect ) = @_;
    # here we are looking for conneciton string that starts with format: <user>/<pwd>@<db_instance>
    if( $dbConnect =~ /^\w+\/\w+@\w+/ ) {
      return 1;
      
    } else {
      return 0;
      
    }
    
}


###################################################################
# Additional Core Subroutines start here                          #
###################################################################


###################################################################
# Core Subroutine: uses -g and -c option to retrieve plaintext    #
# connection value                                                #
###################################################################

sub getConnectForUser ( $ ) {

    my ( $user ) = @_;
    my $connectValue;
    
    $connectValue = `crypt_util.ksh -g $user -c`;
    chomp $connectValue;
    
    return $connectValue;
    
}

###################################################################
# Core Subroutine: gets standard DB_CONNECT using default         #
# DB_CONNECT value                                                #
###################################################################

sub getDBConnect () {

   my $connectValue;
    
    $connectValue = `crypt_util.ksh -g DB_CONNECT`;
    chomp $connectValue;
    
    return $connectValue;
    
}

###################################################################
# Core Subroutine: gets standard DB_PASSWD using default          #
# DBPASSWD value                                                  #
###################################################################

sub getDBPasswd () {

   my $passwordValue;
    
    $passwordValue = `crypt_util.ksh -g DBPASSWD`;
    chomp $passwordValue;
    
    return $passwordValue;
    
}

1;