#!/usr/bin/env perl -w
#---------------------------------------------------------
# (#) ./base/sw/wm_monitor/monitors/wmon_psr.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
# COPYRIGHT 2003 WatchMark Corp.
#---------------------------------------------------------
#

use Env;
use Time::localtime;
use File::Basename;

my $SCRIPT_ID       = basename($0);
my $COMMENT_TAG     = "#";
my $DELIMITER       = "|";

my $MONITOR_HOME    = $ENV{"WM_MONITOR_HOME"} || ".";
my $MONITOR_SQLDIR  = $ENV{"WM_MONITOR_SQLDIR"} || "$MONITOR_HOME/sql";
my $MONITOR_LOGDIR  = $ENV{"WM_MONITOR_LOGDIR"} || "$MONITOR_HOME/logs";
my $MONITOR_SCRIPTS = $ENV{"WM_MONITOR_SCRIPTS"} || "$MONITOR_HOME/monitors";

my $MONITOR_EXECUTOR = "";
my $MONITOR_COMMAND  = "/usr/sbin/psrinfo";
my $MONITOR_OPTIONS  = "";

my $STOP_FILE      = "$MONITOR_HOME/stop";

#----------------------------------------------------------------------
sub process_line($$$)
#----------------------------------------------------------------------
{
   my $Outfile = shift;
   my $line_no = shift;
   my $xline   = shift;

   if ($line_no < 2)
   {
      # add a header before the first line
      print $Outfile $COMMENT_TAG . " "
                   . "PROCESSOR"  . $DELIMITER
                   . "STATUS"     . $DELIMITER
                   . "SPEED\n";
   }

   my @psrinfo = split(' ', $xline);

   my $clock_line = 0;
   my $clock_speed;
   my @clock_info;

   open(CLOCKSPEED, "$MONITOR_COMMAND -v|grep MHz|");
   while (<CLOCKSPEED>)
   {
      chomp;
      $clock_line++;
      if ( $clock_line == $line_no )
      {
         @clock_info = split(' ', $_, 6);
         $clock_speed = $clock_info[5];
         $clock_speed =~ s/,//g;
      }
   }
   close CLOCKSPEED;

   print $Outfile $psrinfo[0]  . $DELIMITER
                . $psrinfo[1]  . $DELIMITER
                . $clock_speed . "\n";
   
}

#----------------------------------------------------------------------
sub format_date
#----------------------------------------------------------------------
{

   my ( $day, $month, $year, $hour, $min, $sec );
   my $date_str;

   my $tm = localtime;
   ( $day, $month, $year, $hour, $min, $sec ) = ( $tm->mday, $tm->mon, $tm->year, $tm->hour, $tm->min, $tm->sec );
   $date_str = sprintf( "%04d%02d%02d:%02d:%02d:%02d", $year+1900, $month+1, $day, $hour, $min, $sec );

   return $date_str;
}

#----------------------------------------------------------------------
sub createLogDir
#----------------------------------------------------------------------
{
   if ( ! -e $MONITOR_LOGDIR )
   {
      mkdir($MONITOR_LOGDIR,"500") or die "Cannot create $MONITOR_LOGDIR: $!\n";
   }
}


#----------------------------------------------------------------------
sub header
#----------------------------------------------------------------------
{

   my ( $fh, $name, $date, $monitor, $options ) = @_;

   printf $fh "$COMMENT_TAG StartHeader\n";
   printf $fh "$COMMENT_TAG Name: %s\n", $name;
   printf $fh "$COMMENT_TAG Date: %s\n", $date;
   printf $fh "$COMMENT_TAG Monitor: %s\n", $monitor;
   printf $fh "$COMMENT_TAG Options: %s\n", $options;
   printf $fh "$COMMENT_TAG EndHeader\n";
}

#----------------------------------------------------------------------
sub error_out
#----------------------------------------------------------------------
{
   my ( $name, $date, $monitor, $options, $error_message ) = @_;
   my $log_str;

   $log_str = sprintf( "%s/%s.%s.err", $MONITOR_LOGDIR, $name, $date );
   open( LOGFILE, ( "> " . $log_str ) ) or die "Can't write $log_str: $!\n";
   $fh = *LOGFILE;
   header( $fh, $name, $date, $monitor, $options );

   print $fh "$error_message\n";
   die $error_message;
}

#----------------------------------------------------------------------
sub new_log
#----------------------------------------------------------------------
{

   my ( $name, $date, $monitor, $options ) = @_;
   my $log_str;

   error_out( $name, $date, $monitor, $options, "Process is blocked. File $STOP_FILE found" )
      if ( -e $STOP_FILE );
   
   $log_str = sprintf( "%s/%s.%s.log", $MONITOR_LOGDIR, $name, $date );
   open( LOGFILE, ( "> " . $log_str ) ) or die "Can't write $log_str: $!\n";

   $fh = *LOGFILE;

   header( $fh, $name, $date, $monitor, $options );

   return $fh;

}

#----------------------------------------------------------------------
sub main
#----------------------------------------------------------------------
{

   createLogDir;

   my $FormattedDate = format_date;
   my $LogFile = new_log($SCRIPT_ID, $FormattedDate, $MONITOR_COMMAND, $MONITOR_OPTIONS);
   my $lineNum = 0;

   open ( MONITOR, "$MONITOR_EXECUTOR $MONITOR_COMMAND $MONITOR_OPTIONS|");
   while ( <MONITOR> )
   {
      chomp;
      $lineNum++;
      process_line($LogFile, $lineNum, $_);
   
   }

   close MONITOR;
   close $LogFile;
}

exit main;
