#!/usr/bin/env perl -w
#---------------------------------------------------------
# (#) ./base/sw/wm_monitor/monitors/wmon_rglogs.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
# COPYRIGHT 2003 WatchMark Corp.
#---------------------------------------------------------
#

use Env;
use Time::localtime;
use File::Basename;

my $SCRIPT_ID       = basename($0);
my $COMMENT_TAG     = "#";
my $DELIMITER       = "|";

my $FLEXPM_HOME = $ENV{"FLEXPM_HOME"} || '';

my $MONITOR_HOME    = $ENV{"WM_MONITOR_HOME"} || ".";
my $MONITOR_SQLDIR  = $ENV{"WM_MONITOR_SQLDIR"} || "$MONITOR_HOME/sql";
my $MONITOR_LOGDIR  = $ENV{"WM_MONITOR_LOGDIR"} || "$MONITOR_HOME/logs";
my $MONITOR_SCRIPTS = $ENV{"WM_MONITOR_SCRIPTS"} || "$MONITOR_HOME/monitors";

# The Monitor command vars will be set by setCommand()
#
my $MONITOR_EXECUTOR = "";
my $MONITOR_COMMAND  = "";
my $MONITOR_OPTIONS  = "";

my $STOP_FILE      = "$MONITOR_HOME/stop";

#----------------------------------------------------------------------
sub process_line($$$$)
#----------------------------------------------------------------------
{
   my $Outfile  = shift;
   my $line_no  = shift;
   my $xline    = shift;
   my $ExecDate = shift;
   my @axline;

   my $filename = $xline;
   my $start_time;
   my $end_time;
   my $slashdate;

   if ($line_no < 2)
   {
      # comment out the first line
      print $Outfile "$COMMENT_TAG LOGFILE|START_TIME|END_TIME\n";
   }

   $xline = `head -3 $filename|grep "wmtm\.greetings"|head -1`;

   $xline =~ s/\[.*\]//g;
   @axline = split(' ', $xline);

  if ( $axline[1] eq $ExecDate )
  {
      $start_time = "$axline[1] $axline[2]";
      $start_time =~ s/:$//g;

      $xline = `grep "Processing finished" $filename|head -1`;

      if ($xline ne '')
      {
         $xline =~ s/\[.*\]//g;
         @axline = split(' ', $xline);

         $end_time = "$axline[1] $axline[2]";
         $end_time =~ s/:$//g;
      }
      else
      {
         $end_time = "";
      }

      print $Outfile basename($filename) . $DELIMITER
                   . $start_time         . $DELIMITER
                   . $end_time           . "\n";
  }

}

#----------------------------------------------------------------------
sub setCommand($)
#----------------------------------------------------------------------
{
   my $parse_str = shift;
   my $rg_logs;

   $MONITOR_EXECUTOR = "grep -l";
   $MONITOR_COMMAND  = "\"$parse_str\"";
   $SCRIPT_ID = $SCRIPT_ID . ":" . $parse_str;

   if ( $parse_str eq "BH_Clean"              ||
        $parse_str eq "summaryClean.CleanAll" ||
        $parse_str eq "part_maint.MaintainDaily"
      )
   {
      $rg_logs="$FLEXPM_HOME/rg/log/wmtm-20-*.log";
   }
   elsif ($parse_str eq "summarize.SummarizeAll")
   {
     $rg_logs="$FLEXPM_HOME/rg/log/wmtm-25-*.log";
   }
   elsif ($parse_str eq "bhupdate")
   {
     $MONITOR_EXECUTOR = "ls";
     $rg_logs="$FLEXPM_HOME/rg/log/wmtm-30-*.log";
   }
   else
   {
     $rg_logs="$FLEXPM_HOME/rg/log/wmtm-*.log";
   }
   $MONITOR_OPTIONS  = "$rg_logs 2>/dev/null";
}

#----------------------------------------------------------------------
sub format_date($$)
#----------------------------------------------------------------------
{

   my $day_diff = shift;
   my $option   = shift;
   my ( $day, $month, $year, $hour, $min, $sec );
   my $date_str;

   my $tm = localtime(time + ($day_diff * 24 * 60 * 60));
   ( $day, $month, $year, $hour, $min, $sec ) = ( $tm->mday, $tm->mon, $tm->year, $tm->hour, $tm->min, $tm->sec );

   if ( $option eq '')
   {
      $date_str = sprintf( "%04d%02d%02d:%02d:%02d:%02d", $year+1900, $month+1, $day, $hour, $min, $sec );
   }
   elsif ($option eq 'SLASH-DATE')
   {
      $date_str = sprintf( "%04d/%02d/%02d", $year+1900, $month+1, $day );
   }
   

   return $date_str;
}

#----------------------------------------------------------------------
sub createLogDir
#----------------------------------------------------------------------
{
   if ( ! -e $MONITOR_LOGDIR )
   {
      mkdir($MONITOR_LOGDIR,"500") or die "Cannot create $MONITOR_LOGDIR: $!\n";
   }
}


#----------------------------------------------------------------------
sub header
#----------------------------------------------------------------------
{

   my ( $fh, $name, $date, $monitor, $options ) = @_;

   printf $fh "$COMMENT_TAG StartHeader\n";
   printf $fh "$COMMENT_TAG Name: %s\n", $name;
   printf $fh "$COMMENT_TAG Date: %s\n", $date;
   printf $fh "$COMMENT_TAG Monitor: %s\n", $monitor;
   printf $fh "$COMMENT_TAG Options: %s\n", $options;
   printf $fh "$COMMENT_TAG EndHeader\n";
}

#----------------------------------------------------------------------
sub error_out
#----------------------------------------------------------------------
{
   my ( $name, $date, $monitor, $options, $error_message ) = @_;
   my $log_str;

   $log_str = sprintf( "%s/%s.%s.err", $MONITOR_LOGDIR, $name, $date );
   open( LOGFILE, ( "> " . $log_str ) ) or die "Can't write $log_str: $!\n";
   $fh = *LOGFILE;
   header( $fh, $name, $date, $monitor, $options );

   print $fh "$error_message\n";
   die $error_message;
}

#----------------------------------------------------------------------
sub new_log
#----------------------------------------------------------------------
{

   my ( $name, $date, $monitor, $options ) = @_;
   my $log_str;

   error_out( $name, $date, $monitor, $options, "Process is blocked. File $STOP_FILE found" )
      if ( -e $STOP_FILE );
   
   $log_str = sprintf( "%s/%s.%s.log", $MONITOR_LOGDIR, $name, $date );
   open( LOGFILE, ( "> " . $log_str ) ) or die "Can't write $log_str: $!\n";

   $fh = *LOGFILE;

   header( $fh, $name, $date, $monitor, $options );

   return $fh;

}

#----------------------------------------------------------------------
sub main
#----------------------------------------------------------------------
{

   createLogDir;

   my $FormattedDate = format_date(0, '');
   my $SlashDate = format_date(-1, 'SLASH-DATE');

   error_out($SCRIPT_ID, $FormattedDate, $MONITOR_COMMAND, $MONITOR_OPTIONS, "String to parse not passed.")
      if ($#ARGV < 0);
   setCommand($ARGV[0]);

   my $LogFile = new_log($SCRIPT_ID, $FormattedDate, $MONITOR_COMMAND, $MONITOR_OPTIONS);
   my $lineNum = 0;

   error_out($SCRIPT_ID, $FormattedDate, $MONITOR_COMMAND, $MONITOR_OPTIONS, "FLEXPM_HOME is not set.")
      if ($FLEXPM_HOME eq '');

   open ( MONITOR, "$MONITOR_EXECUTOR $MONITOR_COMMAND $MONITOR_OPTIONS|");
   while ( <MONITOR> )
   {
      chomp;
      $lineNum++;
      process_line($LogFile, $lineNum, $_, $SlashDate);
   
   }

   close MONITOR;
   close $LogFile;
}

exit main;
