#!/usr/bin/env perl -w
#---------------------------------------------------------
# (#) ./base/sw/wm_monitor/monitors/wmon_sql.pl BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
# COPYRIGHT 2003 WatchMark Corp.
#---------------------------------------------------------
#

#Updated 12/02/2009 - valnt00077479
use pass_crypt;

use Env;
use Time::localtime;
use File::Basename;

my $SCRIPT_ID       = basename($0);
my $COMMENT_TAG     = "#";
my $DELIMITER       = "|";

my $DB_CONNECT =pass_crypt::getConnectForUser($ENV{DBUSER});

my $MONITOR_HOME    = $ENV{"WM_MONITOR_HOME"} || ".";
my $MONITOR_SQLDIR  = $ENV{"WM_MONITOR_SQLDIR"} || "$MONITOR_HOME/sql";
my $MONITOR_LOGDIR  = $ENV{"WM_MONITOR_LOGDIR"} || "$MONITOR_HOME/logs";
my $MONITOR_SCRIPTS = $ENV{"WM_MONITOR_SCRIPTS"} || "$MONITOR_HOME/monitors";

my $MONITOR_EXECUTOR = "sqlplus -S $DB_CONNECT";
my $MONITOR_COMMAND  = "";  #<-- must be provided at command line
my $MONITOR_OPTIONS  = "";

my $STOP_FILE      = "$MONITOR_HOME/stop";

#----------------------------------------------------------------------
sub process_line($$$)
#----------------------------------------------------------------------
{
   my $Outfile = shift;
   my $line_no = shift;
   my $xline   = shift;

   if ($line_no < 2)
   {
      # comment out the first line
      $xline = "$COMMENT_TAG $xline";
   }

   print $Outfile "$xline\n";
}

#----------------------------------------------------------------------
sub DBConnection_OK
#----------------------------------------------------------------------
{
   my $TestStatus = 1;

   if ( $DB_CONNECT eq 'NO-CONNECTION' )
   {
      $TestStatus = 0;
   }
   else
   {
      my $TestConnect = `sqlplus -s $DB_CONNECT <<EOF
         exit 0
      EOF`;
      $TestStatus=$?;

      if ( $TestStatus ne 0 || $TestConnect =~ /SP2/)
      {
         $TestStatus = 0
      }
      else
      {
         $TestStatus = 1;
      }
   }
   return $TestStatus;

}


#----------------------------------------------------------------------
sub format_date
#----------------------------------------------------------------------
{

   my ( $day, $month, $year, $hour, $min, $sec );
   my $date_str;

   my $tm = localtime;
   ( $day, $month, $year, $hour, $min, $sec ) = ( $tm->mday, $tm->mon, $tm->year, $tm->hour, $tm->min, $tm->sec );
   $date_str = sprintf( "%04d%02d%02d:%02d:%02d:%02d", $year+1900, $month+1, $day, $hour, $min, $sec );

   return $date_str;
}

#----------------------------------------------------------------------
sub createLogDir
#----------------------------------------------------------------------
{
   if ( ! -e $MONITOR_LOGDIR )
   {
      mkdir($MONITOR_LOGDIR,"500") or die "Cannot create $MONITOR_LOGDIR: $!\n";
   }
}


#----------------------------------------------------------------------
sub header
#----------------------------------------------------------------------
{

   my ( $fh, $name, $date, $monitor, $options ) = @_;

   printf $fh "$COMMENT_TAG StartHeader\n";
   printf $fh "$COMMENT_TAG Name: %s\n", $name;
   printf $fh "$COMMENT_TAG Date: %s\n", $date;
   printf $fh "$COMMENT_TAG Monitor: %s\n", $monitor;
   printf $fh "$COMMENT_TAG Options: %s\n", $options;
   printf $fh "$COMMENT_TAG EndHeader\n";
}

#----------------------------------------------------------------------
sub error_out
#----------------------------------------------------------------------
{
   my ( $name, $date, $monitor, $options, $error_message ) = @_;
   my $log_str;

   $log_str = sprintf( "%s/%s.%s.err", $MONITOR_LOGDIR, $name, $date );
   open( LOGFILE, ( "> " . $log_str ) ) or die "Can't write $log_str: $!\n";
   $fh = *LOGFILE;
   header( $fh, $name, $date, $monitor, $options );

   print $fh "$error_message\n";
   die $error_message;
}

#----------------------------------------------------------------------
sub new_log
#----------------------------------------------------------------------
{

   my ( $name, $date, $monitor, $options ) = @_;
   my $log_str;

   error_out( $name, $date, $monitor, $options, "Process is blocked. File $STOP_FILE found" )
      if ( -e $STOP_FILE );
   
   $log_str = sprintf( "%s/%s.%s.log", $MONITOR_LOGDIR, $name, $date );
   open( LOGFILE, ( "> " . $log_str ) ) or die "Can't write $log_str: $!\n";

   $fh = *LOGFILE;

   header( $fh, $name, $date, $monitor, $options );

   return $fh;

}

#----------------------------------------------------------------------
sub main
#----------------------------------------------------------------------
{

   my $SQLFile = $ARGV[0];

   createLogDir;
   my $FormattedDate = format_date;

   error_out($SCRIPT_ID, $FormattedDate, $MONITOR_COMMAND, $MONITOR_OPTIONS, "SQL file not passed")
      if ($#ARGV < 0);

   #
   # Get the SQL file to run from the command line
   #
   if (-e $SQLFile)
   {
      $MONITOR_COMMAND = "\@" . $SQLFile;
   }
   elsif (-e "$MONITOR_SQLDIR/$SQLFile")
   {
      $MONITOR_COMMAND = "\@" . "$MONITOR_SQLDIR/$SQLFile";
   }
   else
   {
      error_out($SCRIPT_ID, $FormattedDate, $MONITOR_COMMAND, $MONITOR_OPTIONS, "$SQLFile file not found");
   }
   $SCRIPT_ID = "$SCRIPT_ID:" . basename($MONITOR_COMMAND);

   error_out($SCRIPT_ID, $FormattedDate, $MONITOR_COMMAND, $MONITOR_OPTIONS, "Cannot connect to Oracle")
      if (! DBConnection_OK);

   my $LogFile = new_log($SCRIPT_ID, $FormattedDate, $MONITOR_COMMAND, $MONITOR_OPTIONS);
   my $lineNum = 0;

   open ( MONITOR, "$MONITOR_EXECUTOR $MONITOR_COMMAND $MONITOR_OPTIONS|");
   while ( <MONITOR> )
   {
      chomp;
      $lineNum++;
      process_line($LogFile, $lineNum, $_);
   
   }

   close MONITOR;
   close $LogFile;
}

exit main;
