#! /bin/ksh
# @(#) ./base/tools/popdata/pop_data.sh ; BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
############################################################################
# The following script populates the traffic table(s) with simulation data
############################################################################

SCRIPTNAME=`basename $0`
usage() {
    echo "$SCRIPTNAME -e entity_name -f fullkey -d start_date end_date"
    echo " -e: entity name"
    echo " -f: instance fullkey"
    echo " -d: date range with format YYYYMMDDHHMI"
    echo " example:"
    echo "  $ pop_data.sh -e Cell -f 1-2-3 -d 200301010000 200301010100"
    exit 1
}

if  [ ! $# -eq 7 ]
then
  usage 
  exit 1
fi

while [ $# -gt 0 ]
do
case $1 in
   -e)
        shift;
        ENTITY_NAME="'$1'"
        ;;
   -f)
        shift;
        FULLKEY="'$1'"
        ;;
   -d)
        shift;
        START="'$1'"
        shift;
        END="'$1'"
        ;;
   *)
        usage
        ;;
esac
shift
done

#echo $ENTITY_NAME
#echo $FULLKEY
#echo $START
#echo $END

date

sqlplus -s /NOLOG << !
whenever sqlerror exit sql.sqlcode
set feed off
set serveroutput on size 1000000
connect $DBUSER/$DBPASSWD@$DBSID
declare
 vSQL varchar2(400);
 table_not_exists exception;
 object_exists exception;
 seq_not_exists exception;
 PRAGMA EXCEPTION_INIT(table_not_exists, -942);
 PRAGMA EXCEPTION_INIT(object_exists, -955);
 PRAGMA EXCEPTION_INIT(seq_not_exists, -2289);
begin
 begin -- drop table pop_ent_data_ins_tmp
   vSQL := 'drop table pop_ent_data_ins_tmp';
   execute immediate vSQL;
 exception
   when table_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_ent_data_ins_tmp

 begin -- drop table pop_time_slice_tmp
   vSQL := 'drop table pop_time_slice_tmp';
   execute immediate vSQL;
 exception
   when table_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_time_slice_tmp

 begin -- create table pop_ent_data_ins_tmp
   vSQL := 'create table pop_ent_data_ins_tmp '||
           '(instance_id integer,'||
           'entity_id integer,'||
           'fullkey   varchar2(360))';
   execute immediate vSQL;
 exception
   when object_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- create table pop_ent_data_ins_tmp

 begin -- create table pop_time_slice_tmp
   vSQL := 'create table pop_time_slice_tmp '||
           '(tstamp date )';
   execute immediate vSQL;
 exception
   when object_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- create table pop_time_slice_tmp

 begin -- drop sequence sq_popEntData
   vSQL := 'drop sequence sq_popEntData';
   execute immediate vSQL;
 exception
   when seq_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_time_slice_tmp

 begin -- create sequence sq_popEntData
   vSQL := 'create sequence sq_popEntData start with 1 maxvalue 99 cycle';
   execute immediate vSQL;
 exception
   when object_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- create table pop_time_slice_tmp
 
end;
/
EXIT 0
!

# create package
sqlplus -s /NOLOG << !
whenever sqlerror exit sql.sqlcode
set feed off
set serveroutput on size 1000000
connect $DBUSER/$DBPASSWD@$DBSID
@popentity_pkg.sql
!

# create package body
sqlplus -s /NOLOG << !
whenever sqlerror exit sql.sqlcode
set feed off
set serveroutput on size 1000000
connect $DBUSER/$DBPASSWD@$DBSID
@popentity_pkgb.sql
!

sqlplus -s /NOLOG << !
whenever sqlerror exit sql.sqlcode
set feed off
set serveroutput on size 1000000
connect $DBUSER/$DBPASSWD@$DBSID
begin
null;
POPULATE_ENTITY.pop_ent_data($ENTITY_NAME, $FULLKEY,
      to_date($START,'YYYYMMDDHH24MI'),
      to_date($END,'YYYYMMDDHH24MI'));
end;
/
EXIT 0
!


sqlplus -s /NOLOG << !
whenever sqlerror exit sql.sqlcode
set feed off
set serveroutput on size 1000000
connect $DBUSER/$DBPASSWD@$DBSID
declare
 vSQL varchar2(400);
 table_not_exists exception;
 object_exists exception;
 seq_not_exists exception;
 PRAGMA EXCEPTION_INIT(table_not_exists, -942);
 PRAGMA EXCEPTION_INIT(seq_not_exists, -2289);
begin
 begin -- drop table pop_ent_data_ins_tmp
   vSQL := 'drop table pop_ent_data_ins_tmp';
   execute immediate vSQL;
 exception
   when table_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_ent_data_ins_tmp

 begin -- drop table pop_time_slice_tmp
   vSQL := 'drop table pop_time_slice_tmp';
   execute immediate vSQL;
 exception
   when table_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_time_slice_tmp

 begin -- drop sequence sq_popEntData
   vSQL := 'drop sequence sq_popEntData';
   execute immediate vSQL;
 exception
   when seq_not_exists then
    null;
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop table pop_time_slice_tmp

 begin -- drop package populate_entity
   vSQL := 'drop package populate_entity';
   execute immediate vSQL;
 exception
   when others then
    dbms_output.put_line(SQLERRM);
 end; -- drop package populate_entity
end;
/
EXIT 0
!
date
