#!/bin/ksh
#-----------------------------------------------------------------------
# (#) ./base/tools/wm_consumer/scripts/load_set.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
# Copyright 2003 WatchMark Corp.
#-----------------------------------------------------------------------
#

MYNAME=$(basename $0)
MYDIR=$(pwd)
SCRIPTDIR=$(dirname $0)

SCRIPTLIST="SCRIPTLIST"
SELECTOR="script_selector.pl"

OKDIR="ok"
FAILDIR="fail"
SITEFILE="SITE_INFO"

#----------------------------------------------------------------------
error_out()
#----------------------------------------------------------------------
{
   echo "$MYNAME: $1" >&2
   exit ${2:-9}
}

#----------------------------------------------------------------------
check_dir()
#----------------------------------------------------------------------
{
   if [ ! -d $1 ]
   then
      mkdir $1
      if [ $? -ne 0 ]
      then
         error_out "Cannot create $1"
      fi
   fi
}

#----------------------------------------------------------------------
check_script()
#----------------------------------------------------------------------
{
   if [ ! -f $1 ]
   then
      error_out "Cannot find $1"
   elif [ ! -x $1 ]
   then
      error_out "Cannot execute $1"
   fi
}

#----------------------------------------------------------------------
get_fullpath()
#----------------------------------------------------------------------
{
   CURDIR=`pwd`
   cd $1
   FP=`pwd`
   cd $CURDIR
   echo $FP
}


#===========================================================================
# M A I N
#===========================================================================
#

#----------------------------------------------------------------------
# Test List file
#----------------------------------------------------------------------
if [ $# -lt 1 ]
then
   error_out "Usage: $MYNAME <list-file>"
fi

LISTNAME=$(basename $1);
LISTDIR=`get_fullpath $(dirname $1)`

if [ ! -f $LISTDIR/$LISTNAME ]
then
   error_out "Cannot find archive $LISTDIR/$LISTNAME"
fi

#----------------------------------------------------------------------
# Test required scripts
#----------------------------------------------------------------------

#----------------------------------------------------------------------
# Test required files/directories
#----------------------------------------------------------------------
SITEDIR=`get_fullpath $LISTDIR/..`
SCRIPTDIR=`get_fullpath $SCRIPTDIR`

if [ ! -f $SITEDIR/$SITEFILE ]
then
   error_out "Cannot find $SITEDIR/$SITEFILE"
fi

if [ ! -f $SCRIPTDIR/$SCRIPTLIST ]
then
   error_out "Cannot find $SCRIPTDIR/$SCRIPTLIST" 
fi

check_script $SCRIPTDIR/$SELECTOR

OKDIR=$SITEDIR/$OKDIR
FAILDIR=$SITEDIR/$FAILDIR
check_dir $OKDIR
check_dir $FAILDIR

#----------------------------------------------------------------------
# get SET TSTAMP
#----------------------------------------------------------------------
SETTSTAMP=$(echo $LISTNAME| cut -f2 -d".")
### ADD CHECK HERE!

#----------------------------------------------------------------------
# Process each file
#----------------------------------------------------------------------
grep ".*\.log" $LISTDIR/$LISTNAME | \
while read lfile
do
   logfile=$(basename $lfile)

   consumer=`eval $SCRIPTDIR/$SELECTOR $LISTDIR/$logfile`
   if [ $? -ne 0 ]
   then

      echo "FAILED: $logfile"
      echo "$logfile" >> $FAILDIR/${LISTNAME}.fail
      if [ $LISTDIR != $FAILDIR ]
      then
         mv $LISTDIR/$logfile $FAILDIR
      fi

   else

      eval $SCRIPTDIR/$consumer --logfile $LISTDIR/$logfile --settstamp $SETTSTAMP

      if [ $? -eq 0 ]
      then

         echo "OK:     $logfile"
         echo "$logfile" >> $OKDIR/${LISTNAME}.ok
         if [ $LISTDIR != $OKDIR ]
         then
            mv $LISTDIR/$logfile $OKDIR
         fi

      else

         echo "FAILED: $logfile"
         echo "$logfile" >> $FAILDIR/${LISTNAME}.fail
         if [ $LISTDIR != $FAILDIR ]
         then
            mv $LISTDIR/$logfile $FAILDIR
         fi

      fi

   fi
done
